-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.name = "XMacros"
XMacros.slashName = "xm"
XMacros.version = "v1.3"
XMacros.x = 500
XMacros.y = 500
XMacros.xTooltip = 50
XMacros.yTooltip = 50
XMacros.xMacros = -1
XMacros.yMacros = -1
XMacros.xBars = -1
XMacros.yBars = -1
XMacros.startup = false
XMacros.inCombat = false
XMacros.barEditMode = false
XMacros.isOff = false
XMacros.currentRole = "0"
XMacros.additionalIcons = {}
XMacros.roleSets = {}
XMacros.MaxRoles = 20

--
-- Main functions
--

function XMacros.Create()
	local parent = UI.CreateContext(XMacros.name .. "Context")
	parent:SetSecureMode("restricted")
	XMacros.parentFrame = UI.CreateFrame("Frame", XMacros.name .. "Parent", parent);
	XMacros.parentFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", XMacros.x, XMacros.y)
	XMacros.parentFrame:SetSecureMode("restricted")
	
	XMacros.manager = XMacros.MacroManager.Create(XMacros.macros)
	XMacros.tooltip = XMacros.Tooltip.Create(XMacros.parentFrame, UIParent, XMacros.xTooltip, XMacros.yTooltip, function(a, x, y) XMacros.TooltipMoved(x, y) end)
end

function XMacros.PostCreate()
	local roleSets = XMacros.roleSets[XMacros.playerName]
	if not roleSets then
		roleSets = {}
	end
	
	for role = 1, XMacros.MaxRoles do
		local roleKey = "Role" .. role
		local currentRole = XMacros.roleSets[roleKey]
		if currentRole then
			roleSets[roleKey] = currentRole
			XMacros.roleSets[roleKey] = nil
		end
	end
	
	XMacros.buttonSets = XMacros.ButtonSets.Create(XMacros.parentFrame, XMacros.sets, roleSets, XMacros.tooltip, function(category, name) return XMacros.MacroManager.GetMacro(XMacros.manager, category, name) end)
	
	XMacros.currentRole = XMacros.GetRoleNumber()
	local currentSets = XMacros.ButtonSets.GetRoleSet(XMacros.buttonSets, XMacros.currentRole)
	if currentSets and #currentSets > 0 then
		XMacros.ButtonSets.EnableSets(XMacros.buttonSets, currentSets)
		XMacros.ButtonSets.MakeSetsVisible(XMacros.buttonSets, currentSets, true, XMacros.inCombat)
	end	
	
	XMacros.sets = nil
	XMacros.macros = nil
end

function XMacros.Current(arg)
	XMacros.HideAll()

	if XMacros.isOff == false then
		local setString = ""
		if arg then
			for i=2, #arg do
				setString = setString .. XMacros.Utils.Trim(arg[i])
			end
		end
		
		local setNames = nil
		if string.len(setString) > 0 then
			setNames = XMacros.Utils.Split(setString, ',')
			XMacros.ButtonSets.SetRoleSet(XMacros.buttonSets, XMacros.currentRole, setNames)
		end
		
		setNames = XMacros.ButtonSets.GetRoleSet(XMacros.buttonSets, XMacros.currentRole)
		XMacros.ButtonSets.EnableSets(XMacros.buttonSets, setNames)
		XMacros.ButtonSets.MakeSetsVisible(XMacros.buttonSets, setNames, true, XMacros.inCombat)
	end

	XMacros.Tooltip.HideTooltip(XMacros.tooltip)
end

function XMacros.HideAll()
	local sets = XMacros.ButtonSets.GetSets(XMacros.buttonSets)
	if sets then
		local setNames = {}
		for name, _ in pairs(sets) do
			table.insert(setNames, name)
		end
		
		XMacros.ButtonSets.MakeSetsVisible(XMacros.buttonSets, setNames, false, false)
	end
	
	XMacros.ButtonSets.EnableSets(XMacros.buttonSets, {})
	XMacros.Tooltip.HideTooltip(XMacros.tooltip)
end

function XMacros.GetRoleNumber()
	if Inspect.TEMPORARY ~= nil and Inspect.TEMPORARY.Role ~= nil then
		local ret = Inspect.TEMPORARY.Role()
		if ret ~= nil then
			return tostring(ret)
		end
	end
	
	return "0"
end

function XMacros.EditComplete(buttonBar, row, col, results)
	if results then
		if results.iconPath then
			XMacros.ButtonSets.SetTexture(XMacros.buttonSets, buttonBar.setName, buttonBar.barName, row, col, results.iconAddon, results.iconPath)
		end
		
		if results.abilityName then
			XMacros.ButtonSets.SetAbilityName(XMacros.buttonSets, buttonBar.setName, buttonBar.barName, row, col, results.abilityName)
		end
		
		if results.macros then
			for macroButton, macro in pairs(results.macros) do
				XMacros.ButtonSets.SetMacro(XMacros.buttonSets, buttonBar.setName, buttonBar.barName, row, col, macroButton, macro.category, macro.macro)
			end
		end
		
		if results.iconSize and results.iconSize > 0 then
			XMacros.ButtonSets.SetBarSize(XMacros.buttonSets, buttonBar.setName, buttonBar.barName, results.iconSize)
		end
		
		XMacros.RefreshEditBars()
	end
end

function XMacros.EditCallback(buttonBar, row, col)
	if not XMacros.buttonEditor then
		XMacros.buttonEditor = XMacros.ButtonEditor.Create(XMacros.parentFrame)
	end
	
	local fullCategoryNames = {}
	local fullMacroNames = {}
	local categoryNames = XMacros.MacroManager.GetCategoryNames(XMacros.manager)
	for _, categoryName in ipairs(categoryNames) do
		local macroNames = XMacros.MacroManager.GetMacroNames(XMacros.manager, categoryName)
		for _, macroName in ipairs(macroNames) do
			table.insert(fullCategoryNames, categoryName)
			table.insert(fullMacroNames, macroName)
		end
	end
	
	local macroNames = XMacros.MacroManager.GetMacroNames(XMacros.manager, XMacros.MacroManager.AbilityCategory)
	for _, macroName in ipairs(macroNames) do
		table.insert(fullCategoryNames, XMacros.MacroManager.AbilityCategory)
		table.insert(fullMacroNames, macroName)
	end
	
	XMacros.ButtonEditor.Show(XMacros.buttonEditor, XMacros.buttonSets, buttonBar, row, col, fullCategoryNames, fullMacroNames, function(a, results) XMacros.EditComplete(buttonBar, row, col, results) end, XMacros.RefreshEditBars)
end

function XMacros.EditBarsComplete()
	XMacros.Tooltip.SetDragEnabled(XMacros.tooltip, false)
	XMacros.ButtonEditor.WindowClose(XMacros.buttonEditor)
	XMacros.barEditMode = false
	XMacros.currentRole = XMacros.GetRoleNumber()
	XMacros.Current()
end

function XMacros.RefreshEditBars(role)
	if role ~= nil then
		XMacros.currentRole = role
	end

	local setNames = XMacros.ButtonSets.GetRoleSet(XMacros.buttonSets, XMacros.currentRole)
	XMacros.HideAll()
	XMacros.ButtonSets.EnableSets(XMacros.buttonSets, setNames, XMacros.EditCallback)
	XMacros.ButtonSets.MakeSetsDragable(XMacros.buttonSets, setNames, true)
	XMacros.Tooltip.SetDragEnabled(XMacros.tooltip, true)
end

function XMacros.EditBars()
	XMacros.barEditMode = true
	
	if not XMacros.setEditor then
		XMacros.setEditor = XMacros.SetEditor.Create(XMacros.parentFrame, XMacros.xBars, XMacros.yBars, XMacros.buttonSets, XMacros.RefreshEditBars, XMacros.EditBarsComplete, function(a, x, y) XMacros.BarsMoved(x, y) end)
	end
	
	XMacros.RefreshEditBars(XMacros.currentRole)
	XMacros.SetEditor.Show(XMacros.setEditor, XMacros.currentRole)
end

function XMacros.EditMacrosComplete()
	if XMacros.barEditMode == false then
		XMacros.Current()
	end
end

function XMacros.EditMacros()
	if not XMacros.macroEditor then
		XMacros.macroEditor = XMacros.MacroEditor.Create(XMacros.parentFrame, XMacros.EditMacrosComplete, XMacros.xMacros, XMacros.yMacros, function(a, x, y) XMacros.MacrosMoved(x, y) end)
	end
	XMacros.MacroEditor.Show(XMacros.macroEditor, XMacros.manager)
end

function XMacros.TooltipMoved(x, y)
	XMacros.xTooltip = x
	XMacros.yTooltip = y
end

function XMacros.MacrosMoved(x, y)
	XMacros.xMacros = x
	XMacros.yMacros = y
end

function XMacros.BarsMoved(x, y)
	XMacros.xBars = x
	XMacros.yBars = y
end

function XMacros.SplitArgs(str)
	local result = {}
	local count = 1
	for token in string.gmatch(str, "[^%s]+") do
		result[count] = token
		count = count + 1
	end
	return result
end

function XMacros.GetAdditionalIcons()
	return XMacros.additionalIcons
end

function XMacros.RemoveAdditionalIcon(name)
	XMacros.additionalIcons[name] = nil
end

--
-- Event Handlers
--
function XMacros.SetInCombat()
	if XMacros.inCombat ~= true then
		XMacros.inCombat = true
		if XMacros.barEditMode ~= true and XMacros.isOff ~= true then
			local setNames = XMacros.ButtonSets.GetRoleSet(XMacros.buttonSets, XMacros.currentRole)
			XMacros.ButtonSets.MakeSetsVisible(XMacros.buttonSets, setNames, true, XMacros.inCombat)
		end
	end
end

function XMacros.SetOutCombat()
	if XMacros.inCombat == true then
		XMacros.inCombat = false
		if XMacros.barEditMode ~= true and XMacros.isOff ~= true then
			local setNames = XMacros.ButtonSets.GetRoleSet(XMacros.buttonSets, XMacros.currentRole)
			XMacros.ButtonSets.MakeSetsVisible(XMacros.buttonSets, setNames, true, XMacros.inCombat)
		end
	end
end

function XMacros.SetRole(roleNumber)
	if Inspect.System.Secure() == true then
		XMacros.RoleChanged(roleNumber)
	else
		if roleNumber ~= nil and XMacros.barEditMode == false then
			XMacros.currentRole = tostring(roleNumber)
			XMacros.Current()
		end
	end
end

function XMacros.RoleChanged(roleNumber)
	XMacros.Timer.AddCallback("XMacros.RoleChangedTimer", 0.2, function(frameTime) XMacros.SetRole(roleNumber) end, false)
end

function XMacros.Initialise(addon)
	if addon == XMacros.name then
		XMacros.startup = true
		XMacros.Create()
	end
end

function XMacros.Startup()
	if XMacros.startup == true then
		if XMacros.AbilityManager then	
			XMacros.startup = false
			XMacros.inCombat = Inspect.System.Secure()
			
			local playerDetails = Inspect.Unit.Detail("player")
			XMacros.playerName = playerDetails.name
			
			XMacros.PostCreate()

			XMacros.RegisterPostStartupEvents()

			for i, hook in ipairs(Event.System.Update.Begin) do
				if hook == XMacros.StartupHook then
					table.remove(Event.System.Update.Begin, i)
					return
				end
			end
		end
	end
end

XMacros.StartupHook = { XMacros.Startup, XMacros.name, "XMacros.Startup" }

function XMacros.SlashHandler(args)
	local arg = XMacros.SplitArgs(args)
	
	if arg[1] == "current" then
		XMacros.isOff = false
		XMacros.Current(arg)
	elseif arg[1] == "off" then
		XMacros.isOff = true
		XMacros.HideAll()
	elseif arg[1] == "on" then
		XMacros.isOff = false
		XMacros.Current()
	elseif arg[1] == "bars" then
		XMacros.EditBars()
	elseif arg[1] == "macros" then
		XMacros.EditMacros(arg)
	else
		print("Usage: xm [ current [ setnames ] | off | on | bars | macros ]")
		return
	end
end

function XMacros.SaveVariables(addon)
	if addon == XMacros.name then
		-- now copy saved group to settings so that they can be preserved on logout
		XMacros_SavedVariables = {}
		XMacros_SavedVariables.version = XMacros.version
		XMacros_SavedVariables.x = XMacros.x
		XMacros_SavedVariables.y = XMacros.y
		XMacros_SavedVariables.xTooltip = XMacros.xTooltip
		XMacros_SavedVariables.yTooltip = XMacros.yTooltip
		XMacros_SavedVariables.xMacros = XMacros.xMacros
		XMacros_SavedVariables.yMacros = XMacros.yMacros
		XMacros_SavedVariables.xBars = XMacros.xBars
		XMacros_SavedVariables.yBars = XMacros.yBars
		XMacros_SavedVariables.currentRole = XMacros.currentRole
		XMacros_SavedVariables.additionalIcons = XMacros.additionalIcons
	
		XMacros_SavedVariables.sets = XMacros.ButtonSets.GetSets(XMacros.buttonSets)
		XMacros_SavedVariables.macros = XMacros.MacroManager.GetMacros(XMacros.manager)
		
		if not XMacros.roleSets then
			XMacros.roleSets = {}
		end
		
		XMacros.roleSets[XMacros.playerName] = XMacros.ButtonSets.GetRoleSets(XMacros.buttonSets)
		XMacros_SavedVariables.roleSets = XMacros.roleSets
	end
end

function XMacros.GetSavedValue(value, default)
	if value ~= nil then
		return value
	else
		return default
	end
end

function XMacros.LoadVariables(addon)
	if addon == XMacros.name then
		-- now that variables are loaded and saved positions restored we can create frame
		if XMacros_SavedVariables then
			XMacros.x = XMacros.GetSavedValue(XMacros_SavedVariables.x, 100)
			XMacros.y = XMacros.GetSavedValue(XMacros_SavedVariables.y, 20)
			XMacros.currentRole = XMacros.GetSavedValue(XMacros_SavedVariables.currentRole, "0")
			XMacros.roleSets = XMacros.GetSavedValue(XMacros_SavedVariables.roleSets, {})
			XMacros.sets = XMacros.GetSavedValue(XMacros_SavedVariables.sets, {})
			XMacros.additionalIcons = XMacros.GetSavedValue(XMacros_SavedVariables.additionalIcons, {})
			XMacros.macros = XMacros.GetSavedValue(XMacros_SavedVariables.macros, {})
			XMacros.xTooltip = XMacros.GetSavedValue(XMacros_SavedVariables.xTooltip, 50)
			XMacros.yTooltip = XMacros.GetSavedValue(XMacros_SavedVariables.yTooltip, 50)
			XMacros.xMacros = XMacros.GetSavedValue(XMacros_SavedVariables.xMacros, -1)
			XMacros.yMacros = XMacros.GetSavedValue(XMacros_SavedVariables.yMacros, -1)
			XMacros.xBars = XMacros.GetSavedValue(XMacros_SavedVariables.xBars, -1)
			XMacros.yBars = XMacros.GetSavedValue(XMacros_SavedVariables.yBars, -1)
		end
	end	
end

--
-- Register events
--
function XMacros.RegisterPostStartupEvents()
	table.insert(Command.Slash.Register(XMacros.slashName), { XMacros.SlashHandler, XMacros.name, "XMacros.SlashHandler"})
	table.insert(Event.Addon.SavedVariables.Save.Begin, { XMacros.SaveVariables, XMacros.name, "XMacros.SaveVariables" })
	table.insert(Event.System.Secure.Enter, { XMacros.SetInCombat, XMacros.name, "XMacros.SetInCombat" })
	table.insert(Event.System.Secure.Leave, { XMacros.SetOutCombat, XMacros.name, "XMacros.SetOutCombat" })
	if Event.TEMPORARY ~= nil and Event.TEMPORARY.Role ~= nil then
		table.insert(Event.TEMPORARY.Role, { XMacros.RoleChanged, XMacros.name, "XMacros.RoleChanged" })
	end
end

table.insert(Event.Addon.Load.End, { XMacros.Initialise, XMacros.name, "XMacros.Initialise" })
table.insert(Event.Addon.SavedVariables.Load.End, { XMacros.LoadVariables, XMacros.name, "XMacros.LoadVariables" })
table.insert(Event.System.Update.Begin, XMacros.StartupHook)
