-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.MacroPicker = {}
XMacros.MacroPicker.name = "XMacros.MacroPicker"

--
-- Main functions
--

--
-- UI functions
--

function XMacros.MacroPicker.Create(parentFrame, positionFrame)
	local macroPicker = {}
	macroPicker.parentFrame = parentFrame
	macroPicker.page = 1
	macroPicker.maxPage = 1
	macroPicker.macroNames = {}
	
	macroPicker.textFrame = UI.CreateFrame("RiftWindow", "TextFrame", parentFrame)
	macroPicker.textFrame:SetVisible(false)
	macroPicker.textFrame:SetPoint("CENTER", positionFrame, "CENTER")
	macroPicker.textFrame:SetWidth(400)
	macroPicker.textFrame:SetHeight(628)
	macroPicker.textFrame:SetTitle("Choose macro")
	macroPicker.textFrame:SetLayer(parentFrame:GetLayer() + 3)

	local xMargin = 20
	local yMargin = 20
	macroPicker.okButton = UI.CreateFrame("RiftButton", "OKButton", macroPicker.textFrame)
	macroPicker.okButton:SetPoint("BOTTOMLEFT", macroPicker.textFrame, "BOTTOMLEFT", xMargin, -yMargin)
	macroPicker.okButton:SetText("OK")
	macroPicker.okButton.Event.LeftPress = function() XMacros.MacroPicker.WindowClose(macroPicker, true) end	

	macroPicker.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", macroPicker.textFrame)
	macroPicker.cancelButton:SetPoint("BOTTOMRIGHT", macroPicker.textFrame, "BOTTOMRIGHT", -xMargin, -yMargin)
	macroPicker.cancelButton:SetText("Cancel")
	macroPicker.cancelButton.Event.LeftPress = function() XMacros.MacroPicker.WindowClose(macroPicker, false) end	

	local pathMargin = -yMargin

	macroPicker.pathLabel = UI.CreateFrame("Text", "PathLabel", macroPicker.textFrame)
	macroPicker.pathLabel:SetPoint("BOTTOMLEFT", macroPicker.okButton, "TOPLEFT", 0, pathMargin)
	macroPicker.pathLabel:SetText("Value:")

	macroPicker.pathText = UI.CreateFrame("RiftTextfield", "PathText", macroPicker.textFrame)
	macroPicker.pathText:SetPoint("BOTTOMRIGHT", macroPicker.cancelButton, "TOPRIGHT", 0, pathMargin)
	macroPicker.pathText:SetPoint("BOTTOMLEFT", macroPicker.pathLabel, "BOTTOMRIGHT", xMargin, 0)
	macroPicker.pathText:SetPoint("TOPLEFT", macroPicker.pathLabel, "TOPRIGHT", xMargin, 0)
	macroPicker.pathText:SetText("")
	macroPicker.pathText:SetBackgroundColor(0.15, 0.15, 0.15, 0.4)

	local frameMiddle = math.floor(macroPicker.textFrame:GetWidth() / 2)
	macroPicker.prevButton = UI.CreateFrame("RiftButton", "PrevButton", macroPicker.textFrame)
	macroPicker.prevButton:SetPoint("BOTTOMRIGHT", macroPicker.pathLabel, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)
	macroPicker.prevButton:SetText("<<")
	macroPicker.prevButton.Event.LeftPress = function() XMacros.MacroPicker.PageTurn(macroPicker, -1) end	

	macroPicker.nextButton = UI.CreateFrame("RiftButton", "NextButton", macroPicker.textFrame)
	macroPicker.nextButton:SetPoint("BOTTOMLEFT", macroPicker.pathText, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	macroPicker.nextButton:SetText(">>")
	macroPicker.nextButton.Event.LeftPress = function() XMacros.MacroPicker.PageTurn(macroPicker, 1) end
	
	macroPicker.gridFrame = UI.CreateFrame("Frame", "GridFrame", macroPicker.textFrame)
	macroPicker.gridFrame:SetPoint("TOPLEFT", macroPicker.textFrame, "TOPLEFT", xMargin, yMargin + 40)
	macroPicker.gridFrame:SetPoint("TOPRIGHT", macroPicker.textFrame, "TOPRIGHT", -xMargin, yMargin + 40)
	macroPicker.gridFrame:SetPoint("BOTTOMLEFT", macroPicker.prevButton, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	macroPicker.gridFrame:SetPoint("BOTTOMRIGHT", macroPicker.nextButton, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)

	local margin = 4
	local textSize = macroPicker.pathLabel:GetHeight()
	local rows = math.floor((macroPicker.gridFrame:GetHeight() + margin) / (textSize + margin))
	local cols = 2
	local colWidth = math.floor((macroPicker.textFrame:GetWidth() - (3 * xMargin)) / 2)
	macroPicker.rowLayout = XMacros.RowLayout.Create(macroPicker.gridFrame, rows, cols, textSize, colWidth, margin, margin)
	local txtFrame = XMacros.RowLayout.CreateFrame(macroPicker.rowLayout, 1, 1, "Text")
	txtFrame:SetBackgroundColor(0, 0, 1)
	txtFrame:SetText("CATEGORY")
	local txtFrame = XMacros.RowLayout.CreateFrame(macroPicker.rowLayout, 1, 2, "Text")
	txtFrame:SetBackgroundColor(0, 0, 1)
	txtFrame:SetText("MACRO")
	for col = 1, cols do
		for row = 1, rows - 1 do
			local txtFrame = XMacros.RowLayout.CreateFrame(macroPicker.rowLayout, row + 1, col, "Text")
			if row % 2 == 1 then
				txtFrame:SetBackgroundColor(0, 0, 0.15)
			else
				txtFrame:SetBackgroundColor(0.15, 0.15, 0.15)
			end
		end
	end

	XMacros.MacroPicker.Redraw(macroPicker)
	XMacros.RowLayout.Redraw(macroPicker.rowLayout)

	return macroPicker
end

function XMacros.MacroPicker.SetMacros(macroPicker, categoryNames, macroNames)
	macroPicker.categoryNames = categoryNames
	macroPicker.macroNames = macroNames
end

function XMacros.MacroPicker.Show(macroPicker, callback)
	macroPicker.callback = callback
	macroPicker.page = 1
	macroPicker.maxPage = math.ceil(#macroPicker.macroNames / (XMacros.RowLayout.GetNumRows(macroPicker.rowLayout) - 1))
	if macroPicker.maxPage < 1 then
		macroPicker.maxPage = 1
	end
	macroPicker.pathText:SetText("")
	XMacros.MacroPicker.Redraw(macroPicker)
	macroPicker.textFrame:SetVisible(true)
end

function XMacros.MacroPicker.WindowClose(macroPicker, okPressed)
	macroPicker.pathText:SetKeyFocus(false)
	macroPicker.textFrame:SetVisible(false)
	
	if okPressed then
		local text = macroPicker.pathText:GetText()
		if text and string.len(text) > 1 then
			macroPicker.callback(macroPicker, macroPicker.pathText:GetText())
		else
			macroPicker.callback(macroPicker)
		end
	else
		macroPicker.callback(macroPicker)
	end
end

function XMacros.MacroPicker.Redraw(macroPicker)
	macroPicker.prevButton:SetEnabled(macroPicker.page ~= 1)
	macroPicker.nextButton:SetEnabled(macroPicker.page ~= macroPicker.maxPage)
	
	local rows = XMacros.RowLayout.GetNumRows(macroPicker.rowLayout) - 1
	local cols = XMacros.RowLayout.GetNumCols(macroPicker.rowLayout)
	for row = 1, rows do
		local textIndex = (rows * (macroPicker.page - 1)) + row
		local catFrame = XMacros.RowLayout.GetCellFrame(macroPicker.rowLayout, row + 1, 1)
		local txtFrame = XMacros.RowLayout.GetCellFrame(macroPicker.rowLayout, row + 1, 2)
		if textIndex <= #macroPicker.macroNames then
			catFrame:SetText(macroPicker.categoryNames[textIndex])
			catFrame.Event.LeftClick = function() XMacros.MacroPicker.ChooseText(macroPicker, textIndex) end
			catFrame:SetVisible(true)
			txtFrame:SetText(macroPicker.macroNames[textIndex])
			txtFrame.Event.LeftClick = function() XMacros.MacroPicker.ChooseText(macroPicker, textIndex) end
			txtFrame:SetVisible(true)
		else
			catFrame:SetVisible(false)
			catFrame.Event.LeftClick = function() end
			txtFrame:SetVisible(false)
			txtFrame.Event.LeftClick = function() end
		end
	end
end

function XMacros.MacroPicker.ChooseText(macroPicker, textIndex)
	macroPicker.pathText:SetText(macroPicker.categoryNames[textIndex] .. "/" .. macroPicker.macroNames[textIndex])
end

function XMacros.MacroPicker.PageTurn(macroPicker, pageIncrement)
	macroPicker.page = macroPicker.page + pageIncrement
	if macroPicker.page < 1 then
		macroPicker.page = 1
	end
	if macroPicker.page > macroPicker.maxPage then
		macroPicker.page = macroPicker.maxPage
	end
	
	XMacros.MacroPicker.Redraw(macroPicker)
end
