-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.TexturePicker = {}
XMacros.TexturePicker.name = "XMacros.TexturePicker"

--
-- Main functions
--

--
-- UI functions
--

function XMacros.TexturePicker.Create(parentFrame, positionFrame, callback)
	local txtPicker = {}
	txtPicker.parentFrame = parentFrame
	txtPicker.callback = callback
	txtPicker.textureArray = {}
	txtPicker.page = 1
	txtPicker.maxPage = 1
	
	txtPicker.width = 1188
	txtPicker.height = 628
	txtPicker.textFrame = UI.CreateFrame("RiftWindow", "TextureFrame", parentFrame)
	txtPicker.textFrame:SetPoint("CENTER", positionFrame, "CENTER")
	txtPicker.textFrame:SetWidth(txtPicker.width)
	txtPicker.textFrame:SetHeight(txtPicker.height)
	txtPicker.textFrame:SetTitle("Pick Texture")
	txtPicker.textFrame:SetLayer(positionFrame:GetLayer() + 1)
	txtPicker.textFrame:SetVisible(false)
	txtPicker.x = txtPicker.textFrame:GetLeft()
	txtPicker.y = txtPicker.textFrame:GetTop()

	local xMargin = 20
	local yMargin = 20
	txtPicker.okButton = UI.CreateFrame("RiftButton", "OKButton", txtPicker.textFrame)
	txtPicker.okButton:SetPoint("BOTTOMLEFT", txtPicker.textFrame, "BOTTOMLEFT", xMargin, -yMargin)
	txtPicker.okButton:SetText("OK")
	txtPicker.okButton.Event.LeftPress = function() XMacros.TexturePicker.WindowClose(txtPicker, true) end	

	txtPicker.cancelButton = UI.CreateFrame("RiftButton", "CancelButton", txtPicker.textFrame)
	txtPicker.cancelButton:SetPoint("BOTTOMRIGHT", txtPicker.textFrame, "BOTTOMRIGHT", -xMargin, -yMargin)
	txtPicker.cancelButton:SetText("Cancel")
	txtPicker.cancelButton.Event.LeftPress = function() XMacros.TexturePicker.WindowClose(txtPicker, false) end	

	txtPicker.addonLabel = UI.CreateFrame("Text", "AddonLabel", txtPicker.textFrame)
	txtPicker.addonLabel:SetPoint("BOTTOMLEFT", txtPicker.okButton, "TOPLEFT", 100, -yMargin)
	txtPicker.addonLabel:SetText("Texture Addon:")

	txtPicker.pathLabel = UI.CreateFrame("Text", "PathLabel", txtPicker.textFrame)
	txtPicker.pathLabel:SetPoint("BOTTOMLEFT", txtPicker.addonLabel, "TOPLEFT", 0, -yMargin)
	txtPicker.pathLabel:SetText("Texture Path:")

	txtPicker.nameLabel = UI.CreateFrame("Text", "NameLabel", txtPicker.textFrame)
	txtPicker.nameLabel:SetPoint("BOTTOMLEFT", txtPicker.pathLabel, "TOPLEFT", 0, -yMargin)
	txtPicker.nameLabel:SetText("Texture Name:")
	
	local labelWidth = txtPicker.pathLabel:GetWidth()
	if txtPicker.nameLabel:GetWidth() > labelWidth then
		labelWidth = txtPicker.nameLabel:GetWidth()
	end
	if txtPicker.addonLabel:GetWidth() > labelWidth then
		labelWidth = txtPicker.addonLabel:GetWidth()
	end
	
	txtPicker.pathLabel:SetWidth(labelWidth)
	txtPicker.nameLabel:SetWidth(labelWidth)
	txtPicker.addonLabel:SetWidth(labelWidth)

	txtPicker.addonText = UI.CreateFrame("Text", "PathText", txtPicker.textFrame)
	txtPicker.addonText:SetPoint("BOTTOMRIGHT", txtPicker.cancelButton, "TOPRIGHT", -200, -yMargin)
	txtPicker.addonText:SetPoint("TOPLEFT", txtPicker.addonLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.addonText:SetText("")
	Library.LibSimpleWidgets.SetBorder(txtPicker.addonText, 1, 0.5, 0.5, 0.5, 1)

	txtPicker.pathText = UI.CreateFrame("Text", "PathText", txtPicker.textFrame)
	txtPicker.pathText:SetPoint("BOTTOMRIGHT", txtPicker.addonText, "TOPRIGHT", 0, -yMargin)
	txtPicker.pathText:SetPoint("TOPLEFT", txtPicker.pathLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.pathText:SetText("")
	Library.LibSimpleWidgets.SetBorder(txtPicker.pathText, 1, 0.5, 0.5, 0.5, 1)

	txtPicker.nameText = UI.CreateFrame("Text", "NameText", txtPicker.textFrame)
	txtPicker.nameText:SetPoint("BOTTOMRIGHT", txtPicker.pathText, "TOPRIGHT", 0, -yMargin)
	txtPicker.nameText:SetPoint("TOPLEFT", txtPicker.nameLabel, "TOPRIGHT", xMargin, 0)
	txtPicker.nameText:SetText("")
	Library.LibSimpleWidgets.SetBorder(txtPicker.nameText, 1, 0.5, 0.5, 0.5, 1)

	local frameMiddle = math.floor(txtPicker.textFrame:GetWidth() / 2)
	txtPicker.prevButton = UI.CreateFrame("RiftButton", "PrevButton", txtPicker.textFrame)
	txtPicker.prevButton:SetPoint("BOTTOMRIGHT", txtPicker.nameLabel, "TOPLEFT", frameMiddle - 2 * xMargin - 100, -yMargin)
	txtPicker.prevButton:SetText("<<")
	txtPicker.prevButton.Event.LeftPress = function() XMacros.TexturePicker.PageTurn(txtPicker, -1) end	

	txtPicker.nextButton = UI.CreateFrame("RiftButton", "NextButton", txtPicker.textFrame)
	txtPicker.nextButton:SetPoint("BOTTOMLEFT", txtPicker.nameText, "TOPRIGHT", 200 - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.nextButton:SetText(">>")
	txtPicker.nextButton.Event.LeftPress = function() XMacros.TexturePicker.PageTurn(txtPicker, 1) end
	
	txtPicker.addButton = UI.CreateFrame("RiftButton", "AddButton", txtPicker.textFrame)
	txtPicker.addButton:SetPoint("TOP", txtPicker.nextButton, "TOP")
	txtPicker.addButton:SetPoint("RIGHT", txtPicker.cancelButton, "RIGHT")
	txtPicker.addButton:SetText("Add Icon")
	txtPicker.addButton.Event.LeftPress = function() XMacros.TexturePicker.AddIcon(txtPicker) end
	
	txtPicker.removeButton = UI.CreateFrame("RiftButton", "RemoveButton", txtPicker.textFrame)
	txtPicker.removeButton:SetPoint("TOPLEFT", txtPicker.addButton, "BOTTOMLEFT", 0, 5)
	txtPicker.removeButton:SetText("Remove Icon")
	txtPicker.removeButton:SetEnabled(false)
	
	txtPicker.gridFrame = UI.CreateFrame("Frame", "GridFrame", txtPicker.textFrame)
	txtPicker.gridFrame:SetPoint("TOPLEFT", txtPicker.textFrame, "TOPLEFT", xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("TOPRIGHT", txtPicker.textFrame, "TOPRIGHT", -xMargin, yMargin + 40)
	txtPicker.gridFrame:SetPoint("BOTTOMLEFT", txtPicker.prevButton, "TOPRIGHT", - frameMiddle + 2 * xMargin, -yMargin)
	txtPicker.gridFrame:SetPoint("BOTTOMRIGHT", txtPicker.nextButton, "TOPLEFT", frameMiddle - 2 * xMargin, -yMargin)

	local margin = 4
	local textureSize = 48
	local cols = math.floor((txtPicker.gridFrame:GetWidth() + margin) / (textureSize + margin))
	local rows = math.floor((txtPicker.gridFrame:GetHeight() + margin) / (textureSize + margin))
	txtPicker.rowLayout = XMacros.RowLayout.Create(txtPicker.gridFrame, rows, cols, textureSize, textureSize, margin, margin)
	
	for row = 1, rows do
		for col = 1, cols do
			local txtFrame = XMacros.RowLayout.CreateFrame(txtPicker.rowLayout, row, col, "Texture")
		end
	end

	XMacros.TexturePicker.Redraw(txtPicker)
	
	XMacros.RowLayout.Redraw(txtPicker.rowLayout)
	
	XMacros.TexturePicker.SetDragEnabled(txtPicker, true)

	txtPicker.entWindow = XMacros.EntryWindow.CreateEntryWindow(parentFrame, positionFrame)
	
	return txtPicker
end

function XMacros.TexturePicker.Show(txtPicker, textureArray, removeCallback)
	local cols = XMacros.RowLayout.GetNumCols(txtPicker.rowLayout)
	local rows = XMacros.RowLayout.GetNumRows(txtPicker.rowLayout)
	txtPicker.maxPage = math.ceil(#textureArray / (cols * rows))
	txtPicker.nameText:SetText("")
	txtPicker.pathText:SetText("")
	txtPicker.addonText:SetText("")
	txtPicker.removeButton:SetEnabled(false)
	txtPicker.removeCallback = removeCallback
	txtPicker.textureArray = textureArray
	XMacros.TexturePicker.Redraw(txtPicker)
	txtPicker.textFrame:SetVisible(true)
	txtPicker.x = txtPicker.textFrame:GetLeft()
	txtPicker.y = txtPicker.textFrame:GetTop()
end

function XMacros.TexturePicker.Hide(txtPicker)
	txtPicker.pathText:SetKeyFocus(false)
	txtPicker.nameText:SetKeyFocus(false)
	txtPicker.addonText:SetKeyFocus(false)
	txtPicker.textFrame:SetVisible(false)
end

function XMacros.TexturePicker.WindowClose(txtPicker, okPressed)
	XMacros.TexturePicker.Hide(txtPicker)
	
	if okPressed then
		txtPicker.callback(txtPicker, txtPicker.nameText:GetText(), txtPicker.addonText:GetText(), txtPicker.pathText:GetText())
		
	else
		txtPicker.callback(txtPicker)
	end
end

function XMacros.TexturePicker.Redraw(txtPicker)
	Command.System.Watchdog.Quiet()
	
	txtPicker.prevButton:SetEnabled(txtPicker.page ~= 1)
	txtPicker.nextButton:SetEnabled(txtPicker.page ~= txtPicker.maxPage)
	
	local rows = XMacros.RowLayout.GetNumRows(txtPicker.rowLayout)
	local cols = XMacros.RowLayout.GetNumCols(txtPicker.rowLayout)
	for row = 1, rows do
		for col = 1, cols do
			local txtFrame = XMacros.RowLayout.GetCellFrame(txtPicker.rowLayout, row, col)
			local textureIndex = (rows * cols * (txtPicker.page - 1)) + (cols * (row - 1)) + col
			if textureIndex <= #txtPicker.textureArray then
				txtFrame:SetTexture(txtPicker.textureArray[textureIndex].addon, txtPicker.textureArray[textureIndex].path)
				txtFrame.Event.LeftClick = function() XMacros.TexturePicker.ChooseTexture(txtPicker, textureIndex) end
				txtFrame:SetVisible(true)
			else
				txtFrame:SetVisible(false)
				txtFrame.Event.LeftClick = function() XMacros.TexturePicker.ChooseTexture(nil, nil) end
			end
		end
	end
end

function XMacros.TexturePicker.ChooseTexture(txtPicker, textureIndex)
	if not txtPicker then
		txtPicker.addonText:SetText("")
		txtPicker.nameText:SetText("")
		txtPicker.pathText:SetText("")
		txtPicker.removeButton:SetEnabled(false)
	else
		txtPicker.addonText:SetText(txtPicker.textureArray[textureIndex].addon)
		txtPicker.nameText:SetText(txtPicker.textureArray[textureIndex].name)
		txtPicker.pathText:SetText(txtPicker.textureArray[textureIndex].path)
		if string.find(txtPicker.textureArray[textureIndex].name, "^additional/") ~= nil then
			txtPicker.removeButton.Event.LeftPress = function() XMacros.TexturePicker.RemoveIcon(txtPicker, textureIndex) end
			txtPicker.removeButton:SetEnabled(true)
		else
			txtPicker.removeButton:SetEnabled(false)
		end
	end
end

function XMacros.TexturePicker.PageTurn(txtPicker, pageIncrement)
	txtPicker.page = txtPicker.page + pageIncrement
	if txtPicker.page < 1 then
		txtPicker.page = 1
	end
	if txtPicker.page > txtPicker.maxPage then
		txtPicker.page = txtPicker.maxPage
	end
	
	txtPicker.addonText:SetText("")
	txtPicker.nameText:SetText("")
	txtPicker.pathText:SetText("")
	txtPicker.removeButton:SetEnabled(false)

	XMacros.TexturePicker.Redraw(txtPicker)
end

function XMacros.TexturePicker.AddIcon(txtPicker)
	txtPicker.textFrame:SetVisible(false)
	XMacros.EntryWindow.ShowEntryWindow(txtPicker.entWindow, "Enter the path to your icon file.  The image must be under the XMacros addon directory.\nFor example Textures/number1.png",
									"", function(a, path) XMacros.TexturePicker.AddAditionalIcon(txtPicker, path) end)
end

function XMacros.TexturePicker.RemoveIcon(txtPicker, textureIndex)
	if txtPicker.removeCallback then
		local length = string.len("additional/")
		local name = string.sub(txtPicker.textureArray[textureIndex].name, length + 1)
		txtPicker.removeCallback(name)
		XMacros.TexturePicker.Show(txtPicker, XMacros.TexturePicker.GetTextureArray(), txtPicker.removeCallback)
	end
end

function XMacros.TexturePicker.AddAditionalIcon(txtPicker, path)
	if path == nil or string.len(path) < 1 then
		txtPicker.textFrame:SetVisible(true)
		return
	end
	
	XMacros.TexturePicker.textureArray = nil
	
	local additionalIcons = XMacros.GetAdditionalIcons()
	additionalIcons[path] = path
	
	XMacros.TexturePicker.Show(txtPicker, XMacros.TexturePicker.GetTextureArray(), txtPicker.removeCallback)
end

function XMacros.TexturePicker.GetTextureArray()
	if not XMacros.TexturePicker.textureArray then
		Command.System.Watchdog.Quiet()
		
		local knownAbilities = {}
		for name, abilities in pairs(XMacros.TexturePicker.abilityMap) do
			for _, icon in pairs(abilities) do
				knownAbilities[icon] = 1
			end
		end

		local abilityNumber = 1
		local unknownAbilities = {}
		for _, icon in ipairs(XMacros.TexturePicker.abilityList) do
			if not knownAbilities[icon] then
				unknownAbilities["unknown" .. tostring(abilityNumber)] = icon
				abilityNumber = abilityNumber + 1
			end
		end

		local textureArray = {}

		for _, className in ipairs({ "cleric", "rogue", "mage", "warrior" }) do
			local abilities = XMacros.TexturePicker.abilityMap[className]
			if abilities then
				for name, path in pairs(abilities) do
					local addon = "Rift"
					local texture = { name = className .. "/" .. name, path = path, addon = addon }
					table.insert(textureArray, texture)
				end
			end
		end

		for name, path in pairs(unknownAbilities) do
			local addon = "Rift"			
			local texture = { name = "unknown/" .. name, path = path, addon = addon }
			table.insert(textureArray, texture)
		end

		for name, path in pairs(XMacros.GetAdditionalIcons()) do
			if not XMacros.TexturePicker.additionalAbilities[name] then
				local addon = "XMacros"			
				local texture = { name = "additional/" .. name, path = path, addon = addon }
				table.insert(textureArray, texture)				
			end			
		end
		
		for name, path in pairs(XMacros.TexturePicker.additionalAbilities) do
			local addon = "XMacros"			
			local texture = { name = "additional/" .. name, path = path, addon = addon }
			table.insert(textureArray, texture)				
		end
		
		table.sort(textureArray, function(a, b) return a.name < b.name end)
		
		XMacros.TexturePicker.textureArray = textureArray		
	end
	
	return XMacros.TexturePicker.textureArray
end

local function coBuildAbilityMaps()
	local yield = Inspect.Time.Real() + 0.01 -- Allow 10 miliseconds of execution
	
	local abilityName2IconMap = {}
	local abilityName2NameMap = {}
	for className, abilities in pairs(XMacros.TexturePicker.abilityMap) do
		for abilityName, icon in pairs(abilities) do
			local lowerAbilityName = string.lower(abilityName)
			if abilityName2IconMap[lowerAbilityName] then
				if abilityName2IconMap[lowerAbilityName] ~= icon then
					print("Duplicate " .. lowerAbilityName)
				end
			else
				abilityName2IconMap[lowerAbilityName] = icon
			end
			
			abilityName2NameMap[lowerAbilityName] = abilityName
		end
		
		if(Inspect.Time.Real() >= yield) then
			coroutine.yield()
			yield = Inspect.Time.Real() + 0.01
		end
	end
	
	XMacros.TexturePicker.abilityName2IconMap = abilityName2IconMap
	XMacros.TexturePicker.abilityName2NameMap = abilityName2NameMap
end

function XMacros.TexturePicker.BuildAbilityMaps()
	if not XMacros.TexturePicker.abilityName2IconMap then
		--local job = coroutine.create(coBuildAbilityMaps)
		local coroCaller = coroutine.wrap(coBuildAbilityMaps)
		coroCaller()
		
		if not XMacros.TexturePicker.abilityName2IconMap then
			print "Failed to create maps"
		end
	end
end

function XMacros.TexturePicker.FilterIcons(filter)
	local textures = XMacros.TexturePicker.GetTextureArray()
	local lowerFilter = string.lower(filter)
	local filteredTextures = {}
	for _, texture in ipairs(textures) do
		if string.find(texture.name, lowerFilter) ~= nil or string.find(texture.path, lowerFilter) ~= nil then
			table.insert(filteredTextures, texture)
		end
	end
	
	return filteredTextures
end

function XMacros.TexturePicker.FindIconForAbility(ability)
	XMacros.TexturePicker.BuildAbilityMaps()
	local lowerAbilityName = string.lower(ability)
	return XMacros.TexturePicker.abilityName2IconMap[lowerAbilityName]
end

function XMacros.TexturePicker.FindAbilityName(ability)
	XMacros.TexturePicker.BuildAbilityMaps()
	local lowerAbilityName = string.lower(ability)
	return XMacros.TexturePicker.abilityName2NameMap[lowerAbilityName]
end

function XMacros.TexturePicker.SetDragEnabled(txtPicker, enabled)
	if enabled == false then
		if txtPicker.dragFrame then
			txtPicker.dragFrame.frame.Event.RightUp = nil
			txtPicker.dragFrame.frame.Event.RightUpoutside = nil
			txtPicker.dragFrame.frame.Event.RightDown = nil
			txtPicker.dragFrame.frame.Event.MouseMove = nil
			txtPicker.dragFrame.frame = nil
			txtPicker.dragFrame = nil
		end
	elseif not txtPicker.dragFrame then
		local dragFrame = {}

		dragFrame.dragging = false
		dragFrame.frame = UI.CreateFrame("Frame", "DragFrame", txtPicker.textFrame)
		dragFrame.frame:SetPoint("TOPLEFT", txtPicker.textFrame, "TOPLEFT")
		dragFrame.frame:SetPoint("BOTTOMRIGHT", txtPicker.textFrame, "TOPRIGHT", 0, 50)
		dragFrame.frame:SetLayer(txtPicker.textFrame:GetLayer() + 1)
		dragFrame.frame:SetMouseMasking("full")
		dragFrame.frame.Event.RightUp = function () XMacros.TexturePicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.RightUpoutside = function () XMacros.TexturePicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.RightDown = function () XMacros.TexturePicker.MouseDown(txtPicker) end
		dragFrame.frame.Event.LeftUp = function () XMacros.TexturePicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.LeftUpoutside = function () XMacros.TexturePicker.MouseUp(txtPicker) end
		dragFrame.frame.Event.LeftDown = function () XMacros.TexturePicker.MouseDown(txtPicker) end
		dragFrame.frame.Event.MouseMove = function (event, x, y) XMacros.TexturePicker.MouseMove(txtPicker, x, y) end
		txtPicker.dragFrame = dragFrame
	end
end

function XMacros.TexturePicker.MouseDown(txtPicker)
	if txtPicker.dragFrame.dragging == false then
		local mouse = Inspect.Mouse()
		txtPicker.dragFrame.xOffset = txtPicker.x - mouse.x
		txtPicker.dragFrame.yOffset = txtPicker.y - mouse.y
		txtPicker.dragFrame.dragging = true
	end
end

function XMacros.TexturePicker.MouseMove(txtPicker, x, y)
	if txtPicker.dragFrame.dragging == true then
		local totalWidth = UIParent:GetWidth()
		local totalHeight = UIParent:GetHeight()
		txtPicker.x = x + txtPicker.dragFrame.xOffset
		txtPicker.y = y + txtPicker.dragFrame.yOffset
		if txtPicker.x < 0 then txtPicker.x = 0 end
		if txtPicker.y < 0 then txtPicker.y = 0 end
		if txtPicker.x + txtPicker.width > totalWidth then txtPicker.x = totalWidth - txtPicker.width end
		if txtPicker.y + txtPicker.height > totalHeight then txtPicker.y = totalHeight - txtPicker.height end
		txtPicker.textFrame:ClearPoint("CENTER")
		txtPicker.textFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", txtPicker.x, txtPicker.y)
	end
end

function XMacros.TexturePicker.MouseUp(txtPicker)
	txtPicker.dragFrame.dragging = false
end

XMacros.TexturePicker.additionalAbilities = {
	["Mark 1"] = "Textures/number1.png",
	["Mark 2"] = "Textures/number2.png",
	["Mark 3"] = "Textures/number3.png",
	["Mark 4"] = "Textures/number4.png",
	["Mark 5"] = "Textures/number5.png",
	["Mark 6"] = "Textures/number6.png",
	["Mark 7"] = "Textures/number7.png",
	["Mark 8"] = "Textures/number8.png",
	["LeftArrow"] = "Textures/LeftArrow.png",
	["RightArrow"] = "Textures/RightArrow.png",
	["UpArrow"] = "Textures/UpArrow.png",
	["DownArrow"] = "Textures/DownArrow.png"
}

XMacros.TexturePicker.abilityMap = {
		cleric = {
			Absolution = "Data/UI/ability_icons/absolution_01_a.dds",
			["Ageless Ice"] = "Data/UI/ability_icons/icy_heart1.dds",
			["Aggressive Renewal"] = "Data/UI/ability_icons/manachannel1.dds",
			["Armor of Awakening"] = "Data/UI/ability_icons/castigation3.dds",
			["Armor of Devotion"] = "Data/UI/ability_icons/castigation1c.dds",
			["Armor of Treachery"] = "Data/UI/ability_icons/castigation1b.dds",
			["Avalanche Strike"] = "Data/UI/ability_icons/slam2.dds",
			["Balm of the Woods"] = "Data/UI/ability_icons/triage2a.dds",
			Banish = "Data/UI/ability_icons/spiritstrike5.dds",
			["Battle Charge"] = "Data/UI/ability_icons/slam1.dds",
			Bewilder = "Data/UI/ability_icons/confound2.dds",
			["Bitter Wind"] = "Data/UI/ability_icons/shaman-vengeance_of_bitter_wind.dds",
			["Blessing of the Sea"] = "Data/UI/ability_icons/underwaterbreath2.dds",
			["Bolt of Depravity"] = "Data/UI/ability_icons/spiritblast4b.dds",
			["Bolt of Judgment"] = "Data/UI/ability_icons/spiritblast4bc.dds",
			["Bolt of Radiance"] = "Data/UI/ability_icons/bolt_of_radiance_01_a.dds",
			Bombard = "Data/UI/ability_icons/druid-bombard_a.dds",
			["Bound Fate"] = "Data/UI/ability_icons/entropy_embrace.dds",
			["Break Free"] = "Data/UI/item_icons/magicitem4a.dds",
			["Breath of Life"] = "Data/UI/ability_icons/ressurect4c.dds",
			["Caregiver's Blessing"] = "Data/UI/ability_icons/spiritofsaints3.dds",
			Cascade = "Data/UI/ability_icons/healthchannel3a.dds",
			Cauterize = "Data/UI/ability_icons/sentinel-cleansing_prayer_b.dds",
			Cavalier = "Data/UI/ability_icons/cavalier_01.dds",
			Censure = "Data/UI/ability_icons/censure_01.dds",
			["Circle of Oblivion"] = "Data/UI/ability_icons/exorcize2.dds",
			["Cleansing Prayer"] = "Data/UI/ability_icons/spiritual_will.dds",
			["Cleansing Waters"] = "Data/UI/ability_icons/rally1.dds",
			["Combined Effort"] = "Data/UI/ability_icons/intervene2.dds",
			["Courage of the Bear"] = "Data/UI/ability_icons/shaman-courage_of_the_bear.dds",
			["Courage of the Eagle"] = "Data/UI/ability_icons/ironwill2.dds",
			["Courage of the Jaguar"] = "Data/UI/ability_icons/shaman-courage_of_the_jaguar.dds",
			["Crag Hammer"] = "Data/UI/ability_icons/smite4a.dds",
			["Crushing Blow"] = "Data/UI/ability_icons/frost_hammer1.dds",
			["Crushing Wave"] = "Data/UI/ability_icons/warden-crushing_wave.dds",
			["Curative Waters"] = "Data/UI/ability_icons/rally1a.dds",
			["Curse of Discord"] = "Data/UI/ability_icons/cabalist_sigil_of_ravaging_a.dds",
			["Curse of Solitude"] = "Data/UI/ability_icons/curse_of_solitude.dds",
			["Dark Harbor"] = "Data/UI/ability_icons/dark_harbor.dds",
			["Dark Passage"] = "Data/UI/ability_icons/shadow_warp_b.dds",
			["Dark Water"] = "Data/UI/ability_icons/dark_waters_a.dds",
			Dehydrate = "Data/UI/ability_icons/souldrain3.dds",
			Deluge = "Data/UI/ability_icons/warden-deluge.dds",
			Detaunt = "Data/UI/ability_icons/absolution_01.dds",
			["Dismiss Pet"] = "Data/UI/ability_icons/redirect1a.dds",
			Dissolution = "Data/UI/ability_icons/warden-dissolution.dds",
			["Distorted Shadows"] = "Data/UI/ability_icons/distorted_shadows.dds",
			["Divine Call"] = "Data/UI/ability_icons/spiritofsaints1.dds",
			["Divine Cascade"] = "Data/UI/ability_icons/mend1a.dds",
			["Divine Pardon"] = "Data/UI/ability_icons/shroudofpenitence1a.dds",
			["Doctrine of Authority"] = "Data/UI/ability_icons/doctrine_of_authority_01.dds",
			["Doctrine of Bliss"] = "Data/UI/ability_icons/doctrine_of_bliss_01_b.dds",
			["Doctrine of Loyalty"] = "Data/UI/ability_icons/righteous_invocation_01.dds",
			["Doctrine of Righteousness"] = "Data/UI/ability_icons/righteous_healing_01.dds",
			Downpour = "Data/UI/ability_icons/warden-downpour.dds",
			Drown = "Data/UI/ability_icons/drowning2.dds",
			["Empowering Light"] = "Data/UI/ability_icons/healing_flare_a.dds",
			["Eruption of Life"] = "Data/UI/ability_icons/barbedtrap1.dds",
			["Even Justice"] = "Data/UI/ability_icons/even_justice_01.dds",
			Excommunicate = "Data/UI/ability_icons/excommunicate.dds",
			["Fae Hammer"] = "Data/UI/ability_icons/spiritstrike3z.dds",
			Fanaticism = "Data/UI/ability_icons/fanaticism.dds",
			["Fated Blow"] = "Data/UI/ability_icons/intimidatingweapon1a.dds",
			["Fervent Strike"] = "Data/UI/ability_icons/shockwave2.dds",
			["Fiery Blessing"] = "Data/UI/ability_icons/fiery_blessing.dds",
			["Flames of the Phoenix"] = "Data/UI/ability_icons/ressurection1za.dds",
			Flashover = "Data/UI/ability_icons/flash1.dds",
			["Frozen Wrath"] = "Data/UI/ability_icons/holysmite4.dds",
			["Glacial Shield"] = "Data/UI/ability_icons/shaman-glacial_shield.dds",
			["Glassy Reflection"] = "Data/UI/ability_icons/mirrorandsmoke2.dds",
			["Glory of the Chosen"] = "Data/UI/ability_icons/shaman-glory_of_the_chosen_a.dds",
			["Grasping Waters"] = "Data/UI/ability_icons/darkball2z.dds",
			["Greater Empowerment"] = "Data/UI/ability_icons/releaseheat4.dds",
			["Grim Silence"] = "Data/UI/ability_icons/druid-grim_silence.dds",
			["Harsh Discipline"] = "Data/UI/ability_icons/healthchannel1g.dds",
			["Healer's Covenant"] = "Data/UI/ability_icons/ardent_doctrine_01_a.dds",
			["Healer's Haste"] = "Data/UI/ability_icons/sentinel-refreshing_communion.dds",
			["Healing Benediction"] = "Data/UI/ability_icons/sentinel-healing_benediction_a.dds",
			["Healing Blessing"] = "Data/UI/ability_icons/ward1.dds",
			["Healing Breath"] = "Data/UI/ability_icons/flame_speaker_a.dds",
			["Healing Cataract"] = "Data/UI/ability_icons/warden-healing_cataract.dds",
			["Healing Communion"] = "Data/UI/ability_icons/heal3.dds",
			["Healing Current"] = "Data/UI/ability_icons/spiritblast2.dds",
			["Healing Flame"] = "Data/UI/ability_icons/healing_flame.dds",
			["Healing Flare"] = "Data/UI/ability_icons/healing_flare2.dds",
			["Healing Flood"] = "Data/UI/ability_icons/warden-healing_flood.dds",
			["Healing Grace"] = "Data/UI/ability_icons/restore2b.dds",
			["Healing Invocation"] = "Data/UI/ability_icons/heal2z.dds",
			["Healing Momentum"] = "Data/UI/ability_icons/generic_ability_016.dds",
			["Healing Showers"] = "Data/UI/ability_icons/rainofarrows2.dds",
			["Healing Spray"] = "Data/UI/ability_icons/layonhands2z.dds",
			["Heart of the Fire Mountain"] = "Data/UI/ability_icons/arcaneshield4.dds",
			["Heart of the Frozen Sea"] = "Data/UI/ability_icons/arkanxshield1.dds",
			["Heart of the Twilight Forest"] = "Data/UI/ability_icons/arcaneshield2.dds",
			["Heroic Blessing"] = "Data/UI/ability_icons/holystrength3c.dds",
			["Hidden Path"] = "Data/UI/ability_icons/druid-hidden_path.dds",
			["Howling Death"] = "Data/UI/ability_icons/menacingroar2a.dds",
			Humility = "Data/UI/ability_icons/humbling_blow_01.dds",
			Impede = "Data/UI/ability_icons/exorcize3.dds",
			Interdict = "Data/UI/ability_icons/interdict_01.dds",
			Jolt = "Data/UI/ability_icons/lighting1.dds",
			["Just Defense"] = "Data/UI/ability_icons/paladin-reprisal.dds",
			["Latent Blaze"] = "Data/UI/ability_icons/latent_blaze.dds",
			["Life of the Ages"] = "Data/UI/ability_icons/heal1.dds",
			["Life of the Era"] = "Data/UI/ability_icons/heal1.dds",
			["Life's Return"] = "Data/UI/ability_icons/ressurect4y.dds",
			["Life's Vengeance"] = "Data/UI/ability_icons/holysmite1.dds",
			["Light Bolt"] = "Data/UI/ability_icons/spiritbolt1.dds",
			["Lightning Hammer"] = "Data/UI/ability_icons/shaman-lighting_hammer.dds",
			["Luminous Gaze"] = "Data/UI/ability_icons/blindinglight2.dds",
			["Lurking Decay"] = "Data/UI/ability_icons/devouring_mist_a.dds",
			Maelstrom = "Data/UI/ability_icons/maelstrom.dds",
			["Marked by the Light"] = "Data/UI/ability_icons/mend1.dds",
			["Massive Blow"] = "Data/UI/ability_icons/shockwave2a.dds",
			["Mien of Honor"] = "Data/UI/ability_icons/overwhelming_brilliance_01.dds",
			["Mien of Leadership"] = "Data/UI/ability_icons/requital_01_a.dds",
			["Natural Dedication"] = "Data/UI/ability_icons/doctrine_of_dedication_01.dds",
			["Nebulous Haze"] = "Data/UI/ability_icons/dirtytricks1a.dds",
			["Noble Blessing"] = "Data/UI/ability_icons/holystrength3c.dds",
			["Nysyr's Brand"] = "Data/UI/ability_icons/generic_ability_009.dds",
			["Nysyr's Rebuke"] = "Data/UI/ability_icons/nysyrs_rebuke.dds",
			Obliterate = "Data/UI/ability_icons/reign_of_despair_a.dds",
			["Orbs of the Stream"] = "Data/UI/ability_icons/ward1zz.dds",
			["Orbs of the Tide"] = "Data/UI/ability_icons/warden-orbs_of_the_tide.dds",
			Perseverance = "Data/UI/ability_icons/perseverance.dds",
			["Planar Lure"] = "Data/UI/ability_icons/generic_ability_030.dds",
			["Precept of Refuge"] = "Data/UI/ability_icons/defensivestrike1.dds",
			Provoke = "Data/UI/ability_icons/piety2a.dds",
			Purge = "Data/UI/ability_icons/purge_a.dds",
			Purpose = "Data/UI/ability_icons/generic_ability_024_a.dds",
			Quietus = "Data/UI/ability_icons/sentinel-quietus.dds",
			["Rage of the Fae"] = "Data/UI/ability_icons/berserk4a.dds",
			["Rage of the North"] = "Data/UI/ability_icons/spiritofcrusader2.dds",
			["Ravaging Darkness"] = "Data/UI/ability_icons/binding_of_affliction_01.dds",
			Rebuke = "Data/UI/ability_icons/redirect4b.dds",
			Reparation = "Data/UI/ability_icons/reparation_01.dds",
			Reprieve = "Data/UI/ability_icons/layinghands1x.dds",
			["Resplendent Embrace"] = "Data/UI/ability_icons/paladin-preservation.dds",
			["Restorative Flame"] = "Data/UI/ability_icons/restorative_flame2.dds",
			["Reversal of Fortune"] = "Data/UI/ability_icons/redirect2.dds",
			["Righteous Imperative"] = "Data/UI/ability_icons/ressurection3bb.dds",
			["Righteous Mandate"] = "Data/UI/ability_icons/faithful_doctrine_01_a.dds",
			Ripple = "Data/UI/ability_icons/whirlingblades1.dds",
			["Rite of the Ancestors"] = "Data/UI/ability_icons/spiritofsaints2.dds",
			Salvation = "Data/UI/ability_icons/salvation_01.dds",
			["Sanction Heretic"] = "Data/UI/ability_icons/sanction_heretic_a.dds",
			["Searing Transfusion"] = "Data/UI/ability_icons/internalizeheat2.dds",
			["Shadow's Touch"] = "Data/UI/ability_icons/darkbolt3.dds",
			["Shards of Light"] = "Data/UI/ability_icons/sentinel-shards_of_light_b.dds",
			["Shared Recovery"] = "Data/UI/ability_icons/layonhands3.dds",
			["Shield of Oak"] = "Data/UI/ability_icons/vitalizingshield1b.dds",
			["Shield of the Ancestors"] = "Data/UI/ability_icons/ward2.dds",
			["Shroud Of Agony"] = "Data/UI/ability_icons/shroud_of_agony_a.dds",
			["Sigil of Binding"] = "Data/UI/ability_icons/magicward4.dds",
			["Sigil of Ruin"] = "Data/UI/ability_icons/incapacitate_01_a.dds",
			["Sigil of Secrecy"] = "Data/UI/ability_icons/sigil_of_lurking_decay_a.dds",
			["Sigil of Woe"] = "Data/UI/ability_icons/generic_ability_020_a.dds",
			["Sign of Asias"] = "Data/UI/ability_icons/ebon_fury_01.dds",
			["Slothful Spirit"] = "Data/UI/ability_icons/druid-slothful_spirit.dds",
			Slumber = "Data/UI/ability_icons/ressurect4bl.dds",
			["Soothing Stream"] = "Data/UI/ability_icons/souldrain1.dds",
			["Soul Drain"] = "Data/UI/ability_icons/souldrain4.dds",
			["Soul Lance"] = "Data/UI/ability_icons/lance1_a.dds",
			["Soul Recall"] = "Data/UI/ability_icons/soulbind.dds",
			Sovereignty = "Data/UI/ability_icons/virtuous_conviction_01.dds",
			["Spirit Rupture"] = "Data/UI/ability_icons/internalizeheat3.dds",
			["Spiritual Conflagration"] = "Data/UI/ability_icons/ward3.dds",
			["Spiritual Deficiency"] = "Data/UI/ability_icons/menacingroar4b.dds",
			["Spiritual Protection"] = "Data/UI/ability_icons/magicward1.dds",
			["Spiteful Spirit"] = "Data/UI/ability_icons/druid-spiteful_spirit.dds",
			Sterilize = "Data/UI/ability_icons/sentinel-cleansing_prayer.dds",
			["Stone Burst"] = "Data/UI/ability_icons/druid-stone_strike_a.dds",
			["Strength of the Fae"] = "Data/UI/ability_icons/druid-strength_of_the_fae.dds",
			["Strike of Judgment"] = "Data/UI/ability_icons/strike_of_judgement_01.dds",
			["Strike of the Maelstrom"] = "Data/UI/ability_icons/whirlwind1a.dds",
			["Summon: Faerie"] = "Data/UI/ability_icons/druid-summon_faerie.dds",
			["Summon: Greater Faerie"] = "Data/UI/ability_icons/druid-summon_greater_faerie.dds",
			["Summon: Satyr"] = "Data/UI/ability_icons/druid-summon_satyr.dds",
			["Surging Flames"] = "Data/UI/ability_icons/surging_flames.dds",
			["Tidal Surge"] = "Data/UI/ability_icons/warden-tidal_surge.dds",
			["Touch the Light"] = "Data/UI/ability_icons/redirect4a.dds",
			["Track Fae"] = "Data/UI/ability_icons/generic_ability_009_b.dds",
			["Track Water Creatures"] = "Data/UI/ability_icons/virtuoso_b.dds",
			Trepidation = "Data/UI/ability_icons/confound1.dds",
			["Trickster Spirit"] = "Data/UI/ability_icons/spiritofmongoose3.dds",
			Tyranny = "Data/UI/ability_icons/tyranny_of_death.dds",
			["Vengeance of the Frozen Earth"] = "Data/UI/ability_icons/shaman-vengeance_of_frozen_earth.dds",
			["Vengeance of the Piercing Cold"] = "Data/UI/ability_icons/dark_malady_01_a.dds",
			["Vengeance of the Primal North"] = "Data/UI/ability_icons/chainlighting1.dds",
			["Vengeance of the Winter Storm"] = "Data/UI/ability_icons/shaman-vengeance_of_winter_storm.dds",
			Vex = "Data/UI/ability_icons/vex.dds",
			Vigilance = "Data/UI/ability_icons/blindinglight1.dds",
			["Vile Power"] = "Data/UI/ability_icons/magicward1a.dds",
			["Ward of the Ancestors"] = "Data/UI/ability_icons/magicresist2.dds",
			Waterjet = "Data/UI/ability_icons/warden-waterjet.dds",
			["Well of Life"] = "Data/UI/ability_icons/ressurect4bb.dds",
			["Wild Strike"] = "Data/UI/ability_icons/slam3a.dds",
			["Wisdom of the Ages"] = "Data/UI/ability_icons/inspiration1.dds",
			["Wisdom of the Era"] = "Data/UI/ability_icons/inspiration1.dds"
		},
		mage = {
			["Absolute Zero"] = "Data/UI/ability_icons/rally1a.dds",
			["Accelerated Decay"] = "Data/UI/ability_icons/plague_01.dds",
			["Angelic Flight"] = "Data/UI/ability_icons/latent_blaze_a.dds",
			["Arcane Aegis"] = "Data/UI/ability_icons/invigoration2a.dds",
			["Arctic Blast"] = "Data/UI/ability_icons/icebolt4aa.dds",
			["Arresting Presence"] = "Data/UI/ability_icons/ward1z.dds",
			["Ashen Defense"] = "Data/UI/ability_icons/castigation1a.dds",
			Backdraft = "Data/UI/ability_icons/conflagration3.dds",
			Blight = "Data/UI/ability_icons/flesh_rot_01_b.dds",
			["Blood Binding"] = "Data/UI/ability_icons/master_of_the_abyss_01.dds",
			Bloom = "Data/UI/item_icons/plant14a.dds",
			["Break Free"] = "Data/UI/item_icons/magicitem4a.dds",
			Burn = "Data/UI/ability_icons/deflect1c.dds",
			["Burning Bonds"] = "Data/UI/ability_icons/ensnare1.dds",
			["Burning Ground"] = "Data/UI/ability_icons/conflagration2.dds",
			["Burning Intellect"] = "Data/UI/ability_icons/discombobulate2a.dds",
			["Burning Purpose"] = "Data/UI/ability_icons/ardent_doctrine_01.dds",
			["Burning Shield"] = "Data/UI/ability_icons/releaseheat3a.dds",
			["Channel Elements"] = "Data/UI/ability_icons/healthchannel3a.dds",
			["Charged Shield"] = "Data/UI/ability_icons/lightingbolt4y.dds",
			Choke = "Data/UI/ability_icons/secondwind1.dds",
			["Cinder Burst"] = "Data/UI/ability_icons/coalesceheat1.dds",
			["Cleansing Flames"] = "Data/UI/ability_icons/doctrine_of_bliss_01_c.dds",
			Cloudburst = "Data/UI/ability_icons/doctrine_of_bliss_01.dds",
			["Consuming Flames"] = "Data/UI/ability_icons/elemental_attunement1.dds",
			Consumption = "Data/UI/ability_icons/necropotence4z.dds",
			["Corpse Explosion"] = "Data/UI/ability_icons/necromancer-corpse_explosion.dds",
			Corrosion = "Data/UI/ability_icons/poison_gas_01.dds",
			Countdown = "Data/UI/ability_icons/pyroclasty2.dds",
			["Crumbling Resistance"] = "Data/UI/ability_icons/archon_crumbling_resistance_01.dds",
			["Crystalline Missiles"] = "Data/UI/ability_icons/spiritblast4bc.dds",
			["Dark Armor"] = "Data/UI/ability_icons/darkball2.dds",
			["Dark Fury"] = "Data/UI/ability_icons/elemres4b.dds",
			["Dark Touch"] = "Data/UI/ability_icons/darkball2z.dds",
			["Death's Edict"] = "Data/UI/ability_icons/binding_of_affliction_01_a.dds",
			["Death's Grasp"] = "Data/UI/ability_icons/impede1a.dds",
			Defile = "Data/UI/ability_icons/void_knight-rift_summon_a.dds",
			Degeneration = "Data/UI/ability_icons/rogue_shadow_shift.dds",
			Deny = "Data/UI/ability_icons/ebon_fury_01.dds",
			Detaunt = "Data/UI/ability_icons/absolution_01.dds",
			["Devouring Shadows"] = "Data/UI/ability_icons/poisoned.dds",
			["Dismiss Pet"] = "Data/UI/ability_icons/redirect1a.dds",
			Disorient = "Data/UI/ability_icons/stunning_blow_01.dds",
			["Draining Bolt"] = "Data/UI/ability_icons/darkbolt5.dds",
			["Earthen Barrage"] = "Data/UI/ability_icons/archon-earthen_barrage.dds",
			Electrocute = "Data/UI/ability_icons/chainlighting4.dds",
			["Elemental Armor"] = "Data/UI/ability_icons/arcaneshield2.dds",
			["Elemental Forces"] = "Data/UI/ability_icons/elemresist1.dds",
			["Empathic Bond"] = "Data/UI/ability_icons/ressurect4b.dds",
			["Empowered Darkness"] = "Data/UI/ability_icons/necropotence1.dds",
			["Empty the Crypts"] = "Data/UI/ability_icons/souldrain4.dds",
			Encase = "Data/UI/ability_icons/shreddingtrap2.dds",
			["Entropic Veil"] = "Data/UI/ability_icons/chloromancer_entropic_veil_01.dds",
			["Essence Link"] = "Data/UI/ability_icons/dark_containment_01.dds",
			["Essence Surge"] = "Data/UI/ability_icons/invigorate1.dds",
			Exhume = "Data/UI/ability_icons/binding_of_death_01.dds",
			Exposure = "Data/UI/ability_icons/elemresistance3.dds",
			["Eye of the Storm"] = "Data/UI/ability_icons/eye_of_the_storm.dds",
			["Fast Summon"] = "Data/UI/ability_icons/generic_ability_031_2.dds",
			Fear = "Data/UI/ability_icons/finalstrike3.dds",
			["Feign Death"] = "Data/UI/ability_icons/necromancer-feign_death.dds",
			["Fiery Assault"] = "Data/UI/ability_icons/elemental_bolt1b.dds",
			["Fire Armor"] = "Data/UI/ability_icons/elemward1.dds",
			["Fire Storm"] = "Data/UI/ability_icons/conflagration1.dds",
			Fireball = "Data/UI/ability_icons/fireball.dds",
			["Flame Bolt"] = "Data/UI/ability_icons/energybeam1.dds",
			["Flame Jet"] = "Data/UI/ability_icons/flamejet1.dds",
			["Flaring Intellect"] = "Data/UI/ability_icons/discombobulate2a.dds",
			["Flaring Power"] = "Data/UI/ability_icons/humbling_blow_01.dds",
			["Flash Freeze"] = "Data/UI/ability_icons/icebolt3.dds",
			Flashfire = "Data/UI/ability_icons/releaseheat2.dds",
			Flicker = "Data/UI/ability_icons/burning.dds",
			Flourish = "Data/UI/ability_icons/mend2b.dds",
			["Flowing Sand"] = "Data/UI/ability_icons/requital_01.dds",
			["Forked Lightning"] = "Data/UI/ability_icons/chainlighting1.dds",
			Fulminate = "Data/UI/ability_icons/intense_flames.dds",
			["Glyphs of Power"] = "Data/UI/ability_icons/dirtytricks1c.dds",
			["Grasping Void"] = "Data/UI/ability_icons/dusk_to_dawn_01.dds",
			["Grave Rot"] = "Data/UI/ability_icons/flesh_rot_01.dds",
			["Ground of Strength"] = "Data/UI/ability_icons/dirtytricks1b.dds",
			Hailstorm = "Data/UI/ability_icons/icestormdark.dds",
			["Haunting Pain"] = "Data/UI/ability_icons/necropotence4.dds",
			["Heat Wave"] = "Data/UI/ability_icons/bloodfire1.dds",
			["Ice Shear"] = "Data/UI/ability_icons/twilight_force_01.dds",
			["Ice Shield"] = "Data/UI/ability_icons/arcaneshield3.dds",
			Icicle = "Data/UI/ability_icons/icebolt1.dds",
			["Icy Carapace"] = "Data/UI/ability_icons/shaman-vengeance_of_frozen_earth_c.dds",
			["Icy Gale"] = "Data/UI/ability_icons/removeheat1.dds",
			["Icy Vortex"] = "Data/UI/ability_icons/void_knight-improved_rift_summon_a.dds",
			Ignite = "Data/UI/ability_icons/sacrificepet1.dds",
			Illuminate = "Data/UI/ability_icons/riftblade-surging_energy.dds",
			Incompetence = "Data/UI/ability_icons/energybeam2a.dds",
			Inferno = "Data/UI/ability_icons/internalizeheat1.dds",
			["Intensify Elements"] = "Data/UI/ability_icons/elementalres2b.dds",
			["Internalize Charge"] = "Data/UI/ability_icons/releaseheat4b.dds",
			["Last Gasp"] = "Data/UI/ability_icons/menacingroar2.dds",
			["Lava Field"] = "Data/UI/ability_icons/discombobulate3a.dds",
			["Leeching Flames"] = "Data/UI/ability_icons/magicresist1.dds",
			["Lich Form"] = "Data/UI/ability_icons/invoker_prerogative1.dds",
			["Life Leech"] = "Data/UI/ability_icons/drainlife1.dds",
			["Life Shift"] = "Data/UI/ability_icons/mend4.dds",
			["Lifebound Veil"] = "Data/UI/ability_icons/generic_ability_011.dds",
			["Lifegiving Veil"] = "Data/UI/ability_icons/vitalizingshield1b.dds",
			["Lightning Burst"] = "Data/UI/ability_icons/riftblade-surging_energy_c.dds",
			["Lightning Field"] = "Data/UI/ability_icons/lightingbolt4.dds",
			["Lightning Storm"] = "Data/UI/ability_icons/stormcaller_lightning_storm_02_b.dds",
			["Lightning Strike"] = "Data/UI/ability_icons/lighting2a.dds",
			["Lingering Dust"] = "Data/UI/ability_icons/explosive_infestation_01_a.dds",
			["Living Energy"] = "Data/UI/ability_icons/opportunity_01_a.dds",
			["Living Shell"] = "Data/UI/ability_icons/spiritofturtle2.dds",
			Lockdown = "Data/UI/ability_icons/elemres4d.dds",
			["Looming Demise"] = "Data/UI/ability_icons/primal_death_01.dds",
			["Mana Wrench"] = "Data/UI/ability_icons/manachannel1.dds",
			["Mass Betrayal"] = "Data/UI/ability_icons/hasted_time_a.dds",
			["Mass Charged Shield"] = "Data/UI/ability_icons/lightingbolt4y.dds",
			["Mass Exhaustion"] = "Data/UI/ability_icons/dominator-mass_exhaustion_b.dds",
			["Mass Fear"] = "Data/UI/ability_icons/finalstrike3a.dds",
			["Mass Grave"] = "Data/UI/ability_icons/incite_01_a.dds",
			["Memory Wipe"] = "Data/UI/ability_icons/ressurect4.dds",
			["Mental Flare"] = "Data/UI/ability_icons/manadrain1.dds",
			["Mental Shock"] = "Data/UI/ability_icons/holystrength2.dds",
			Mortality = "Data/UI/ability_icons/channelhealth4.dds",
			["Natural Conversion"] = "Data/UI/ability_icons/chloromancer-natural_conversion.dds",
			["Natural Healing"] = "Data/UI/ability_icons/invigorate2.dds",
			["Natural Splendor"] = "Data/UI/ability_icons/healthchannel2.dds",
			["Nature's Cleansing"] = "Data/UI/ability_icons/mend1c.dds",
			["Nature's Fury"] = "Data/UI/ability_icons/healthchannel1g.dds",
			["Nature's Touch"] = "Data/UI/ability_icons/layonhands2.dds",
			Necrosis = "Data/UI/ability_icons/poison_gas_02.dds",
			["Neddra's Essence"] = "Data/UI/ability_icons/summonskeleton1.dds",
			["Neddra's Grasp"] = "Data/UI/ability_icons/drainlife3.dds",
			["Neddra's Might"] = "Data/UI/ability_icons/channelhealth3.dds",
			["Neddra's Strength"] = "Data/UI/ability_icons/channelhealth3.dds",
			["Neddra's Torture"] = "Data/UI/ability_icons/fatique.dds",
			["Nenvin's Lament"] = "Data/UI/ability_icons/generic_ability_006.dds",
			["Neural Prod"] = "Data/UI/ability_icons/lighting_fury_a.dds",
			["Omen Sight"] = "Data/UI/item_icons/plant39a.dds",
			["Pillaging Stone"] = "Data/UI/ability_icons/concussion_01_b.dds",
			["Plague Bolt"] = "Data/UI/ability_icons/darkbolt2.dds",
			["Planar Lure"] = "Data/UI/ability_icons/generic_ability_030.dds",
			["Power Drain"] = "Data/UI/ability_icons/healthchannel1b.dds",
			["Priest's Lament"] = "Data/UI/ability_icons/righteousness1.dds",
			Purification = "Data/UI/ability_icons/heal1.dds",
			["Pyromancer's Armor"] = "Data/UI/ability_icons/arcaneshield4.dds",
			["Quicken Elements"] = "Data/UI/ability_icons/healthchannel3.dds",
			["Radiant Spores"] = "Data/UI/ability_icons/devout_conviction_01.dds",
			["Radiate Death"] = "Data/UI/ability_icons/summonskel5.dds",
			["Raging Storm"] = "Data/UI/ability_icons/souldrain1.dds",
			["Reclaim Power"] = "Data/UI/ability_icons/channelhealth2.dds",
			Reconstruct = "Data/UI/ability_icons/lifedrain4.dds",
			["Reflective Command"] = "Data/UI/ability_icons/deflect2.dds",
			["Reflective Presence"] = "Data/UI/ability_icons/magicward1a.dds",
			Revitalize = "Data/UI/ability_icons/elementalward3.dds",
			Revivify = "Data/UI/ability_icons/healthchannel1g.dds",
			["Ride the Wind"] = "Data/UI/ability_icons/tumble1.dds",
			["Rock Slide"] = "Data/UI/ability_icons/archon_rock_slide.dds",
			Ruin = "Data/UI/ability_icons/druid-hidden_path.dds",
			["Rune Shield"] = "Data/UI/ability_icons/fire_death_attunement_01.dds",
			Sacrifice = "Data/UI/ability_icons/sacrificepet2.dds",
			["Sacrifice Life: Damage"] = "Data/UI/ability_icons/sacrificepet3.dds",
			["Sacrifice Life: Mana"] = "Data/UI/ability_icons/sacrificepet4b.dds",
			["Searing Vitality"] = "Data/UI/ability_icons/bleeding_aa.dds",
			["Seed of Life"] = "Data/UI/ability_icons/restore1.dds",
			["Sever Bonds"] = "Data/UI/ability_icons/magicward4.dds",
			["Shadow Life"] = "Data/UI/ability_icons/necropotence4za.dds",
			["Shared Vigor"] = "Data/UI/ability_icons/rally1x.dds",
			["Smoldering Power"] = "Data/UI/ability_icons/elemward1a.dds",
			["Soul Purge"] = "Data/UI/ability_icons/healthchannel3b.dds",
			["Soul Recall"] = "Data/UI/ability_icons/soulbind.dds",
			["Soul Tether"] = "Data/UI/ability_icons/vitalizingshield2.dds",
			["Spark of Life"] = "Data/UI/ability_icons/enchant1.dds",
			["Split Personality"] = "Data/UI/ability_icons/dominator-split_personality.dds",
			["Static Barrier"] = "Data/UI/ability_icons/energyblast3.dds",
			["Static Discharge"] = "Data/UI/ability_icons/energyburst5.dds",
			["Static Flux"] = "Data/UI/ability_icons/elemres4.dds",
			["Stone Armor"] = "Data/UI/ability_icons/ward4.dds",
			["Storm Guard"] = "Data/UI/ability_icons/arkanxshield1.dds",
			["Storm Shackle"] = "Data/UI/ability_icons/energyburst3f.dds",
			["Stream of Reclamation"] = "Data/UI/ability_icons/redirect1b.dds",
			["Summon: Greater Air Elemental"] = "Data/UI/ability_icons/generic_ability_025.dds",
			["Summon: Greater Earth Elemental"] = "Data/UI/ability_icons/steelskin2.dds",
			["Summon: Greater Water Elemental"] = "Data/UI/ability_icons/riftblade-avatar_of_water.dds",
			["Summon: Lesser Air Elemental"] = "Data/UI/ability_icons/whirlwind4.dds",
			["Summon: Lesser Earth Elemental"] = "Data/UI/ability_icons/ironskin1.dds",
			["Summon: Shadow Revenant"] = "Data/UI/ability_icons/creeping_death_01.dds",
			["Summon: Skeletal Horror"] = "Data/UI/ability_icons/necromancer-summon_zealot.dds",
			["Summon: Skeletal Knight"] = "Data/UI/ability_icons/skeleton2c.dds",
			["Summon: Skeletal Stalwart"] = "Data/UI/ability_icons/summonskeleton2.dds",
			["Summon: Skeletal Zealot"] = "Data/UI/ability_icons/necromancer-summon_zealot_a.dds",
			["Surging Flare"] = "Data/UI/ability_icons/flash1.dds",
			["Swarming Flames"] = "Data/UI/ability_icons/internalizeheat3b.dds",
			Synergize = "Data/UI/ability_icons/doctrine_of_bliss_01_b.dds",
			Synthesis = "Data/UI/ability_icons/magicresist3.dds",
			["Tempered Armor"] = "Data/UI/ability_icons/castigation2.dds",
			["Thunder Blast"] = "Data/UI/ability_icons/lighting2.dds",
			Thunderbolt = "Data/UI/ability_icons/lighting1.dds",
			["Track Elementals"] = "Data/UI/ability_icons/releaseheat5.dds",
			["Track Fire Creatures"] = "Data/UI/ability_icons/increased_explosive_power.dds",
			["Track Life Creatures"] = "Data/UI/ability_icons/spiritoftree1.dds",
			["Track Plants"] = "Data/UI/item_icons/plant1.dds",
			["Track Undead"] = "Data/UI/ability_icons/illumination_01.dds",
			["Track Wood"] = "Data/UI/item_icons/wood3.dds",
			["Traitorous Influence"] = "Data/UI/ability_icons/hasted_time.dds",
			Transference = "Data/UI/ability_icons/redirect2a.dds",
			Transmogrify = "Data/UI/ability_icons/spiritofmongoose2.dds",
			["Vile Spores"] = "Data/UI/ability_icons/sanction_heretic.dds",
			["Vitality of Stone"] = "Data/UI/ability_icons/mend1f.dds",
			["Void Bolt"] = "Data/UI/ability_icons/darkbolt3.dds",
			["Void Life"] = "Data/UI/ability_icons/chloromancer_void_life_01_a.dds",
			["Void Shroud"] = "Data/UI/ability_icons/blackout_01_a.dds",
			["Volcanic Bomb"] = "Data/UI/ability_icons/slam4.dds",
			["Waning Power"] = "Data/UI/ability_icons/smokeandmirrors3.dds",
			["Ward of Detection"] = "Data/UI/ability_icons/awareness_01.dds",
			["Warlock Armor"] = "Data/UI/ability_icons/darkball1.dds",
			["Wild Abandon"] = "Data/UI/ability_icons/triage2a.dds",
			["Wild Growth"] = "Data/UI/ability_icons/spiritoftree1a.dds",
			["Wind Chill"] = "Data/UI/ability_icons/secondwind2.dds",
			Wither = "Data/UI/ability_icons/snaretrap1b.dds",
			["Withering Flames"] = "Data/UI/ability_icons/pyroclasty1.dds",
			["Withering Vine"] = "Data/UI/item_icons/plant3b.dds"
		},
		rogue = {
			["Adhesive Bomb"] = "Data/UI/ability_icons/bomb_adhesive.dds",
			["Angelic Flight"] = "Data/UI/ability_icons/latent_blaze_a.dds",
			Annihilate = "Data/UI/ability_icons/shadow_assault_a.dds",
			["Annihilation Bomb"] = "Data/UI/ability_icons/bomb_annihilation.dds",
			["Anthem of Competence"] = "Data/UI/ability_icons/anthem_of_competence.dds",
			["Anthem of Defiance"] = "Data/UI/ability_icons/anthem_of_defiance.dds",
			["Anthem of Fervor"] = "Data/UI/ability_icons/anthem_of_fervor.dds",
			["Anthem of Glory"] = "Data/UI/ability_icons/anthem_of_glory.dds",
			Assassinate = "Data/UI/ability_icons/generic_ability_015_a.dds",
			Backstab = "Data/UI/ability_icons/backstab.dds",
			["Baneful Touch"] = "Data/UI/ability_icons/baneful_touch_01.dds",
			["Barbed Shot"] = "Data/UI/ability_icons/mortalshot2.dds",
			["Barbed Trap"] = "Data/UI/ability_icons/shreddingtrap1.dds",
			["Blade Tempo"] = "Data/UI/ability_icons/blade_tempo.dds",
			["Blade and Soul Parity"] = "Data/UI/ability_icons/revitalize1c.dds",
			["Blast Charge"] = "Data/UI/ability_icons/charge_blast.dds",
			["Blazing Strike"] = "Data/UI/ability_icons/holysweep3a.dds",
			["Blinding Powder"] = "Data/UI/ability_icons/poison_gas_01_a.dds",
			["Booby Trap"] = "Data/UI/ability_icons/barbedtrap4a.dds",
			["Break Free"] = "Data/UI/item_icons/magicitem4a.dds",
			["Bull's Eye"] = "Data/UI/ability_icons/headshot2.dds",
			Cadence = "Data/UI/ability_icons/cadence_a.dds",
			["Call: Blood Raptor"] = "Data/UI/item_icons/pet_raptor.dds",
			["Call: Dire Wolf"] = "Data/UI/item_icons/pet_dire-wolf.dds",
			["Call: Greater Dire Wolf"] = "Data/UI/item_icons/pet_dire-wolf_a.dds",
			["Call: Greater Razorbeast"] = "Data/UI/item_icons/pet_razorbeast_a.dds",
			["Call: Razorbeast"] = "Data/UI/item_icons/pet_razorbeast.dds",
			["Caltrop Charge"] = "Data/UI/ability_icons/charge_caltrop.dds",
			["Carpet Bombing"] = "Data/UI/ability_icons/carpet_bombing_a.dds",
			["Charge Booster"] = "Data/UI/ability_icons/increased_explosive_power_a.dds",
			["Chemical Bomb"] = "Data/UI/ability_icons/bomb_chemical.dds",
			["Choking Gas Bomb"] = "Data/UI/ability_icons/bomb_choking_gas_a.dds",
			["Cleanse Soul"] = "Data/UI/ability_icons/shroudofpenitent2.dds",
			["Cloudy Poison"] = "Data/UI/ability_icons/stunning_blow_01.dds",
			["Coda of Cowardice"] = "Data/UI/ability_icons/coda_of_cowardice.dds",
			["Coda of Distress"] = "Data/UI/ability_icons/coda_of_distress.dds",
			["Coda of Fury"] = "Data/UI/ability_icons/coda_of_fury.dds",
			["Coda of Jeopardy"] = "Data/UI/ability_icons/coda_of_distress_a.dds",
			["Coda of Restoration"] = "Data/UI/ability_icons/coda_of_restoration_a.dds",
			["Coda of Wrath"] = "Data/UI/ability_icons/coda_of_wrath_a.dds",
			["Combat Pose"] = "Data/UI/ability_icons/combat_pose.dds",
			["Combat Preparation"] = "Data/UI/ability_icons/combat_pose1_a.dds",
			["Compound Attack"] = "Data/UI/ability_icons/charge2.dds",
			Conceal = "Data/UI/ability_icons/stealth_z.dds",
			["Concussion Charge"] = "Data/UI/ability_icons/charge_concussion_a.dds",
			["Concussive Blast"] = "Data/UI/ability_icons/concussive_blast_01.dds",
			["Crippling Shot"] = "Data/UI/ability_icons/crippling_shot_01.dds",
			Crossfire = "Data/UI/ability_icons/rainofarrows4a.dds",
			["Cut and Run"] = "Data/UI/ability_icons/silent_footsteps_01_a.dds",
			["Dancing Steel"] = "Data/UI/ability_icons/dancing_steel.dds",
			["Dark Containment"] = "Data/UI/ability_icons/dark_containment_01.dds",
			["Dark Malady"] = "Data/UI/ability_icons/dark_malady_01.dds",
			["Dauntless Strike"] = "Data/UI/ability_icons/finalstrike2.dds",
			Deaden = "Data/UI/ability_icons/fire_and_forget.dds",
			["Deadeye Shot"] = "Data/UI/ability_icons/headshot1b.dds",
			["Deadly Strike"] = "Data/UI/ability_icons/roguestrike2a.dds",
			["Debilitating Poison"] = "Data/UI/ability_icons/debilitating_poison_01_a.dds",
			["Debilitating Trap"] = "Data/UI/ability_icons/shreddingtrap3.dds",
			["Defer Death"] = "Data/UI/ability_icons/defer_death.dds",
			Detonate = "Data/UI/ability_icons/detonate.dds",
			Diffuse = "Data/UI/ability_icons/diffuse_01.dds",
			Disassemble = "Data/UI/ability_icons/disabling4.dds",
			Disengage = "Data/UI/ability_icons/defensivestrike3.dds",
			["Dismiss Pet"] = "Data/UI/ability_icons/redirect1a.dds",
			["Divert Rage"] = "Data/UI/ability_icons/divert_rage_01.dds",
			["Double Coup"] = "Data/UI/ability_icons/deflection4cc.dds",
			Dualism = "Data/UI/ability_icons/deflection1a.dds",
			["Dusk Strike"] = "Data/UI/ability_icons/necropotence4a.dds",
			["Dusk to Dawn"] = "Data/UI/ability_icons/dusk_to_dawn_01_a.dds",
			["Ebon Fury"] = "Data/UI/ability_icons/ebon_fury_01.dds",
			["Ebon Terror"] = "Data/UI/ability_icons/ebon_terror_01.dds",
			["Electrified Munitions"] = "Data/UI/ability_icons/lance1.dds",
			Elusiveness = "Data/UI/ability_icons/survival2.dds",
			["Embers Charge"] = "Data/UI/ability_icons/charge_ember.dds",
			["Empowered Shot"] = "Data/UI/ability_icons/rapidshot4a.dds",
			["Enduring Brew"] = "Data/UI/ability_icons/poisontwo3.dds",
			["Entangling Trap"] = "Data/UI/ability_icons/snaretrap1.dds",
			Eradicate = "Data/UI/ability_icons/master_archer_b.dds",
			["Escape Artist"] = "Data/UI/ability_icons/escape_artist_01.dds",
			["Expose Weakness"] = "Data/UI/ability_icons/exposeweakness4.dds",
			["False Blade"] = "Data/UI/ability_icons/swiftstrike1.dds",
			["Fan Out"] = "Data/UI/ability_icons/rainofarrows1.dds",
			["Fanfare of Knowledge"] = "Data/UI/ability_icons/fanfare_of_knowledge.dds",
			["Fanfare of Power"] = "Data/UI/ability_icons/fanfare_of_power.dds",
			["Fanfare of Vigor"] = "Data/UI/ability_icons/fanfare_of_vigor.dds",
			["Fell Blades"] = "Data/UI/ability_icons/sacrificepet4a.dds",
			["Feral Aggression"] = "Data/UI/ability_icons/shaman-courage_of_the_bear_a.dds",
			["Feral Instincts"] = "Data/UI/ability_icons/menacingroar2.dds",
			["Fiery Chains"] = "Data/UI/ability_icons/fiery_chains_01_a.dds",
			["Fiery Spike"] = "Data/UI/ability_icons/fiery_spike_01.dds",
			["Final Blow"] = "Data/UI/ability_icons/sweep2.dds",
			["Flame Thrust"] = "Data/UI/ability_icons/flame_thrust_01.dds",
			["Flash of Steel"] = "Data/UI/ability_icons/charge3.dds",
			Flashback = "Data/UI/ability_icons/flashback_a.dds",
			["Fleeting Instincts"] = "Data/UI/ability_icons/fleeting_instincts_02_b.dds",
			["Foul Play"] = "Data/UI/ability_icons/foul_play_02.dds",
			["Fragmentation Bomb"] = "Data/UI/ability_icons/bomb_fragment_a.dds",
			["Guarded Steel"] = "Data/UI/ability_icons/deflection4c.dds",
			["Guardian Phase"] = "Data/UI/ability_icons/guardian_phase.dds",
			["Hasted Shot"] = "Data/UI/ability_icons/rapidshot3.dds",
			["Hasten Call"] = "Data/UI/item_icons/trinket4.dds",
			["Head Shot"] = "Data/UI/ability_icons/headshot3.dds",
			["Hellfire Blades"] = "Data/UI/ability_icons/hellfire_blades_01.dds",
			["Hidden Veil"] = "Data/UI/ability_icons/hidden_veil_01.dds",
			Impale = "Data/UI/ability_icons/exposeweak1b.dds",
			Incapacitate = "Data/UI/ability_icons/incapacitate_01.dds",
			Incriminate = "Data/UI/item_icons/generic_ability_032.dds",
			Instigate = "Data/UI/ability_icons/foul_play_01.dds",
			["Jagged Strike"] = "Data/UI/ability_icons/roguestrike1.dds",
			["Keen Strike"] = "Data/UI/ability_icons/disabling1a.dds",
			["Land Mines"] = "Data/UI/ability_icons/land_mines.dds",
			["Leeching Poison"] = "Data/UI/ability_icons/debilitating_poison_01_b.dds",
			["Lethal Poison"] = "Data/UI/ability_icons/poisonone2a.dds",
			["Lightning Fury"] = "Data/UI/ability_icons/lighting_fury.dds",
			["Living Flame"] = "Data/UI/ability_icons/living_flame_01.dds",
			["Lost Hope"] = "Data/UI/ability_icons/discombobulate2.dds",
			Maim = "Data/UI/ability_icons/cripple1.dds",
			["Malicious Strike"] = "Data/UI/ability_icons/malicious_strike_01.dds",
			["Mass Subdue"] = "Data/UI/ability_icons/mass_subdue.dds",
			["Memory Capture"] = "Data/UI/ability_icons/memory_capture.dds",
			["Motif of Bravery"] = "Data/UI/ability_icons/motif_of_bravery.dds",
			["Motif of Encouragement"] = "Data/UI/ability_icons/motif_of_encouragement.dds",
			["Motif of Focus"] = "Data/UI/ability_icons/motif_of_focus.dds",
			["Motif of Grandeur"] = "Data/UI/ability_icons/motif_of_grandeur.dds",
			["Motif of Regeneration"] = "Data/UI/ability_icons/motif_of_regeneration_a.dds",
			["Motif of Renewal"] = "Data/UI/ability_icons/motif_of_regeneration.dds",
			["Motif of Tenacity"] = "Data/UI/ability_icons/motif_of_tenacity.dds",
			["On the Double"] = "Data/UI/ability_icons/fleeting_instincts_01_a.dds",
			["Paralyzing Strike"] = "Data/UI/ability_icons/paralyzing_strike_01.dds",
			["Phantom Blow"] = "Data/UI/ability_icons/phantom_blow_a.dds",
			["Physical Wellness"] = "Data/UI/ability_icons/riftstalker_toughened_soul_01.dds",
			["Piercing Shot"] = "Data/UI/ability_icons/piercing_shot_01_a.dds",
			["Pin Down"] = "Data/UI/ability_icons/pin_down_01.dds",
			["Planar Attraction"] = "Data/UI/ability_icons/riftstalker_planar_repulsion_01_a.dds",
			["Planar Disruption"] = "Data/UI/ability_icons/riftblade-surging_energy_b.dds",
			["Planar Lure"] = "Data/UI/ability_icons/generic_ability_030.dds",
			["Planar Refuge"] = "Data/UI/ability_icons/riftstalker_planar_refuge_01_a.dds",
			["Planar Strike"] = "Data/UI/ability_icons/disabling2.dds",
			["Planar Switch"] = "Data/UI/ability_icons/sigil_of_lurking_decay_b.dds",
			["Planebound Resilience"] = "Data/UI/ability_icons/rift_affinity.dds",
			["Poison Gas"] = "Data/UI/ability_icons/poison_gas_01.dds",
			["Poison Malice"] = "Data/UI/ability_icons/poisontwo4.dds",
			["Power Chord"] = "Data/UI/ability_icons/power_chord.dds",
			["Precision Strike"] = "Data/UI/ability_icons/critical2a.dds",
			["Predatory Instincts"] = "Data/UI/ability_icons/spiritoflion3.dds",
			["Primal Strike"] = "Data/UI/ability_icons/sweep3c.dds",
			Puncture = "Data/UI/ability_icons/exposeweakness2a.dds",
			["Quick Reload"] = "Data/UI/ability_icons/quick_reload_a.dds",
			["Quick Shot"] = "Data/UI/ability_icons/rapidshot1.dds",
			["Quick Strike"] = "Data/UI/ability_icons/sweep3b.dds",
			["Rain of Arrows"] = "Data/UI/ability_icons/rainofarrows2.dds",
			["Rapid Fire Shot"] = "Data/UI/ability_icons/rapidshot1a.dds",
			["Rapid Setup"] = "Data/UI/ability_icons/rapid_setup.dds",
			Rejuvenate = "Data/UI/ability_icons/triage3.dds",
			["Rending Munitions"] = "Data/UI/ability_icons/piercing_shot_01.dds",
			["Repelling Shot"] = "Data/UI/ability_icons/repelling_shot.dds",
			Reprisal = "Data/UI/ability_icons/deflection3.dds",
			Retreat = "Data/UI/ability_icons/retreat.dds",
			Riff = "Data/UI/ability_icons/riff.dds",
			["Rift Disturbance"] = "Data/UI/ability_icons/criticalsweep1b.dds",
			["Rift Prison"] = "Data/UI/ability_icons/rift_prison.dds",
			["Savage Strike"] = "Data/UI/ability_icons/disabling3a.dds",
			["Scatter the Shadows"] = "Data/UI/ability_icons/scatter_the_shadows.dds",
			["Scourge of Darkness"] = "Data/UI/ability_icons/scourge_of_darkness_01_a.dds",
			["Serpent Strike"] = "Data/UI/ability_icons/generic_ability_002.dds",
			["Shadow Assault"] = "Data/UI/ability_icons/shadow_assault.dds",
			["Shadow Blitz"] = "Data/UI/ability_icons/shadow_blitz_a.dds",
			["Shadow Fire"] = "Data/UI/ability_icons/rapidshot2.dds",
			["Shadow Shift"] = "Data/UI/ability_icons/shadow_shift_b.dds",
			["Shadow Stalk"] = "Data/UI/ability_icons/shadow_stalk.dds",
			["Shadow Warp"] = "Data/UI/ability_icons/shadow_warp.dds",
			["Shrapnel Charge"] = "Data/UI/ability_icons/charge_shrapnel.dds",
			["Side Steps"] = "Data/UI/ability_icons/side_steps.dds",
			["Silver Tip Munitions"] = "Data/UI/ability_icons/mortalshot1b.dds",
			["Slip Away"] = "Data/UI/ability_icons/slip_away_03.dds",
			["Smoldering Blades"] = "Data/UI/ability_icons/rampage1.dds",
			Smother = "Data/UI/ability_icons/smother_01.dds",
			["Soul Recall"] = "Data/UI/ability_icons/soulbind.dds",
			["Spike Charge"] = "Data/UI/ability_icons/charge_spike.dds",
			["Splinter Charge"] = "Data/UI/ability_icons/charge_splinter.dds",
			["Splinter Shot"] = "Data/UI/ability_icons/killer_instinct.dds",
			Sprint = "Data/UI/ability_icons/tumble1.dds",
			["Stalker Phase"] = "Data/UI/ability_icons/stalking_phase_a.dds",
			["Static Shot"] = "Data/UI/ability_icons/static_shot.dds",
			Stealth = "Data/UI/ability_icons/stealth_w.dds",
			Strafe = "Data/UI/ability_icons/strafe_a.dds",
			["Swift Shot"] = "Data/UI/ability_icons/rapidshot4.dds",
			["Time Bomb"] = "Data/UI/ability_icons/bomb_time.dds",
			["Track Air Creatures"] = "Data/UI/ability_icons/surge_b.dds",
			["Track Beasts"] = "Data/UI/ability_icons/survival1.dds",
			["Track Constructs"] = "Data/UI/ability_icons/doctrine_of_loyalty_01.dds",
			["Track Humanoids"] = "Data/UI/ability_icons/track_humanoids_01.dds",
			["Trick Shot"] = "Data/UI/ability_icons/ricochet1.dds",
			["Twilight Force"] = "Data/UI/ability_icons/twilight_force_01_b.dds",
			["Twilight Shelter"] = "Data/UI/ability_icons/magicresist1.dds",
			["Twilight Transcendence"] = "Data/UI/ability_icons/inspiration3.dds",
			["Twin Strike"] = "Data/UI/ability_icons/twin_strike_a.dds",
			Untangle = "Data/UI/ability_icons/untangle.dds",
			["Vampiric Munitions"] = "Data/UI/ability_icons/mortalshot1.dds",
			["Verse of Captivation"] = "Data/UI/ability_icons/verse_of_fascination_a.dds",
			["Verse of Fascination"] = "Data/UI/ability_icons/verse_of_fascination.dds",
			["Verse of Joy"] = "Data/UI/ability_icons/verse_of_joy.dds",
			["Verse of Rebirth"] = "Data/UI/ability_icons/triumphant_spirit_a.dds",
			["Verse of Vitality"] = "Data/UI/ability_icons/verse_of_vitality.dds",
			Virtuoso = "Data/UI/ability_icons/virtuoso_b.dds",
			["Virulent Poison"] = "Data/UI/ability_icons/poisonone2b.dds",
			["Weapon Barrage"] = "Data/UI/ability_icons/weapon_barrage.dds",
			["Weapon Flare"] = "Data/UI/ability_icons/holysweep1.dds"
		},
		warrior = {
			["Accord of Emptiness"] = "Data/UI/ability_icons/curse_of_solitude.dds",
			["Aegis of Salvation"] = "Data/UI/ability_icons/blazeofglory3.dds",
			["Aegis of Vitality"] = "Data/UI/ability_icons/faithful_doctrine_01.dds",
			["Aggressive Block"] = "Data/UI/ability_icons/defstrike2.dds",
			["Aid Command"] = "Data/UI/ability_icons/spiritofsaints3.dds",
			Airburst = "Data/UI/ability_icons/energyburst5.dds",
			["Aspect of the Elements"] = "Data/UI/ability_icons/elementalward2a.dds",
			["Aspect of the Fallen Hero"] = "Data/UI/ability_icons/ressurect4.dds",
			["Assault Command"] = "Data/UI/ability_icons/blazeofglory1a.dds",
			["Aura of Vengeful Wrath"] = "Data/UI/ability_icons/righteousness2.dds",
			["Avatar of Water"] = "Data/UI/ability_icons/riftblade-avatar_of_water_a.dds",
			["Avatar of Wind"] = "Data/UI/ability_icons/generic_ability_025_a.dds",
			["Avatar of the Rift"] = "Data/UI/ability_icons/ebon_fury_01.dds",
			["Backhanded Blow"] = "Data/UI/ability_icons/sweep1.dds",
			Bash = "Data/UI/ability_icons/bash1.dds",
			["Battlefield Awareness"] = "Data/UI/ability_icons/fanaticism.dds",
			["Battlefield Distraction"] = "Data/UI/ability_icons/menacingroar4.dds",
			["Battlefield Intimidation"] = "Data/UI/ability_icons/berserk1a.dds",
			["Bend Like the Reed"] = "Data/UI/ability_icons/deflection2.dds",
			["Binding of Affliction"] = "Data/UI/ability_icons/binding_of_affliction_01_a.dds",
			["Binding of Atrophy"] = "Data/UI/ability_icons/binding_of_atrophy_01.dds",
			["Binding of Death"] = "Data/UI/ability_icons/binding_of_death_01.dds",
			["Blade of Elemental Affinity"] = "Data/UI/ability_icons/holysweep3aa.dds",
			Bladefury = "Data/UI/ability_icons/criticalsweep3c.dds",
			["Blood Fever"] = "Data/UI/ability_icons/bloodfire1.dds",
			Bloodthirst = "Data/UI/ability_icons/criticalsweep2.dds",
			["Bond of Alacrity"] = "Data/UI/ability_icons/swiftstrike2.dds",
			["Bond of Feline Grace"] = "Data/UI/ability_icons/beastmaster-natural_instincts.dds",
			["Bond of Might"] = "Data/UI/ability_icons/beastmaster_bond_of_might_a.dds",
			["Bond of Vigor"] = "Data/UI/ability_icons/berserk1.dds",
			["Break Free"] = "Data/UI/item_icons/magicitem4a.dds",
			["Bull Rush"] = "Data/UI/ability_icons/champion-bull_rush.dds",
			["Call to Battle"] = "Data/UI/ability_icons/warcry.dds",
			["Call to Entrench"] = "Data/UI/ability_icons/ward4.dds",
			["Call to Regroup"] = "Data/UI/ability_icons/righteous_invocation_01.dds",
			["Calming Influence"] = "Data/UI/ability_icons/beastmaster_beast_whisperer_01.dds",
			Catalyze = "Data/UI/ability_icons/voidknight_catalyze_01_b.dds",
			["Companion's Call"] = "Data/UI/ability_icons/shaman-courage_of_the_jaguar.dds",
			Concussion = "Data/UI/ability_icons/concussion_01_a.dds",
			["Cornered Beast"] = "Data/UI/ability_icons/whirlwind2.dds",
			["Cracking Skulls"] = "Data/UI/ability_icons/warlord-cracking_skulls_a.dds",
			["Creeping Death"] = "Data/UI/ability_icons/creeping_death_01_a.dds",
			["Crest of the Abyss"] = "Data/UI/ability_icons/elemward1.dds",
			["Cutting Distraction"] = "Data/UI/ability_icons/warlord-cutting_distraction.dds",
			["Cyclone Strike"] = "Data/UI/ability_icons/whirlwind4.dds",
			["Death Touch"] = "Data/UI/ability_icons/piercingenergy3.dds",
			Deathblow = "Data/UI/ability_icons/critical3.dds",
			["Debilitating Strike"] = "Data/UI/ability_icons/criticalstrike1.dds",
			["Destroyer's Bearing"] = "Data/UI/ability_icons/champion-destroyers_bearing.dds",
			["Devouring Blow"] = "Data/UI/ability_icons/paralyzing_strike_01.dds",
			["Dire Blow"] = "Data/UI/ability_icons/roguestrike1a.dds",
			["Disarming Counterblow"] = "Data/UI/ability_icons/lifes_devotion_01.dds",
			Discharge = "Data/UI/ability_icons/layonhands2z.dds",
			["Dismiss Pet"] = "Data/UI/ability_icons/redirect1a.dds",
			Disrupt = "Data/UI/ability_icons/confound1.dds",
			["Disruptive Strike"] = "Data/UI/ability_icons/generic_ability_021.dds",
			["Diversion Strike"] = "Data/UI/ability_icons/deflection3c.dds",
			["Dual Strike"] = "Data/UI/ability_icons/double_cross_01.dds",
			["Earth Burst"] = "Data/UI/ability_icons/riftblade-earth_burst_a.dds",
			["Empowering Strike"] = "Data/UI/ability_icons/rampage1.dds",
			["Enraged Companion"] = "Data/UI/ability_icons/beastmaster_enraged_companion_01.dds",
			["Enraged Essence"] = "Data/UI/ability_icons/shatter1.dds",
			["Face Slam"] = "Data/UI/ability_icons/paladin_face_slam.dds",
			["Feral Sweep"] = "Data/UI/ability_icons/whirlwind3.dds",
			["Fierce Strike"] = "Data/UI/ability_icons/sweep4a.dds",
			["Fiery Burst"] = "Data/UI/ability_icons/slam4a.dds",
			["Fight as One"] = "Data/UI/ability_icons/generic_ability_004.dds",
			Flamespear = "Data/UI/ability_icons/critical2a.dds",
			["Fleet of Foot"] = "Data/UI/ability_icons/kick4.dds",
			["Flesh Rip"] = "Data/UI/ability_icons/critical4.dds",
			["Flesh Rot"] = "Data/UI/ability_icons/flesh_rot_01_b.dds",
			["Flinching Strike"] = "Data/UI/ability_icons/roguestrike2a.dds",
			Flurry = "Data/UI/ability_icons/whirlingblades3.dds",
			["Focus of Body"] = "Data/UI/ability_icons/strong_constitution_01_a.dds",
			["Focus of Strength"] = "Data/UI/ability_icons/smite5.dds",
			Fork = "Data/UI/ability_icons/cripplingshot1.dds",
			["Freeze Armor"] = "Data/UI/item_icons/plate_114a_chest_a.dds",
			["Frenzied Strike"] = "Data/UI/ability_icons/cleave1a.dds",
			["Frost Strike"] = "Data/UI/ability_icons/riftblade_frost_strike_01_a.dds",
			["Furious Rage"] = "Data/UI/ability_icons/shatter3.dds",
			["Fusion of Flesh"] = "Data/UI/ability_icons/healthchannel3.dds",
			["General's Order"] = "Data/UI/ability_icons/increased_explosive_power_a.dds",
			["Grim Lure"] = "Data/UI/ability_icons/life-drain5.dds",
			Guard = "Data/UI/ability_icons/exposeweakness4.dds",
			["Hold the Line"] = "Data/UI/ability_icons/spiritofcrusader1.dds",
			["Impassable Guard"] = "Data/UI/ability_icons/bulwark1.dds",
			Imposing = "Data/UI/ability_icons/berserk2.dds",
			Incite = "Data/UI/ability_icons/incite_01.dds",
			["Inescapable Fury"] = "Data/UI/ability_icons/holysweep2a.dds",
			Infestation = "Data/UI/ability_icons/infestation_01.dds",
			Intercept = "Data/UI/ability_icons/steelskin3.dds",
			Judgment = "Data/UI/ability_icons/doctrine_of_bliss_01_c.dds",
			["Killing Field"] = "Data/UI/ability_icons/warlord-killing_field.dds",
			Kinship = "Data/UI/ability_icons/beastmaster_kinship_01.dds",
			["Leader's Mark"] = "Data/UI/ability_icons/rampage2.dds",
			["Leg Sweep"] = "Data/UI/ability_icons/cripple1a.dds",
			["Life's Grace"] = "Data/UI/ability_icons/ressurection1za.dds",
			["Life's Rapture"] = "Data/UI/ability_icons/paladin-life_rapture_a.dds",
			["Light's Decree"] = "Data/UI/ability_icons/generic_ability_013.dds",
			["Light's Hammer"] = "Data/UI/ability_icons/smite2.dds",
			["Locked Down"] = "Data/UI/ability_icons/hamstring2.dds",
			["Lure of the Eye"] = "Data/UI/item_icons/1h_mace_air_epic_b.dds",
			["Mark of Extermination"] = "Data/UI/ability_icons/finalstrike2.dds",
			["Mark of Inevitability"] = "Data/UI/ability_icons/exposeweakness1.dds",
			["Master of the Abyss"] = "Data/UI/ability_icons/master_of_the_abyss_01.dds",
			["Mighty Blow"] = "Data/UI/ability_icons/sweep4b.dds",
			["Motivational Roar"] = "Data/UI/ability_icons/combat_survival.dds",
			["Necrotic Wounds"] = "Data/UI/ability_icons/necropotence4a.dds",
			["No Permission to Die"] = "Data/UI/ability_icons/righteousness2.dds",
			["Omen Sight"] = "Data/UI/item_icons/plant39a.dds",
			["Pacifying Strike"] = "Data/UI/ability_icons/spiritstrike5.dds",
			["Pact Conversion"] = "Data/UI/ability_icons/enchant1.dds",
			["Paired Strike"] = "Data/UI/ability_icons/nightstalker_01.dds",
			["Paladin's Devotion"] = "Data/UI/ability_icons/resolve2a.dds",
			["Paladin's Reprisal"] = "Data/UI/ability_icons/paladin-reprisal.dds",
			["Path of the Hurricane"] = "Data/UI/ability_icons/generic_ability_005.dds",
			["Path of the Raptor"] = "Data/UI/ability_icons/staggeringshot1.dds",
			["Path of the Tempest"] = "Data/UI/ability_icons/paragon_path_of_the_tempest_01_b.dds",
			["Path of the Wind"] = "Data/UI/ability_icons/twilight_force_01.dds",
			Phalanx = "Data/UI/ability_icons/shielded.dds",
			["Pin Target"] = "Data/UI/ability_icons/critical2.dds",
			["Plague Bringer"] = "Data/UI/ability_icons/plague_01_a.dds",
			["Planar Blade"] = "Data/UI/ability_icons/shockwave3d.dds",
			["Planar Lure"] = "Data/UI/ability_icons/generic_ability_030.dds",
			["Power Sink"] = "Data/UI/ability_icons/magicresist1.dds",
			["Power Strike"] = "Data/UI/ability_icons/charge4z.dds",
			["Predictable Movements"] = "Data/UI/ability_icons/paragon-predicatable_movements_a.dds",
			Preservation = "Data/UI/ability_icons/paladin-preservation_a.dds",
			["Primal Recovery"] = "Data/UI/ability_icons/beastmaster_primal_recovery_01_a.dds",
			["Promise of Steel"] = "Data/UI/ability_icons/dancing_steel.dds",
			["Proper Timing"] = "Data/UI/item_icons/trinket4.dds",
			["Protective Companion"] = "Data/UI/ability_icons/beastmaster_protective_companion_01.dds",
			["Protective Instincts"] = "Data/UI/ability_icons/beastmaster_protective_instincts.dds",
			["Punishing Blow"] = "Data/UI/ability_icons/charge3a.dds",
			Ragestorm = "Data/UI/ability_icons/elemres4.dds",
			["Rallying Command"] = "Data/UI/ability_icons/blazeofglory3.dds",
			["Ravaging Strike"] = "Data/UI/ability_icons/charge1.dds",
			["Reaping Harvest"] = "Data/UI/ability_icons/twin_strike_a.dds",
			["Reckless Strike"] = "Data/UI/ability_icons/swiftstrike1a.dds",
			Retaliation = "Data/UI/ability_icons/deflection2a.dds",
			["Reverent Protection"] = "Data/UI/ability_icons/spiritofcrusader2.dds",
			["Rift Shield"] = "Data/UI/ability_icons/spiritofcrusader2.dds",
			["Rift Storm"] = "Data/UI/ability_icons/generic_ability_031.dds",
			["Rift Strike"] = "Data/UI/ability_icons/shadow_assault.dds",
			["Rift Summon"] = "Data/UI/ability_icons/void_knight-rift_summon.dds",
			["Rift Surge"] = "Data/UI/ability_icons/energyburst5.dds",
			["Rift Tap"] = "Data/UI/ability_icons/chainlighting2.dds",
			Riftwalk = "Data/UI/ability_icons/generic_ability_022.dds",
			["Righteous Blow"] = "Data/UI/ability_icons/flare_of_light_01.dds",
			["Rising Waterfall"] = "Data/UI/ability_icons/cleave1.dds",
			["Ruthless Pursuit"] = "Data/UI/ability_icons/secondwind1.dds",
			["Scales of Justice"] = "Data/UI/ability_icons/paladin-scales_of_justice.dds",
			["Searing Strike"] = "Data/UI/ability_icons/shockwave1.dds",
			["Second Wind"] = "Data/UI/ability_icons/mend4.dds",
			["Sergeant's Order"] = "Data/UI/ability_icons/berserk1b.dds",
			["Shadow of Dread"] = "Data/UI/ability_icons/souldrain4.dds",
			["Shield Charge"] = "Data/UI/ability_icons/defstrike3.dds",
			["Shield Throw"] = "Data/UI/ability_icons/defensivestrike1.dds",
			["Shield of the Chosen"] = "Data/UI/ability_icons/generic_ability_016.dds",
			["Shield of the Hero"] = "Data/UI/item_icons/1h_shield_054a.dds",
			["Shifting Blades"] = "Data/UI/ability_icons/paragon_shifting_blades_01.dds",
			["Shock Burst"] = "Data/UI/ability_icons/ward1zz.dds",
			["Shroud of Entropy"] = "Data/UI/ability_icons/hidden_veil_01.dds",
			Singularity = "Data/UI/ability_icons/generic_ability_028.dds",
			["Slashing Strike"] = "Data/UI/ability_icons/malicious_strike_01_a.dds",
			["Slayer's Bearing"] = "Data/UI/ability_icons/exposeweak1b.dds",
			["Soldier's Bearing"] = "Data/UI/ability_icons/berserk3.dds",
			["Soul Pillage"] = "Data/UI/ability_icons/spiritblast2.dds",
			["Soul Recall"] = "Data/UI/ability_icons/soulbind.dds",
			["Soul Sickness"] = "Data/UI/ability_icons/necropotence2.dds",
			Spark = "Data/UI/ability_icons/spiritblast3.dds",
			["Spell Sunder"] = "Data/UI/ability_icons/shatter1.dds",
			Spellbreaker = "Data/UI/ability_icons/magicward4.dds",
			["Spoils of the Hunt"] = "Data/UI/ability_icons/spiritofmongoose3.dds",
			["Spotter's Order"] = "Data/UI/ability_icons/awareness_01.dds",
			["Stand Off"] = "Data/UI/ability_icons/bindingforce2.dds",
			Stoneshield = "Data/UI/ability_icons/riftblade_stone_shield_01_b.dds",
			Stonespear = "Data/UI/item_icons/2h_polearm_019_a.dds",
			["Storm Blade"] = "Data/UI/ability_icons/shockwave3.dds",
			["Storm Burst"] = "Data/UI/ability_icons/chainlighting5a.dds",
			["Strike Like Iron"] = "Data/UI/ability_icons/intimidatingweapon2.dds",
			["Summon Greater Primal Companion"] = "Data/UI/ability_icons/beastmaster-greater_corporeal_manifestation.dds",
			["Summon Lesser Primal Companion"] = "Data/UI/ability_icons/beastmaster-lesser_corporeal_manifestation_a.dds",
			["Sweeping Strike"] = "Data/UI/ability_icons/criticalsweep1b.dds",
			["Swift Blades"] = "Data/UI/ability_icons/blade_tempo.dds",
			["Tearing Slash"] = "Data/UI/ability_icons/sweep3c.dds",
			["Tempered Will"] = "Data/UI/ability_icons/discombobulate2a.dds",
			Tempest = "Data/UI/ability_icons/void_knight-tempest.dds",
			["Thread the Trees"] = "Data/UI/ability_icons/tumble1.dds",
			["Thunder Strike"] = "Data/UI/ability_icons/riftblade_thunder_strike_01.dds",
			["Thunderous Kick"] = "Data/UI/ability_icons/kick1.dds",
			["Thunderous Leap"] = "Data/UI/ability_icons/fleeting_instincts_01_a.dds",
			["Titan's Strike"] = "Data/UI/ability_icons/generic_ability_007.dds",
			["Touch of Life"] = "Data/UI/ability_icons/layonhands5.dds",
			["Touch of Tranquility"] = "Data/UI/ability_icons/redirect4b.dds",
			["Track Death Creatures"] = "Data/UI/ability_icons/druid-spiteful_spirit_a.dds",
			["Track Earth Creatures"] = "Data/UI/ability_icons/druid-stone_strike.dds",
			["Track Ore"] = "Data/UI/item_icons/pick4.dds",
			Trauma = "Data/UI/ability_icons/cripple2c.dds",
			["Turn the Blade"] = "Data/UI/ability_icons/deflection4c.dds",
			Unrestrained = "Data/UI/ability_icons/strike_of_judgement_01.dds",
			["Unstable Reaction"] = "Data/UI/ability_icons/energybeam2a.dds",
			["Unstoppable Force"] = "Data/UI/ability_icons/spiritofcrusader2.dds",
			["Vicious Cleave"] = "Data/UI/ability_icons/debilitating_poison_01.dds",
			Void = "Data/UI/ability_icons/void_knight-void_a.dds",
			["Way of the Mountain"] = "Data/UI/ability_icons/paragon-way_of_mountain_a.dds",
			["Way of the River"] = "Data/UI/ability_icons/paragon_way_of_the_river_01.dds",
			["Way of the Wind"] = "Data/UI/ability_icons/secondwind2a.dds",
			["Weakening Essence"] = "Data/UI/ability_icons/weakening_essence_01.dds",
			["Weapon Blow"] = "Data/UI/ability_icons/disabling4a.dds",
			Windspear = "Data/UI/ability_icons/lance1.dds",
			["Wrist Strike"] = "Data/UI/ability_icons/disabling4.dds"
		}
	}

XMacros.TexturePicker.abilityList = {"Data/UI/ability_icons/absolution_01.dds",
"Data/UI/ability_icons/absolution_01_a.dds",
"Data/UI/ability_icons/alchemy1a.dds",
"Data/UI/ability_icons/alchemy1b.dds",
"Data/UI/ability_icons/alchemy2d.dds",
"Data/UI/ability_icons/alchemy2e.dds",
"Data/UI/ability_icons/alchemy2n.dds",
"Data/UI/ability_icons/ambidextrous.dds",
"Data/UI/ability_icons/anthem_of_competence.dds",
"Data/UI/ability_icons/anthem_of_defiance.dds",
"Data/UI/ability_icons/anthem_of_fervor.dds",
"Data/UI/ability_icons/anthem_of_fervor_a.dds",
"Data/UI/ability_icons/anthem_of_fervor_b.dds",
"Data/UI/ability_icons/anthem_of_glory.dds",
"Data/UI/ability_icons/anthem_of_glory_a.dds",
"Data/UI/ability_icons/arcaneshield2.dds",
"Data/UI/ability_icons/arcaneshield3.dds",
"Data/UI/ability_icons/arcaneshield4.dds",
"Data/UI/ability_icons/arcaneshield4a.dds",
"Data/UI/ability_icons/arcaneshield4aa.dds",
"Data/UI/ability_icons/archon-earthen_barrage.dds",
"Data/UI/ability_icons/archon-earthen_barrage_a.dds",
"Data/UI/ability_icons/archon-earthen_barrage_b.dds",
"Data/UI/ability_icons/archon_crumbling_resistance_01.dds",
"Data/UI/ability_icons/archon_crumbling_resistance_01_a.dds",
"Data/UI/ability_icons/archon_crumbling_resistance_01_b.dds",
"Data/UI/ability_icons/archon_rock_slide.dds",
"Data/UI/ability_icons/archon_rock_slide_a.dds",
"Data/UI/ability_icons/ardent_doctrine_01.dds",
"Data/UI/ability_icons/ardent_doctrine_01_a.dds",
"Data/UI/ability_icons/arkanxshield1.dds",
"Data/UI/ability_icons/armorsmith1.dds",
"Data/UI/ability_icons/armorsmith2.dds",
"Data/UI/ability_icons/armor_of_virtue_01.dds",
"Data/UI/ability_icons/artificer1.dds",
"Data/UI/ability_icons/artificer2a.dds",
"Data/UI/ability_icons/artificer2b.dds",
"Data/UI/ability_icons/auto_attack_1.dds",
"Data/UI/ability_icons/auto_attack_2.dds",
"Data/UI/ability_icons/awareness_01.dds",
"Data/UI/ability_icons/backstab.dds",
"Data/UI/ability_icons/backstab_a.dds",
"Data/UI/ability_icons/baneful_touch_01.dds",
"Data/UI/ability_icons/baneful_touch_01_a.dds",
"Data/UI/ability_icons/baneful_touch_01_b.dds",
"Data/UI/ability_icons/barbedtrap1.dds",
"Data/UI/ability_icons/barbedtrap1b.dds",
"Data/UI/ability_icons/barbedtrap3.dds",
"Data/UI/ability_icons/barbedtrap4.dds",
"Data/UI/ability_icons/barbedtrap4a.dds",
"Data/UI/ability_icons/barbedtrap4b.dds",
"Data/UI/ability_icons/bardic_inspiration.dds",
"Data/UI/ability_icons/bash1.dds",
"Data/UI/ability_icons/bash1a.dds",
"Data/UI/ability_icons/beastmaster-greater_corporeal_manifestation.dds",
"Data/UI/ability_icons/beastmaster-greater_corporeal_manifestation_a.dds",
"Data/UI/ability_icons/beastmaster-greater_corporeal_manifestation_b.dds",
"Data/UI/ability_icons/beastmaster-lesser_corporeal_manifestation.dds",
"Data/UI/ability_icons/beastmaster-lesser_corporeal_manifestation_a.dds",
"Data/UI/ability_icons/beastmaster-natural_instincts.dds",
"Data/UI/ability_icons/beastmaster_beast_whisperer_01.dds",
"Data/UI/ability_icons/beastmaster_bond_of_might.dds",
"Data/UI/ability_icons/beastmaster_bond_of_might_a.dds",
"Data/UI/ability_icons/beastmaster_bond_of_might_b.dds",
"Data/UI/ability_icons/beastmaster_bond_of_might_c.dds",
"Data/UI/ability_icons/beastmaster_bond_of_might_d.dds",
"Data/UI/ability_icons/beastmaster_enraged_companion_01.dds",
"Data/UI/ability_icons/beastmaster_enraged_companion_01_a.dds",
"Data/UI/ability_icons/beastmaster_enraged_companion_aura_01.dds",
"Data/UI/ability_icons/beastmaster_enraged_companion_aura_01_a.dds",
"Data/UI/ability_icons/beastmaster_imp_prot_instincts.dds",
"Data/UI/ability_icons/beastmaster_imp_prot_instincts_a.dds",
"Data/UI/ability_icons/beastmaster_kinship_01.dds",
"Data/UI/ability_icons/beastmaster_kinship_01_a.dds",
"Data/UI/ability_icons/beastmaster_kinship_01_b.dds",
"Data/UI/ability_icons/beastmaster_natural_instincts.dds",
"Data/UI/ability_icons/beastmaster_natural_instincts_a.dds",
"Data/UI/ability_icons/beastmaster_primal_recovery_01.dds",
"Data/UI/ability_icons/beastmaster_primal_recovery_01_a.dds",
"Data/UI/ability_icons/beastmaster_protective_companion_01.dds",
"Data/UI/ability_icons/beastmaster_protective_companion_01_a.dds",
"Data/UI/ability_icons/beastmaster_protective_companion_aura_01.dds",
"Data/UI/ability_icons/beastmaster_protective_companion_aura_01_a.dds",
"Data/UI/ability_icons/beastmaster_protective_companion_aura_01_b.dds",
"Data/UI/ability_icons/beastmaster_protective_companion_aura_01_c.dds",
"Data/UI/ability_icons/beastmaster_protective_instincts.dds",
"Data/UI/ability_icons/beastmaster_protective_instincts_a.dds",
"Data/UI/ability_icons/beastmaster_shared_bonds.dds",
"Data/UI/ability_icons/beastmaster_shared_bonds_a.dds",
"Data/UI/ability_icons/beastmaster_shared_bonds_b.dds",
"Data/UI/ability_icons/beastmaster_shared_bonds_c.dds",
"Data/UI/ability_icons/beastmaster_strike_to_maim.dds",
"Data/UI/ability_icons/beastmaster_strike_to_maim_a.dds",
"Data/UI/ability_icons/beastmaster_strike_to_maim_b.dds",
"Data/UI/ability_icons/beastmaster_survival_of_the_fittest.dds",
"Data/UI/ability_icons/beastmaster_survival_of_the_fittest_a.dds",
"Data/UI/ability_icons/beastmaster_survival_of_the_fittest_b.dds",
"Data/UI/ability_icons/beastmaster_survival_of_the_fittest_c.dds",
"Data/UI/ability_icons/beastmaster_ties_that_bind.dds",
"Data/UI/ability_icons/beastmaster_ties_that_bind_a.dds",
"Data/UI/ability_icons/berserk1.dds",
"Data/UI/ability_icons/berserk1a.dds",
"Data/UI/ability_icons/berserk1b.dds",
"Data/UI/ability_icons/berserk2.dds",
"Data/UI/ability_icons/berserk3.dds",
"Data/UI/ability_icons/berserk4.dds",
"Data/UI/ability_icons/berserk4a.dds",
"Data/UI/ability_icons/berserk4b.dds",
"Data/UI/ability_icons/bindingforce1.dds",
"Data/UI/ability_icons/bindingforce2.dds",
"Data/UI/ability_icons/binding_of_affliction_01.dds",
"Data/UI/ability_icons/binding_of_affliction_01_a.dds",
"Data/UI/ability_icons/binding_of_affliction_01_b.dds",
"Data/UI/ability_icons/binding_of_atrophy_01.dds",
"Data/UI/ability_icons/binding_of_authority_01.dds",
"Data/UI/ability_icons/binding_of_death_01.dds",
"Data/UI/ability_icons/binding_of_death_01_a.dds",
"Data/UI/ability_icons/blackout_01.dds",
"Data/UI/ability_icons/blackout_01_a.dds",
"Data/UI/ability_icons/bladeturn1.dds",
"Data/UI/ability_icons/blade_tempo.dds",
"Data/UI/ability_icons/blazeofglory1.dds",
"Data/UI/ability_icons/blazeofglory1a.dds",
"Data/UI/ability_icons/blazeofglory2.dds",
"Data/UI/ability_icons/blazeofglory3.dds",
"Data/UI/ability_icons/bleeding.dds",
"Data/UI/ability_icons/bleeding_a.dds",
"Data/UI/ability_icons/bleeding_aa.dds",
"Data/UI/ability_icons/blessed.dds",
"Data/UI/ability_icons/blessing_of_the_flames.dds",
"Data/UI/ability_icons/blessing_of_the_flames_a.dds",
"Data/UI/ability_icons/blindinglight1.dds",
"Data/UI/ability_icons/blindinglight1a.dds",
"Data/UI/ability_icons/blindinglight2.dds",
"Data/UI/ability_icons/blindinglight3.dds",
"Data/UI/ability_icons/bloodfire1.dds",
"Data/UI/ability_icons/bolster_01.dds",
"Data/UI/ability_icons/bolt_of_radiance_01.dds",
"Data/UI/ability_icons/bolt_of_radiance_01_a.dds",
"Data/UI/ability_icons/bomb_adhesive.dds",
"Data/UI/ability_icons/bomb_adhesive_a.dds",
"Data/UI/ability_icons/bomb_annihilation.dds",
"Data/UI/ability_icons/bomb_chemical.dds",
"Data/UI/ability_icons/bomb_choking_gas.dds",
"Data/UI/ability_icons/bomb_choking_gas_a.dds",
"Data/UI/ability_icons/bomb_fragment.dds",
"Data/UI/ability_icons/bomb_fragment_a.dds",
"Data/UI/ability_icons/bomb_specialist.dds",
"Data/UI/ability_icons/bomb_time.dds",
"Data/UI/ability_icons/bowyer1.dds",
"Data/UI/ability_icons/bowyer2.dds",
"Data/UI/ability_icons/bulwark1.dds",
"Data/UI/ability_icons/bulwark1a.dds",
"Data/UI/ability_icons/bulwark2.dds",
"Data/UI/ability_icons/burning.dds",
"Data/UI/ability_icons/cabalist-binding_waters.dds",
"Data/UI/ability_icons/cabalist-binding_waters_a.dds",
"Data/UI/ability_icons/cabalist_sigil_of_ravaging.dds",
"Data/UI/ability_icons/cabalist_sigil_of_ravaging_a.dds",
"Data/UI/ability_icons/cadence.dds",
"Data/UI/ability_icons/cadence_a.dds",
"Data/UI/ability_icons/campfire1.dds",
"Data/UI/ability_icons/carpet_bombing.dds",
"Data/UI/ability_icons/carpet_bombing_a.dds",
"Data/UI/ability_icons/castigation1a.dds",
"Data/UI/ability_icons/castigation1b.dds",
"Data/UI/ability_icons/castigation1c.dds",
"Data/UI/ability_icons/castigation1d.dds",
"Data/UI/ability_icons/castigation2.dds",
"Data/UI/ability_icons/castigation3.dds",
"Data/UI/ability_icons/cauterize.dds",
"Data/UI/ability_icons/cauterize1.dds",
"Data/UI/ability_icons/cautious_stalker.dds",
"Data/UI/ability_icons/cavalier_01.dds",
"Data/UI/ability_icons/cavalier_01_a.dds",
"Data/UI/ability_icons/censure_01.dds",
"Data/UI/ability_icons/chainlighting1.dds",
"Data/UI/ability_icons/chainlighting2.dds",
"Data/UI/ability_icons/chainlighting2b.dds",
"Data/UI/ability_icons/chainlighting4.dds",
"Data/UI/ability_icons/chainlighting5.dds",
"Data/UI/ability_icons/chainlighting5a.dds",
"Data/UI/ability_icons/chainsofice1.dds",
"Data/UI/ability_icons/chainsofice3.dds",
"Data/UI/ability_icons/chainsofice3a.dds",
"Data/UI/ability_icons/champion-bull_rush.dds",
"Data/UI/ability_icons/champion-bull_rush_overrun.dds",
"Data/UI/ability_icons/champion-bull_rush_overrun_a.dds",
"Data/UI/ability_icons/champion-destroyers_bearing.dds",
"Data/UI/ability_icons/champion-destroyers_bearing_a.dds",
"Data/UI/ability_icons/channelhealth.dds",
"Data/UI/ability_icons/channelhealth1a.dds",
"Data/UI/ability_icons/channelhealth2.dds",
"Data/UI/ability_icons/channelhealth3.dds",
"Data/UI/ability_icons/channelhealth4.dds",
"Data/UI/ability_icons/channelhealth4a.dds",
"Data/UI/ability_icons/charge1.dds",
"Data/UI/ability_icons/charge1a.dds",
"Data/UI/ability_icons/charge2.dds",
"Data/UI/ability_icons/charge3.dds",
"Data/UI/ability_icons/charge3a.dds",
"Data/UI/ability_icons/charge4.dds",
"Data/UI/ability_icons/charge4a.dds",
"Data/UI/ability_icons/charge4z.dds",
"Data/UI/ability_icons/charge_blast.dds",
"Data/UI/ability_icons/charge_caltrop.dds",
"Data/UI/ability_icons/charge_concussion.dds",
"Data/UI/ability_icons/charge_concussion_a.dds",
"Data/UI/ability_icons/charge_ember.dds",
"Data/UI/ability_icons/charge_shrapnel.dds",
"Data/UI/ability_icons/charge_spike.dds",
"Data/UI/ability_icons/charge_splinter.dds",
"Data/UI/ability_icons/chloromancer-natural_conversion.dds",
"Data/UI/ability_icons/chloromancer_entropic_veil_01.dds",
"Data/UI/ability_icons/chloromancer_entropic_veil_01_a.dds",
"Data/UI/ability_icons/chloromancer_void_life_01.dds",
"Data/UI/ability_icons/chloromancer_void_life_01_a.dds",
"Data/UI/ability_icons/cleave1.dds",
"Data/UI/ability_icons/cleave1a.dds",
"Data/UI/ability_icons/close_combat.dds",
"Data/UI/ability_icons/coalesceheat1.dds",
"Data/UI/ability_icons/coalesceheat2.dds",
"Data/UI/ability_icons/coalesceheat3.dds",
"Data/UI/ability_icons/coda_of_cowardice.dds",
"Data/UI/ability_icons/coda_of_cowardice_a.dds",
"Data/UI/ability_icons/coda_of_distress.dds",
"Data/UI/ability_icons/coda_of_distress_a.dds",
"Data/UI/ability_icons/coda_of_fury.dds",
"Data/UI/ability_icons/coda_of_fury_a.dds",
"Data/UI/ability_icons/coda_of_restoration.dds",
"Data/UI/ability_icons/coda_of_restoration_a.dds",
"Data/UI/ability_icons/coda_of_wrath.dds",
"Data/UI/ability_icons/coda_of_wrath_a.dds",
"Data/UI/ability_icons/combat_culmination.dds",
"Data/UI/ability_icons/combat_pose.dds",
"Data/UI/ability_icons/combat_pose1.dds",
"Data/UI/ability_icons/combat_pose1_a.dds",
"Data/UI/ability_icons/combat_survival.dds",
"Data/UI/ability_icons/concussion_01.dds",
"Data/UI/ability_icons/concussion_01_a.dds",
"Data/UI/ability_icons/concussion_01_b.dds",
"Data/UI/ability_icons/concussive_blast_01.dds",
"Data/UI/ability_icons/conflagration1.dds",
"Data/UI/ability_icons/conflagration2.dds",
"Data/UI/ability_icons/conflagration3.dds",
"Data/UI/ability_icons/conflagration4.dds",
"Data/UI/ability_icons/confound1.dds",
"Data/UI/ability_icons/confound2.dds",
"Data/UI/ability_icons/confound3.dds",
"Data/UI/ability_icons/controlled_fire.dds",
"Data/UI/ability_icons/conviction_01.dds",
"Data/UI/ability_icons/cooking1.dds",
"Data/UI/ability_icons/creeping_death_01.dds",
"Data/UI/ability_icons/creeping_death_01_a.dds",
"Data/UI/ability_icons/cripple1.dds",
"Data/UI/ability_icons/cripple1a.dds",
"Data/UI/ability_icons/cripple2a.dds",
"Data/UI/ability_icons/cripple2c.dds",
"Data/UI/ability_icons/cripple2f.dds",
"Data/UI/ability_icons/cripplingshot1.dds",
"Data/UI/ability_icons/crippling_shot_01.dds",
"Data/UI/ability_icons/critical2.dds",
"Data/UI/ability_icons/critical2a.dds",
"Data/UI/ability_icons/critical3.dds",
"Data/UI/ability_icons/critical4.dds",
"Data/UI/ability_icons/critical4a.dds",
"Data/UI/ability_icons/criticalblow4m.dds",
"Data/UI/ability_icons/criticalblow4n.dds",
"Data/UI/ability_icons/criticalstrike1.dds",
"Data/UI/ability_icons/criticalsweep1b.dds",
"Data/UI/ability_icons/criticalsweep2.dds",
"Data/UI/ability_icons/criticalsweep3a.dds",
"Data/UI/ability_icons/criticalsweep3b.dds",
"Data/UI/ability_icons/criticalsweep3c.dds",
"Data/UI/ability_icons/critical_sweep1.dds",
"Data/UI/ability_icons/cruel_vengeance_01.dds",
"Data/UI/ability_icons/curse_of_solitude.dds",
"Data/UI/ability_icons/customcraft1.dds",
"Data/UI/ability_icons/customcraft1a.dds",
"Data/UI/ability_icons/customcraft1b.dds",
"Data/UI/ability_icons/dancing_steel.dds",
"Data/UI/ability_icons/darkball1.dds",
"Data/UI/ability_icons/darkball2.dds",
"Data/UI/ability_icons/darkball2z.dds",
"Data/UI/ability_icons/darkball3.dds",
"Data/UI/ability_icons/darkball4.dds",
"Data/UI/ability_icons/darkbolt1.dds",
"Data/UI/ability_icons/darkbolt2.dds",
"Data/UI/ability_icons/darkbolt3.dds",
"Data/UI/ability_icons/darkbolt5.dds",
"Data/UI/ability_icons/dark_containment_01.dds",
"Data/UI/ability_icons/dark_harbor.dds",
"Data/UI/ability_icons/dark_malady_01.dds",
"Data/UI/ability_icons/dark_malady_01_a.dds",
"Data/UI/ability_icons/dark_waters.dds",
"Data/UI/ability_icons/dark_waters_a.dds",
"Data/UI/ability_icons/deafening_music.dds",
"Data/UI/ability_icons/debilitating_poison_01.dds",
"Data/UI/ability_icons/debilitating_poison_01_a.dds",
"Data/UI/ability_icons/debilitating_poison_01_b.dds",
"Data/UI/ability_icons/defensivestrike1.dds",
"Data/UI/ability_icons/defensivestrike2.dds",
"Data/UI/ability_icons/defensivestrike3.dds",
"Data/UI/ability_icons/defer_death.dds",
"Data/UI/ability_icons/deflect1.dds",
"Data/UI/ability_icons/deflect1a.dds",
"Data/UI/ability_icons/deflect1b.dds",
"Data/UI/ability_icons/deflect1c.dds",
"Data/UI/ability_icons/deflect2.dds",
"Data/UI/ability_icons/deflection1.dds",
"Data/UI/ability_icons/deflection1a.dds",
"Data/UI/ability_icons/deflection2.dds",
"Data/UI/ability_icons/deflection2a.dds",
"Data/UI/ability_icons/deflection2b.dds",
"Data/UI/ability_icons/deflection3.dds",
"Data/UI/ability_icons/deflection3a.dds",
"Data/UI/ability_icons/deflection3aa.dds",
"Data/UI/ability_icons/deflection3b.dds",
"Data/UI/ability_icons/deflection3c.dds",
"Data/UI/ability_icons/deflection4b.dds",
"Data/UI/ability_icons/deflection4c.dds",
"Data/UI/ability_icons/deflection4cc.dds",
"Data/UI/ability_icons/deflection4ccc.dds",
"Data/UI/ability_icons/defstrike2.dds",
"Data/UI/ability_icons/defstrike3.dds",
"Data/UI/ability_icons/demolition_specialist.dds",
"Data/UI/ability_icons/determination1.dds",
"Data/UI/ability_icons/determination2.dds",
"Data/UI/ability_icons/detonate.dds",
"Data/UI/ability_icons/detonate_a.dds",
"Data/UI/ability_icons/devouring_mist.dds",
"Data/UI/ability_icons/devouring_mist_a.dds",
"Data/UI/ability_icons/devout_conviction_01.dds",
"Data/UI/ability_icons/devout_conviction_01_a.dds",
"Data/UI/ability_icons/diffuse_01.dds",
"Data/UI/ability_icons/diffuse_01_a.dds",
"Data/UI/ability_icons/dig1.dds",
"Data/UI/ability_icons/dirtytricks1a.dds",
"Data/UI/ability_icons/dirtytricks1b.dds",
"Data/UI/ability_icons/dirtytricks1c.dds",
"Data/UI/ability_icons/dirtytricks2.dds",
"Data/UI/ability_icons/dirtytricks2a.dds",
"Data/UI/ability_icons/dirtytricks2b.dds",
"Data/UI/ability_icons/dirtytricks3.dds",
"Data/UI/ability_icons/dirtytricks4.dds",
"Data/UI/ability_icons/disabling1.dds",
"Data/UI/ability_icons/disabling1a.dds",
"Data/UI/ability_icons/disabling2.dds",
"Data/UI/ability_icons/disabling3.dds",
"Data/UI/ability_icons/disabling3a.dds",
"Data/UI/ability_icons/disabling4.dds",
"Data/UI/ability_icons/disabling4a.dds",
"Data/UI/ability_icons/disabling4b.dds",
"Data/UI/ability_icons/discombobulate1.dds",
"Data/UI/ability_icons/discombobulate2.dds",
"Data/UI/ability_icons/discombobulate2a.dds",
"Data/UI/ability_icons/discombobulate3.dds",
"Data/UI/ability_icons/discombobulate3a.dds",
"Data/UI/ability_icons/disenchant1.dds",
"Data/UI/ability_icons/distorted_shadows.dds",
"Data/UI/ability_icons/distorted_shadows_a.dds",
"Data/UI/ability_icons/distorted_shadows_b.dds",
"Data/UI/ability_icons/divert_rage_01.dds",
"Data/UI/ability_icons/divert_rage_01_a.dds",
"Data/UI/ability_icons/doctrine_of_authority_01.dds",
"Data/UI/ability_icons/doctrine_of_authority_01_a.dds",
"Data/UI/ability_icons/doctrine_of_bliss_01.dds",
"Data/UI/ability_icons/doctrine_of_bliss_01_a.dds",
"Data/UI/ability_icons/doctrine_of_bliss_01_b.dds",
"Data/UI/ability_icons/doctrine_of_bliss_01_c.dds",
"Data/UI/ability_icons/doctrine_of_dedication_01.dds",
"Data/UI/ability_icons/doctrine_of_dedication_01_a.dds",
"Data/UI/ability_icons/doctrine_of_faithful_01.dds",
"Data/UI/ability_icons/doctrine_of_loyalty_01.dds",
"Data/UI/ability_icons/doctrine_of_loyalty_01_a.dds",
"Data/UI/ability_icons/doctrine_of_virtue_01.dds",
"Data/UI/ability_icons/dominator-mass_exhaustion.dds",
"Data/UI/ability_icons/dominator-mass_exhaustion_a.dds",
"Data/UI/ability_icons/dominator-mass_exhaustion_b.dds",
"Data/UI/ability_icons/dominator-split_personality.dds",
"Data/UI/ability_icons/dominator-split_personality_a.dds",
"Data/UI/ability_icons/dominator_draining_presence_01.dds",
"Data/UI/ability_icons/dominator_draining_presence_01_a.dds",
"Data/UI/ability_icons/dominator_draining_presence_01_b.dds",
"Data/UI/ability_icons/double_cross_01.dds",
"Data/UI/ability_icons/double_cross_01_a.dds",
"Data/UI/ability_icons/double_shot_01.dds",
"Data/UI/ability_icons/double_shot_01_a.dds",
"Data/UI/ability_icons/drainlife1.dds",
"Data/UI/ability_icons/drainlife3.dds",
"Data/UI/ability_icons/drowning1.dds",
"Data/UI/ability_icons/drowning2.dds",
"Data/UI/ability_icons/druid-bombard.dds",
"Data/UI/ability_icons/druid-bombard_a.dds",
"Data/UI/ability_icons/druid-grim_silence.dds",
"Data/UI/ability_icons/druid-hidden_path.dds",
"Data/UI/ability_icons/druid-slothful_spirit.dds",
"Data/UI/ability_icons/druid-slothful_spirit_a.dds",
"Data/UI/ability_icons/druid-spiteful_spirit.dds",
"Data/UI/ability_icons/druid-spiteful_spirit_a.dds",
"Data/UI/ability_icons/druid-stone_strike.dds",
"Data/UI/ability_icons/druid-stone_strike_a.dds",
"Data/UI/ability_icons/druid-strength_of_the_fae.dds",
"Data/UI/ability_icons/druid-summon_faerie.dds",
"Data/UI/ability_icons/druid-summon_greater_faerie.dds",
"Data/UI/ability_icons/druid-summon_satyr.dds",
"Data/UI/ability_icons/druid-summon_satyr_a.dds",
"Data/UI/ability_icons/dusk_to_dawn_01.dds",
"Data/UI/ability_icons/dusk_to_dawn_01_a.dds",
"Data/UI/ability_icons/ebon_fury_01.dds",
"Data/UI/ability_icons/ebon_terror_01.dds",
"Data/UI/ability_icons/elementalres2b.dds",
"Data/UI/ability_icons/elementalward2a.dds",
"Data/UI/ability_icons/elementalward3.dds",
"Data/UI/ability_icons/elemental_attunement1.dds",
"Data/UI/ability_icons/elemental_bolt1a.dds",
"Data/UI/ability_icons/elemental_bolt1b.dds",
"Data/UI/ability_icons/elemres4.dds",
"Data/UI/ability_icons/elemres4b.dds",
"Data/UI/ability_icons/elemres4d.dds",
"Data/UI/ability_icons/elemresist1.dds",
"Data/UI/ability_icons/elemresistance3.dds",
"Data/UI/ability_icons/elemward1.dds",
"Data/UI/ability_icons/elemward1a.dds",
"Data/UI/ability_icons/elemward4.dds",
"Data/UI/ability_icons/embolden1.dds",
"Data/UI/ability_icons/embolden1a.dds",
"Data/UI/ability_icons/embolden1aa.dds",
"Data/UI/ability_icons/embolden1b.dds",
"Data/UI/ability_icons/embolden2a.dds",
"Data/UI/ability_icons/enchant1.dds",
"Data/UI/ability_icons/energybeam1.dds",
"Data/UI/ability_icons/energybeam1a.dds",
"Data/UI/ability_icons/energybeam1b.dds",
"Data/UI/ability_icons/energybeam2.dds",
"Data/UI/ability_icons/energybeam2a.dds",
"Data/UI/ability_icons/energyblast1.dds",
"Data/UI/ability_icons/energyblast2.dds",
"Data/UI/ability_icons/energyblast3.dds",
"Data/UI/ability_icons/energybolt1.dds",
"Data/UI/ability_icons/energybolt2.dds",
"Data/UI/ability_icons/energybolt3.dds",
"Data/UI/ability_icons/energybolt4.dds",
"Data/UI/ability_icons/energyburst3f.dds",
"Data/UI/ability_icons/energyburst3g.dds",
"Data/UI/ability_icons/energyburst5.dds",
"Data/UI/ability_icons/energy_overload1.dds",
"Data/UI/ability_icons/enflamed_rejuvenation.dds",
"Data/UI/ability_icons/enhanced_weapon_enchantments_01.dds",
"Data/UI/ability_icons/enhanced_weapon_enchantments_01_a.dds",
"Data/UI/ability_icons/enraged.dds",
"Data/UI/ability_icons/enraged_a.dds",
"Data/UI/ability_icons/ensnare1.dds",
"Data/UI/ability_icons/ensnare2.dds",
"Data/UI/ability_icons/ensnare3.dds",
"Data/UI/ability_icons/ensnare3a.dds",
"Data/UI/ability_icons/ensnare5.dds",
"Data/UI/ability_icons/entropy_embrace.dds",
"Data/UI/ability_icons/escape_artist_01.dds",
"Data/UI/ability_icons/escape_artist_01_a.dds",
"Data/UI/ability_icons/escape_artist_01_b.dds",
"Data/UI/ability_icons/even_justice_01.dds",
"Data/UI/ability_icons/excommunicate.dds",
"Data/UI/ability_icons/exorcize1.dds",
"Data/UI/ability_icons/exorcize2.dds",
"Data/UI/ability_icons/exorcize2a.dds",
"Data/UI/ability_icons/exorcize3.dds",
"Data/UI/ability_icons/explosivetrap1.dds",
"Data/UI/ability_icons/explosivetrap2.dds",
"Data/UI/ability_icons/explosivetrap3.dds",
"Data/UI/ability_icons/explosive_infestation_01.dds",
"Data/UI/ability_icons/explosive_infestation_01_a.dds",
"Data/UI/ability_icons/exposeweak1.dds",
"Data/UI/ability_icons/exposeweak1a.dds",
"Data/UI/ability_icons/exposeweak1b.dds",
"Data/UI/ability_icons/exposeweak1d.dds",
"Data/UI/ability_icons/exposeweakness1.dds",
"Data/UI/ability_icons/exposeweakness2.dds",
"Data/UI/ability_icons/exposeweakness2a.dds",
"Data/UI/ability_icons/exposeweakness3.dds",
"Data/UI/ability_icons/exposeweakness4.dds",
"Data/UI/ability_icons/eye_of_the_storm.dds",
"Data/UI/ability_icons/eye_of_the_storm_a.dds",
"Data/UI/ability_icons/faithful_doctrine_01.dds",
"Data/UI/ability_icons/faithful_doctrine_01_a.dds",
"Data/UI/ability_icons/fanaticism.dds",
"Data/UI/ability_icons/fanfare_of_knowledge.dds",
"Data/UI/ability_icons/fanfare_of_knowledge_a.dds",
"Data/UI/ability_icons/fanfare_of_power.dds",
"Data/UI/ability_icons/fanfare_of_power_a.dds",
"Data/UI/ability_icons/fanfare_of_vigor.dds",
"Data/UI/ability_icons/fatique.dds",
"Data/UI/ability_icons/fiery_blessing.dds",
"Data/UI/ability_icons/fiery_blessing_a.dds",
"Data/UI/ability_icons/fiery_chains_01.dds",
"Data/UI/ability_icons/fiery_chains_01_a.dds",
"Data/UI/ability_icons/fiery_spike_01.dds",
"Data/UI/ability_icons/finalstrike1.dds",
"Data/UI/ability_icons/finalstrike1a.dds",
"Data/UI/ability_icons/finalstrike2.dds",
"Data/UI/ability_icons/finalstrike3.dds",
"Data/UI/ability_icons/finalstrike3a.dds",
"Data/UI/ability_icons/fireball.dds",
"Data/UI/ability_icons/fire_and_forget.dds",
"Data/UI/ability_icons/fire_death_attunement_01.dds",
"Data/UI/ability_icons/flamejet1.dds",
"Data/UI/ability_icons/flamejet2big.dds",
"Data/UI/ability_icons/flame_speaker.dds",
"Data/UI/ability_icons/flame_speaker1.dds",
"Data/UI/ability_icons/flame_speaker_a.dds",
"Data/UI/ability_icons/flame_thrust_01.dds",
"Data/UI/ability_icons/flame_thrust_01_a.dds",
"Data/UI/ability_icons/flare_of_light_01.dds",
"Data/UI/ability_icons/flash1.dds",
"Data/UI/ability_icons/flash2.dds",
"Data/UI/ability_icons/flashback.dds",
"Data/UI/ability_icons/flashback_a.dds",
"Data/UI/ability_icons/fleeting_instincts_01.dds",
"Data/UI/ability_icons/fleeting_instincts_01_a.dds",
"Data/UI/ability_icons/fleeting_instincts_02.dds",
"Data/UI/ability_icons/fleeting_instincts_02_a.dds",
"Data/UI/ability_icons/fleeting_instincts_02_b.dds",
"Data/UI/ability_icons/fleeting_instincts_02_c.dds",
"Data/UI/ability_icons/flesh_rot_01.dds",
"Data/UI/ability_icons/flesh_rot_01_a.dds",
"Data/UI/ability_icons/flesh_rot_01_b.dds",
"Data/UI/ability_icons/foraging1.dds",
"Data/UI/ability_icons/foraging2.dds",
"Data/UI/ability_icons/foraging2a.dds",
"Data/UI/ability_icons/force_to_kneel_01.dds",
"Data/UI/ability_icons/foul_play_01.dds",
"Data/UI/ability_icons/foul_play_01_a.dds",
"Data/UI/ability_icons/foul_play_02.dds",
"Data/UI/ability_icons/foul_play_02_a.dds",
"Data/UI/ability_icons/freedom_of_movement.dds",
"Data/UI/ability_icons/frost_hammer1.dds",
"Data/UI/ability_icons/furious_strike.dds",
"Data/UI/ability_icons/generic_ability_001.dds",
"Data/UI/ability_icons/generic_ability_002.dds",
"Data/UI/ability_icons/generic_ability_002_a.dds",
"Data/UI/ability_icons/generic_ability_003.dds",
"Data/UI/ability_icons/generic_ability_004.dds",
"Data/UI/ability_icons/generic_ability_005.dds",
"Data/UI/ability_icons/generic_ability_006.dds",
"Data/UI/ability_icons/generic_ability_007.dds",
"Data/UI/ability_icons/generic_ability_007_a.dds",
"Data/UI/ability_icons/generic_ability_007_b.dds",
"Data/UI/ability_icons/generic_ability_007_c.dds",
"Data/UI/ability_icons/generic_ability_008.dds",
"Data/UI/ability_icons/generic_ability_009.dds",
"Data/UI/ability_icons/generic_ability_009_a.dds",
"Data/UI/ability_icons/generic_ability_009_b.dds",
"Data/UI/ability_icons/generic_ability_010.dds",
"Data/UI/ability_icons/generic_ability_010_a.dds",
"Data/UI/ability_icons/generic_ability_011.dds",
"Data/UI/ability_icons/generic_ability_012.dds",
"Data/UI/ability_icons/generic_ability_012_a.dds",
"Data/UI/ability_icons/generic_ability_012_b.dds",
"Data/UI/ability_icons/generic_ability_013.dds",
"Data/UI/ability_icons/generic_ability_014.dds",
"Data/UI/ability_icons/generic_ability_014_a.dds",
"Data/UI/ability_icons/generic_ability_014_b.dds",
"Data/UI/ability_icons/generic_ability_015.dds",
"Data/UI/ability_icons/generic_ability_015_a.dds",
"Data/UI/ability_icons/generic_ability_015_aa.dds",
"Data/UI/ability_icons/generic_ability_015_z.dds",
"Data/UI/ability_icons/generic_ability_016.dds",
"Data/UI/ability_icons/generic_ability_017.dds",
"Data/UI/ability_icons/generic_ability_017_a.dds",
"Data/UI/ability_icons/generic_ability_018.dds",
"Data/UI/ability_icons/generic_ability_019.dds",
"Data/UI/ability_icons/generic_ability_020.dds",
"Data/UI/ability_icons/generic_ability_020_a.dds",
"Data/UI/ability_icons/generic_ability_021.dds",
"Data/UI/ability_icons/generic_ability_022.dds",
"Data/UI/ability_icons/generic_ability_023.dds",
"Data/UI/ability_icons/generic_ability_024.dds",
"Data/UI/ability_icons/generic_ability_024_a.dds",
"Data/UI/ability_icons/generic_ability_025.dds",
"Data/UI/ability_icons/generic_ability_025_a.dds",
"Data/UI/ability_icons/generic_ability_026.dds",
"Data/UI/ability_icons/generic_ability_026_a.dds",
"Data/UI/ability_icons/generic_ability_027.dds",
"Data/UI/ability_icons/generic_ability_027_a.dds",
"Data/UI/ability_icons/generic_ability_028.dds",
"Data/UI/ability_icons/generic_ability_029.dds",
"Data/UI/ability_icons/generic_ability_030.dds",
"Data/UI/ability_icons/generic_ability_030_a.dds",
"Data/UI/ability_icons/generic_ability_031.dds",
"Data/UI/ability_icons/generic_ability_031_2.dds",
"Data/UI/ability_icons/good_health.dds",
"Data/UI/ability_icons/grim_blade_01.dds",
"Data/UI/ability_icons/grim_blade_01_a.dds",
"Data/UI/ability_icons/grim_blade_01_b.dds",
"Data/UI/ability_icons/guardian_phase.dds",
"Data/UI/ability_icons/guardian_phase_a.dds",
"Data/UI/ability_icons/hammer_of_virtue_01.dds",
"Data/UI/ability_icons/hamstring1.dds",
"Data/UI/ability_icons/hamstring2.dds",
"Data/UI/ability_icons/hardened_stalker.dds",
"Data/UI/ability_icons/hasted_time.dds",
"Data/UI/ability_icons/hasted_time_a.dds",
"Data/UI/ability_icons/headshot1b.dds",
"Data/UI/ability_icons/headshot2.dds",
"Data/UI/ability_icons/headshot3.dds",
"Data/UI/ability_icons/heal1.dds",
"Data/UI/ability_icons/heal1a.dds",
"Data/UI/ability_icons/heal2.dds",
"Data/UI/ability_icons/heal2a.dds",
"Data/UI/ability_icons/heal2z.dds",
"Data/UI/ability_icons/heal3.dds",
"Data/UI/ability_icons/healed.dds",
"Data/UI/ability_icons/healing_flame.dds",
"Data/UI/ability_icons/healing_flare.dds",
"Data/UI/ability_icons/healing_flare2.dds",
"Data/UI/ability_icons/healing_flare2_a.dds",
"Data/UI/ability_icons/healing_flare_a.dds",
"Data/UI/ability_icons/healing_flare_b.dds",
"Data/UI/ability_icons/healthchannel1.dds",
"Data/UI/ability_icons/healthchannel1a.dds",
"Data/UI/ability_icons/healthchannel1b.dds",
"Data/UI/ability_icons/healthchannel1g.dds",
"Data/UI/ability_icons/healthchannel1gg.dds",
"Data/UI/ability_icons/healthchannel1gy.dds",
"Data/UI/ability_icons/healthchannel2.dds",
"Data/UI/ability_icons/healthchannel3.dds",
"Data/UI/ability_icons/healthchannel3a.dds",
"Data/UI/ability_icons/healthchannel3b.dds",
"Data/UI/ability_icons/heavybleeding.dds",
"Data/UI/ability_icons/hellfire_blades_01.dds",
"Data/UI/ability_icons/heroicsurge1.dds",
"Data/UI/ability_icons/heroicsurge1a.dds",
"Data/UI/ability_icons/hidden_veil_01.dds",
"Data/UI/ability_icons/hit_and_run.dds",
"Data/UI/ability_icons/hit_and_run_a.dds",
"Data/UI/ability_icons/holysmite1.dds",
"Data/UI/ability_icons/holysmite3.dds",
"Data/UI/ability_icons/holysmite4.dds",
"Data/UI/ability_icons/holystrength1.dds",
"Data/UI/ability_icons/holystrength1b.dds",
"Data/UI/ability_icons/holystrength2.dds",
"Data/UI/ability_icons/holystrength3.dds",
"Data/UI/ability_icons/holystrength3a.dds",
"Data/UI/ability_icons/holystrength3aa.dds",
"Data/UI/ability_icons/holystrength3c.dds",
"Data/UI/ability_icons/holysweep1.dds",
"Data/UI/ability_icons/holysweep1z.dds",
"Data/UI/ability_icons/holysweep2.dds",
"Data/UI/ability_icons/holysweep2a.dds",
"Data/UI/ability_icons/holysweep3.dds",
"Data/UI/ability_icons/holysweep3a.dds",
"Data/UI/ability_icons/holysweep3aa.dds",
"Data/UI/ability_icons/holysweep3c.dds",
"Data/UI/ability_icons/holysweep3cc.dds",
"Data/UI/ability_icons/holysweep4d.dds",
"Data/UI/ability_icons/holysweep4g.dds",
"Data/UI/ability_icons/holysweep4ga.dds",
"Data/UI/ability_icons/humbling_blow_01.dds",
"Data/UI/ability_icons/icebolt1.dds",
"Data/UI/ability_icons/icebolt2.dds",
"Data/UI/ability_icons/icebolt3.dds",
"Data/UI/ability_icons/icebolt4.dds",
"Data/UI/ability_icons/icebolt4a.dds",
"Data/UI/ability_icons/icebolt4aa.dds",
"Data/UI/ability_icons/icestorm-light.dds",
"Data/UI/ability_icons/icestormdark.dds",
"Data/UI/ability_icons/icy_heart1.dds",
"Data/UI/ability_icons/icy_heart2.dds",
"Data/UI/ability_icons/illumination_01.dds",
"Data/UI/ability_icons/illumination_01_a.dds",
"Data/UI/ability_icons/impede1.dds",
"Data/UI/ability_icons/impede1a.dds",
"Data/UI/ability_icons/improved_master_of_the_abyss_01.dds",
"Data/UI/ability_icons/incapacitate_01.dds",
"Data/UI/ability_icons/incapacitate_01_a.dds",
"Data/UI/ability_icons/incite_01.dds",
"Data/UI/ability_icons/incite_01_a.dds",
"Data/UI/ability_icons/increased_explosive_power.dds",
"Data/UI/ability_icons/increased_explosive_power_a.dds",
"Data/UI/ability_icons/infestation_01.dds",
"Data/UI/ability_icons/infestation_01_a.dds",
"Data/UI/ability_icons/inspiration1.dds",
"Data/UI/ability_icons/inspiration1a.dds",
"Data/UI/ability_icons/inspiration2.dds",
"Data/UI/ability_icons/inspiration2z.dds",
"Data/UI/ability_icons/inspiration3.dds",
"Data/UI/ability_icons/inspiration4.dds",
"Data/UI/ability_icons/intense_flames.dds",
"Data/UI/ability_icons/intense_flames_a.dds",
"Data/UI/ability_icons/interdict_01.dds",
"Data/UI/ability_icons/internalizeheat1.dds",
"Data/UI/ability_icons/internalizeheat2.dds",
"Data/UI/ability_icons/internalizeheat3.dds",
"Data/UI/ability_icons/internalizeheat3b.dds",
"Data/UI/ability_icons/intervene1.dds",
"Data/UI/ability_icons/intervene2.dds",
"Data/UI/ability_icons/intimidatingweapon1.dds",
"Data/UI/ability_icons/intimidatingweapon1a.dds",
"Data/UI/ability_icons/intimidatingweapon2.dds",
"Data/UI/ability_icons/invigorate1.dds",
"Data/UI/ability_icons/invigorate2.dds",
"Data/UI/ability_icons/invigoration1.dds",
"Data/UI/ability_icons/invigoration1a.dds",
"Data/UI/ability_icons/invigoration1b.dds",
"Data/UI/ability_icons/invigoration1r.dds",
"Data/UI/ability_icons/invigoration2.dds",
"Data/UI/ability_icons/invigoration2a.dds",
"Data/UI/ability_icons/invigoration3.dds",
"Data/UI/ability_icons/invoker_prerogative1.dds",
"Data/UI/ability_icons/ironskin1.dds",
"Data/UI/ability_icons/ironwill1.dds",
"Data/UI/ability_icons/ironwill2.dds",
"Data/UI/ability_icons/ironwill2a.dds",
"Data/UI/ability_icons/jaggedlighting1.dds",
"Data/UI/ability_icons/jaggedlighting2.dds",
"Data/UI/ability_icons/jaggedlighting2a.dds",
"Data/UI/ability_icons/keen_eye.dds",
"Data/UI/ability_icons/kick1.dds",
"Data/UI/ability_icons/kick2.dds",
"Data/UI/ability_icons/kick2x.dds",
"Data/UI/ability_icons/kick3.dds",
"Data/UI/ability_icons/kick4.dds",
"Data/UI/ability_icons/killer_instinct.dds",
"Data/UI/ability_icons/killing_focus_01.dds",
"Data/UI/ability_icons/lance1.dds",
"Data/UI/ability_icons/lance1_a.dds",
"Data/UI/ability_icons/land_mines.dds",
"Data/UI/ability_icons/last_stand.dds",
"Data/UI/ability_icons/last_stand_a.dds",
"Data/UI/ability_icons/latent_blaze.dds",
"Data/UI/ability_icons/latent_blaze_a.dds",
"Data/UI/ability_icons/layinghands1.dds",
"Data/UI/ability_icons/layinghands1x.dds",
"Data/UI/ability_icons/layonhands2.dds",
"Data/UI/ability_icons/layonhands2z.dds",
"Data/UI/ability_icons/layonhands3.dds",
"Data/UI/ability_icons/layonhands4a.dds",
"Data/UI/ability_icons/layonhands4aa.dds",
"Data/UI/ability_icons/layonhands5.dds",
"Data/UI/ability_icons/life-drain3.dds",
"Data/UI/ability_icons/life-drain5.dds",
"Data/UI/ability_icons/lifedrain4.dds",
"Data/UI/ability_icons/lifes_devotion_01.dds",
"Data/UI/ability_icons/life_and_death_concord.dds",
"Data/UI/ability_icons/life_and_death_concord_a.dds",
"Data/UI/ability_icons/life_and_death_concord_b.dds",
"Data/UI/ability_icons/lighting1.dds",
"Data/UI/ability_icons/lighting2.dds",
"Data/UI/ability_icons/lighting2a.dds",
"Data/UI/ability_icons/lightingbolt4.dds",
"Data/UI/ability_icons/lightingbolt4y.dds",
"Data/UI/ability_icons/lightingbolt5.dds",
"Data/UI/ability_icons/lighting_fury.dds",
"Data/UI/ability_icons/lighting_fury_a.dds",
"Data/UI/ability_icons/ligthtingbolt4r.dds",
"Data/UI/ability_icons/living_flame_01.dds",
"Data/UI/ability_icons/long_range_bombing.dds",
"Data/UI/ability_icons/lowblow1.dds",
"Data/UI/ability_icons/maelstrom.dds",
"Data/UI/ability_icons/maelstrom_a.dds",
"Data/UI/ability_icons/magicres4.dds",
"Data/UI/ability_icons/magicresist1.dds",
"Data/UI/ability_icons/magicresist2.dds",
"Data/UI/ability_icons/magicresist3.dds",
"Data/UI/ability_icons/magicward1.dds",
"Data/UI/ability_icons/magicward1a.dds",
"Data/UI/ability_icons/magicward2.dds",
"Data/UI/ability_icons/magicward3.dds",
"Data/UI/ability_icons/magicward4.dds",
"Data/UI/ability_icons/magnify_pain_01.dds",
"Data/UI/ability_icons/maim1.dds",
"Data/UI/ability_icons/malicious_strike_01.dds",
"Data/UI/ability_icons/malicious_strike_01_a.dds",
"Data/UI/ability_icons/manachannel1.dds",
"Data/UI/ability_icons/manadrain1.dds",
"Data/UI/ability_icons/marksman_pedestal.dds",
"Data/UI/ability_icons/marksman_pedestal_a.dds",
"Data/UI/ability_icons/mass_subdue.dds",
"Data/UI/ability_icons/master_archer.dds",
"Data/UI/ability_icons/master_archer_a.dds",
"Data/UI/ability_icons/master_archer_b.dds",
"Data/UI/ability_icons/master_of_the_abyss_01.dds",
"Data/UI/ability_icons/memory_capture.dds",
"Data/UI/ability_icons/menacingroar1.dds",
"Data/UI/ability_icons/menacingroar2.dds",
"Data/UI/ability_icons/menacingroar2a.dds",
"Data/UI/ability_icons/menacingroar3.dds",
"Data/UI/ability_icons/menacingroar3a.dds",
"Data/UI/ability_icons/menacingroar4.dds",
"Data/UI/ability_icons/menacingroar4b.dds",
"Data/UI/ability_icons/menacingroar4c.dds",
"Data/UI/ability_icons/mend1.dds",
"Data/UI/ability_icons/mend1a.dds",
"Data/UI/ability_icons/mend1c.dds",
"Data/UI/ability_icons/mend1f.dds",
"Data/UI/ability_icons/mend2a.dds",
"Data/UI/ability_icons/mend2b.dds",
"Data/UI/ability_icons/mend4.dds",
"Data/UI/ability_icons/merchant1.dds",
"Data/UI/ability_icons/mining1.dds",
"Data/UI/ability_icons/mining2.dds",
"Data/UI/ability_icons/mining2a.dds",
"Data/UI/ability_icons/mining3.dds",
"Data/UI/ability_icons/mirrorandsmoke2.dds",
"Data/UI/ability_icons/mortalshot1.dds",
"Data/UI/ability_icons/mortalshot1b.dds",
"Data/UI/ability_icons/mortalshot2.dds",
"Data/UI/ability_icons/motif_of_bravery.dds",
"Data/UI/ability_icons/motif_of_encouragement.dds",
"Data/UI/ability_icons/motif_of_focus.dds",
"Data/UI/ability_icons/motif_of_grandeur.dds",
"Data/UI/ability_icons/motif_of_regeneration.dds",
"Data/UI/ability_icons/motif_of_regeneration_a.dds",
"Data/UI/ability_icons/motif_of_tenacity.dds",
"Data/UI/ability_icons/necromancer-corpse_explosion.dds",
"Data/UI/ability_icons/necromancer-corpse_explosion_a.dds",
"Data/UI/ability_icons/necromancer-feign_death.dds",
"Data/UI/ability_icons/necromancer-feign_death_a.dds",
"Data/UI/ability_icons/necromancer-summon_zealot.dds",
"Data/UI/ability_icons/necromancer-summon_zealot_a.dds",
"Data/UI/ability_icons/necromancer_ancient_tomb_01.dds",
"Data/UI/ability_icons/necromancer_detonate_bones_02.dds",
"Data/UI/ability_icons/necromancer_detonate_bones_02_a.dds",
"Data/UI/ability_icons/necromancer_detonate_bones_02_b.dds",
"Data/UI/ability_icons/necropotence1.dds",
"Data/UI/ability_icons/necropotence2.dds",
"Data/UI/ability_icons/necropotence4.dds",
"Data/UI/ability_icons/necropotence4a.dds",
"Data/UI/ability_icons/necropotence4b.dds",
"Data/UI/ability_icons/necropotence4c.dds",
"Data/UI/ability_icons/necropotence4z.dds",
"Data/UI/ability_icons/necropotence4za.dds",
"Data/UI/ability_icons/necropotence4zb.dds",
"Data/UI/ability_icons/nightstalker_01.dds",
"Data/UI/ability_icons/nimble_fingers.dds",
"Data/UI/ability_icons/nysyrs_rebuke.dds",
"Data/UI/ability_icons/opportunity_01.dds",
"Data/UI/ability_icons/opportunity_01_a.dds",
"Data/UI/ability_icons/opportunity_01_b.dds",
"Data/UI/ability_icons/outfitter1.dds",
"Data/UI/ability_icons/outfitter2.dds",
"Data/UI/ability_icons/overwhelm1.dds",
"Data/UI/ability_icons/overwhelming_brilliance_01.dds",
"Data/UI/ability_icons/paladin-graceful_under_pressure.dds",
"Data/UI/ability_icons/paladin-graceful_under_pressure_a.dds",
"Data/UI/ability_icons/paladin-life_rapture.dds",
"Data/UI/ability_icons/paladin-life_rapture_a.dds",
"Data/UI/ability_icons/paladin-life_rapture_b.dds",
"Data/UI/ability_icons/paladin-preservation.dds",
"Data/UI/ability_icons/paladin-preservation_a.dds",
"Data/UI/ability_icons/paladin-preservation_b.dds",
"Data/UI/ability_icons/paladin-reprisal.dds",
"Data/UI/ability_icons/paladin-scales_of_justice.dds",
"Data/UI/ability_icons/paladin-vengeful_wrath.dds",
"Data/UI/ability_icons/paladin_face_slam.dds",
"Data/UI/ability_icons/paragon-improved_predicatable_movements.dds",
"Data/UI/ability_icons/paragon-improved_predicatable_movements_a.dds",
"Data/UI/ability_icons/paragon-improved_predicatable_movements_b.dds",
"Data/UI/ability_icons/paragon-improved_predicatable_movements_c.dds",
"Data/UI/ability_icons/paragon-predicatable_movements.dds",
"Data/UI/ability_icons/paragon-predicatable_movements_a.dds",
"Data/UI/ability_icons/paragon-predicatable_movements_b.dds",
"Data/UI/ability_icons/paragon-way_of_mountain.dds",
"Data/UI/ability_icons/paragon-way_of_mountain_a.dds",
"Data/UI/ability_icons/paragon-way_of_mountain_b.dds",
"Data/UI/ability_icons/paragon-way_of_mountain_c.dds",
"Data/UI/ability_icons/paragon_flowing_strikes.dds",
"Data/UI/ability_icons/paragon_flowing_strikes_a.dds",
"Data/UI/ability_icons/paragon_improved_flowing_strikes_01_a.dds",
"Data/UI/ability_icons/paragon_improved_flowing_strikes_01_b.dds",
"Data/UI/ability_icons/paragon_path_of_the_tempest_01.dds",
"Data/UI/ability_icons/paragon_path_of_the_tempest_01_a.dds",
"Data/UI/ability_icons/paragon_path_of_the_tempest_01_b.dds",
"Data/UI/ability_icons/paragon_shifting_blades_01.dds",
"Data/UI/ability_icons/paragon_way_of_the_river_01.dds",
"Data/UI/ability_icons/paragon_way_of_the_river_01_a.dds",
"Data/UI/ability_icons/paralyzing_strike_01.dds",
"Data/UI/ability_icons/perseverance.dds",
"Data/UI/ability_icons/perseverance_a.dds",
"Data/UI/ability_icons/phantom_blow.dds",
"Data/UI/ability_icons/phantom_blow_a.dds",
"Data/UI/ability_icons/physical_trauma_01.dds",
"Data/UI/ability_icons/piercingenergy1.dds",
"Data/UI/ability_icons/piercingenergy1a.dds",
"Data/UI/ability_icons/piercingenergy2a.dds",
"Data/UI/ability_icons/piercingenergy3.dds",
"Data/UI/ability_icons/piercingshot1.dds",
"Data/UI/ability_icons/piercingshot2.dds",
"Data/UI/ability_icons/piercingshot2a.dds",
"Data/UI/ability_icons/piercingshot3.dds",
"Data/UI/ability_icons/piercingshot3a.dds",
"Data/UI/ability_icons/piercing_shot_01.dds",
"Data/UI/ability_icons/piercing_shot_01_a.dds",
"Data/UI/ability_icons/piety1a.dds",
"Data/UI/ability_icons/piety1d.dds",
"Data/UI/ability_icons/piety1z.dds",
"Data/UI/ability_icons/piety2.dds",
"Data/UI/ability_icons/piety2a.dds",
"Data/UI/ability_icons/piety3.dds",
"Data/UI/ability_icons/pin_down_01.dds",
"Data/UI/ability_icons/pin_down_01_a.dds",
"Data/UI/ability_icons/plague_01.dds",
"Data/UI/ability_icons/plague_01_a.dds",
"Data/UI/ability_icons/poisoned.dds",
"Data/UI/ability_icons/poisonone1a.dds",
"Data/UI/ability_icons/poisonone2a.dds",
"Data/UI/ability_icons/poisonone2b.dds",
"Data/UI/ability_icons/poisontwo1.dds",
"Data/UI/ability_icons/poisontwo2.dds",
"Data/UI/ability_icons/poisontwo3.dds",
"Data/UI/ability_icons/poisontwo4.dds",
"Data/UI/ability_icons/poison_gas_01.dds",
"Data/UI/ability_icons/poison_gas_01_a.dds",
"Data/UI/ability_icons/poison_gas_02.dds",
"Data/UI/ability_icons/potshot1.dds",
"Data/UI/ability_icons/power_chord.dds",
"Data/UI/ability_icons/power_chord_a.dds",
"Data/UI/ability_icons/power_chord_b.dds",
"Data/UI/ability_icons/preparation1.dds",
"Data/UI/ability_icons/primal_death_01.dds",
"Data/UI/ability_icons/primal_death_01_a.dds",
"Data/UI/ability_icons/provoke1.dds",
"Data/UI/ability_icons/provoke1a.dds",
"Data/UI/ability_icons/puncture1.dds",
"Data/UI/ability_icons/puncture2.dds",
"Data/UI/ability_icons/purge.dds",
"Data/UI/ability_icons/purge_a.dds",
"Data/UI/ability_icons/pyroclasty1.dds",
"Data/UI/ability_icons/pyroclasty2.dds",
"Data/UI/ability_icons/quick_reflexes.dds",
"Data/UI/ability_icons/quick_reload.dds",
"Data/UI/ability_icons/quick_reload_a.dds",
"Data/UI/ability_icons/rainofarrows1.dds",
"Data/UI/ability_icons/rainofarrows1z.dds",
"Data/UI/ability_icons/rainofarrows2.dds",
"Data/UI/ability_icons/rainofarrows3.dds",
"Data/UI/ability_icons/rainofarrows4.dds",
"Data/UI/ability_icons/rainofarrows4a.dds",
"Data/UI/ability_icons/rally1.dds",
"Data/UI/ability_icons/rally1a.dds",
"Data/UI/ability_icons/rally1x.dds",
"Data/UI/ability_icons/rally3.dds",
"Data/UI/ability_icons/rampage1.dds",
"Data/UI/ability_icons/rampage2.dds",
"Data/UI/ability_icons/rampage2a.dds",
"Data/UI/ability_icons/rampage3.dds",
"Data/UI/ability_icons/rapidarrow3a.dds",
"Data/UI/ability_icons/rapidshot1.dds",
"Data/UI/ability_icons/rapidshot1a.dds",
"Data/UI/ability_icons/rapidshot2.dds",
"Data/UI/ability_icons/rapidshot3.dds",
"Data/UI/ability_icons/rapidshot4.dds",
"Data/UI/ability_icons/rapidshot4a.dds",
"Data/UI/ability_icons/rapid_setup.dds",
"Data/UI/ability_icons/rapid_setup_a.dds",
"Data/UI/ability_icons/redirect1.dds",
"Data/UI/ability_icons/redirect1a.dds",
"Data/UI/ability_icons/redirect1b.dds",
"Data/UI/ability_icons/redirect2.dds",
"Data/UI/ability_icons/redirect2a.dds",
"Data/UI/ability_icons/redirect4a.dds",
"Data/UI/ability_icons/redirect4b.dds",
"Data/UI/ability_icons/reign_of_despair.dds",
"Data/UI/ability_icons/reign_of_despair_a.dds",
"Data/UI/ability_icons/releaseheat2.dds",
"Data/UI/ability_icons/releaseheat2f.dds",
"Data/UI/ability_icons/releaseheat3.dds",
"Data/UI/ability_icons/releaseheat3a.dds",
"Data/UI/ability_icons/releaseheat3b.dds",
"Data/UI/ability_icons/releaseheat4.dds",
"Data/UI/ability_icons/releaseheat4a.dds",
"Data/UI/ability_icons/releaseheat4b.dds",
"Data/UI/ability_icons/releaseheat5.dds",
"Data/UI/ability_icons/releaseheat5a.dds",
"Data/UI/ability_icons/remote_clipping.dds",
"Data/UI/ability_icons/remote_clipping_a.dds",
"Data/UI/ability_icons/removeheat1.dds",
"Data/UI/ability_icons/reparation_01.dds",
"Data/UI/ability_icons/reparation_01_a.dds",
"Data/UI/ability_icons/repelling_shot.dds",
"Data/UI/ability_icons/reprieve_01.dds",
"Data/UI/ability_icons/requital_01.dds",
"Data/UI/ability_icons/requital_01_a.dds",
"Data/UI/ability_icons/resolve1.dds",
"Data/UI/ability_icons/resolve2.dds",
"Data/UI/ability_icons/resolve2a.dds",
"Data/UI/ability_icons/resolve2b.dds",
"Data/UI/ability_icons/resonance.dds",
"Data/UI/ability_icons/ressurect2.dds",
"Data/UI/ability_icons/ressurect4.dds",
"Data/UI/ability_icons/ressurect4b.dds",
"Data/UI/ability_icons/ressurect4bb.dds",
"Data/UI/ability_icons/ressurect4bl.dds",
"Data/UI/ability_icons/ressurect4c.dds",
"Data/UI/ability_icons/ressurect4y.dds",
"Data/UI/ability_icons/ressurection1.dds",
"Data/UI/ability_icons/ressurection1a.dds",
"Data/UI/ability_icons/ressurection1z.dds",
"Data/UI/ability_icons/ressurection1za.dds",
"Data/UI/ability_icons/ressurection3b.dds",
"Data/UI/ability_icons/ressurection3bb.dds",
"Data/UI/ability_icons/ressurection3bbb.dds",
"Data/UI/ability_icons/rest1.dds",
"Data/UI/ability_icons/restorative_flame.dds",
"Data/UI/ability_icons/restorative_flame2.dds",
"Data/UI/ability_icons/restorative_flame_a.dds",
"Data/UI/ability_icons/restore1.dds",
"Data/UI/ability_icons/restore1a.dds",
"Data/UI/ability_icons/restore2a.dds",
"Data/UI/ability_icons/restore2b.dds",
"Data/UI/ability_icons/retreat.dds",
"Data/UI/ability_icons/retribution1.dds",
"Data/UI/ability_icons/revitalize1c.dds",
"Data/UI/ability_icons/revitalize1cc.dds",
"Data/UI/ability_icons/revitalize1f.dds",
"Data/UI/ability_icons/revitalize1u.dds",
"Data/UI/ability_icons/ricochet1.dds",
"Data/UI/ability_icons/ricochet2.dds",
"Data/UI/ability_icons/riff.dds",
"Data/UI/ability_icons/riftblade-avatar_of_water.dds",
"Data/UI/ability_icons/riftblade-avatar_of_water_a.dds",
"Data/UI/ability_icons/riftblade-earth_burst.dds",
"Data/UI/ability_icons/riftblade-earth_burst_a.dds",
"Data/UI/ability_icons/riftblade-surging_energy.dds",
"Data/UI/ability_icons/riftblade-surging_energy_a.dds",
"Data/UI/ability_icons/riftblade-surging_energy_b.dds",
"Data/UI/ability_icons/riftblade-surging_energy_c.dds",
"Data/UI/ability_icons/riftblade_frost_strike_01.dds",
"Data/UI/ability_icons/riftblade_frost_strike_01_a.dds",
"Data/UI/ability_icons/riftblade_stone_shield_01.dds",
"Data/UI/ability_icons/riftblade_stone_shield_01_a.dds",
"Data/UI/ability_icons/riftblade_stone_shield_01_b.dds",
"Data/UI/ability_icons/riftblade_stone_shield_01_c.dds",
"Data/UI/ability_icons/riftblade_stone_shield_01_d.dds",
"Data/UI/ability_icons/riftblade_stone_shield_01_e.dds",
"Data/UI/ability_icons/riftblade_thunder_strike_01.dds",
"Data/UI/ability_icons/riftblade_thunder_strike_01_a.dds",
"Data/UI/ability_icons/riftblade_thunder_strike_01_b.dds",
"Data/UI/ability_icons/riftstalker_great_fortitude_01.dds",
"Data/UI/ability_icons/riftstalker_great_fortitude_01_a.dds",
"Data/UI/ability_icons/riftstalker_planar_refuge_01.dds",
"Data/UI/ability_icons/riftstalker_planar_refuge_01_a.dds",
"Data/UI/ability_icons/riftstalker_planar_repulsion_01.dds",
"Data/UI/ability_icons/riftstalker_planar_repulsion_01_a.dds",
"Data/UI/ability_icons/riftstalker_rift_scavenger_01.dds",
"Data/UI/ability_icons/riftstalker_rift_scavenger_01_a.dds",
"Data/UI/ability_icons/riftstalker_rift_scavenger_01_b.dds",
"Data/UI/ability_icons/riftstalker_toughened_soul_01.dds",
"Data/UI/ability_icons/riftstalker_toughened_soul_01_a.dds",
"Data/UI/ability_icons/riftstalker_toughened_soul_01_b.dds",
"Data/UI/ability_icons/riftstalker_unseen_fury_01.dds",
"Data/UI/ability_icons/riftstalker_unseen_fury_01_a.dds",
"Data/UI/ability_icons/riftstalker_unseen_fury_01_b.dds",
"Data/UI/ability_icons/riftstalker_unseen_fury_01_c.dds",
"Data/UI/ability_icons/rift_affinity.dds",
"Data/UI/ability_icons/rift_prison.dds",
"Data/UI/ability_icons/righteousness1.dds",
"Data/UI/ability_icons/righteousness2.dds",
"Data/UI/ability_icons/righteousness2a.dds",
"Data/UI/ability_icons/righteousness3.dds",
"Data/UI/ability_icons/righteous_healing_01.dds",
"Data/UI/ability_icons/righteous_healing_01_a.dds",
"Data/UI/ability_icons/righteous_invocation_01.dds",
"Data/UI/ability_icons/rogueshot1.dds",
"Data/UI/ability_icons/rogueshot2.dds",
"Data/UI/ability_icons/rogueshot2a.dds",
"Data/UI/ability_icons/rogueshot3.dds",
"Data/UI/ability_icons/rogueshottwo1.dds",
"Data/UI/ability_icons/rogueshottwo1a.dds",
"Data/UI/ability_icons/rogueshottwo2.dds",
"Data/UI/ability_icons/rogueshottwo3.dds",
"Data/UI/ability_icons/roguestrike1.dds",
"Data/UI/ability_icons/roguestrike1a.dds",
"Data/UI/ability_icons/roguestrike2.dds",
"Data/UI/ability_icons/roguestrike2a.dds",
"Data/UI/ability_icons/rogue_shadow_shift.dds",
"Data/UI/ability_icons/rudeawake1.dds",
"Data/UI/ability_icons/ruthless_stalker.dds",
"Data/UI/ability_icons/sacrificepet1.dds",
"Data/UI/ability_icons/sacrificepet2.dds",
"Data/UI/ability_icons/sacrificepet3.dds",
"Data/UI/ability_icons/sacrificepet4.dds",
"Data/UI/ability_icons/sacrificepet4a.dds",
"Data/UI/ability_icons/sacrificepet4b.dds",
"Data/UI/ability_icons/salvage1.dds",
"Data/UI/ability_icons/salvage2.dds",
"Data/UI/ability_icons/salvation_01.dds",
"Data/UI/ability_icons/salvation_01_a.dds",
"Data/UI/ability_icons/sanction_heretic.dds",
"Data/UI/ability_icons/sanction_heretic_a.dds",
"Data/UI/ability_icons/scatter_the_shadows.dds",
"Data/UI/ability_icons/scourge_of_darkness_01.dds",
"Data/UI/ability_icons/scourge_of_darkness_01_a.dds",
"Data/UI/ability_icons/scrape1.dds",
"Data/UI/ability_icons/scrape2.dds",
"Data/UI/ability_icons/scrape3.dds",
"Data/UI/ability_icons/secondwind1.dds",
"Data/UI/ability_icons/secondwind2.dds",
"Data/UI/ability_icons/secondwind2a.dds",
"Data/UI/ability_icons/sentinel-cleansing_prayer.dds",
"Data/UI/ability_icons/sentinel-cleansing_prayer_a.dds",
"Data/UI/ability_icons/sentinel-cleansing_prayer_b.dds",
"Data/UI/ability_icons/sentinel-healing_benediction.dds",
"Data/UI/ability_icons/sentinel-healing_benediction_a.dds",
"Data/UI/ability_icons/sentinel-healing_benediction_b.dds",
"Data/UI/ability_icons/sentinel-protect_the_flock.dds",
"Data/UI/ability_icons/sentinel-protect_the_flock_a.dds",
"Data/UI/ability_icons/sentinel-quietus.dds",
"Data/UI/ability_icons/sentinel-refreshing_communion.dds",
"Data/UI/ability_icons/sentinel-refreshing_communion_a.dds",
"Data/UI/ability_icons/sentinel-shards_of_light.dds",
"Data/UI/ability_icons/sentinel-shards_of_light_a.dds",
"Data/UI/ability_icons/sentinel-shards_of_light_b.dds",
"Data/UI/ability_icons/shadow_assault.dds",
"Data/UI/ability_icons/shadow_assault_a.dds",
"Data/UI/ability_icons/shadow_blitz.dds",
"Data/UI/ability_icons/shadow_blitz_a.dds",
"Data/UI/ability_icons/shadow_blitz_b.dds",
"Data/UI/ability_icons/shadow_shift.dds",
"Data/UI/ability_icons/shadow_shift_a.dds",
"Data/UI/ability_icons/shadow_shift_b.dds",
"Data/UI/ability_icons/shadow_shift_c.dds",
"Data/UI/ability_icons/shadow_stalk.dds",
"Data/UI/ability_icons/shadow_stalk_a.dds",
"Data/UI/ability_icons/shadow_warp.dds",
"Data/UI/ability_icons/shadow_warp_a.dds",
"Data/UI/ability_icons/shadow_warp_b.dds",
"Data/UI/ability_icons/shaman-courage_of_the_bear.dds",
"Data/UI/ability_icons/shaman-courage_of_the_bear_a.dds",
"Data/UI/ability_icons/shaman-courage_of_the_jaguar.dds",
"Data/UI/ability_icons/shaman-glacial_shield.dds",
"Data/UI/ability_icons/shaman-glory_of_the_chosen.dds",
"Data/UI/ability_icons/shaman-glory_of_the_chosen_a.dds",
"Data/UI/ability_icons/shaman-lighting_hammer.dds",
"Data/UI/ability_icons/shaman-vengeance_of_bitter_wind.dds",
"Data/UI/ability_icons/shaman-vengeance_of_frozen_earth.dds",
"Data/UI/ability_icons/shaman-vengeance_of_frozen_earth_a.dds",
"Data/UI/ability_icons/shaman-vengeance_of_frozen_earth_b.dds",
"Data/UI/ability_icons/shaman-vengeance_of_frozen_earth_c.dds",
"Data/UI/ability_icons/shaman-vengeance_of_winter_storm.dds",
"Data/UI/ability_icons/shaman-vengeance_of_winter_storm_a.dds",
"Data/UI/ability_icons/shank1.dds",
"Data/UI/ability_icons/shank1a.dds",
"Data/UI/ability_icons/shank1b.dds",
"Data/UI/ability_icons/shatter1.dds",
"Data/UI/ability_icons/shatter3.dds",
"Data/UI/ability_icons/shatter3a.dds",
"Data/UI/ability_icons/shielded.dds",
"Data/UI/ability_icons/shockandawe1.dds",
"Data/UI/ability_icons/shockwave1.dds",
"Data/UI/ability_icons/shockwave1a.dds",
"Data/UI/ability_icons/shockwave2.dds",
"Data/UI/ability_icons/shockwave2a.dds",
"Data/UI/ability_icons/shockwave3.dds",
"Data/UI/ability_icons/shockwave3a.dds",
"Data/UI/ability_icons/shockwave3b.dds",
"Data/UI/ability_icons/shockwave3d.dds",
"Data/UI/ability_icons/shockwave4.dds",
"Data/UI/ability_icons/shreddingtrap1.dds",
"Data/UI/ability_icons/shreddingtrap2.dds",
"Data/UI/ability_icons/shreddingtrap3.dds",
"Data/UI/ability_icons/shreddingtrap3a.dds",
"Data/UI/ability_icons/shroudofpenitence1a.dds",
"Data/UI/ability_icons/shroudofpenitent2.dds",
"Data/UI/ability_icons/shroud_of_agony.dds",
"Data/UI/ability_icons/shroud_of_agony_a.dds",
"Data/UI/ability_icons/side_steps.dds",
"Data/UI/ability_icons/sigil_of_lurking_decay.dds",
"Data/UI/ability_icons/sigil_of_lurking_decay_a.dds",
"Data/UI/ability_icons/sigil_of_lurking_decay_b.dds",
"Data/UI/ability_icons/silent_footsteps_01.dds",
"Data/UI/ability_icons/silent_footsteps_01_a.dds",
"Data/UI/ability_icons/silent_footsteps_01_b.dds",
"Data/UI/ability_icons/silent_setup.dds",
"Data/UI/ability_icons/single_minded_focus.dds",
"Data/UI/ability_icons/skeleton2c.dds",
"Data/UI/ability_icons/slam1.dds",
"Data/UI/ability_icons/slam2.dds",
"Data/UI/ability_icons/slam3.dds",
"Data/UI/ability_icons/slam3a.dds",
"Data/UI/ability_icons/slam4.dds",
"Data/UI/ability_icons/slam4a.dds",
"Data/UI/ability_icons/slipaway1.dds",
"Data/UI/ability_icons/slip_away_03.dds",
"Data/UI/ability_icons/slip_away_03_a.dds",
"Data/UI/ability_icons/smelting1.dds",
"Data/UI/ability_icons/smelting1a.dds",
"Data/UI/ability_icons/smite1.dds",
"Data/UI/ability_icons/smite1a.dds",
"Data/UI/ability_icons/smite2.dds",
"Data/UI/ability_icons/smite4.dds",
"Data/UI/ability_icons/smite4a.dds",
"Data/UI/ability_icons/smite5.dds",
"Data/UI/ability_icons/smokeandmirrors1.dds",
"Data/UI/ability_icons/smokeandmirrors3.dds",
"Data/UI/ability_icons/smother_01.dds",
"Data/UI/ability_icons/snaretrap1.dds",
"Data/UI/ability_icons/snaretrap1a.dds",
"Data/UI/ability_icons/snaretrap1b.dds",
"Data/UI/ability_icons/snaretrap1g.dds",
"Data/UI/ability_icons/snaretrap2.dds",
"Data/UI/ability_icons/snaretrap3.dds",
"Data/UI/ability_icons/snaretrap3a.dds",
"Data/UI/ability_icons/soothe_the_flames.dds",
"Data/UI/ability_icons/soothe_the_flames_a.dds",
"Data/UI/ability_icons/soulbind.dds",
"Data/UI/ability_icons/souldrain1.dds",
"Data/UI/ability_icons/souldrain2.dds",
"Data/UI/ability_icons/souldrain3.dds",
"Data/UI/ability_icons/souldrain4.dds",
"Data/UI/ability_icons/sovereign_hammer_01.dds",
"Data/UI/ability_icons/spirit-of-mongoose1.dds",
"Data/UI/ability_icons/spirit-of-mongoose1a.dds",
"Data/UI/ability_icons/spiritblast1.dds",
"Data/UI/ability_icons/spiritblast2.dds",
"Data/UI/ability_icons/spiritblast3.dds",
"Data/UI/ability_icons/spiritblast4.dds",
"Data/UI/ability_icons/spiritblast4a.dds",
"Data/UI/ability_icons/spiritblast4b.dds",
"Data/UI/ability_icons/spiritblast4bc.dds",
"Data/UI/ability_icons/spiritblast4d.dds",
"Data/UI/ability_icons/spiritbolt1.dds",
"Data/UI/ability_icons/spiritbolt1a.dds",
"Data/UI/ability_icons/spiritbolt2.dds",
"Data/UI/ability_icons/spiritofcrusader1.dds",
"Data/UI/ability_icons/spiritofcrusader2.dds",
"Data/UI/ability_icons/spiritofcrusader3.dds",
"Data/UI/ability_icons/spiritofcrusader3z.dds",
"Data/UI/ability_icons/spiritoflion2.dds",
"Data/UI/ability_icons/spiritoflion3.dds",
"Data/UI/ability_icons/spiritofmongoose2.dds",
"Data/UI/ability_icons/spiritofmongoose3.dds",
"Data/UI/ability_icons/spiritofsaints1.dds",
"Data/UI/ability_icons/spiritofsaints1a.dds",
"Data/UI/ability_icons/spiritofsaints2.dds",
"Data/UI/ability_icons/spiritofsaints3.dds",
"Data/UI/ability_icons/spiritofsorcery1a.dds",
"Data/UI/ability_icons/spiritofsorcery1b.dds",
"Data/UI/ability_icons/spiritoftree1.dds",
"Data/UI/ability_icons/spiritoftree1a.dds",
"Data/UI/ability_icons/spiritofturtle2.dds",
"Data/UI/ability_icons/spiritofturtle3.dds",
"Data/UI/ability_icons/spiritofvaliant1.dds",
"Data/UI/ability_icons/spiritofvaliant1a.dds",
"Data/UI/ability_icons/spiritstrike2.dds",
"Data/UI/ability_icons/spiritstrike2a.dds",
"Data/UI/ability_icons/spiritstrike3.dds",
"Data/UI/ability_icons/spiritstrike3x.dds",
"Data/UI/ability_icons/spiritstrike3y.dds",
"Data/UI/ability_icons/spiritstrike3z.dds",
"Data/UI/ability_icons/spiritstrike4v.dds",
"Data/UI/ability_icons/spiritstrike4v1.dds",
"Data/UI/ability_icons/spiritstrike4v2.dds",
"Data/UI/ability_icons/spiritstrike4v2a.dds",
"Data/UI/ability_icons/spiritstrike5.dds",
"Data/UI/ability_icons/spiritstrike5a.dds",
"Data/UI/ability_icons/spiritual_will.dds",
"Data/UI/ability_icons/spirit_of_lion1.dds",
"Data/UI/ability_icons/spirit_of_turtle1.dds",
"Data/UI/ability_icons/stage_presence.dds",
"Data/UI/ability_icons/stage_presence_a.dds",
"Data/UI/ability_icons/staggeringshot1.dds",
"Data/UI/ability_icons/staggeringshot1a.dds",
"Data/UI/ability_icons/staggeringshot2.dds",
"Data/UI/ability_icons/staggeringshot3.dds",
"Data/UI/ability_icons/staggeringshot3a.dds",
"Data/UI/ability_icons/stalking_phase.dds",
"Data/UI/ability_icons/stalking_phase_a.dds",
"Data/UI/ability_icons/stalking_phase_b.dds",
"Data/UI/ability_icons/static_shot.dds",
"Data/UI/ability_icons/static_shot_a.dds",
"Data/UI/ability_icons/stealth_w.dds",
"Data/UI/ability_icons/stealth_y.dds",
"Data/UI/ability_icons/stealth_z.dds",
"Data/UI/ability_icons/steelskin2.dds",
"Data/UI/ability_icons/steelskin3.dds",
"Data/UI/ability_icons/sterilize1.dds",
"Data/UI/ability_icons/sterilize1_a.dds",
"Data/UI/ability_icons/stormcaller_lightning_storm_01.dds",
"Data/UI/ability_icons/stormcaller_lightning_storm_01_a.dds",
"Data/UI/ability_icons/stormcaller_lightning_storm_02.dds",
"Data/UI/ability_icons/stormcaller_lightning_storm_02_a.dds",
"Data/UI/ability_icons/stormcaller_lightning_storm_02_b.dds",
"Data/UI/ability_icons/strafe.dds",
"Data/UI/ability_icons/strafe_a.dds",
"Data/UI/ability_icons/street_performer.dds",
"Data/UI/ability_icons/street_smart.dds",
"Data/UI/ability_icons/strike_of_judgement_01.dds",
"Data/UI/ability_icons/strong_constitution_01.dds",
"Data/UI/ability_icons/strong_constitution_01_a.dds",
"Data/UI/ability_icons/stunning_blow_01.dds",
"Data/UI/ability_icons/summonskel5.dds",
"Data/UI/ability_icons/summonskeleton-3.dds",
"Data/UI/ability_icons/summonskeleton1.dds",
"Data/UI/ability_icons/summonskeleton2.dds",
"Data/UI/ability_icons/summonskeleton4.dds",
"Data/UI/ability_icons/surge.dds",
"Data/UI/ability_icons/surge_a.dds",
"Data/UI/ability_icons/surge_b.dds",
"Data/UI/ability_icons/surging_flames.dds",
"Data/UI/ability_icons/survival1.dds",
"Data/UI/ability_icons/survival2.dds",
"Data/UI/ability_icons/survival2a.dds",
"Data/UI/ability_icons/survival3.dds",
"Data/UI/ability_icons/sweep1.dds",
"Data/UI/ability_icons/sweep2.dds",
"Data/UI/ability_icons/sweep3.dds",
"Data/UI/ability_icons/sweep3b.dds",
"Data/UI/ability_icons/sweep3c.dds",
"Data/UI/ability_icons/sweep4.dds",
"Data/UI/ability_icons/sweep4a.dds",
"Data/UI/ability_icons/sweep4b.dds",
"Data/UI/ability_icons/swiftstrike1.dds",
"Data/UI/ability_icons/swiftstrike1a.dds",
"Data/UI/ability_icons/swiftstrike2.dds",
"Data/UI/ability_icons/swordsmith1.dds",
"Data/UI/ability_icons/swordsmith2.dds",
"Data/UI/ability_icons/talented_composer.dds",
"Data/UI/ability_icons/tenderize1.dds",
"Data/UI/ability_icons/tenderize2.dds",
"Data/UI/ability_icons/track_humanoids_01.dds",
"Data/UI/ability_icons/track_humanoids_01_a.dds",
"Data/UI/ability_icons/treasurehunt1.dds",
"Data/UI/ability_icons/treasurehunt2.dds",
"Data/UI/ability_icons/triage1.dds",
"Data/UI/ability_icons/triage1a.dds",
"Data/UI/ability_icons/triage2.dds",
"Data/UI/ability_icons/triage2a.dds",
"Data/UI/ability_icons/triage3.dds",
"Data/UI/ability_icons/triumphant_spirit.dds",
"Data/UI/ability_icons/triumphant_spirit_a.dds",
"Data/UI/ability_icons/tumble1.dds",
"Data/UI/ability_icons/tumble1a.dds",
"Data/UI/ability_icons/tumblevanish2.dds",
"Data/UI/ability_icons/twilight_force_01.dds",
"Data/UI/ability_icons/twilight_force_01_a.dds",
"Data/UI/ability_icons/twilight_force_01_b.dds",
"Data/UI/ability_icons/twin_strike.dds",
"Data/UI/ability_icons/twin_strike_a.dds",
"Data/UI/ability_icons/tyranny_of_death.dds",
"Data/UI/ability_icons/tyranny_of_death_a.dds",
"Data/UI/ability_icons/undertow.dds",
"Data/UI/ability_icons/underwaterbreath1a.dds",
"Data/UI/ability_icons/underwaterbreath1b.dds",
"Data/UI/ability_icons/underwaterbreath2.dds",
"Data/UI/ability_icons/underwaterbreath2a.dds",
"Data/UI/ability_icons/underwaterbreath3.dds",
"Data/UI/ability_icons/underwaterbreath3c.dds",
"Data/UI/ability_icons/untangle.dds",
"Data/UI/ability_icons/untangle_a.dds",
"Data/UI/ability_icons/untangle_b.dds",
"Data/UI/ability_icons/upperhand1.dds",
"Data/UI/ability_icons/vengeful_justice_01.dds",
"Data/UI/ability_icons/venomoustrap1.dds",
"Data/UI/ability_icons/venomoustrap1a.dds",
"Data/UI/ability_icons/venomoustrap2.dds",
"Data/UI/ability_icons/venomoustrap2a.dds",
"Data/UI/ability_icons/venomoustrap3.dds",
"Data/UI/ability_icons/venomtrap4.dds",
"Data/UI/ability_icons/verse_of_fascination.dds",
"Data/UI/ability_icons/verse_of_fascination_a.dds",
"Data/UI/ability_icons/verse_of_joy.dds",
"Data/UI/ability_icons/verse_of_joy_a.dds",
"Data/UI/ability_icons/verse_of_restoration.dds",
"Data/UI/ability_icons/verse_of_vitality.dds",
"Data/UI/ability_icons/vex.dds",
"Data/UI/ability_icons/virtuoso.dds",
"Data/UI/ability_icons/virtuoso_a.dds",
"Data/UI/ability_icons/virtuoso_b.dds",
"Data/UI/ability_icons/virtuous_conviction_01.dds",
"Data/UI/ability_icons/virtuous_conviction_01_a.dds",
"Data/UI/ability_icons/vitalizingshield1b.dds",
"Data/UI/ability_icons/vitalizingshield2.dds",
"Data/UI/ability_icons/voidknight_catalyze_01.dds",
"Data/UI/ability_icons/voidknight_catalyze_01_a.dds",
"Data/UI/ability_icons/voidknight_catalyze_01_b.dds",
"Data/UI/ability_icons/voidknight_ravenous_defence_01.dds",
"Data/UI/ability_icons/voidknight_ravenous_defence_01_a.dds",
"Data/UI/ability_icons/void_knight-improved_rift_summon.dds",
"Data/UI/ability_icons/void_knight-improved_rift_summon_a.dds",
"Data/UI/ability_icons/void_knight-insatiable_hunger.dds",
"Data/UI/ability_icons/void_knight-insatiable_hunger_a.dds",
"Data/UI/ability_icons/void_knight-pact.dds",
"Data/UI/ability_icons/void_knight-pact_a.dds",
"Data/UI/ability_icons/void_knight-ravenous_strength.dds",
"Data/UI/ability_icons/void_knight-ravenous_strength_a.dds",
"Data/UI/ability_icons/void_knight-ravenous_strength_b.dds",
"Data/UI/ability_icons/void_knight-rift_summon.dds",
"Data/UI/ability_icons/void_knight-rift_summon_a.dds",
"Data/UI/ability_icons/void_knight-tempest.dds",
"Data/UI/ability_icons/void_knight-void.dds",
"Data/UI/ability_icons/void_knight-void_a.dds",
"Data/UI/ability_icons/warcry.dds",
"Data/UI/ability_icons/ward1.dds",
"Data/UI/ability_icons/ward1a.dds",
"Data/UI/ability_icons/ward1z.dds",
"Data/UI/ability_icons/ward1zz.dds",
"Data/UI/ability_icons/ward2.dds",
"Data/UI/ability_icons/ward3.dds",
"Data/UI/ability_icons/ward3b.dds",
"Data/UI/ability_icons/ward4.dds",
"Data/UI/ability_icons/warden-crushing_wave.dds",
"Data/UI/ability_icons/warden-deluge.dds",
"Data/UI/ability_icons/warden-deluge_a.dds",
"Data/UI/ability_icons/warden-dissolution.dds",
"Data/UI/ability_icons/warden-dissolution_a.dds",
"Data/UI/ability_icons/warden-dissolution_b.dds",
"Data/UI/ability_icons/warden-downpour.dds",
"Data/UI/ability_icons/warden-healing_cataract.dds",
"Data/UI/ability_icons/warden-healing_flood.dds",
"Data/UI/ability_icons/warden-healing_flood_a.dds",
"Data/UI/ability_icons/warden-orbs_of_the_tide.dds",
"Data/UI/ability_icons/warden-overflow.dds",
"Data/UI/ability_icons/warden-overflow_a.dds",
"Data/UI/ability_icons/warden-overflow_b.dds",
"Data/UI/ability_icons/warden-overflow_c.dds",
"Data/UI/ability_icons/warden-tidal_surge.dds",
"Data/UI/ability_icons/warden-waterjet.dds",
"Data/UI/ability_icons/warden-waterjet_a.dds",
"Data/UI/ability_icons/warlord-cracking_skulls.dds",
"Data/UI/ability_icons/warlord-cracking_skulls_a.dds",
"Data/UI/ability_icons/warlord-cracking_skulls_b.dds",
"Data/UI/ability_icons/warlord-cutting_distraction.dds",
"Data/UI/ability_icons/warlord-cutting_distraction_a.dds",
"Data/UI/ability_icons/warlord-killing_field.dds",
"Data/UI/ability_icons/weakening_essence_01.dds",
"Data/UI/ability_icons/weaponsmith1.dds",
"Data/UI/ability_icons/weaponsmith1a.dds",
"Data/UI/ability_icons/weaponsmith2.dds",
"Data/UI/ability_icons/weaponsmith3.dds",
"Data/UI/ability_icons/weapon_barrage.dds",
"Data/UI/ability_icons/whirlingblades1.dds",
"Data/UI/ability_icons/whirlingblades2.dds",
"Data/UI/ability_icons/whirlingblades3.dds",
"Data/UI/ability_icons/whirlingblades3a.dds",
"Data/UI/ability_icons/whirlwind1.dds",
"Data/UI/ability_icons/whirlwind1a.dds",
"Data/UI/ability_icons/whirlwind2.dds",
"Data/UI/ability_icons/whirlwind3.dds",
"Data/UI/ability_icons/whirlwind3a.dds",
"Data/UI/ability_icons/whirlwind4.dds",
"Data/UI/ability_icons/whirlwind4n.dds",
"Data/UI/ability_icons/whrilwind4a.dds",
"Data/UI/ability_icons/whrilwind4b.dds",
"Data/UI/ability_icons/whrilwind4c.dds",
"Data/UI/item_icons/1h_axe_010.dds",
"Data/UI/item_icons/1h_axe_013.dds",
"Data/UI/item_icons/1h_axe_013a.dds",
"Data/UI/item_icons/1h_axe_014.dds",
"Data/UI/item_icons/1h_axe_014a.dds",
"Data/UI/item_icons/1h_axe_015.dds",
"Data/UI/item_icons/1h_axe_015a.dds",
"Data/UI/item_icons/1h_axe_016.dds",
"Data/UI/item_icons/1h_axe_016a.dds",
"Data/UI/item_icons/1h_axe_017.dds",
"Data/UI/item_icons/1h_axe_017a.dds",
"Data/UI/item_icons/1h_axe_017b.dds",
"Data/UI/item_icons/1h_axe_017c.dds",
"Data/UI/item_icons/1h_axe_018.dds",
"Data/UI/item_icons/1h_axe_018a.dds",
"Data/UI/item_icons/1h_axe_019.dds",
"Data/UI/item_icons/1h_axe_019_a.dds",
"Data/UI/item_icons/1h_axe_020.dds",
"Data/UI/item_icons/1h_axe_020a.dds",
"Data/UI/item_icons/1h_axe_021.dds",
"Data/UI/item_icons/1h_axe_022.dds",
"Data/UI/item_icons/1h_axe_022_a.dds",
"Data/UI/item_icons/1h_axe_023.dds",
"Data/UI/item_icons/1h_axe_023a.dds",
"Data/UI/item_icons/1h_axe_024.dds",
"Data/UI/item_icons/1h_axe_025.dds",
"Data/UI/item_icons/1h_axe_025a.dds",
"Data/UI/item_icons/1h_axe_026.dds",
"Data/UI/item_icons/1h_axe_026a.dds",
"Data/UI/item_icons/1h_axe_026b.dds",
"Data/UI/item_icons/1h_axe_027.dds",
"Data/UI/item_icons/1h_axe_027a.dds",
"Data/UI/item_icons/1h_axe_028.dds",
"Data/UI/item_icons/1h_axe_028_a.dds",
"Data/UI/item_icons/1h_axe_029.dds",
"Data/UI/item_icons/1h_axe_030.dds",
"Data/UI/item_icons/1h_axe_031.dds",
"Data/UI/item_icons/1h_axe_032.dds",
"Data/UI/item_icons/1h_axe_033.dds",
"Data/UI/item_icons/1h_axe_034.dds",
"Data/UI/item_icons/1h_axe_035.dds",
"Data/UI/item_icons/1h_axe_036.dds",
"Data/UI/item_icons/1h_axe_037.dds",
"Data/UI/item_icons/1h_axe_037a.dds",
"Data/UI/item_icons/1h_axe_038.dds",
"Data/UI/item_icons/1h_axe_038a.dds",
"Data/UI/item_icons/1h_axe_039.dds",
"Data/UI/item_icons/1h_axe_039_a.dds",
"Data/UI/item_icons/1h_axe_040.dds",
"Data/UI/item_icons/1h_axe_040_a.dds",
"Data/UI/item_icons/1h_axe_041.dds",
"Data/UI/item_icons/1h_axe_041_a.dds",
"Data/UI/item_icons/1h_axe_042.dds",
"Data/UI/item_icons/1h_axe_042_a.dds",
"Data/UI/item_icons/1h_axe_043.dds",
"Data/UI/item_icons/1h_axe_044.dds",
"Data/UI/item_icons/1h_axe_044a.dds",
"Data/UI/item_icons/1h_axe_045.dds",
"Data/UI/item_icons/1h_axe_045a.dds",
"Data/UI/item_icons/1h_axe_046.dds",
"Data/UI/item_icons/1h_axe_047.dds",
"Data/UI/item_icons/1h_axe_047a.dds",
"Data/UI/item_icons/1h_axe_048.dds",
"Data/UI/item_icons/1h_axe_048a.dds",
"Data/UI/item_icons/1h_axe_049.dds",
"Data/UI/item_icons/1h_axe_049a.dds",
"Data/UI/item_icons/1h_axe_050.dds",
"Data/UI/item_icons/1h_axe_050a.dds",
"Data/UI/item_icons/1h_axe_051.dds",
"Data/UI/item_icons/1h_axe_051a.dds",
"Data/UI/item_icons/1h_axe_051b.dds",
"Data/UI/item_icons/1h_axe_051c.dds",
"Data/UI/item_icons/1h_axe_052.dds",
"Data/UI/item_icons/1h_axe_052a.dds",
"Data/UI/item_icons/1h_axe_053.dds",
"Data/UI/item_icons/1h_axe_053a.dds",
"Data/UI/item_icons/1h_axe_053b.dds",
"Data/UI/item_icons/1h_axe_054.dds",
"Data/UI/item_icons/1h_axe_054a.dds",
"Data/UI/item_icons/1h_axe_055.dds",
"Data/UI/item_icons/1h_axe_055a.dds",
"Data/UI/item_icons/1h_axe_056.dds",
"Data/UI/item_icons/1h_axe_056a.dds",
"Data/UI/item_icons/1h_axe_057.dds",
"Data/UI/item_icons/1h_axe_057a.dds",
"Data/UI/item_icons/1h_axe_058.dds",
"Data/UI/item_icons/1h_axe_058a.dds",
"Data/UI/item_icons/1h_axe_058b.dds",
"Data/UI/item_icons/1h_axe_059.dds",
"Data/UI/item_icons/1h_axe_059a.dds",
"Data/UI/item_icons/1h_axe_060.dds",
"Data/UI/item_icons/1h_axe_061.dds",
"Data/UI/item_icons/1h_axe_061_a.dds",
"Data/UI/item_icons/1h_axe_062.dds",
"Data/UI/item_icons/1h_axe_062_a.dds",
"Data/UI/item_icons/1h_axe_063.dds",
"Data/UI/item_icons/1h_axe_063_a.dds",
"Data/UI/item_icons/1h_axe_064.dds",
"Data/UI/item_icons/1h_axe_064_a.dds",
"Data/UI/item_icons/1h_axe_065.dds",
"Data/UI/item_icons/1h_axe_066.dds",
"Data/UI/item_icons/1h_axe_066a.dds",
"Data/UI/item_icons/1h_axe_067.dds",
"Data/UI/item_icons/1h_axe_067a.dds",
"Data/UI/item_icons/1h_axe_068.dds",
"Data/UI/item_icons/1h_axe_069.dds",
"Data/UI/item_icons/1h_axe_069a.dds",
"Data/UI/item_icons/1h_axe_070.dds",
"Data/UI/item_icons/1h_axe_070a.dds",
"Data/UI/item_icons/1h_axe_071.dds",
"Data/UI/item_icons/1h_axe_071a.dds",
"Data/UI/item_icons/1h_axe_072.dds",
"Data/UI/item_icons/1h_axe_072a.dds",
"Data/UI/item_icons/1h_axe_073.dds",
"Data/UI/item_icons/1h_axe_073a.dds",
"Data/UI/item_icons/1h_axe_074.dds",
"Data/UI/item_icons/1h_axe_074a.dds",
"Data/UI/item_icons/1h_axe_075.dds",
"Data/UI/item_icons/1h_axe_075a.dds",
"Data/UI/item_icons/1h_axe_075b.dds",
"Data/UI/item_icons/1h_axe_076.dds",
"Data/UI/item_icons/1h_axe_076a.dds",
"Data/UI/item_icons/1h_axe_076b.dds",
"Data/UI/item_icons/1h_axe_077.dds",
"Data/UI/item_icons/1h_axe_077a.dds",
"Data/UI/item_icons/1h_axe_078.dds",
"Data/UI/item_icons/1h_axe_078a.dds",
"Data/UI/item_icons/1h_axe_079.dds",
"Data/UI/item_icons/1h_axe_079a.dds",
"Data/UI/item_icons/1h_axe_080.dds",
"Data/UI/item_icons/1h_axe_080a.dds",
"Data/UI/item_icons/1h_axe_081.dds",
"Data/UI/item_icons/1h_axe_081a.dds",
"Data/UI/item_icons/1h_axe_082.dds",
"Data/UI/item_icons/1h_axe_082a.dds",
"Data/UI/item_icons/1h_axe_083.dds",
"Data/UI/item_icons/1h_axe_083a.dds",
"Data/UI/item_icons/1h_axe_084.dds",
"Data/UI/item_icons/1h_axe_084a.dds",
"Data/UI/item_icons/1h_axe_085.dds",
"Data/UI/item_icons/1h_axe_085a.dds",
"Data/UI/item_icons/1h_axe_086.dds",
"Data/UI/item_icons/1h_axe_086a.dds",
"Data/UI/item_icons/1h_axe_087.dds",
"Data/UI/item_icons/1h_axe_087a.dds",
"Data/UI/item_icons/1h_axe_088.dds",
"Data/UI/item_icons/1h_axe_088a.dds",
"Data/UI/item_icons/1h_axe_air_mundane.dds",
"Data/UI/item_icons/1h_axe_air_mundane_a.dds",
"Data/UI/item_icons/1h_axe_earth_epic.dds",
"Data/UI/item_icons/1h_axe_earth_epic_a.dds",
"Data/UI/item_icons/1h_axe_fire_mundane.dds",
"Data/UI/item_icons/1h_axe_fire_mundane_a.dds",
"Data/UI/item_icons/1h_dagger_012.dds",
"Data/UI/item_icons/1h_dagger_012_a.dds",
"Data/UI/item_icons/1h_dagger_013.dds",
"Data/UI/item_icons/1h_dagger_014.dds",
"Data/UI/item_icons/1h_dagger_014a.dds",
"Data/UI/item_icons/1h_dagger_015.dds",
"Data/UI/item_icons/1h_dagger_016.dds",
"Data/UI/item_icons/1h_dagger_017.dds",
"Data/UI/item_icons/1h_dagger_017a.dds",
"Data/UI/item_icons/1h_dagger_018.dds",
"Data/UI/item_icons/1h_dagger_019.dds",
"Data/UI/item_icons/1h_dagger_020.dds",
"Data/UI/item_icons/1h_dagger_020a.dds",
"Data/UI/item_icons/1h_dagger_021.dds",
"Data/UI/item_icons/1h_dagger_022.dds",
"Data/UI/item_icons/1h_dagger_023.dds",
"Data/UI/item_icons/1h_dagger_023a.dds",
"Data/UI/item_icons/1h_dagger_024.dds",
"Data/UI/item_icons/1h_dagger_024a.dds",
"Data/UI/item_icons/1h_dagger_025.dds",
"Data/UI/item_icons/1h_dagger_025_a.dds",
"Data/UI/item_icons/1h_dagger_026.dds",
"Data/UI/item_icons/1h_dagger_026a.dds",
"Data/UI/item_icons/1h_dagger_027.dds",
"Data/UI/item_icons/1h_dagger_027a.dds",
"Data/UI/item_icons/1h_dagger_028.dds",
"Data/UI/item_icons/1h_dagger_028_a.dds",
"Data/UI/item_icons/1h_dagger_028_b.dds",
"Data/UI/item_icons/1h_dagger_029.dds",
"Data/UI/item_icons/1h_dagger_029a.dds",
"Data/UI/item_icons/1h_dagger_030.dds",
"Data/UI/item_icons/1h_dagger_030a.dds",
"Data/UI/item_icons/1h_dagger_030b.dds",
"Data/UI/item_icons/1h_dagger_031.dds",
"Data/UI/item_icons/1h_dagger_032.dds",
"Data/UI/item_icons/1h_dagger_032a.dds",
"Data/UI/item_icons/1h_dagger_033.dds",
"Data/UI/item_icons/1h_dagger_033a.dds",
"Data/UI/item_icons/1h_dagger_034.dds",
"Data/UI/item_icons/1h_dagger_034a.dds",
"Data/UI/item_icons/1h_dagger_035.dds",
"Data/UI/item_icons/1h_dagger_036.dds",
"Data/UI/item_icons/1h_dagger_036a.dds",
"Data/UI/item_icons/1h_dagger_037.dds",
"Data/UI/item_icons/1h_dagger_037a.dds",
"Data/UI/item_icons/1h_dagger_038.dds",
"Data/UI/item_icons/1h_dagger_039.dds",
"Data/UI/item_icons/1h_dagger_039_a.dds",
"Data/UI/item_icons/1h_dagger_040.dds",
"Data/UI/item_icons/1h_dagger_040_a.dds",
"Data/UI/item_icons/1h_dagger_040_b.dds",
"Data/UI/item_icons/1h_dagger_041.dds",
"Data/UI/item_icons/1h_dagger_041_a.dds",
"Data/UI/item_icons/1h_dagger_041_b.dds",
"Data/UI/item_icons/1h_dagger_042.dds",
"Data/UI/item_icons/1h_dagger_042_a.dds",
"Data/UI/item_icons/1h_dagger_043.dds",
"Data/UI/item_icons/1h_dagger_043_a.dds",
"Data/UI/item_icons/1h_dagger_043_b.dds",
"Data/UI/item_icons/1h_dagger_044.dds",
"Data/UI/item_icons/1h_dagger_044_a.dds",
"Data/UI/item_icons/1h_dagger_044_b.dds",
"Data/UI/item_icons/1h_dagger_045.dds",
"Data/UI/item_icons/1h_dagger_045_a.dds",
"Data/UI/item_icons/1h_dagger_045_c_x.dds",
"Data/UI/item_icons/1h_dagger_046.dds",
"Data/UI/item_icons/1h_dagger_046_a.dds",
"Data/UI/item_icons/1h_dagger_046_b.dds",
"Data/UI/item_icons/1h_dagger_046_c_x.dds",
"Data/UI/item_icons/1h_dagger_048a.dds",
"Data/UI/item_icons/1h_dagger_049.dds",
"Data/UI/item_icons/1h_dagger_049a.dds",
"Data/UI/item_icons/1h_dagger_049b.dds",
"Data/UI/item_icons/1h_dagger_050.dds",
"Data/UI/item_icons/1h_dagger_050a.dds",
"Data/UI/item_icons/1h_dagger_051.dds",
"Data/UI/item_icons/1h_dagger_051a.dds",
"Data/UI/item_icons/1h_dagger_051b.dds",
"Data/UI/item_icons/1h_dagger_052.dds",
"Data/UI/item_icons/1h_dagger_052a.dds",
"Data/UI/item_icons/1h_dagger_052b.dds",
"Data/UI/item_icons/1h_dagger_053.dds",
"Data/UI/item_icons/1h_dagger_053a.dds",
"Data/UI/item_icons/1h_dagger_053b.dds",
"Data/UI/item_icons/1h_dagger_053c.dds",
"Data/UI/item_icons/1h_dagger_054.dds",
"Data/UI/item_icons/1h_dagger_054a.dds",
"Data/UI/item_icons/1h_dagger_054b.dds",
"Data/UI/item_icons/1h_dagger_055.dds",
"Data/UI/item_icons/1h_dagger_055a.dds",
"Data/UI/item_icons/1h_dagger_055b.dds",
"Data/UI/item_icons/1h_dagger_056.dds",
"Data/UI/item_icons/1h_dagger_056a.dds",
"Data/UI/item_icons/1h_dagger_057.dds",
"Data/UI/item_icons/1h_dagger_057a.dds",
"Data/UI/item_icons/1h_dagger_058.dds",
"Data/UI/item_icons/1h_dagger_058a.dds",
"Data/UI/item_icons/1h_dagger_059.dds",
"Data/UI/item_icons/1h_dagger_059a.dds",
"Data/UI/item_icons/1h_dagger_059b.dds",
"Data/UI/item_icons/1h_dagger_060.dds",
"Data/UI/item_icons/1h_dagger_060a.dds",
"Data/UI/item_icons/1h_dagger_061.dds",
"Data/UI/item_icons/1h_dagger_061a.dds",
"Data/UI/item_icons/1h_dagger_062.dds",
"Data/UI/item_icons/1h_dagger_062a.dds",
"Data/UI/item_icons/1h_dagger_063.dds",
"Data/UI/item_icons/1h_dagger_063a.dds",
"Data/UI/item_icons/1h_dagger_063b.dds",
"Data/UI/item_icons/1h_dagger_064.dds",
"Data/UI/item_icons/1h_dagger_065.dds",
"Data/UI/item_icons/1h_dagger_065a.dds",
"Data/UI/item_icons/1h_dagger_066.dds",
"Data/UI/item_icons/1h_dagger_066a.dds",
"Data/UI/item_icons/1h_dagger_067.dds",
"Data/UI/item_icons/1h_dagger_067a.dds",
"Data/UI/item_icons/1h_dagger_068.dds",
"Data/UI/item_icons/1h_dagger_068a.dds",
"Data/UI/item_icons/1h_dagger_069.dds",
"Data/UI/item_icons/1h_dagger_069a.dds",
"Data/UI/item_icons/1h_dagger_070.dds",
"Data/UI/item_icons/1h_dagger_070a.dds",
"Data/UI/item_icons/1h_dagger_071.dds",
"Data/UI/item_icons/1h_dagger_071a.dds",
"Data/UI/item_icons/1h_dagger_072.dds",
"Data/UI/item_icons/1h_dagger_072a.dds",
"Data/UI/item_icons/1h_dagger_072b.dds",
"Data/UI/item_icons/1h_dagger_073.dds",
"Data/UI/item_icons/1h_dagger_073a.dds",
"Data/UI/item_icons/1h_dagger_073b.dds",
"Data/UI/item_icons/1h_dagger_074.dds",
"Data/UI/item_icons/1h_dagger_074a.dds",
"Data/UI/item_icons/1h_dagger_075.dds",
"Data/UI/item_icons/1h_dagger_075a.dds",
"Data/UI/item_icons/1h_dagger_076.dds",
"Data/UI/item_icons/1h_dagger_076a.dds",
"Data/UI/item_icons/1h_dagger_077.dds",
"Data/UI/item_icons/1h_dagger_077a.dds",
"Data/UI/item_icons/1h_dagger_077b.dds",
"Data/UI/item_icons/1h_dagger_078.dds",
"Data/UI/item_icons/1h_dagger_078a.dds",
"Data/UI/item_icons/1h_dagger_079.dds",
"Data/UI/item_icons/1h_dagger_079a.dds",
"Data/UI/item_icons/1h_dagger_080.dds",
"Data/UI/item_icons/1h_dagger_080a.dds",
"Data/UI/item_icons/1h_dagger_081.dds",
"Data/UI/item_icons/1h_dagger_081a.dds",
"Data/UI/item_icons/1h_dagger_082.dds",
"Data/UI/item_icons/1h_dagger_082a.dds",
"Data/UI/item_icons/1h_dagger_083.dds",
"Data/UI/item_icons/1h_dagger_083a.dds",
"Data/UI/item_icons/1h_dagger_084.dds",
"Data/UI/item_icons/1h_dagger_084a.dds",
"Data/UI/item_icons/1h_dagger_085.dds",
"Data/UI/item_icons/1h_dagger_085a.dds",
"Data/UI/item_icons/1h_dagger_086.dds",
"Data/UI/item_icons/1h_dagger_086a.dds",
"Data/UI/item_icons/1h_dagger_47.dds",
"Data/UI/item_icons/1h_dagger_47a.dds",
"Data/UI/item_icons/1h_dagger_48.dds",
"Data/UI/item_icons/1h_dagger_air_epic.dds",
"Data/UI/item_icons/1h_dagger_air_epic_a.dds",
"Data/UI/item_icons/1h_dagger_death_epic.dds",
"Data/UI/item_icons/1h_dagger_death_epic_a.dds",
"Data/UI/item_icons/1h_dagger_fire_epic.dds",
"Data/UI/item_icons/1h_dagger_fire_epic_a.dds",
"Data/UI/item_icons/1h_dagger_water_epic.dds",
"Data/UI/item_icons/1h_dagger_water_epic_a.dds",
"Data/UI/item_icons/1h_dagger_water_mundane.dds",
"Data/UI/item_icons/1h_dagger_water_mundane_a.dds",
"Data/UI/item_icons/1h_dagger_water_mundane_b.dds",
"Data/UI/item_icons/1h_mace_016.dds",
"Data/UI/item_icons/1h_mace_017.dds",
"Data/UI/item_icons/1h_mace_018.dds",
"Data/UI/item_icons/1h_mace_018_a.dds",
"Data/UI/item_icons/1h_mace_019.dds",
"Data/UI/item_icons/1h_mace_019a.dds",
"Data/UI/item_icons/1h_mace_020.dds",
"Data/UI/item_icons/1h_mace_020a.dds",
"Data/UI/item_icons/1h_mace_020b.dds",
"Data/UI/item_icons/1h_mace_021.dds",
"Data/UI/item_icons/1h_mace_021a.dds",
"Data/UI/item_icons/1h_mace_022.dds",
"Data/UI/item_icons/1h_mace_022a.dds",
"Data/UI/item_icons/1h_mace_022b.dds",
"Data/UI/item_icons/1h_mace_023.dds",
"Data/UI/item_icons/1h_mace_024.dds",
"Data/UI/item_icons/1h_mace_024a.dds",
"Data/UI/item_icons/1h_mace_025.dds",
"Data/UI/item_icons/1h_mace_025a.dds",
"Data/UI/item_icons/1h_mace_026.dds",
"Data/UI/item_icons/1h_mace_026_a.dds",
"Data/UI/item_icons/1h_mace_027.dds",
"Data/UI/item_icons/1h_mace_028.dds",
"Data/UI/item_icons/1h_mace_029.dds",
"Data/UI/item_icons/1h_mace_029a.dds",
"Data/UI/item_icons/1h_mace_030.dds",
"Data/UI/item_icons/1h_mace_031.dds",
"Data/UI/item_icons/1h_mace_032.dds",
"Data/UI/item_icons/1h_mace_032a.dds",
"Data/UI/item_icons/1h_mace_033.dds",
"Data/UI/item_icons/1h_mace_033a.dds",
"Data/UI/item_icons/1h_mace_036.dds",
"Data/UI/item_icons/1h_mace_036a.dds",
"Data/UI/item_icons/1h_mace_038.dds",
"Data/UI/item_icons/1h_mace_038a.dds",
"Data/UI/item_icons/1h_mace_040.dds",
"Data/UI/item_icons/1h_mace_040_a.dds",
"Data/UI/item_icons/1h_mace_041.dds",
"Data/UI/item_icons/1h_mace_041_a.dds",
"Data/UI/item_icons/1h_mace_042.dds",
"Data/UI/item_icons/1h_mace_042_a.dds",
"Data/UI/item_icons/1h_mace_048.dds",
"Data/UI/item_icons/1h_mace_049.dds",
"Data/UI/item_icons/1h_mace_050.dds",
"Data/UI/item_icons/1h_mace_051.dds",
"Data/UI/item_icons/1h_mace_051a.dds",
"Data/UI/item_icons/1h_mace_052.dds",
"Data/UI/item_icons/1h_mace_052a.dds",
"Data/UI/item_icons/1h_mace_053.dds",
"Data/UI/item_icons/1h_mace_053a.dds",
"Data/UI/item_icons/1h_mace_054.dds",
"Data/UI/item_icons/1h_mace_054a.dds",
"Data/UI/item_icons/1h_mace_055.dds",
"Data/UI/item_icons/1h_mace_055a.dds",
"Data/UI/item_icons/1h_mace_056.dds",
"Data/UI/item_icons/1h_mace_056a.dds",
"Data/UI/item_icons/1h_mace_057.dds",
"Data/UI/item_icons/1h_mace_057a.dds",
"Data/UI/item_icons/1h_mace_058.dds",
"Data/UI/item_icons/1h_mace_058a.dds",
"Data/UI/item_icons/1h_mace_063.dds",
"Data/UI/item_icons/1h_mace_063a.dds",
"Data/UI/item_icons/1h_mace_064.dds",
"Data/UI/item_icons/1h_mace_064a.dds",
"Data/UI/item_icons/1h_mace_065.dds",
"Data/UI/item_icons/1h_mace_065a.dds",
"Data/UI/item_icons/1h_mace_065b.dds",
"Data/UI/item_icons/1h_mace_066.dds",
"Data/UI/item_icons/1h_mace_066a.dds",
"Data/UI/item_icons/1h_mace_067.dds",
"Data/UI/item_icons/1h_mace_067a.dds",
"Data/UI/item_icons/1h_mace_068.dds",
"Data/UI/item_icons/1h_mace_068a.dds",
"Data/UI/item_icons/1h_mace_069.dds",
"Data/UI/item_icons/1h_mace_069a.dds",
"Data/UI/item_icons/1h_mace_070.dds",
"Data/UI/item_icons/1h_mace_070a.dds",
"Data/UI/item_icons/1h_mace_070b.dds",
"Data/UI/item_icons/1h_mace_071.dds",
"Data/UI/item_icons/1h_mace_071a.dds",
"Data/UI/item_icons/1h_mace_072.dds",
"Data/UI/item_icons/1h_mace_072a.dds",
"Data/UI/item_icons/1h_mace_073.dds",
"Data/UI/item_icons/1h_mace_073a.dds",
"Data/UI/item_icons/1h_mace_074.dds",
"Data/UI/item_icons/1h_mace_075.dds",
"Data/UI/item_icons/1h_mace_075a.dds",
"Data/UI/item_icons/1h_mace_076.dds",
"Data/UI/item_icons/1h_mace_076a.dds",
"Data/UI/item_icons/1h_mace_077.dds",
"Data/UI/item_icons/1h_mace_077a.dds",
"Data/UI/item_icons/1h_mace_078.dds",
"Data/UI/item_icons/1h_mace_079.dds",
"Data/UI/item_icons/1h_mace_079a.dds",
"Data/UI/item_icons/1h_mace_080.dds",
"Data/UI/item_icons/1h_mace_080a.dds",
"Data/UI/item_icons/1h_mace_081.dds",
"Data/UI/item_icons/1h_mace_081a.dds",
"Data/UI/item_icons/1h_mace_082.dds",
"Data/UI/item_icons/1h_mace_082a.dds",
"Data/UI/item_icons/1h_mace_083.dds",
"Data/UI/item_icons/1h_mace_083a.dds",
"Data/UI/item_icons/1h_mace_084.dds",
"Data/UI/item_icons/1h_mace_084a.dds",
"Data/UI/item_icons/1h_mace_085.dds",
"Data/UI/item_icons/1h_mace_085a.dds",
"Data/UI/item_icons/1h_mace_086.dds",
"Data/UI/item_icons/1h_mace_086a.dds",
"Data/UI/item_icons/1h_mace_087.dds",
"Data/UI/item_icons/1h_mace_087a.dds",
"Data/UI/item_icons/1h_mace_088.dds",
"Data/UI/item_icons/1h_mace_088_a.dds",
"Data/UI/item_icons/1h_mace_088_b.dds",
"Data/UI/item_icons/1h_mace_088_c.dds",
"Data/UI/item_icons/1h_mace_089.dds",
"Data/UI/item_icons/1h_mace_089_a.dds",
"Data/UI/item_icons/1h_mace_089_b.dds",
"Data/UI/item_icons/1h_mace_090.dds",
"Data/UI/item_icons/1h_mace_090_a.dds",
"Data/UI/item_icons/1h_mace_59.dds",
"Data/UI/item_icons/1h_mace_59a.dds",
"Data/UI/item_icons/1h_mace_60.dds",
"Data/UI/item_icons/1h_mace_60a.dds",
"Data/UI/item_icons/1h_mace_61.dds",
"Data/UI/item_icons/1h_mace_61a.dds",
"Data/UI/item_icons/1h_mace_62.dds",
"Data/UI/item_icons/1h_mace_62a.dds",
"Data/UI/item_icons/1h_mace_air_epic.dds",
"Data/UI/item_icons/1h_mace_air_epic_a.dds",
"Data/UI/item_icons/1h_mace_air_epic_b.dds",
"Data/UI/item_icons/1h_mace_death_epic.dds",
"Data/UI/item_icons/1h_mace_death_epic_a.dds",
"Data/UI/item_icons/1h_mace_earth_epic.dds",
"Data/UI/item_icons/1h_mace_earth_epic_a.dds",
"Data/UI/item_icons/1h_mace_fire_epic.dds",
"Data/UI/item_icons/1h_mace_fire_epic_a.dds",
"Data/UI/item_icons/1h_mace_life_01.dds",
"Data/UI/item_icons/1h_mace_life_01_a.dds",
"Data/UI/item_icons/1h_mace_water_mundane.dds",
"Data/UI/item_icons/1h_mace_water_mundane_a.dds",
"Data/UI/item_icons/1h_shield_0.dds",
"Data/UI/item_icons/1h_shield_001.dds",
"Data/UI/item_icons/1h_shield_002.dds",
"Data/UI/item_icons/1h_shield_002_a.dds",
"Data/UI/item_icons/1h_shield_002_b.dds",
"Data/UI/item_icons/1h_shield_003.dds",
"Data/UI/item_icons/1h_shield_003_a.dds",
"Data/UI/item_icons/1h_shield_004.dds",
"Data/UI/item_icons/1h_shield_004_a.dds",
"Data/UI/item_icons/1h_shield_005.dds",
"Data/UI/item_icons/1h_shield_006.dds",
"Data/UI/item_icons/1h_shield_007.dds",
"Data/UI/item_icons/1h_shield_007_a.dds",
"Data/UI/item_icons/1h_shield_008.dds",
"Data/UI/item_icons/1h_shield_009.dds",
"Data/UI/item_icons/1h_shield_010.dds",
"Data/UI/item_icons/1h_shield_011.dds",
"Data/UI/item_icons/1h_shield_012.dds",
"Data/UI/item_icons/1h_shield_013.dds",
"Data/UI/item_icons/1h_shield_014.dds",
"Data/UI/item_icons/1h_shield_015.dds",
"Data/UI/item_icons/1h_shield_016.dds",
"Data/UI/item_icons/1h_shield_016a.dds",
"Data/UI/item_icons/1h_shield_017.dds",
"Data/UI/item_icons/1h_shield_017a.dds",
"Data/UI/item_icons/1h_shield_018.dds",
"Data/UI/item_icons/1h_shield_019.dds",
"Data/UI/item_icons/1h_shield_019a.dds",
"Data/UI/item_icons/1h_shield_020.dds",
"Data/UI/item_icons/1h_shield_021.dds",
"Data/UI/item_icons/1h_shield_022.dds",
"Data/UI/item_icons/1h_shield_022a.dds",
"Data/UI/item_icons/1h_shield_023.dds",
"Data/UI/item_icons/1h_shield_024.dds",
"Data/UI/item_icons/1h_shield_025.dds",
"Data/UI/item_icons/1h_shield_025_a.dds",
"Data/UI/item_icons/1h_shield_025_b.dds",
"Data/UI/item_icons/1h_shield_026.dds",
"Data/UI/item_icons/1h_shield_026_a.dds",
"Data/UI/item_icons/1h_shield_027.dds",
"Data/UI/item_icons/1h_shield_027_a.dds",
"Data/UI/item_icons/1h_shield_027_b.dds",
"Data/UI/item_icons/1h_shield_028.dds",
"Data/UI/item_icons/1h_shield_028_a.dds",
"Data/UI/item_icons/1h_shield_028_b.dds",
"Data/UI/item_icons/1h_shield_029.dds",
"Data/UI/item_icons/1h_shield_029a.dds",
"Data/UI/item_icons/1h_shield_030.dds",
"Data/UI/item_icons/1h_shield_030a.dds",
"Data/UI/item_icons/1h_shield_031.dds",
"Data/UI/item_icons/1h_shield_031a.dds",
"Data/UI/item_icons/1h_shield_032.dds",
"Data/UI/item_icons/1h_shield_032a.dds",
"Data/UI/item_icons/1h_shield_033.dds",
"Data/UI/item_icons/1h_shield_033a.dds",
"Data/UI/item_icons/1h_shield_034.dds",
"Data/UI/item_icons/1h_shield_034a.dds",
"Data/UI/item_icons/1h_shield_035.dds",
"Data/UI/item_icons/1h_shield_035a.dds",
"Data/UI/item_icons/1h_shield_036.dds",
"Data/UI/item_icons/1h_shield_036a.dds",
"Data/UI/item_icons/1h_shield_037.dds",
"Data/UI/item_icons/1h_shield_037a.dds",
"Data/UI/item_icons/1h_shield_038.dds",
"Data/UI/item_icons/1h_shield_038a.dds",
"Data/UI/item_icons/1h_shield_039a.dds",
"Data/UI/item_icons/1h_shield_040.dds",
"Data/UI/item_icons/1h_shield_040a.dds",
"Data/UI/item_icons/1h_shield_041.dds",
"Data/UI/item_icons/1h_shield_041a.dds",
"Data/UI/item_icons/1h_shield_042.dds",
"Data/UI/item_icons/1h_shield_042a.dds",
"Data/UI/item_icons/1h_shield_043.dds",
"Data/UI/item_icons/1h_shield_043a.dds",
"Data/UI/item_icons/1h_shield_043b.dds",
"Data/UI/item_icons/1h_shield_044.dds",
"Data/UI/item_icons/1h_shield_044a.dds",
"Data/UI/item_icons/1h_shield_045.dds",
"Data/UI/item_icons/1h_shield_045a.dds",
"Data/UI/item_icons/1h_shield_046.dds",
"Data/UI/item_icons/1h_shield_046a.dds",
"Data/UI/item_icons/1h_shield_047.dds",
"Data/UI/item_icons/1h_shield_048.dds",
"Data/UI/item_icons/1h_shield_048a.dds",
"Data/UI/item_icons/1h_shield_049.dds",
"Data/UI/item_icons/1h_shield_049a.dds",
"Data/UI/item_icons/1h_shield_050.dds",
"Data/UI/item_icons/1h_shield_050a.dds",
"Data/UI/item_icons/1h_shield_050b.dds",
"Data/UI/item_icons/1h_shield_051.dds",
"Data/UI/item_icons/1h_shield_051a.dds",
"Data/UI/item_icons/1h_shield_051b.dds",
"Data/UI/item_icons/1h_shield_052.dds",
"Data/UI/item_icons/1h_shield_052a.dds",
"Data/UI/item_icons/1h_shield_053.dds",
"Data/UI/item_icons/1h_shield_053a.dds",
"Data/UI/item_icons/1h_shield_054.dds",
"Data/UI/item_icons/1h_shield_054a.dds",
"Data/UI/item_icons/1h_shield_054b.dds",
"Data/UI/item_icons/1h_shield_054c.dds",
"Data/UI/item_icons/1h_shield_055.dds",
"Data/UI/item_icons/1h_shield_055a.dds",
"Data/UI/item_icons/1h_shield_056.dds",
"Data/UI/item_icons/1h_shield_056a.dds",
"Data/UI/item_icons/1h_shield_057.dds",
"Data/UI/item_icons/1h_shield_057a.dds",
"Data/UI/item_icons/1h_shield_058.dds",
"Data/UI/item_icons/1h_shield_058a.dds",
"Data/UI/item_icons/1h_shield_0_a.dds",
"Data/UI/item_icons/1h_shield_life_01.dds",
"Data/UI/item_icons/1h_shield_life_02.dds",
"Data/UI/item_icons/1h_shield_life_02_a.dds",
"Data/UI/item_icons/1h_sword_011.dds",
"Data/UI/item_icons/1h_sword_015.dds",
"Data/UI/item_icons/1h_sword_015_a.dds",
"Data/UI/item_icons/1h_sword_016.dds",
"Data/UI/item_icons/1h_sword_016a.dds",
"Data/UI/item_icons/1h_sword_017.dds",
"Data/UI/item_icons/1h_sword_018.dds",
"Data/UI/item_icons/1h_sword_019.dds",
"Data/UI/item_icons/1h_sword_019a.dds",
"Data/UI/item_icons/1h_sword_020.dds",
"Data/UI/item_icons/1h_sword_021.dds",
"Data/UI/item_icons/1h_sword_021a.dds",
"Data/UI/item_icons/1h_sword_022.dds",
"Data/UI/item_icons/1h_sword_022a.dds",
"Data/UI/item_icons/1h_sword_024.dds",
"Data/UI/item_icons/1h_sword_024a.dds",
"Data/UI/item_icons/1h_sword_025.dds",
"Data/UI/item_icons/1h_sword_026.dds",
"Data/UI/item_icons/1h_sword_026a.dds",
"Data/UI/item_icons/1h_sword_027.dds",
"Data/UI/item_icons/1h_sword_028.dds",
"Data/UI/item_icons/1h_sword_028a.dds",
"Data/UI/item_icons/1h_sword_028b.dds",
"Data/UI/item_icons/1h_sword_029.dds",
"Data/UI/item_icons/1h_sword_029a.dds",
"Data/UI/item_icons/1h_sword_030.dds",
"Data/UI/item_icons/1h_sword_030_a.dds",
"Data/UI/item_icons/1h_sword_031.dds",
"Data/UI/item_icons/1h_sword_031a.dds",
"Data/UI/item_icons/1h_sword_032.dds",
"Data/UI/item_icons/1h_sword_033.dds",
"Data/UI/item_icons/1h_sword_033a.dds",
"Data/UI/item_icons/1h_sword_034.dds",
"Data/UI/item_icons/1h_sword_035.dds",
"Data/UI/item_icons/1h_sword_036.dds",
"Data/UI/item_icons/1h_sword_036a.dds",
"Data/UI/item_icons/1h_sword_037.dds",
"Data/UI/item_icons/1h_sword_038.dds",
"Data/UI/item_icons/1h_sword_038a.dds",
"Data/UI/item_icons/1h_sword_039.dds",
"Data/UI/item_icons/1h_sword_039a.dds",
"Data/UI/item_icons/1h_sword_040.dds",
"Data/UI/item_icons/1h_sword_041.dds",
"Data/UI/item_icons/1h_sword_041_a.dds",
"Data/UI/item_icons/1h_sword_042.dds",
"Data/UI/item_icons/1h_sword_042_a.dds",
"Data/UI/item_icons/1h_sword_042_b.dds",
"Data/UI/item_icons/1h_sword_043.dds",
"Data/UI/item_icons/1h_sword_043_a.dds",
"Data/UI/item_icons/1h_sword_044.dds",
"Data/UI/item_icons/1h_sword_044_a.dds",
"Data/UI/item_icons/1h_sword_045.dds",
"Data/UI/item_icons/1h_sword_046.dds",
"Data/UI/item_icons/1h_sword_046a.dds",
"Data/UI/item_icons/1h_sword_046b.dds",
"Data/UI/item_icons/1h_sword_046c.dds",
"Data/UI/item_icons/1h_sword_047.dds",
"Data/UI/item_icons/1h_sword_047a.dds",
"Data/UI/item_icons/1h_sword_048.dds",
"Data/UI/item_icons/1h_sword_048a.dds",
"Data/UI/item_icons/1h_sword_048b.dds",
"Data/UI/item_icons/1h_sword_049.dds",
"Data/UI/item_icons/1h_sword_050.dds",
"Data/UI/item_icons/1h_sword_051.dds",
"Data/UI/item_icons/1h_sword_051a.dds",
"Data/UI/item_icons/1h_sword_052.dds",
"Data/UI/item_icons/1h_sword_052a.dds",
"Data/UI/item_icons/1h_sword_055a.dds",
"Data/UI/item_icons/1h_sword_058b.dds",
"Data/UI/item_icons/1h_sword_058c.dds",
"Data/UI/item_icons/1h_sword_059b.dds",
"Data/UI/item_icons/1h_sword_059c.dds",
"Data/UI/item_icons/1h_sword_060b.dds",
"Data/UI/item_icons/1h_sword_060c.dds",
"Data/UI/item_icons/1h_sword_064b.dds",
"Data/UI/item_icons/1h_sword_064c.dds",
"Data/UI/item_icons/1h_sword_065b.dds",
"Data/UI/item_icons/1h_sword_065c.dds",
"Data/UI/item_icons/1h_sword_067a.dds",
"Data/UI/item_icons/1h_sword_068b.dds",
"Data/UI/item_icons/1h_sword_068c.dds",
"Data/UI/item_icons/1h_sword_069b.dds",
"Data/UI/item_icons/1h_sword_069c.dds",
"Data/UI/item_icons/1h_sword_071.dds",
"Data/UI/item_icons/1h_sword_071a.dds",
"Data/UI/item_icons/1h_sword_072.dds",
"Data/UI/item_icons/1h_sword_072a.dds",
"Data/UI/item_icons/1h_sword_073.dds",
"Data/UI/item_icons/1h_sword_073a.dds",
"Data/UI/item_icons/1h_sword_073b.dds",
"Data/UI/item_icons/1h_sword_074.dds",
"Data/UI/item_icons/1h_sword_074a.dds",
"Data/UI/item_icons/1h_sword_075.dds",
"Data/UI/item_icons/1h_sword_075a.dds",
"Data/UI/item_icons/1h_sword_075b.dds",
"Data/UI/item_icons/1h_sword_075c.dds",
"Data/UI/item_icons/1h_sword_076.dds",
"Data/UI/item_icons/1h_sword_076a.dds",
"Data/UI/item_icons/1h_sword_076b.dds",
"Data/UI/item_icons/1h_sword_076c.dds",
"Data/UI/item_icons/1h_sword_077.dds",
"Data/UI/item_icons/1h_sword_077a.dds",
"Data/UI/item_icons/1h_sword_078.dds",
"Data/UI/item_icons/1h_sword_078a.dds",
"Data/UI/item_icons/1h_sword_079.dds",
"Data/UI/item_icons/1h_sword_079a.dds",
"Data/UI/item_icons/1h_sword_080.dds",
"Data/UI/item_icons/1h_sword_080a.dds",
"Data/UI/item_icons/1h_sword_080b.dds",
"Data/UI/item_icons/1h_sword_081.dds",
"Data/UI/item_icons/1h_sword_081a.dds",
"Data/UI/item_icons/1h_sword_082.dds",
"Data/UI/item_icons/1h_sword_082a.dds",
"Data/UI/item_icons/1h_sword_083.dds",
"Data/UI/item_icons/1h_sword_083a.dds",
"Data/UI/item_icons/1h_sword_084.dds",
"Data/UI/item_icons/1h_sword_084a.dds",
"Data/UI/item_icons/1h_sword_084b.dds",
"Data/UI/item_icons/1h_sword_084c.dds",
"Data/UI/item_icons/1h_sword_085.dds",
"Data/UI/item_icons/1h_sword_085a.dds",
"Data/UI/item_icons/1h_sword_085b.dds",
"Data/UI/item_icons/1h_sword_085c.dds",
"Data/UI/item_icons/1h_sword_086.dds",
"Data/UI/item_icons/1h_sword_086a.dds",
"Data/UI/item_icons/1h_sword_087.dds",
"Data/UI/item_icons/1h_sword_087a.dds",
"Data/UI/item_icons/1h_sword_088.dds",
"Data/UI/item_icons/1h_sword_088a.dds",
"Data/UI/item_icons/1h_sword_089.dds",
"Data/UI/item_icons/1h_sword_089a.dds",
"Data/UI/item_icons/1h_sword_090.dds",
"Data/UI/item_icons/1h_sword_090a.dds",
"Data/UI/item_icons/1h_sword_091.dds",
"Data/UI/item_icons/1h_sword_091a.dds",
"Data/UI/item_icons/1h_sword_091b.dds",
"Data/UI/item_icons/1h_sword_092.dds",
"Data/UI/item_icons/1h_sword_092a.dds",
"Data/UI/item_icons/1h_sword_093.dds",
"Data/UI/item_icons/1h_sword_093a.dds",
"Data/UI/item_icons/1h_sword_094.dds",
"Data/UI/item_icons/1h_sword_094a.dds",
"Data/UI/item_icons/1h_sword_094b.dds",
"Data/UI/item_icons/1h_sword_094c.dds",
"Data/UI/item_icons/1h_sword_095.dds",
"Data/UI/item_icons/1h_sword_095a.dds",
"Data/UI/item_icons/1h_sword_096.dds",
"Data/UI/item_icons/1h_sword_096a.dds",
"Data/UI/item_icons/1h_sword_096b.dds",
"Data/UI/item_icons/1h_sword_097.dds",
"Data/UI/item_icons/1h_sword_097_a.dds",
"Data/UI/item_icons/1h_sword_097_b.dds",
"Data/UI/item_icons/1h_sword_53.dds",
"Data/UI/item_icons/1h_sword_53a.dds",
"Data/UI/item_icons/1h_sword_53b.dds",
"Data/UI/item_icons/1h_sword_53c.dds",
"Data/UI/item_icons/1h_sword_54.dds",
"Data/UI/item_icons/1h_sword_54a.dds",
"Data/UI/item_icons/1h_sword_54b.dds",
"Data/UI/item_icons/1h_sword_55.dds",
"Data/UI/item_icons/1h_sword_56.dds",
"Data/UI/item_icons/1h_sword_56a.dds",
"Data/UI/item_icons/1h_sword_57.dds",
"Data/UI/item_icons/1h_sword_57a.dds",
"Data/UI/item_icons/1h_sword_58.dds",
"Data/UI/item_icons/1h_sword_58a.dds",
"Data/UI/item_icons/1h_sword_59.dds",
"Data/UI/item_icons/1h_sword_59a.dds",
"Data/UI/item_icons/1h_sword_60.dds",
"Data/UI/item_icons/1h_sword_60a.dds",
"Data/UI/item_icons/1h_sword_61.dds",
"Data/UI/item_icons/1h_sword_61a.dds",
"Data/UI/item_icons/1h_sword_62.dds",
"Data/UI/item_icons/1h_sword_62a.dds",
"Data/UI/item_icons/1h_sword_63.dds",
"Data/UI/item_icons/1h_sword_63a.dds",
"Data/UI/item_icons/1h_sword_63b.dds",
"Data/UI/item_icons/1h_sword_64.dds",
"Data/UI/item_icons/1h_sword_64a.dds",
"Data/UI/item_icons/1h_sword_65.dds",
"Data/UI/item_icons/1h_sword_65a.dds",
"Data/UI/item_icons/1h_sword_66.dds",
"Data/UI/item_icons/1h_sword_66a.dds",
"Data/UI/item_icons/1h_sword_67.dds",
"Data/UI/item_icons/1h_sword_68.dds",
"Data/UI/item_icons/1h_sword_68a.dds",
"Data/UI/item_icons/1h_sword_69.dds",
"Data/UI/item_icons/1h_sword_69a.dds",
"Data/UI/item_icons/1h_sword_70.dds",
"Data/UI/item_icons/1h_sword_70a.dds",
"Data/UI/item_icons/1h_sword_air_epic.dds",
"Data/UI/item_icons/1h_sword_air_epic_a.dds",
"Data/UI/item_icons/1h_sword_air_epic_b.dds",
"Data/UI/item_icons/1h_sword_earth_mundane.dds",
"Data/UI/item_icons/1h_sword_earth_mundane_a.dds",
"Data/UI/item_icons/1h_sword_fire_epic.dds",
"Data/UI/item_icons/1h_sword_fire_epic_a.dds",
"Data/UI/item_icons/1h_sword_water_epic.dds",
"Data/UI/item_icons/1h_sword_water_epic_a.dds",
"Data/UI/item_icons/1h_sword_water_epic_b.dds",
"Data/UI/item_icons/1h_sword_water_epic_c.dds",
"Data/UI/item_icons/1h_wand_air_mundane.dds",
"Data/UI/item_icons/1h_wand_air_mundane_a.dds",
"Data/UI/item_icons/1h_wand_death_epic.dds",
"Data/UI/item_icons/1h_wand_death_epic_a.dds",
"Data/UI/item_icons/1h_wand_earth_epic.dds",
"Data/UI/item_icons/1h_wand_earth_epic_a.dds",
"Data/UI/item_icons/1h_wand_earth_epic_b.dds",
"Data/UI/item_icons/1h_wand_fire_epic.dds",
"Data/UI/item_icons/1h_wand_fire_epic_a.dds",
"Data/UI/item_icons/1h_wand_fire_epic_b.dds",
"Data/UI/item_icons/1h_wand_water_mundane.dds",
"Data/UI/item_icons/1h_wand_water_mundane_a.dds",
"Data/UI/item_icons/2h_axe_010.dds",
"Data/UI/item_icons/2h_axe_010a.dds",
"Data/UI/item_icons/2h_axe_011.dds",
"Data/UI/item_icons/2h_axe_011a.dds",
"Data/UI/item_icons/2h_axe_012.dds",
"Data/UI/item_icons/2h_axe_012a.dds",
"Data/UI/item_icons/2h_axe_013.dds",
"Data/UI/item_icons/2h_axe_014.dds",
"Data/UI/item_icons/2h_axe_015.dds",
"Data/UI/item_icons/2h_axe_017.dds",
"Data/UI/item_icons/2h_axe_017a.dds",
"Data/UI/item_icons/2h_axe_018.dds",
"Data/UI/item_icons/2h_axe_018_a.dds",
"Data/UI/item_icons/2h_axe_019.dds",
"Data/UI/item_icons/2h_axe_019_a.dds",
"Data/UI/item_icons/2h_axe_020.dds",
"Data/UI/item_icons/2h_axe_021.dds",
"Data/UI/item_icons/2h_axe_021a.dds",
"Data/UI/item_icons/2h_axe_021b.dds",
"Data/UI/item_icons/2h_axe_022.dds",
"Data/UI/item_icons/2h_axe_022a.dds",
"Data/UI/item_icons/2h_axe_023.dds",
"Data/UI/item_icons/2h_axe_023a.dds",
"Data/UI/item_icons/2h_axe_024.dds",
"Data/UI/item_icons/2h_axe_025.dds",
"Data/UI/item_icons/2h_axe_025a.dds",
"Data/UI/item_icons/2h_axe_026.dds",
"Data/UI/item_icons/2h_axe_026a.dds",
"Data/UI/item_icons/2h_axe_027.dds",
"Data/UI/item_icons/2h_axe_027_a.dds",
"Data/UI/item_icons/2h_axe_028.dds",
"Data/UI/item_icons/2h_axe_029.dds",
"Data/UI/item_icons/2h_axe_029_a.dds",
"Data/UI/item_icons/2h_axe_030.dds",
"Data/UI/item_icons/2h_axe_031.dds",
"Data/UI/item_icons/2h_axe_032.dds",
"Data/UI/item_icons/2h_axe_033.dds",
"Data/UI/item_icons/2h_axe_034.dds",
"Data/UI/item_icons/2h_axe_035.dds",
"Data/UI/item_icons/2h_axe_036.dds",
"Data/UI/item_icons/2h_axe_036_a.dds",
"Data/UI/item_icons/2h_axe_037.dds",
"Data/UI/item_icons/2h_axe_037_a.dds",
"Data/UI/item_icons/2h_axe_038.dds",
"Data/UI/item_icons/2h_axe_038_a.dds",
"Data/UI/item_icons/2h_axe_039.dds",
"Data/UI/item_icons/2h_axe_039_a.dds",
"Data/UI/item_icons/2h_axe_039_b.dds",
"Data/UI/item_icons/2h_axe_040.dds",
"Data/UI/item_icons/2h_axe_040_a.dds",
"Data/UI/item_icons/2h_axe_041.dds",
"Data/UI/item_icons/2h_axe_041_a.dds",
"Data/UI/item_icons/2h_axe_042.dds",
"Data/UI/item_icons/2h_axe_042_a.dds",
"Data/UI/item_icons/2h_axe_042_b.dds",
"Data/UI/item_icons/2h_axe_043.dds",
"Data/UI/item_icons/2h_axe_043_a.dds",
"Data/UI/item_icons/2h_axe_044.dds",
"Data/UI/item_icons/2h_axe_044a.dds",
"Data/UI/item_icons/2h_axe_045.dds",
"Data/UI/item_icons/2h_axe_045a.dds",
"Data/UI/item_icons/2h_axe_045b.dds",
"Data/UI/item_icons/2h_axe_046.dds",
"Data/UI/item_icons/2h_axe_046a.dds",
"Data/UI/item_icons/2h_axe_046b.dds",
"Data/UI/item_icons/2h_axe_058.dds",
"Data/UI/item_icons/2h_axe_058a.dds",
"Data/UI/item_icons/2h_axe_059.dds",
"Data/UI/item_icons/2h_axe_059a.dds",
"Data/UI/item_icons/2h_axe_060.dds",
"Data/UI/item_icons/2h_axe_060a.dds",
"Data/UI/item_icons/2h_axe_060b.dds",
"Data/UI/item_icons/2h_axe_061.dds",
"Data/UI/item_icons/2h_axe_061a.dds",
"Data/UI/item_icons/2h_axe_062.dds",
"Data/UI/item_icons/2h_axe_062a.dds",
"Data/UI/item_icons/2h_axe_062b.dds",
"Data/UI/item_icons/2h_axe_063.dds",
"Data/UI/item_icons/2h_axe_063a.dds",
"Data/UI/item_icons/2h_axe_064.dds",
"Data/UI/item_icons/2h_axe_064a.dds",
"Data/UI/item_icons/2h_axe_065.dds",
"Data/UI/item_icons/2h_axe_065a.dds",
"Data/UI/item_icons/2h_axe_066.dds",
"Data/UI/item_icons/2h_axe_066a.dds",
"Data/UI/item_icons/2h_axe_067.dds",
"Data/UI/item_icons/2h_axe_067a.dds",
"Data/UI/item_icons/2h_axe_068.dds",
"Data/UI/item_icons/2h_axe_068a.dds",
"Data/UI/item_icons/2h_axe_069.dds",
"Data/UI/item_icons/2h_axe_069a.dds",
"Data/UI/item_icons/2h_axe_070.dds",
"Data/UI/item_icons/2h_axe_070a.dds",
"Data/UI/item_icons/2h_axe_071.dds",
"Data/UI/item_icons/2h_axe_071a.dds",
"Data/UI/item_icons/2h_axe_072.dds",
"Data/UI/item_icons/2h_axe_073.dds",
"Data/UI/item_icons/2h_axe_073a.dds",
"Data/UI/item_icons/2h_axe_074.dds",
"Data/UI/item_icons/2h_axe_074a.dds",
"Data/UI/item_icons/2h_axe_075.dds",
"Data/UI/item_icons/2h_axe_075a.dds",
"Data/UI/item_icons/2h_axe_076.dds",
"Data/UI/item_icons/2h_axe_076a.dds",
"Data/UI/item_icons/2h_axe_077.dds",
"Data/UI/item_icons/2h_axe_077a.dds",
"Data/UI/item_icons/2h_axe_078.dds",
"Data/UI/item_icons/2h_axe_078a.dds",
"Data/UI/item_icons/2h_axe_079.dds",
"Data/UI/item_icons/2h_axe_079a.dds",
"Data/UI/item_icons/2h_axe_080.dds",
"Data/UI/item_icons/2h_axe_080a.dds",
"Data/UI/item_icons/2h_axe_081.dds",
"Data/UI/item_icons/2h_axe_081a.dds",
"Data/UI/item_icons/2h_axe_082.dds",
"Data/UI/item_icons/2h_axe_082a.dds",
"Data/UI/item_icons/2h_axe_083.dds",
"Data/UI/item_icons/2h_axe_083a.dds",
"Data/UI/item_icons/2h_axe_47.dds",
"Data/UI/item_icons/2h_axe_47a.dds",
"Data/UI/item_icons/2h_axe_48.dds",
"Data/UI/item_icons/2h_axe_48a.dds",
"Data/UI/item_icons/2h_axe_49.dds",
"Data/UI/item_icons/2h_axe_49a.dds",
"Data/UI/item_icons/2h_axe_50.dds",
"Data/UI/item_icons/2h_axe_50a.dds",
"Data/UI/item_icons/2h_axe_51.dds",
"Data/UI/item_icons/2h_axe_51a.dds",
"Data/UI/item_icons/2h_axe_52.dds",
"Data/UI/item_icons/2h_axe_52a.dds",
"Data/UI/item_icons/2h_axe_53.dds",
"Data/UI/item_icons/2h_axe_53a.dds",
"Data/UI/item_icons/2h_axe_54.dds",
"Data/UI/item_icons/2h_axe_54a.dds",
"Data/UI/item_icons/2h_axe_55.dds",
"Data/UI/item_icons/2h_axe_55a.dds",
"Data/UI/item_icons/2h_axe_56.dds",
"Data/UI/item_icons/2h_axe_57.dds",
"Data/UI/item_icons/2h_axe_57a.dds",
"Data/UI/item_icons/2h_axe_air_epic.dds",
"Data/UI/item_icons/2h_axe_air_epic_a.dds",
"Data/UI/item_icons/2h_axe_death_epic.dds",
"Data/UI/item_icons/2h_axe_death_epic_a.dds",
"Data/UI/item_icons/2h_axe_earth_mundane.dds",
"Data/UI/item_icons/2h_axe_earth_mundane_a.dds",
"Data/UI/item_icons/2h_axe_earth_mundane_b.dds",
"Data/UI/item_icons/2h_axe_water_mundane.dds",
"Data/UI/item_icons/2h_axe_water_mundane_a.dds",
"Data/UI/item_icons/2h_hammer_air_epic.dds",
"Data/UI/item_icons/2h_hammer_air_epic_a.dds",
"Data/UI/item_icons/2h_hammer_earth_epic.dds",
"Data/UI/item_icons/2h_hammer_earth_epic_a.dds",
"Data/UI/item_icons/2h_hammer_fire_mundane.dds",
"Data/UI/item_icons/2h_hammer_fire_mundane_a.dds",
"Data/UI/item_icons/2h_hammer_life_epic.dds",
"Data/UI/item_icons/2h_hammer_life_epic_a.dds",
"Data/UI/item_icons/2h_hammer_water_epic.dds",
"Data/UI/item_icons/2h_hammer_water_epic_a.dds",
"Data/UI/item_icons/2h_mace_016.dds",
"Data/UI/item_icons/2h_mace_016_a.dds",
"Data/UI/item_icons/2h_mace_017.dds",
"Data/UI/item_icons/2h_mace_021.dds",
"Data/UI/item_icons/2h_mace_024.dds",
"Data/UI/item_icons/2h_mace_026.dds",
"Data/UI/item_icons/2h_mace_040.dds",
"Data/UI/item_icons/2h_mace_040a.dds",
"Data/UI/item_icons/2h_mace_041.dds",
"Data/UI/item_icons/2h_mace_041a.dds",
"Data/UI/item_icons/2h_mace_042.dds",
"Data/UI/item_icons/2h_mace_042a.dds",
"Data/UI/item_icons/2h_mace_042b.dds",
"Data/UI/item_icons/2h_mace_043.dds",
"Data/UI/item_icons/2h_mace_043a.dds",
"Data/UI/item_icons/2h_mace_044.dds",
"Data/UI/item_icons/2h_mace_044a.dds",
"Data/UI/item_icons/2h_mace_044b.dds",
"Data/UI/item_icons/2h_mace_045.dds",
"Data/UI/item_icons/2h_mace_045a.dds",
"Data/UI/item_icons/2h_mace_046.dds",
"Data/UI/item_icons/2h_mace_046a.dds",
"Data/UI/item_icons/2h_mace_052.dds",
"Data/UI/item_icons/2h_mace_052a.dds",
"Data/UI/item_icons/2h_mace_053.dds",
"Data/UI/item_icons/2h_mace_053a.dds",
"Data/UI/item_icons/2h_mace_054.dds",
"Data/UI/item_icons/2h_mace_054a.dds",
"Data/UI/item_icons/2h_mace_055.dds",
"Data/UI/item_icons/2h_mace_055a.dds",
"Data/UI/item_icons/2h_mace_056.dds",
"Data/UI/item_icons/2h_mace_056a.dds",
"Data/UI/item_icons/2h_mace_057.dds",
"Data/UI/item_icons/2h_mace_057a.dds",
"Data/UI/item_icons/2h_mace_058.dds",
"Data/UI/item_icons/2h_mace_058a.dds",
"Data/UI/item_icons/2h_mace_059.dds",
"Data/UI/item_icons/2h_mace_059a.dds",
"Data/UI/item_icons/2h_mace_059b.dds",
"Data/UI/item_icons/2h_mace_060.dds",
"Data/UI/item_icons/2h_mace_060a.dds",
"Data/UI/item_icons/2h_mace_061.dds",
"Data/UI/item_icons/2h_mace_061a.dds",
"Data/UI/item_icons/2h_mace_062.dds",
"Data/UI/item_icons/2h_mace_062a.dds",
"Data/UI/item_icons/2h_mace_063.dds",
"Data/UI/item_icons/2h_mace_063a.dds",
"Data/UI/item_icons/2h_mace_063b.dds",
"Data/UI/item_icons/2h_mace_064.dds",
"Data/UI/item_icons/2h_mace_064a.dds",
"Data/UI/item_icons/2h_mace_065.dds",
"Data/UI/item_icons/2h_mace_065a.dds",
"Data/UI/item_icons/2h_mace_066.dds",
"Data/UI/item_icons/2h_mace_066a.dds",
"Data/UI/item_icons/2h_mace_067.dds",
"Data/UI/item_icons/2h_mace_067a.dds",
"Data/UI/item_icons/2h_mace_068.dds",
"Data/UI/item_icons/2h_mace_068a.dds",
"Data/UI/item_icons/2h_mace_069.dds",
"Data/UI/item_icons/2h_mace_069a.dds",
"Data/UI/item_icons/2h_mace_070a.dds",
"Data/UI/item_icons/2h_mace_070_48.dds",
"Data/UI/item_icons/2h_mace_071.dds",
"Data/UI/item_icons/2h_mace_071a.dds",
"Data/UI/item_icons/2h_mace_072.dds",
"Data/UI/item_icons/2h_mace_072a.dds",
"Data/UI/item_icons/2h_mace_073.dds",
"Data/UI/item_icons/2h_mace_073a.dds",
"Data/UI/item_icons/2h_mace_074.dds",
"Data/UI/item_icons/2h_mace_074a.dds",
"Data/UI/item_icons/2h_mace_075a.dds",
"Data/UI/item_icons/2h_mace_075_48.dds",
"Data/UI/item_icons/2h_mace_076.dds",
"Data/UI/item_icons/2h_mace_076a.dds",
"Data/UI/item_icons/2h_mace_077.dds",
"Data/UI/item_icons/2h_mace_077a.dds",
"Data/UI/item_icons/2h_mace_078.dds",
"Data/UI/item_icons/2h_mace_078a.dds",
"Data/UI/item_icons/2h_mace_079.dds",
"Data/UI/item_icons/2h_mace_079a.dds",
"Data/UI/item_icons/2h_mace_081.dds",
"Data/UI/item_icons/2h_mace_081_a.dds",
"Data/UI/item_icons/2h_mace_47.dds",
"Data/UI/item_icons/2h_mace_47a.dds",
"Data/UI/item_icons/2h_mace_48.dds",
"Data/UI/item_icons/2h_mace_48a.dds",
"Data/UI/item_icons/2h_mace_49.dds",
"Data/UI/item_icons/2h_mace_49a.dds",
"Data/UI/item_icons/2h_mace_50.dds",
"Data/UI/item_icons/2h_mace_50a.dds",
"Data/UI/item_icons/2h_mace_51.dds",
"Data/UI/item_icons/2h_mace_51a.dds",
"Data/UI/item_icons/2h_mace_life_01.dds",
"Data/UI/item_icons/2h_mace_life_01_a.dds",
"Data/UI/item_icons/2h_mace_waterlord.dds",
"Data/UI/item_icons/2h_mace_waterlord_a.dds",
"Data/UI/item_icons/2h_mace_waterlord_b.dds",
"Data/UI/item_icons/2h_polearm_012.dds",
"Data/UI/item_icons/2h_polearm_012_a.dds",
"Data/UI/item_icons/2h_polearm_013.dds",
"Data/UI/item_icons/2h_polearm_013a.dds",
"Data/UI/item_icons/2h_polearm_014.dds",
"Data/UI/item_icons/2h_polearm_014a.dds",
"Data/UI/item_icons/2h_polearm_015.dds",
"Data/UI/item_icons/2h_polearm_015_a.dds",
"Data/UI/item_icons/2h_polearm_016.dds",
"Data/UI/item_icons/2h_polearm_016_a.dds",
"Data/UI/item_icons/2h_polearm_018.dds",
"Data/UI/item_icons/2h_polearm_018_a.dds",
"Data/UI/item_icons/2h_polearm_019.dds",
"Data/UI/item_icons/2h_polearm_019_a.dds",
"Data/UI/item_icons/2h_polearm_023.dds",
"Data/UI/item_icons/2h_polearm_023_1.dds",
"Data/UI/item_icons/2h_polearm_023_1a.dds",
"Data/UI/item_icons/2h_polearm_023_1b.dds",
"Data/UI/item_icons/2h_polearm_023_a.dds",
"Data/UI/item_icons/2h_polearm_023_b.dds",
"Data/UI/item_icons/2h_polearm_024.dds",
"Data/UI/item_icons/2h_polearm_025.dds",
"Data/UI/item_icons/2h_polearm_025a.dds",
"Data/UI/item_icons/2h_polearm_026.dds",
"Data/UI/item_icons/2h_polearm_026a.dds",
"Data/UI/item_icons/2h_polearm_027.dds",
"Data/UI/item_icons/2h_polearm_027a.dds",
"Data/UI/item_icons/2h_polearm_028.dds",
"Data/UI/item_icons/2h_polearm_028a.dds",
"Data/UI/item_icons/2h_polearm_029a.dds",
"Data/UI/item_icons/2h_polearm_030.dds",
"Data/UI/item_icons/2h_polearm_030a.dds",
"Data/UI/item_icons/2h_polearm_031.dds",
"Data/UI/item_icons/2h_polearm_031a.dds",
"Data/UI/item_icons/2h_polearm_031b.dds",
"Data/UI/item_icons/2h_polearm_032.dds",
"Data/UI/item_icons/2h_polearm_032a.dds",
"Data/UI/item_icons/2h_polearm_033.dds",
"Data/UI/item_icons/2h_polearm_033a.dds",
"Data/UI/item_icons/2h_polearm_034.dds",
"Data/UI/item_icons/2h_polearm_034a.dds",
"Data/UI/item_icons/2h_polearm_035.dds",
"Data/UI/item_icons/2h_polearm_035a.dds",
"Data/UI/item_icons/2h_polearm_036.dds",
"Data/UI/item_icons/2h_polearm_036a.dds",
"Data/UI/item_icons/2h_polearm_037.dds",
"Data/UI/item_icons/2h_polearm_038.dds",
"Data/UI/item_icons/2h_polearm_038a.dds",
"Data/UI/item_icons/2h_polearm_039.dds",
"Data/UI/item_icons/2h_polearm_039a.dds",
"Data/UI/item_icons/2h_polearm_040.dds",
"Data/UI/item_icons/2h_polearm_040_a.dds",
"Data/UI/item_icons/2h_polearm_041.dds",
"Data/UI/item_icons/2h_polearm_041_a.dds",
"Data/UI/item_icons/2h_polearm_042.dds",
"Data/UI/item_icons/2h_polearm_042_a.dds",
"Data/UI/item_icons/2h_polearm_043.dds",
"Data/UI/item_icons/2h_polearm_043_a.dds",
"Data/UI/item_icons/2h_polearm_044.dds",
"Data/UI/item_icons/2h_polearm_044_a.dds",
"Data/UI/item_icons/2h_polearm_045.dds",
"Data/UI/item_icons/2h_polearm_045_a.dds",
"Data/UI/item_icons/2h_polearm_046.dds",
"Data/UI/item_icons/2h_polearm_046_a.dds",
"Data/UI/item_icons/2h_polearm_047.dds",
"Data/UI/item_icons/2h_polearm_047_a.dds",
"Data/UI/item_icons/2h_polearm_048.dds",
"Data/UI/item_icons/2h_polearm_048_a.dds",
"Data/UI/item_icons/2h_polearm_049.dds",
"Data/UI/item_icons/2h_polearm_049_a.dds",
"Data/UI/item_icons/2h_polearm_050.dds",
"Data/UI/item_icons/2h_polearm_050_a.dds",
"Data/UI/item_icons/2h_polearm_051.dds",
"Data/UI/item_icons/2h_polearm_051_a.dds",
"Data/UI/item_icons/2h_polearm_052.dds",
"Data/UI/item_icons/2h_polearm_053.dds",
"Data/UI/item_icons/2h_polearm_054.dds",
"Data/UI/item_icons/2h_polearm_054a.dds",
"Data/UI/item_icons/2h_polearm_054b.dds",
"Data/UI/item_icons/2h_polearm_055.dds",
"Data/UI/item_icons/2h_polearm_055a.dds",
"Data/UI/item_icons/2h_polearm_056.dds",
"Data/UI/item_icons/2h_polearm_057.dds",
"Data/UI/item_icons/2h_polearm_057a.dds",
"Data/UI/item_icons/2h_polearm_058.dds",
"Data/UI/item_icons/2h_polearm_058a.dds",
"Data/UI/item_icons/2h_polearm_059.dds",
"Data/UI/item_icons/2h_polearm_059a.dds",
"Data/UI/item_icons/2h_polearm_060.dds",
"Data/UI/item_icons/2h_polearm_060a.dds",
"Data/UI/item_icons/2h_polearm_061.dds",
"Data/UI/item_icons/2h_polearm_062.dds",
"Data/UI/item_icons/2h_polearm_062a.dds",
"Data/UI/item_icons/2h_polearm_063.dds",
"Data/UI/item_icons/2h_polearm_063a.dds",
"Data/UI/item_icons/2h_polearm_064.dds",
"Data/UI/item_icons/2h_polearm_065.dds",
"Data/UI/item_icons/2h_polearm_065_a.dds",
"Data/UI/item_icons/2h_polearm_065_b.dds",
"Data/UI/item_icons/2h_polearm_065_c.dds",
"Data/UI/item_icons/2h_polearm_fire_mundane.dds",
"Data/UI/item_icons/2h_polearm_fire_mundane_a.dds",
"Data/UI/item_icons/2h_polearm_water_epic.dds",
"Data/UI/item_icons/2h_polearm_water_epic_a.dds",
"Data/UI/item_icons/2h_staff_012.dds",
"Data/UI/item_icons/2h_staff_013.dds",
"Data/UI/item_icons/2h_staff_013_a.dds",
"Data/UI/item_icons/2h_staff_014.dds",
"Data/UI/item_icons/2h_staff_015.dds",
"Data/UI/item_icons/2h_staff_015_a.dds",
"Data/UI/item_icons/2h_staff_016.dds",
"Data/UI/item_icons/2h_staff_017.dds",
"Data/UI/item_icons/2h_staff_018.dds",
"Data/UI/item_icons/2h_staff_019.dds",
"Data/UI/item_icons/2h_staff_020.dds",
"Data/UI/item_icons/2h_staff_021.dds",
"Data/UI/item_icons/2h_staff_021a.dds",
"Data/UI/item_icons/2h_staff_021b.dds",
"Data/UI/item_icons/2h_staff_022.dds",
"Data/UI/item_icons/2h_staff_022a.dds",
"Data/UI/item_icons/2h_staff_022b.dds",
"Data/UI/item_icons/2h_staff_023.dds",
"Data/UI/item_icons/2h_staff_023_a.dds",
"Data/UI/item_icons/2h_staff_024.dds",
"Data/UI/item_icons/2h_staff_025.dds",
"Data/UI/item_icons/2h_staff_026.dds",
"Data/UI/item_icons/2h_staff_027.dds",
"Data/UI/item_icons/2h_staff_027a.dds",
"Data/UI/item_icons/2h_staff_028.dds",
"Data/UI/item_icons/2h_staff_028a.dds",
"Data/UI/item_icons/2h_staff_029.dds",
"Data/UI/item_icons/2h_staff_030.dds",
"Data/UI/item_icons/2h_staff_031.dds",
"Data/UI/item_icons/2h_staff_031a.dds",
"Data/UI/item_icons/2h_staff_032.dds",
"Data/UI/item_icons/2h_staff_033.dds",
"Data/UI/item_icons/2h_staff_034.dds",
"Data/UI/item_icons/2h_staff_035.dds",
"Data/UI/item_icons/2h_staff_036.dds",
"Data/UI/item_icons/2h_staff_036_a.dds",
"Data/UI/item_icons/2h_staff_037.dds",
"Data/UI/item_icons/2h_staff_037_a.dds",
"Data/UI/item_icons/2h_staff_038.dds",
"Data/UI/item_icons/2h_staff_038_a.dds",
"Data/UI/item_icons/2h_staff_039.dds",
"Data/UI/item_icons/2h_staff_039_a.dds",
"Data/UI/item_icons/2h_staff_041.dds",
"Data/UI/item_icons/2h_staff_041a.dds",
"Data/UI/item_icons/2h_staff_042.dds",
"Data/UI/item_icons/2h_staff_042a.dds",
"Data/UI/item_icons/2h_staff_043.dds",
"Data/UI/item_icons/2h_staff_043a.dds",
"Data/UI/item_icons/2h_staff_044.dds",
"Data/UI/item_icons/2h_staff_044a.dds",
"Data/UI/item_icons/2h_staff_045.dds",
"Data/UI/item_icons/2h_staff_045a.dds",
"Data/UI/item_icons/2h_staff_046.dds",
"Data/UI/item_icons/2h_staff_046a.dds",
"Data/UI/item_icons/2h_staff_047.dds",
"Data/UI/item_icons/2h_staff_047a.dds",
"Data/UI/item_icons/2h_staff_048.dds",
"Data/UI/item_icons/2h_staff_048a.dds",
"Data/UI/item_icons/2h_staff_048b.dds",
"Data/UI/item_icons/2h_staff_049.dds",
"Data/UI/item_icons/2h_staff_049a.dds",
"Data/UI/item_icons/2h_staff_050.dds",
"Data/UI/item_icons/2h_staff_050a.dds",
"Data/UI/item_icons/2h_staff_050b.dds",
"Data/UI/item_icons/2h_staff_051.dds",
"Data/UI/item_icons/2h_staff_051a.dds",
"Data/UI/item_icons/2h_staff_052.dds",
"Data/UI/item_icons/2h_staff_052a.dds",
"Data/UI/item_icons/2h_staff_053.dds",
"Data/UI/item_icons/2h_staff_053a.dds",
"Data/UI/item_icons/2h_staff_054.dds",
"Data/UI/item_icons/2h_staff_054a.dds",
"Data/UI/item_icons/2h_staff_055.dds",
"Data/UI/item_icons/2h_staff_055a.dds",
"Data/UI/item_icons/2h_staff_056.dds",
"Data/UI/item_icons/2h_staff_056a.dds",
"Data/UI/item_icons/2h_staff_057.dds",
"Data/UI/item_icons/2h_staff_057a.dds",
"Data/UI/item_icons/2h_staff_057b.dds",
"Data/UI/item_icons/2h_staff_058.dds",
"Data/UI/item_icons/2h_staff_058a.dds",
"Data/UI/item_icons/2h_staff_059.dds",
"Data/UI/item_icons/2h_staff_059a.dds",
"Data/UI/item_icons/2h_staff_060.dds",
"Data/UI/item_icons/2h_staff_060a.dds",
"Data/UI/item_icons/2h_staff_061.dds",
"Data/UI/item_icons/2h_staff_061a.dds",
"Data/UI/item_icons/2h_staff_062.dds",
"Data/UI/item_icons/2h_staff_062a.dds",
"Data/UI/item_icons/2h_staff_062b.dds",
"Data/UI/item_icons/2h_staff_063.dds",
"Data/UI/item_icons/2h_staff_064.dds",
"Data/UI/item_icons/2h_staff_064a.dds",
"Data/UI/item_icons/2h_staff_065.dds",
"Data/UI/item_icons/2h_staff_065a.dds",
"Data/UI/item_icons/2h_staff_066.dds",
"Data/UI/item_icons/2h_staff_066a.dds",
"Data/UI/item_icons/2h_staff_067.dds",
"Data/UI/item_icons/2h_staff_067a.dds",
"Data/UI/item_icons/2h_staff_068.dds",
"Data/UI/item_icons/2h_staff_068a.dds",
"Data/UI/item_icons/2h_staff_069.dds",
"Data/UI/item_icons/2h_staff_069a.dds",
"Data/UI/item_icons/2h_staff_070.dds",
"Data/UI/item_icons/2h_staff_070a.dds",
"Data/UI/item_icons/2h_staff_071.dds",
"Data/UI/item_icons/2h_staff_071a.dds",
"Data/UI/item_icons/2h_staff_072.dds",
"Data/UI/item_icons/2h_staff_072a.dds",
"Data/UI/item_icons/2h_staff_073.dds",
"Data/UI/item_icons/2h_staff_073a.dds",
"Data/UI/item_icons/2h_staff_074.dds",
"Data/UI/item_icons/2h_staff_074a.dds",
"Data/UI/item_icons/2h_staff_075.dds",
"Data/UI/item_icons/2h_staff_075a.dds",
"Data/UI/item_icons/2h_staff_076.dds",
"Data/UI/item_icons/2h_staff_076a.dds",
"Data/UI/item_icons/2h_staff_077.dds",
"Data/UI/item_icons/2h_staff_077a.dds",
"Data/UI/item_icons/2h_staff_078.dds",
"Data/UI/item_icons/2h_staff_078a.dds",
"Data/UI/item_icons/2h_staff_079.dds",
"Data/UI/item_icons/2h_staff_079_a.dds",
"Data/UI/item_icons/2h_staff_079_b.dds",
"Data/UI/item_icons/2h_staff_079_c1.dds",
"Data/UI/item_icons/2h_staff_080.dds",
"Data/UI/item_icons/2h_staff_080_a.dds",
"Data/UI/item_icons/2h_staff_40.dds",
"Data/UI/item_icons/2h_staff_40a.dds",
"Data/UI/item_icons/2h_staff_air_epic.dds",
"Data/UI/item_icons/2h_staff_air_epic_a.dds",
"Data/UI/item_icons/2h_staff_air_mundane.dds",
"Data/UI/item_icons/2h_staff_air_mundane_a.dds",
"Data/UI/item_icons/2h_staff_death_epic_a.dds",
"Data/UI/item_icons/2h_staff_death_epic_b.dds",
"Data/UI/item_icons/2h_staff_earth_epic.dds",
"Data/UI/item_icons/2h_staff_earth_epic_a.dds",
"Data/UI/item_icons/2h_staff_earth_mundane.dds",
"Data/UI/item_icons/2h_staff_earth_mundane_a.dds",
"Data/UI/item_icons/2h_staff_fire_epic.dds",
"Data/UI/item_icons/2h_staff_fire_epic_a.dds",
"Data/UI/item_icons/2h_staff_fire_epic_b.dds",
"Data/UI/item_icons/2h_staff_fire_mundane.dds",
"Data/UI/item_icons/2h_staff_fire_mundane_a.dds",
"Data/UI/item_icons/2h_staff_life_01.dds",
"Data/UI/item_icons/2h_staff_life_01_a.dds",
"Data/UI/item_icons/2h_staff_water_epic.dds",
"Data/UI/item_icons/2h_staff_water_epic_a.dds",
"Data/UI/item_icons/2h_staff_water_mundane.dds",
"Data/UI/item_icons/2h_staff_water_mundane_a.dds",
"Data/UI/item_icons/2h_sword_011.dds",
"Data/UI/item_icons/2h_sword_012.dds",
"Data/UI/item_icons/2h_sword_013.dds",
"Data/UI/item_icons/2h_sword_013a.dds",
"Data/UI/item_icons/2h_sword_015.dds",
"Data/UI/item_icons/2h_sword_015a.dds",
"Data/UI/item_icons/2h_sword_015b.dds",
"Data/UI/item_icons/2h_sword_016.dds",
"Data/UI/item_icons/2h_sword_017.dds",
"Data/UI/item_icons/2h_sword_017_a.dds",
"Data/UI/item_icons/2h_sword_018.dds",
"Data/UI/item_icons/2h_sword_018_a.dds",
"Data/UI/item_icons/2h_sword_019.dds",
"Data/UI/item_icons/2h_sword_020.dds",
"Data/UI/item_icons/2h_sword_020_a.dds",
"Data/UI/item_icons/2h_sword_021.dds",
"Data/UI/item_icons/2h_sword_021_a.dds",
"Data/UI/item_icons/2h_sword_022.dds",
"Data/UI/item_icons/2h_sword_022_a.dds",
"Data/UI/item_icons/2h_sword_023.dds",
"Data/UI/item_icons/2h_sword_024.dds",
"Data/UI/item_icons/2h_sword_024a.dds",
"Data/UI/item_icons/2h_sword_024b.dds",
"Data/UI/item_icons/2h_sword_025.dds",
"Data/UI/item_icons/2h_sword_025a.dds",
"Data/UI/item_icons/2h_sword_026.dds",
"Data/UI/item_icons/2h_sword_027.dds",
"Data/UI/item_icons/2h_sword_028.dds",
"Data/UI/item_icons/2h_sword_028a.dds",
"Data/UI/item_icons/2h_sword_029.dds",
"Data/UI/item_icons/2h_sword_030.dds",
"Data/UI/item_icons/2h_sword_030a.dds",
"Data/UI/item_icons/2h_sword_031.dds",
"Data/UI/item_icons/2h_sword_031a.dds",
"Data/UI/item_icons/2h_sword_032.dds",
"Data/UI/item_icons/2h_sword_032a.dds",
"Data/UI/item_icons/2h_sword_033.dds",
"Data/UI/item_icons/2h_sword_034.dds",
"Data/UI/item_icons/2h_sword_034a.dds",
"Data/UI/item_icons/2h_sword_034b.dds",
"Data/UI/item_icons/2h_sword_034c.dds",
"Data/UI/item_icons/2h_sword_035.dds",
"Data/UI/item_icons/2h_sword_036.dds",
"Data/UI/item_icons/2h_sword_036a.dds",
"Data/UI/item_icons/2h_sword_037.dds",
"Data/UI/item_icons/2h_sword_037_a.dds",
"Data/UI/item_icons/2h_sword_038.dds",
"Data/UI/item_icons/2h_sword_038_a.dds",
"Data/UI/item_icons/2h_sword_039.dds",
"Data/UI/item_icons/2h_sword_039_a.dds",
"Data/UI/item_icons/2h_sword_039_b.dds",
"Data/UI/item_icons/2h_sword_040.dds",
"Data/UI/item_icons/2h_sword_040_a.dds",
"Data/UI/item_icons/2h_sword_041.dds",
"Data/UI/item_icons/2h_sword_041_a.dds",
"Data/UI/item_icons/2h_sword_041_b.dds",
"Data/UI/item_icons/2h_sword_042.dds",
"Data/UI/item_icons/2h_sword_042_a.dds",
"Data/UI/item_icons/2h_sword_042_b.dds",
"Data/UI/item_icons/2h_sword_042_c.dds",
"Data/UI/item_icons/2h_sword_043.dds",
"Data/UI/item_icons/2h_sword_043a.dds",
"Data/UI/item_icons/2h_sword_043b.dds",
"Data/UI/item_icons/2h_sword_044.dds",
"Data/UI/item_icons/2h_sword_044a.dds",
"Data/UI/item_icons/2h_sword_044b.dds",
"Data/UI/item_icons/2h_sword_045.dds",
"Data/UI/item_icons/2h_sword_045a.dds",
"Data/UI/item_icons/2h_sword_046.dds",
"Data/UI/item_icons/2h_sword_047.dds",
"Data/UI/item_icons/2h_sword_047a.dds",
"Data/UI/item_icons/2h_sword_047b.dds",
"Data/UI/item_icons/2h_sword_047c.dds",
"Data/UI/item_icons/2h_sword_048.dds",
"Data/UI/item_icons/2h_sword_048a.dds",
"Data/UI/item_icons/2h_sword_049.dds",
"Data/UI/item_icons/2h_sword_049a.dds",
"Data/UI/item_icons/2h_sword_050.dds",
"Data/UI/item_icons/2h_sword_050a.dds",
"Data/UI/item_icons/2h_sword_051.dds",
"Data/UI/item_icons/2h_sword_051a.dds",
"Data/UI/item_icons/2h_sword_051b.dds",
"Data/UI/item_icons/2h_sword_052.dds",
"Data/UI/item_icons/2h_sword_052a.dds",
"Data/UI/item_icons/2h_sword_053.dds",
"Data/UI/item_icons/2h_sword_053a.dds",
"Data/UI/item_icons/2h_sword_054.dds",
"Data/UI/item_icons/2h_sword_054a.dds",
"Data/UI/item_icons/2h_sword_055.dds",
"Data/UI/item_icons/2h_sword_055a.dds",
"Data/UI/item_icons/2h_sword_056.dds",
"Data/UI/item_icons/2h_sword_057.dds",
"Data/UI/item_icons/2h_sword_057a.dds",
"Data/UI/item_icons/2h_sword_058.dds",
"Data/UI/item_icons/2h_sword_058a.dds",
"Data/UI/item_icons/2h_sword_059.dds",
"Data/UI/item_icons/2h_sword_059a.dds",
"Data/UI/item_icons/2h_sword_060a.dds",
"Data/UI/item_icons/2h_sword_060b.dds",
"Data/UI/item_icons/2h_sword_061.dds",
"Data/UI/item_icons/2h_sword_061a.dds",
"Data/UI/item_icons/2h_sword_062.dds",
"Data/UI/item_icons/2h_sword_062a.dds",
"Data/UI/item_icons/2h_sword_063.dds",
"Data/UI/item_icons/2h_sword_063a.dds",
"Data/UI/item_icons/2h_sword_064.dds",
"Data/UI/item_icons/2h_sword_064a.dds",
"Data/UI/item_icons/2h_sword_065.dds",
"Data/UI/item_icons/2h_sword_065a.dds",
"Data/UI/item_icons/2h_sword_066.dds",
"Data/UI/item_icons/2h_sword_066a.dds",
"Data/UI/item_icons/2h_sword_067.dds",
"Data/UI/item_icons/2h_sword_067a.dds",
"Data/UI/item_icons/2h_sword_067b.dds",
"Data/UI/item_icons/2h_sword_068.dds",
"Data/UI/item_icons/2h_sword_068a.dds",
"Data/UI/item_icons/2h_sword_069.dds",
"Data/UI/item_icons/2h_sword_069a.dds",
"Data/UI/item_icons/2h_sword_070.dds",
"Data/UI/item_icons/2h_sword_070a.dds",
"Data/UI/item_icons/2h_sword_071.dds",
"Data/UI/item_icons/2h_sword_071a.dds",
"Data/UI/item_icons/2h_sword_072.dds",
"Data/UI/item_icons/2h_sword_072a.dds",
"Data/UI/item_icons/2h_sword_073.dds",
"Data/UI/item_icons/2h_sword_073a.dds",
"Data/UI/item_icons/2h_sword_074.dds",
"Data/UI/item_icons/2h_sword_074a.dds",
"Data/UI/item_icons/2h_sword_075.dds",
"Data/UI/item_icons/2h_sword_075a.dds",
"Data/UI/item_icons/2h_sword_076.dds",
"Data/UI/item_icons/2h_sword_076a.dds",
"Data/UI/item_icons/2h_sword_077.dds",
"Data/UI/item_icons/2h_sword_077a.dds",
"Data/UI/item_icons/2h_sword_air_mundane.dds",
"Data/UI/item_icons/2h_sword_air_mundane_a.dds",
"Data/UI/item_icons/2h_sword_fire_epic.dds",
"Data/UI/item_icons/2h_sword_fire_epic_a.dds",
"Data/UI/item_icons/2h_sword_life_01.dds",
"Data/UI/item_icons/2h_sword_life_01_a.dds",
"Data/UI/item_icons/2h_sword_life_mundane.dds",
"Data/UI/item_icons/2h_sword_life_mundane_a.dds",
"Data/UI/item_icons/2h_sword_life_mundane_b.dds",
"Data/UI/item_icons/2h_sword_life_mundane_c.dds",
"Data/UI/item_icons/abyssal_opal_1.dds",
"Data/UI/item_icons/abyssal_opal_2.dds",
"Data/UI/item_icons/abyssal_opal_3.dds",
"Data/UI/item_icons/aelfwar_artefact.dds",
"Data/UI/item_icons/aelfwar_contract.dds",
"Data/UI/item_icons/aelfwar_contract_a.dds",
"Data/UI/item_icons/agate.dds",
"Data/UI/item_icons/alchemy_item1.dds",
"Data/UI/item_icons/alchemy_item2.dds",
"Data/UI/item_icons/alchemy_item2a.dds",
"Data/UI/item_icons/alchemy_item3.dds",
"Data/UI/item_icons/alchemy_item4.dds",
"Data/UI/item_icons/alchemy_item4a.dds",
"Data/UI/item_icons/alchemy_item4b.dds",
"Data/UI/item_icons/alchemy_item5.dds",
"Data/UI/item_icons/ammo_coating_01.dds",
"Data/UI/item_icons/ammo_coating_02.dds",
"Data/UI/item_icons/ammo_coating_02_a.dds",
"Data/UI/item_icons/ammo_coating_03.dds",
"Data/UI/item_icons/ammo_coating_03_a.dds",
"Data/UI/item_icons/ammo_coating_03_b.dds",
"Data/UI/item_icons/amorian_root1.dds",
"Data/UI/item_icons/amorian_root1_a.dds",
"Data/UI/item_icons/amorian_root2.dds",
"Data/UI/item_icons/amorian_root2_a.dds",
"Data/UI/item_icons/amorian_root2_b.dds",
"Data/UI/item_icons/ancient_charm.dds",
"Data/UI/item_icons/ancient_charm_a.dds",
"Data/UI/item_icons/ancient_fragment.dds",
"Data/UI/item_icons/angered_roots.dds",
"Data/UI/item_icons/angered_vines.dds",
"Data/UI/item_icons/anti-nature_weapon.dds",
"Data/UI/item_icons/anti-nature_weapon_a.dds",
"Data/UI/item_icons/apple.dds",
"Data/UI/item_icons/apple_a.dds",
"Data/UI/item_icons/apple_b.dds",
"Data/UI/item_icons/arcane_potion.dds",
"Data/UI/item_icons/arcane_potion_a.dds",
"Data/UI/item_icons/armor_plating_01.dds",
"Data/UI/item_icons/armor_plating_02.dds",
"Data/UI/item_icons/armor_plating_03.dds",
"Data/UI/item_icons/armor_plating_04.dds",
"Data/UI/item_icons/armor_plating_04_a.dds",
"Data/UI/item_icons/armor_plating_04_b.dds",
"Data/UI/item_icons/artefact1.dds",
"Data/UI/item_icons/artefact1a.dds",
"Data/UI/item_icons/artefact2.dds",
"Data/UI/item_icons/artefact2z.dds",
"Data/UI/item_icons/astral_ruby.dds",
"Data/UI/item_icons/astral_ruby_a.dds",
"Data/UI/item_icons/badges.dds",
"Data/UI/item_icons/badges_b.dds",
"Data/UI/item_icons/bag09.dds",
"Data/UI/item_icons/bag09a.dds",
"Data/UI/item_icons/bag1.dds",
"Data/UI/item_icons/bag10.dds",
"Data/UI/item_icons/bag11.dds",
"Data/UI/item_icons/bag12.dds",
"Data/UI/item_icons/bag13.dds",
"Data/UI/item_icons/bag14.dds",
"Data/UI/item_icons/bag15.dds",
"Data/UI/item_icons/bag15a.dds",
"Data/UI/item_icons/bag16.dds",
"Data/UI/item_icons/bag17.dds",
"Data/UI/item_icons/bag18.dds",
"Data/UI/item_icons/bag19.dds",
"Data/UI/item_icons/bag19a.dds",
"Data/UI/item_icons/bag2.dds",
"Data/UI/item_icons/bag20.dds",
"Data/UI/item_icons/bag2a.dds",
"Data/UI/item_icons/bag3.dds",
"Data/UI/item_icons/bag4.dds",
"Data/UI/item_icons/bag5.dds",
"Data/UI/item_icons/bag6.dds",
"Data/UI/item_icons/bag7.dds",
"Data/UI/item_icons/bag7a.dds",
"Data/UI/item_icons/bag8.dds",
"Data/UI/item_icons/banana.dds",
"Data/UI/item_icons/banana_a.dds",
"Data/UI/item_icons/bandage_enchated.dds",
"Data/UI/item_icons/bandage_enchated_a.dds",
"Data/UI/item_icons/bandage_heavy.dds",
"Data/UI/item_icons/bandage_light.dds",
"Data/UI/item_icons/bandage_medium.dds",
"Data/UI/item_icons/bandage_medium_a.dds",
"Data/UI/item_icons/bandage_runed.dds",
"Data/UI/item_icons/bandage_simple.dds",
"Data/UI/item_icons/bandage_small.dds",
"Data/UI/item_icons/bandage_special1.dds",
"Data/UI/item_icons/bandage_special2.dds",
"Data/UI/item_icons/bandage_special3.dds",
"Data/UI/item_icons/bandage_special4.dds",
"Data/UI/item_icons/bandage_special4_a.dds",
"Data/UI/item_icons/bandage_special5.dds",
"Data/UI/item_icons/bandage_special5_a.dds",
"Data/UI/item_icons/bandage_special5_b.dds",
"Data/UI/item_icons/bandage_starwart.dds",
"Data/UI/item_icons/bandage_starwart_a.dds",
"Data/UI/item_icons/bandage_superior.dds",
"Data/UI/item_icons/bandage_superior_b.dds",
"Data/UI/item_icons/bandage_thick.dds",
"Data/UI/item_icons/bandage_thick_a.dds",
"Data/UI/item_icons/banner_death.dds",
"Data/UI/item_icons/banner_death_a.dds",
"Data/UI/item_icons/banner_death_b.dds",
"Data/UI/item_icons/banner_port_scion.dds",
"Data/UI/item_icons/banner_port_scion2.dds",
"Data/UI/item_icons/banner_port_scion2a.dds",
"Data/UI/item_icons/banner_port_scion_a.dds",
"Data/UI/item_icons/banner_port_scion_b.dds",
"Data/UI/item_icons/banner_port_scion_c.dds",
"Data/UI/item_icons/banner_water.dds",
"Data/UI/item_icons/banner_water_a.dds",
"Data/UI/item_icons/banner_water_b.dds",
"Data/UI/item_icons/banner_water_z.dds",
"Data/UI/item_icons/bar01.dds",
"Data/UI/item_icons/bar02.dds",
"Data/UI/item_icons/bar03.dds",
"Data/UI/item_icons/bar04.dds",
"Data/UI/item_icons/bar05.dds",
"Data/UI/item_icons/bar06.dds",
"Data/UI/item_icons/bar07.dds",
"Data/UI/item_icons/bar08.dds",
"Data/UI/item_icons/bar08b.dds",
"Data/UI/item_icons/bar09.dds",
"Data/UI/item_icons/bar10a.dds",
"Data/UI/item_icons/bar11.dds",
"Data/UI/item_icons/bar11a.dds",
"Data/UI/item_icons/bar12a.dds",
"Data/UI/item_icons/bar12b.dds",
"Data/UI/item_icons/bar13.dds",
"Data/UI/item_icons/bar13a.dds",
"Data/UI/item_icons/bar13b.dds",
"Data/UI/item_icons/bar14.dds",
"Data/UI/item_icons/bar14a.dds",
"Data/UI/item_icons/bar14b.dds",
"Data/UI/item_icons/bar14c.dds",
"Data/UI/item_icons/bar15b.dds",
"Data/UI/item_icons/bar16a.dds",
"Data/UI/item_icons/bar16b.dds",
"Data/UI/item_icons/bar16z.dds",
"Data/UI/item_icons/barghest_claw.dds",
"Data/UI/item_icons/barghest_fang1.dds",
"Data/UI/item_icons/barghest_fang1_a.dds",
"Data/UI/item_icons/barghest_fang2.dds",
"Data/UI/item_icons/barghest_fang2_a.dds",
"Data/UI/item_icons/barghest_fur2.dds",
"Data/UI/item_icons/barghest_fur2_a.dds",
"Data/UI/item_icons/barghest_fur_1.dds",
"Data/UI/item_icons/barnacled_shell.dds",
"Data/UI/item_icons/barnacled_shell_a.dds",
"Data/UI/item_icons/bar_of_dark_chocolate.dds",
"Data/UI/item_icons/bar_of_soap.dds",
"Data/UI/item_icons/bar_of_soap_a.dds",
"Data/UI/item_icons/bar_of_soap_b.dds",
"Data/UI/item_icons/bar_of_soap_mod.dds",
"Data/UI/item_icons/battler_potion.dds",
"Data/UI/item_icons/battler_potion_a.dds",
"Data/UI/item_icons/battle_of_leolis.dds",
"Data/UI/item_icons/batwing1.dds",
"Data/UI/item_icons/beer_bottle.dds",
"Data/UI/item_icons/bell_bronze.dds",
"Data/UI/item_icons/bell_gold.dds",
"Data/UI/item_icons/bell_silver.dds",
"Data/UI/item_icons/beryl.dds",
"Data/UI/item_icons/beryl_a.dds",
"Data/UI/item_icons/bethany_bed_roll.dds",
"Data/UI/item_icons/bethany_bed_roll_a.dds",
"Data/UI/item_icons/binding_charm.dds",
"Data/UI/item_icons/binding_charm_a.dds",
"Data/UI/item_icons/birdwing1.dds",
"Data/UI/item_icons/black_heart.dds",
"Data/UI/item_icons/black_ocular.dds",
"Data/UI/item_icons/blank_parchment.dds",
"Data/UI/item_icons/blood-soakedpaw_a.dds",
"Data/UI/item_icons/blood-soaked_paw.dds",
"Data/UI/item_icons/bloodied_backpack.dds",
"Data/UI/item_icons/bloodied_backpack_a.dds",
"Data/UI/item_icons/bloodied_fur_black.dds",
"Data/UI/item_icons/bloodied_fur_brown.dds",
"Data/UI/item_icons/bloodied_fur_redbrown.dds",
"Data/UI/item_icons/bloodied_fur_redbrown_a.dds",
"Data/UI/item_icons/bloodied_fur_tan.dds",
"Data/UI/item_icons/bloodied_fur_white.dds",
"Data/UI/item_icons/bloodied_fur_white_a.dds",
"Data/UI/item_icons/bloodied_fur_white_b.dds",
"Data/UI/item_icons/bloodsample1.dds",
"Data/UI/item_icons/bloodsample1a.dds",
"Data/UI/item_icons/bloodsample2.dds",
"Data/UI/item_icons/bloody_flipper.dds",
"Data/UI/item_icons/bloody_saw_tooth.dds",
"Data/UI/item_icons/bloody_stinger.dds",
"Data/UI/item_icons/blood_berylite.dds",
"Data/UI/item_icons/blood_berylite_a.dds",
"Data/UI/item_icons/blue_flesh.dds",
"Data/UI/item_icons/blue_flesh_a.dds",
"Data/UI/item_icons/blue_pill.dds",
"Data/UI/item_icons/blue_pill_a.dds",
"Data/UI/item_icons/boarhoof1.dds",
"Data/UI/item_icons/bogling_brain.dds",
"Data/UI/item_icons/bogling_brain_a.dds",
"Data/UI/item_icons/bogling_chef.dds",
"Data/UI/item_icons/bogling_chef_a.dds",
"Data/UI/item_icons/bogling_frontal_lobe.dds",
"Data/UI/item_icons/bogling_frontal_lobe_a.dds",
"Data/UI/item_icons/bolt_of_coarse_linen.dds",
"Data/UI/item_icons/bolt_of_composite_cloth.dds",
"Data/UI/item_icons/bolt_of_fine_linen.dds",
"Data/UI/item_icons/bolt_of_fine_linen_a.dds",
"Data/UI/item_icons/bolt_of_silk_cloth.dds",
"Data/UI/item_icons/bolt_of_silk_cloth_a.dds",
"Data/UI/item_icons/bolt_of_spellspun_silk.dds",
"Data/UI/item_icons/bolt_of_spellspun_silk_a.dds",
"Data/UI/item_icons/bolt_of_witchweave.dds",
"Data/UI/item_icons/bolt_of_witchweave_a.dds",
"Data/UI/item_icons/book1.dds",
"Data/UI/item_icons/book1a.dds",
"Data/UI/item_icons/book2.dds",
"Data/UI/item_icons/book3.dds",
"Data/UI/item_icons/book4.dds",
"Data/UI/item_icons/book4a.dds",
"Data/UI/item_icons/book4b.dds",
"Data/UI/item_icons/book5.dds",
"Data/UI/item_icons/book6.dds",
"Data/UI/item_icons/book6a.dds",
"Data/UI/item_icons/book7.dds",
"Data/UI/item_icons/book8.dds",
"Data/UI/item_icons/book_of_enemies.dds",
"Data/UI/item_icons/bottle1.dds",
"Data/UI/item_icons/bottle1a.dds",
"Data/UI/item_icons/bottle2.dds",
"Data/UI/item_icons/bottle3.dds",
"Data/UI/item_icons/bottle4.dds",
"Data/UI/item_icons/bottle4a.dds",
"Data/UI/item_icons/bottle5.dds",
"Data/UI/item_icons/bottle6.dds",
"Data/UI/item_icons/bottle7.dds",
"Data/UI/item_icons/bottle8.dds",
"Data/UI/item_icons/bottle8a.dds",
"Data/UI/item_icons/bottle8b.dds",
"Data/UI/item_icons/bottle8c.dds",
"Data/UI/item_icons/bottle_01.dds",
"Data/UI/item_icons/bottle_01_a.dds",
"Data/UI/item_icons/bottle_01_b.dds",
"Data/UI/item_icons/bottle_01_c.dds",
"Data/UI/item_icons/bottle_02.dds",
"Data/UI/item_icons/bottle_03.dds",
"Data/UI/item_icons/bottle_03_a.dds",
"Data/UI/item_icons/bottle_04.dds",
"Data/UI/item_icons/bottle_04_a.dds",
"Data/UI/item_icons/bottle_of_gin.dds",
"Data/UI/item_icons/bottle_of_gin_a.dds",
"Data/UI/item_icons/bottle_of_rum.dds",
"Data/UI/item_icons/bottle_of_rum_a.dds",
"Data/UI/item_icons/bottle_of_rum_b.dds",
"Data/UI/item_icons/bowl_of_fruit.dds",
"Data/UI/item_icons/bowl_of_vegetable_stew.dds",
"Data/UI/item_icons/bracelet_beaded_damaged_01.dds",
"Data/UI/item_icons/bracelet_beaded_damaged_01_a.dds",
"Data/UI/item_icons/bracelet_beaded_normal_01.dds",
"Data/UI/item_icons/bracelet_beaded_normal_01_a.dds",
"Data/UI/item_icons/bracelet_beaded_ornate_01.dds",
"Data/UI/item_icons/bracelet_beaded_ornate_01_a.dds",
"Data/UI/item_icons/bracelet_beaded_ornate_01_b.dds",
"Data/UI/item_icons/bracelet_beaded_ornate_01_c.dds",
"Data/UI/item_icons/bracelet_beaded_ornate_01_z.dds",
"Data/UI/item_icons/bracelet_chain_damaged_01.dds",
"Data/UI/item_icons/bracelet_chain_damaged_01_a.dds",
"Data/UI/item_icons/bracelet_chain_damaged_01_b.dds",
"Data/UI/item_icons/bracelet_chain_normal_01.dds",
"Data/UI/item_icons/bracelet_chain_normal_01_a.dds",
"Data/UI/item_icons/bracelet_chain_normal_01_b.dds",
"Data/UI/item_icons/bracelet_chain_ornate_01.dds",
"Data/UI/item_icons/bracelet_chain_ornate_01_a.dds",
"Data/UI/item_icons/bracelet_chain_ornate_01_b.dds",
"Data/UI/item_icons/bracelet_chain_ornate_c.dds",
"Data/UI/item_icons/bracelet_solid_damaged_01.dds",
"Data/UI/item_icons/bracelet_solid_damaged_01_a.dds",
"Data/UI/item_icons/bracelet_solid_damaged_01_b.dds",
"Data/UI/item_icons/bracelet_solid_normal_01.dds",
"Data/UI/item_icons/bracelet_solid_normal_01_a.dds",
"Data/UI/item_icons/bracelet_solid_normal_02.dds",
"Data/UI/item_icons/bracelet_solid_normal_02_a.dds",
"Data/UI/item_icons/bracelet_solid_normal_02_z.dds",
"Data/UI/item_icons/bracelet_solid_ornate_01.dds",
"Data/UI/item_icons/bracelet_solid_ornate_01_a.dds",
"Data/UI/item_icons/bracelet_solid_ornate_01_b.dds",
"Data/UI/item_icons/bracelet_solid_ornate_01_c.dds",
"Data/UI/item_icons/bracelet_solid_ornate_01_d.dds",
"Data/UI/item_icons/brain.dds",
"Data/UI/item_icons/brain_a.dds",
"Data/UI/item_icons/brain_stew.dds",
"Data/UI/item_icons/brilliant_diamond.dds",
"Data/UI/item_icons/broken_bottle.dds",
"Data/UI/item_icons/broken_candy_cane.dds",
"Data/UI/item_icons/broken_cutlass_hilt.dds",
"Data/UI/item_icons/broken_dart.dds",
"Data/UI/item_icons/broken_dart_a.dds",
"Data/UI/item_icons/broken_flintlock_mechanism.dds",
"Data/UI/item_icons/broken_rigged_dice.dds",
"Data/UI/item_icons/broken_seashell.dds",
"Data/UI/item_icons/broken_sextant.dds",
"Data/UI/item_icons/bronze_bars.dds",
"Data/UI/item_icons/bronze_bell.dds",
"Data/UI/item_icons/bronze_goblet.dds",
"Data/UI/item_icons/bs_hammer1.dds",
"Data/UI/item_icons/bs_hammer2.dds",
"Data/UI/item_icons/bs_hammer3.dds",
"Data/UI/item_icons/bs_hammer4.dds",
"Data/UI/item_icons/bs_hammer5.dds",
"Data/UI/item_icons/bucket_of_water.dds",
"Data/UI/item_icons/bundle_of_artifacts.dds",
"Data/UI/item_icons/burdick_bedroll.dds",
"Data/UI/item_icons/burlap_b.dds",
"Data/UI/item_icons/burning_torch.dds",
"Data/UI/item_icons/burning_torch_a.dds",
"Data/UI/item_icons/cake_01.dds",
"Data/UI/item_icons/cake_01_a.dds",
"Data/UI/item_icons/cake_01_slice.dds",
"Data/UI/item_icons/cake_02.dds",
"Data/UI/item_icons/cake_02_a.dds",
"Data/UI/item_icons/cake_02_slice.dds",
"Data/UI/item_icons/cake_03.dds",
"Data/UI/item_icons/cake_03_a.dds",
"Data/UI/item_icons/cake_03_b.dds",
"Data/UI/item_icons/cake_03_c.dds",
"Data/UI/item_icons/cake_03_slice.dds",
"Data/UI/item_icons/cake_03_slice_a.dds",
"Data/UI/item_icons/cake_04.dds",
"Data/UI/item_icons/cake_04_a.dds",
"Data/UI/item_icons/cake_04_slice.dds",
"Data/UI/item_icons/cake_05.dds",
"Data/UI/item_icons/cake_05_a.dds",
"Data/UI/item_icons/cake_05_slice.dds",
"Data/UI/item_icons/card1.dds",
"Data/UI/item_icons/card10.dds",
"Data/UI/item_icons/card10a.dds",
"Data/UI/item_icons/card11.dds",
"Data/UI/item_icons/card11a.dds",
"Data/UI/item_icons/card12.dds",
"Data/UI/item_icons/card12a.dds",
"Data/UI/item_icons/card13.dds",
"Data/UI/item_icons/card13a.dds",
"Data/UI/item_icons/card14.dds",
"Data/UI/item_icons/card14a.dds",
"Data/UI/item_icons/card15.dds",
"Data/UI/item_icons/card15a.dds",
"Data/UI/item_icons/card16.dds",
"Data/UI/item_icons/card16a.dds",
"Data/UI/item_icons/card17.dds",
"Data/UI/item_icons/card17a.dds",
"Data/UI/item_icons/card18.dds",
"Data/UI/item_icons/card18a.dds",
"Data/UI/item_icons/card19.dds",
"Data/UI/item_icons/card19a.dds",
"Data/UI/item_icons/card1a.dds",
"Data/UI/item_icons/card2.dds",
"Data/UI/item_icons/card20.dds",
"Data/UI/item_icons/card20a.dds",
"Data/UI/item_icons/card21.dds",
"Data/UI/item_icons/card21a.dds",
"Data/UI/item_icons/card22.dds",
"Data/UI/item_icons/card22a.dds",
"Data/UI/item_icons/card22b.dds",
"Data/UI/item_icons/card23.dds",
"Data/UI/item_icons/card23a.dds",
"Data/UI/item_icons/card24.dds",
"Data/UI/item_icons/card24a.dds",
"Data/UI/item_icons/card24b.dds",
"Data/UI/item_icons/card25.dds",
"Data/UI/item_icons/card25a.dds",
"Data/UI/item_icons/card26.dds",
"Data/UI/item_icons/card26a.dds",
"Data/UI/item_icons/card26b.dds",
"Data/UI/item_icons/card27.dds",
"Data/UI/item_icons/card27a.dds",
"Data/UI/item_icons/card28a.dds",
"Data/UI/item_icons/card28b.dds",
"Data/UI/item_icons/card29.dds",
"Data/UI/item_icons/card29a.dds",
"Data/UI/item_icons/card29b.dds",
"Data/UI/item_icons/card2a.dds",
"Data/UI/item_icons/card3.dds",
"Data/UI/item_icons/card30a.dds",
"Data/UI/item_icons/card30b.dds",
"Data/UI/item_icons/card30c.dds",
"Data/UI/item_icons/card31.dds",
"Data/UI/item_icons/card31a.dds",
"Data/UI/item_icons/card31c.dds",
"Data/UI/item_icons/card32.dds",
"Data/UI/item_icons/card33.dds",
"Data/UI/item_icons/card33a.dds",
"Data/UI/item_icons/card34.dds",
"Data/UI/item_icons/card35.dds",
"Data/UI/item_icons/card36a.dds",
"Data/UI/item_icons/card36b.dds",
"Data/UI/item_icons/card37.dds",
"Data/UI/item_icons/card38.dds",
"Data/UI/item_icons/card38a.dds",
"Data/UI/item_icons/card38b.dds",
"Data/UI/item_icons/card39.dds",
"Data/UI/item_icons/card39a.dds",
"Data/UI/item_icons/card3a.dds",
"Data/UI/item_icons/card3b.dds",
"Data/UI/item_icons/card4.dds",
"Data/UI/item_icons/card40.dds",
"Data/UI/item_icons/card40a.dds",
"Data/UI/item_icons/card4a.dds",
"Data/UI/item_icons/card5.dds",
"Data/UI/item_icons/card5a.dds",
"Data/UI/item_icons/card6.dds",
"Data/UI/item_icons/card6a.dds",
"Data/UI/item_icons/card6b.dds",
"Data/UI/item_icons/card6bb.dds",
"Data/UI/item_icons/card7.dds",
"Data/UI/item_icons/card7a.dds",
"Data/UI/item_icons/card8.dds",
"Data/UI/item_icons/card8a.dds",
"Data/UI/item_icons/card9.dds",
"Data/UI/item_icons/card9a.dds",
"Data/UI/item_icons/casting_stone.dds",
"Data/UI/item_icons/catfish_jerky.dds",
"Data/UI/item_icons/caustic_artifact.dds",
"Data/UI/item_icons/celestite.dds",
"Data/UI/item_icons/celestite_a.dds",
"Data/UI/item_icons/century_diary.dds",
"Data/UI/item_icons/chain_000_boots.dds",
"Data/UI/item_icons/chain_000_boots_a.dds",
"Data/UI/item_icons/chain_000_chest.dds",
"Data/UI/item_icons/chain_000_chest_a.dds",
"Data/UI/item_icons/chain_000_leggings.dds",
"Data/UI/item_icons/chain_000_leggings_a.dds",
"Data/UI/item_icons/chain_100_chest.dds",
"Data/UI/item_icons/chain_100_chest_a.dds",
"Data/UI/item_icons/chain_101_belt.dds",
"Data/UI/item_icons/chain_101_boots.dds",
"Data/UI/item_icons/chain_101_chest.dds",
"Data/UI/item_icons/chain_101_gloves.dds",
"Data/UI/item_icons/chain_101_helm.dds",
"Data/UI/item_icons/chain_101_leggings.dds",
"Data/UI/item_icons/chain_101_shoulders.dds",
"Data/UI/item_icons/chain_102_belt.dds",
"Data/UI/item_icons/chain_102_boots.dds",
"Data/UI/item_icons/chain_102_chest.dds",
"Data/UI/item_icons/chain_102_gloves.dds",
"Data/UI/item_icons/chain_102_helm.dds",
"Data/UI/item_icons/chain_102_leggings.dds",
"Data/UI/item_icons/chain_102_shoulders.dds",
"Data/UI/item_icons/chain_102_shoulders_a.dds",
"Data/UI/item_icons/chain_103_belt.dds",
"Data/UI/item_icons/chain_103_belt_a.dds",
"Data/UI/item_icons/chain_103_boots.dds",
"Data/UI/item_icons/chain_103_boots_a.dds",
"Data/UI/item_icons/chain_103_chest.dds",
"Data/UI/item_icons/chain_103_chest_a.dds",
"Data/UI/item_icons/chain_103_gloves.dds",
"Data/UI/item_icons/chain_103_gloves_a.dds",
"Data/UI/item_icons/chain_103_helm.dds",
"Data/UI/item_icons/chain_103_helm_a.dds",
"Data/UI/item_icons/chain_103_leggings.dds",
"Data/UI/item_icons/chain_103_leggings_a.dds",
"Data/UI/item_icons/chain_103_shoulders.dds",
"Data/UI/item_icons/chain_103_shoulders_a.dds",
"Data/UI/item_icons/chain_104a_belt.dds",
"Data/UI/item_icons/chain_104a_belt_a.dds",
"Data/UI/item_icons/chain_104a_boots.dds",
"Data/UI/item_icons/chain_104a_boots_a.dds",
"Data/UI/item_icons/chain_104a_chest.dds",
"Data/UI/item_icons/chain_104a_chest_a.dds",
"Data/UI/item_icons/chain_104a_gloves.dds",
"Data/UI/item_icons/chain_104a_gloves_a.dds",
"Data/UI/item_icons/chain_104a_helm.dds",
"Data/UI/item_icons/chain_104a_helm_a.dds",
"Data/UI/item_icons/chain_104a_helm_b.dds",
"Data/UI/item_icons/chain_104a_helm_c.dds",
"Data/UI/item_icons/chain_104a_leggings.dds",
"Data/UI/item_icons/chain_104a_leggings_a.dds",
"Data/UI/item_icons/chain_104a_shoulder.dds",
"Data/UI/item_icons/chain_104a_shoulder_a.dds",
"Data/UI/item_icons/chain_104b_helm.dds",
"Data/UI/item_icons/chain_104b_helm_a.dds",
"Data/UI/item_icons/chain_104b_shoulders.dds",
"Data/UI/item_icons/chain_104b_shoulders_a.dds",
"Data/UI/item_icons/chain_104_belt.dds",
"Data/UI/item_icons/chain_104_boots.dds",
"Data/UI/item_icons/chain_104_chest.dds",
"Data/UI/item_icons/chain_104_gloves.dds",
"Data/UI/item_icons/chain_104_helm.dds",
"Data/UI/item_icons/chain_104_leggings.dds",
"Data/UI/item_icons/chain_104_shoulders.dds",
"Data/UI/item_icons/chain_105a_belt.dds",
"Data/UI/item_icons/chain_105a_belt_a.dds",
"Data/UI/item_icons/chain_105a_boots.dds",
"Data/UI/item_icons/chain_105a_boots_a.dds",
"Data/UI/item_icons/chain_105a_chest.dds",
"Data/UI/item_icons/chain_105a_chest_a.dds",
"Data/UI/item_icons/chain_105a_gloves.dds",
"Data/UI/item_icons/chain_105a_gloves_a.dds",
"Data/UI/item_icons/chain_105a_helm.dds",
"Data/UI/item_icons/chain_105a_helm_a.dds",
"Data/UI/item_icons/chain_105a_leggings.dds",
"Data/UI/item_icons/chain_105a_leggings_a.dds",
"Data/UI/item_icons/chain_105a_shoulders.dds",
"Data/UI/item_icons/chain_105a_shoulders_a.dds",
"Data/UI/item_icons/chain_105b_belt.dds",
"Data/UI/item_icons/chain_105b_belt_a.dds",
"Data/UI/item_icons/chain_105b_boots.dds",
"Data/UI/item_icons/chain_105b_boots_a.dds",
"Data/UI/item_icons/chain_105b_chest.dds",
"Data/UI/item_icons/chain_105b_chest_a.dds",
"Data/UI/item_icons/chain_105b_gloves.dds",
"Data/UI/item_icons/chain_105b_gloves_a.dds",
"Data/UI/item_icons/chain_105b_helm.dds",
"Data/UI/item_icons/chain_105b_helm_a.dds",
"Data/UI/item_icons/chain_105b_helm_c.dds",
"Data/UI/item_icons/chain_105b_helm_d.dds",
"Data/UI/item_icons/chain_105b_leggings.dds",
"Data/UI/item_icons/chain_105b_leggings_a.dds",
"Data/UI/item_icons/chain_105b_shoulders.dds",
"Data/UI/item_icons/chain_105b_shoulders_a.dds",
"Data/UI/item_icons/chain_105_belt.dds",
"Data/UI/item_icons/chain_105_boots.dds",
"Data/UI/item_icons/chain_105_chest.dds",
"Data/UI/item_icons/chain_105_gloves.dds",
"Data/UI/item_icons/chain_105_helm.dds",
"Data/UI/item_icons/chain_105_leggings.dds",
"Data/UI/item_icons/chain_105_shoulders.dds",
"Data/UI/item_icons/chain_106_belt.dds",
"Data/UI/item_icons/chain_106_boots.dds",
"Data/UI/item_icons/chain_106_chest.dds",
"Data/UI/item_icons/chain_106_gloves.dds",
"Data/UI/item_icons/chain_106_helm.dds",
"Data/UI/item_icons/chain_106_leggings.dds",
"Data/UI/item_icons/chain_106_shoulders.dds",
"Data/UI/item_icons/chain_107_belt.dds",
"Data/UI/item_icons/chain_107_belt_a.dds",
"Data/UI/item_icons/chain_107_boots.dds",
"Data/UI/item_icons/chain_107_boots_a.dds",
"Data/UI/item_icons/chain_107_chest.dds",
"Data/UI/item_icons/chain_107_chest_a.dds",
"Data/UI/item_icons/chain_107_chest_b.dds",
"Data/UI/item_icons/chain_107_gloves.dds",
"Data/UI/item_icons/chain_107_gloves_a.dds",
"Data/UI/item_icons/chain_107_helm.dds",
"Data/UI/item_icons/chain_107_helm_a.dds",
"Data/UI/item_icons/chain_107_helm_y.dds",
"Data/UI/item_icons/chain_107_helm_z.dds",
"Data/UI/item_icons/chain_107_leggings.dds",
"Data/UI/item_icons/chain_107_leggings_a.dds",
"Data/UI/item_icons/chain_107_shoulders.dds",
"Data/UI/item_icons/chain_107_shoulders_a.dds",
"Data/UI/item_icons/chain_108_belt.dds",
"Data/UI/item_icons/chain_108_boots.dds",
"Data/UI/item_icons/chain_108_chest.dds",
"Data/UI/item_icons/chain_108_gloves.dds",
"Data/UI/item_icons/chain_108_helm.dds",
"Data/UI/item_icons/chain_108_leggings.dds",
"Data/UI/item_icons/chain_108_shoulders.dds",
"Data/UI/item_icons/chain_109_belt.dds",
"Data/UI/item_icons/chain_109_boots.dds",
"Data/UI/item_icons/chain_109_chest.dds",
"Data/UI/item_icons/chain_109_gloves.dds",
"Data/UI/item_icons/chain_109_helm.dds",
"Data/UI/item_icons/chain_109_leggings.dds",
"Data/UI/item_icons/chain_109_shoulders.dds",
"Data/UI/item_icons/chain_110a_chest.dds",
"Data/UI/item_icons/chain_110a_chest_a.dds",
"Data/UI/item_icons/chain_110a_helm.dds",
"Data/UI/item_icons/chain_110a_helm_a.dds",
"Data/UI/item_icons/chain_110a_shoulders.dds",
"Data/UI/item_icons/chain_110a_shoulders_a.dds",
"Data/UI/item_icons/chain_110b_helm.dds",
"Data/UI/item_icons/chain_110b_helm_a.dds",
"Data/UI/item_icons/chain_110b_shoulders.dds",
"Data/UI/item_icons/chain_110b_shoulders_a.dds",
"Data/UI/item_icons/chain_110_belt.dds",
"Data/UI/item_icons/chain_110_boots.dds",
"Data/UI/item_icons/chain_110_chest.dds",
"Data/UI/item_icons/chain_110_chest_a.dds",
"Data/UI/item_icons/chain_110_gloves.dds",
"Data/UI/item_icons/chain_110_helm.dds",
"Data/UI/item_icons/chain_110_leggings.dds",
"Data/UI/item_icons/chain_110_shoulders.dds",
"Data/UI/item_icons/chain_111a_belt.dds",
"Data/UI/item_icons/chain_111a_belt_a.dds",
"Data/UI/item_icons/chain_111a_boots.dds",
"Data/UI/item_icons/chain_111a_boots_a.dds",
"Data/UI/item_icons/chain_111a_chest.dds",
"Data/UI/item_icons/chain_111a_chest_a.dds",
"Data/UI/item_icons/chain_111a_gloves.dds",
"Data/UI/item_icons/chain_111a_gloves_a.dds",
"Data/UI/item_icons/chain_111a_helm.dds",
"Data/UI/item_icons/chain_111a_helm_a.dds",
"Data/UI/item_icons/chain_111a_leggings.dds",
"Data/UI/item_icons/chain_111a_leggings_a.dds",
"Data/UI/item_icons/chain_111a_shoulder.dds",
"Data/UI/item_icons/chain_111a_shoulder_a.dds",
"Data/UI/item_icons/chain_111_belt.dds",
"Data/UI/item_icons/chain_111_boots.dds",
"Data/UI/item_icons/chain_111_chest.dds",
"Data/UI/item_icons/chain_111_gloves.dds",
"Data/UI/item_icons/chain_111_helm.dds",
"Data/UI/item_icons/chain_111_leggings.dds",
"Data/UI/item_icons/chain_111_shoulders.dds",
"Data/UI/item_icons/chain_112a_helm_a.dds",
"Data/UI/item_icons/chain_112a_shoulders.dds",
"Data/UI/item_icons/chain_112a_shoulders_a.dds",
"Data/UI/item_icons/chain_112_belt.dds",
"Data/UI/item_icons/chain_112_belt_a.dds",
"Data/UI/item_icons/chain_112_boots.dds",
"Data/UI/item_icons/chain_112_boots_a.dds",
"Data/UI/item_icons/chain_112_chest.dds",
"Data/UI/item_icons/chain_112_chest_a.dds",
"Data/UI/item_icons/chain_112_gloves.dds",
"Data/UI/item_icons/chain_112_gloves_a.dds",
"Data/UI/item_icons/chain_112_helm.dds",
"Data/UI/item_icons/chain_112_helm_a.dds",
"Data/UI/item_icons/chain_112_leggings.dds",
"Data/UI/item_icons/chain_112_leggings_a.dds",
"Data/UI/item_icons/chain_112_shoulders.dds",
"Data/UI/item_icons/chain_112_shoulders_a.dds",
"Data/UI/item_icons/chain_112_shoulders_b.dds",
"Data/UI/item_icons/chain_113A_belt.dds",
"Data/UI/item_icons/chain_113A_belt_a.dds",
"Data/UI/item_icons/chain_113A_boots.dds",
"Data/UI/item_icons/chain_113A_boots_a.dds",
"Data/UI/item_icons/chain_113A_chest.dds",
"Data/UI/item_icons/chain_113A_chest_a.dds",
"Data/UI/item_icons/chain_113A_gloves.dds",
"Data/UI/item_icons/chain_113A_gloves_a.dds",
"Data/UI/item_icons/chain_113A_helm.dds",
"Data/UI/item_icons/chain_113A_helm_a.dds",
"Data/UI/item_icons/chain_113A_leggings.dds",
"Data/UI/item_icons/chain_113A_leggings_a.dds",
"Data/UI/item_icons/chain_113A_shoulders.dds",
"Data/UI/item_icons/chain_113A_shoulders_a.dds",
"Data/UI/item_icons/chain_113B_belt.dds",
"Data/UI/item_icons/chain_113B_belt_a.dds",
"Data/UI/item_icons/chain_113B_boots.dds",
"Data/UI/item_icons/chain_113B_boots_a.dds",
"Data/UI/item_icons/chain_113B_chest.dds",
"Data/UI/item_icons/chain_113B_chest_a.dds",
"Data/UI/item_icons/chain_113B_gloves.dds",
"Data/UI/item_icons/chain_113B_gloves_a.dds",
"Data/UI/item_icons/chain_113B_helm.dds",
"Data/UI/item_icons/chain_113B_helm_a.dds",
"Data/UI/item_icons/chain_113B_leggings.dds",
"Data/UI/item_icons/chain_113B_leggings_a.dds",
"Data/UI/item_icons/chain_113B_shoulders.dds",
"Data/UI/item_icons/chain_113B_shoulders_a.dds",
"Data/UI/item_icons/chain_113C_belt.dds",
"Data/UI/item_icons/chain_113C_belt_a.dds",
"Data/UI/item_icons/chain_113C_boots.dds",
"Data/UI/item_icons/chain_113C_boots_a.dds",
"Data/UI/item_icons/chain_113C_chest.dds",
"Data/UI/item_icons/chain_113C_chest_a.dds",
"Data/UI/item_icons/chain_113C_gloves.dds",
"Data/UI/item_icons/chain_113C_gloves_a.dds",
"Data/UI/item_icons/chain_113C_helm.dds",
"Data/UI/item_icons/chain_113C_helm_a.dds",
"Data/UI/item_icons/chain_113C_leggings.dds",
"Data/UI/item_icons/chain_113C_leggings_a.dds",
"Data/UI/item_icons/chain_113C_shoulders.dds",
"Data/UI/item_icons/chain_113C_shoulders_a.dds",
"Data/UI/item_icons/chain_114a_belt.dds",
"Data/UI/item_icons/chain_114a_belt_a.dds",
"Data/UI/item_icons/chain_114a_boots.dds",
"Data/UI/item_icons/chain_114a_boots_a.dds",
"Data/UI/item_icons/chain_114a_chest.dds",
"Data/UI/item_icons/chain_114a_chest_a.dds",
"Data/UI/item_icons/chain_114a_gloves.dds",
"Data/UI/item_icons/chain_114a_gloves_a.dds",
"Data/UI/item_icons/chain_114a_helm.dds",
"Data/UI/item_icons/chain_114a_helm_a.dds",
"Data/UI/item_icons/chain_114a_leggings.dds",
"Data/UI/item_icons/chain_114a_leggings_a.dds",
"Data/UI/item_icons/chain_114a_shoulder.dds",
"Data/UI/item_icons/chain_114a_shoulder_a.dds",
"Data/UI/item_icons/chain_114_belt.dds",
"Data/UI/item_icons/chain_114_belt_a.dds",
"Data/UI/item_icons/chain_114_boots.dds",
"Data/UI/item_icons/chain_114_boots_a.dds",
"Data/UI/item_icons/chain_114_chest.dds",
"Data/UI/item_icons/chain_114_chest_a.dds",
"Data/UI/item_icons/chain_114_gloves.dds",
"Data/UI/item_icons/chain_114_gloves_a.dds",
"Data/UI/item_icons/chain_114_helm.dds",
"Data/UI/item_icons/chain_114_helm_a.dds",
"Data/UI/item_icons/chain_114_leggings.dds",
"Data/UI/item_icons/chain_114_leggings_a.dds",
"Data/UI/item_icons/chain_114_shoulders.dds",
"Data/UI/item_icons/chain_114_shoulders_a.dds",
"Data/UI/item_icons/chain_115a_helm.dds",
"Data/UI/item_icons/chain_115a_helm_a.dds",
"Data/UI/item_icons/chain_115a_shoulder.dds",
"Data/UI/item_icons/chain_115a_shoulder_a.dds",
"Data/UI/item_icons/chain_115_belt.dds",
"Data/UI/item_icons/chain_115_belt_a.dds",
"Data/UI/item_icons/chain_115_boots.dds",
"Data/UI/item_icons/chain_115_boots_a.dds",
"Data/UI/item_icons/chain_115_chest.dds",
"Data/UI/item_icons/chain_115_chest_a.dds",
"Data/UI/item_icons/chain_115_gloves.dds",
"Data/UI/item_icons/chain_115_gloves_a.dds",
"Data/UI/item_icons/chain_115_helm.dds",
"Data/UI/item_icons/chain_115_helm_a.dds",
"Data/UI/item_icons/chain_115_leggings.dds",
"Data/UI/item_icons/chain_115_leggings_a.dds",
"Data/UI/item_icons/chain_115_shoulders.dds",
"Data/UI/item_icons/chain_115_shoulders_a.dds",
"Data/UI/item_icons/chain_116_belt.dds",
"Data/UI/item_icons/chain_116_belt_a.dds",
"Data/UI/item_icons/chain_116_boots.dds",
"Data/UI/item_icons/chain_116_boots_a.dds",
"Data/UI/item_icons/chain_116_boots_b.dds",
"Data/UI/item_icons/chain_116_chest.dds",
"Data/UI/item_icons/chain_116_chest_a.dds",
"Data/UI/item_icons/chain_116_gloves.dds",
"Data/UI/item_icons/chain_116_gloves_a.dds",
"Data/UI/item_icons/chain_116_helm.dds",
"Data/UI/item_icons/chain_116_helm_a.dds",
"Data/UI/item_icons/chain_116_leggings.dds",
"Data/UI/item_icons/chain_116_leggings_a.dds",
"Data/UI/item_icons/chain_116_shoulders.dds",
"Data/UI/item_icons/chain_116_shoulders_a.dds",
"Data/UI/item_icons/chain_117_belt.dds",
"Data/UI/item_icons/chain_117_belt_a.dds",
"Data/UI/item_icons/chain_117_boots.dds",
"Data/UI/item_icons/chain_117_boots_a.dds",
"Data/UI/item_icons/chain_117_chest.dds",
"Data/UI/item_icons/chain_117_chest_a.dds",
"Data/UI/item_icons/chain_117_gloves.dds",
"Data/UI/item_icons/chain_117_gloves_a.dds",
"Data/UI/item_icons/chain_117_helm.dds",
"Data/UI/item_icons/chain_117_helm_a.dds",
"Data/UI/item_icons/chain_117_leggings.dds",
"Data/UI/item_icons/chain_117_leggings_a.dds",
"Data/UI/item_icons/chain_117_shoulders.dds",
"Data/UI/item_icons/chain_117_shoulders_a.dds",
"Data/UI/item_icons/chain_118_belt.dds",
"Data/UI/item_icons/chain_118_belt_a.dds",
"Data/UI/item_icons/chain_118_boots.dds",
"Data/UI/item_icons/chain_118_boots_a.dds",
"Data/UI/item_icons/chain_118_boots_b.dds",
"Data/UI/item_icons/chain_118_boots_c.dds",
"Data/UI/item_icons/chain_118_chest.dds",
"Data/UI/item_icons/chain_118_chest_a.dds",
"Data/UI/item_icons/chain_118_gloves.dds",
"Data/UI/item_icons/chain_118_gloves_a.dds",
"Data/UI/item_icons/chain_118_gloves_b.dds",
"Data/UI/item_icons/chain_118_helm.dds",
"Data/UI/item_icons/chain_118_helm_a.dds",
"Data/UI/item_icons/chain_118_leggings.dds",
"Data/UI/item_icons/chain_118_leggings_a.dds",
"Data/UI/item_icons/chain_118_shoulders.dds",
"Data/UI/item_icons/chain_118_shoulders_a.dds",
"Data/UI/item_icons/chain_belt_damaged_01.dds",
"Data/UI/item_icons/chain_belt_damaged_01_a.dds",
"Data/UI/item_icons/chain_boots_damaged_01.dds",
"Data/UI/item_icons/chain_boots_damaged_01_a.dds",
"Data/UI/item_icons/chain_chest_damaged_01.dds",
"Data/UI/item_icons/chain_chest_damaged_01_a.dds",
"Data/UI/item_icons/chain_gloves_damaged_01.dds",
"Data/UI/item_icons/chain_gloves_damaged_01_a.dds",
"Data/UI/item_icons/chain_helm_damaged_01.dds",
"Data/UI/item_icons/chain_helm_damaged_01_a.dds",
"Data/UI/item_icons/chain_leggings_damaged_01.dds",
"Data/UI/item_icons/chain_leggings_damaged_01_a.dds",
"Data/UI/item_icons/chain_shoulders_damaged_01.dds",
"Data/UI/item_icons/chain_shoulders_damaged_01_a.dds",
"Data/UI/item_icons/chaps_01.dds",
"Data/UI/item_icons/chaps_02.dds",
"Data/UI/item_icons/chaps_02_a.dds",
"Data/UI/item_icons/chaps_03.dds",
"Data/UI/item_icons/chaps_04.dds",
"Data/UI/item_icons/chaps_04_a.dds",
"Data/UI/item_icons/charcoal.dds",
"Data/UI/item_icons/charcoal_and_parchment.dds",
"Data/UI/item_icons/charlie_road_head.dds",
"Data/UI/item_icons/chest1.dds",
"Data/UI/item_icons/chest10.dds",
"Data/UI/item_icons/chest10a.dds",
"Data/UI/item_icons/chest10b.dds",
"Data/UI/item_icons/chest14.dds",
"Data/UI/item_icons/chest15.dds",
"Data/UI/item_icons/chest15a.dds",
"Data/UI/item_icons/chest1a.dds",
"Data/UI/item_icons/chest2.dds",
"Data/UI/item_icons/chest3.dds",
"Data/UI/item_icons/chest3a.dds",
"Data/UI/item_icons/chest4.dds",
"Data/UI/item_icons/chest4a.dds",
"Data/UI/item_icons/chest5.dds",
"Data/UI/item_icons/chest6.dds",
"Data/UI/item_icons/chest7.dds",
"Data/UI/item_icons/chest7a.dds",
"Data/UI/item_icons/chest8a.dds",
"Data/UI/item_icons/chest8b.dds",
"Data/UI/item_icons/chest9.dds",
"Data/UI/item_icons/chest_14.dds",
"Data/UI/item_icons/chest_14a.dds",
"Data/UI/item_icons/chest_15.dds",
"Data/UI/item_icons/chest_15a.dds",
"Data/UI/item_icons/chest_16.dds",
"Data/UI/item_icons/chest_17.dds",
"Data/UI/item_icons/chest_17a.dds",
"Data/UI/item_icons/chewed_bone.dds",
"Data/UI/item_icons/cinerous_coal.dds",
"Data/UI/item_icons/cinerous_coal_a.dds",
"Data/UI/item_icons/civilian_corpse.dds",
"Data/UI/item_icons/civilian_corpse_a.dds",
"Data/UI/item_icons/classical_escutcheon.dds",
"Data/UI/item_icons/cleansed_gnarl_of_tavra.dds",
"Data/UI/item_icons/cleansed_gnarl_of_tavra_a.dds",
"Data/UI/item_icons/cleansing_fire.dds",
"Data/UI/item_icons/cliff_bambler_horn.dds",
"Data/UI/item_icons/cloth1.dds",
"Data/UI/item_icons/cloth10.dds",
"Data/UI/item_icons/cloth11.dds",
"Data/UI/item_icons/cloth12.dds",
"Data/UI/item_icons/cloth12a.dds",
"Data/UI/item_icons/cloth12b.dds",
"Data/UI/item_icons/cloth13.dds",
"Data/UI/item_icons/cloth14.dds",
"Data/UI/item_icons/cloth15.dds",
"Data/UI/item_icons/cloth16a.dds",
"Data/UI/item_icons/cloth16b.dds",
"Data/UI/item_icons/cloth16c.dds",
"Data/UI/item_icons/cloth2.dds",
"Data/UI/item_icons/cloth3.dds",
"Data/UI/item_icons/cloth4.dds",
"Data/UI/item_icons/cloth4b.dds",
"Data/UI/item_icons/cloth5.dds",
"Data/UI/item_icons/cloth6.dds",
"Data/UI/item_icons/cloth6a.dds",
"Data/UI/item_icons/cloth7.dds",
"Data/UI/item_icons/cloth8.dds",
"Data/UI/item_icons/cloth9.dds",
"Data/UI/item_icons/cloth_000_boots.dds",
"Data/UI/item_icons/cloth_000_boots_a.dds",
"Data/UI/item_icons/cloth_000_chest.dds",
"Data/UI/item_icons/cloth_000_chest_a.dds",
"Data/UI/item_icons/cloth_000_leggings.dds",
"Data/UI/item_icons/cloth_000_leggings_a.dds",
"Data/UI/item_icons/cloth_100_boots.dds",
"Data/UI/item_icons/cloth_100_chest.dds",
"Data/UI/item_icons/cloth_100_leggings.dds",
"Data/UI/item_icons/cloth_101_belt.dds",
"Data/UI/item_icons/cloth_101_boots.dds",
"Data/UI/item_icons/cloth_101_chest.dds",
"Data/UI/item_icons/cloth_101_gloves.dds",
"Data/UI/item_icons/cloth_101_helm.dds",
"Data/UI/item_icons/cloth_101_leggings.dds",
"Data/UI/item_icons/cloth_101_shoulders.dds",
"Data/UI/item_icons/cloth_102_belt.dds",
"Data/UI/item_icons/cloth_102_boots.dds",
"Data/UI/item_icons/cloth_102_chest.dds",
"Data/UI/item_icons/cloth_102_gloves.dds",
"Data/UI/item_icons/cloth_102_gloves_a.dds",
"Data/UI/item_icons/cloth_102_helm.dds",
"Data/UI/item_icons/cloth_102_leggings.dds",
"Data/UI/item_icons/cloth_102_shoulders.dds",
"Data/UI/item_icons/cloth_103_belt.dds",
"Data/UI/item_icons/cloth_103_boots.dds",
"Data/UI/item_icons/cloth_103_chest.dds",
"Data/UI/item_icons/cloth_103_gloves.dds",
"Data/UI/item_icons/cloth_103_helm.dds",
"Data/UI/item_icons/cloth_103_helm_a.dds",
"Data/UI/item_icons/cloth_103_leggings.dds",
"Data/UI/item_icons/cloth_103_leggings_a.dds",
"Data/UI/item_icons/cloth_103_shoulders.dds",
"Data/UI/item_icons/cloth_104_belt.dds",
"Data/UI/item_icons/cloth_104_boots.dds",
"Data/UI/item_icons/cloth_104_chest.dds",
"Data/UI/item_icons/cloth_104_gloves.dds",
"Data/UI/item_icons/cloth_104_hat.dds",
"Data/UI/item_icons/cloth_104_leggings.dds",
"Data/UI/item_icons/cloth_104_shoulders.dds",
"Data/UI/item_icons/cloth_105_belt.dds",
"Data/UI/item_icons/cloth_105_boots.dds",
"Data/UI/item_icons/cloth_105_chest.dds",
"Data/UI/item_icons/cloth_105_gloves.dds",
"Data/UI/item_icons/cloth_105_helm.dds",
"Data/UI/item_icons/cloth_105_leggings.dds",
"Data/UI/item_icons/cloth_105_shoulders.dds",
"Data/UI/item_icons/cloth_106_belt.dds",
"Data/UI/item_icons/cloth_106_boots.dds",
"Data/UI/item_icons/cloth_106_chest.dds",
"Data/UI/item_icons/cloth_106_gloves.dds",
"Data/UI/item_icons/cloth_106_helm.dds",
"Data/UI/item_icons/cloth_106_leggings.dds",
"Data/UI/item_icons/cloth_106_shoulders.dds",
"Data/UI/item_icons/cloth_106_shoulders_a.dds",
"Data/UI/item_icons/cloth_107a_helm.dds",
"Data/UI/item_icons/cloth_107a_helm_a.dds",
"Data/UI/item_icons/cloth_107a_shoulders.dds",
"Data/UI/item_icons/cloth_107a_shoulders_a.dds",
"Data/UI/item_icons/cloth_107b_helm.dds",
"Data/UI/item_icons/cloth_107b_helm_a.dds",
"Data/UI/item_icons/cloth_107b_shoulders.dds",
"Data/UI/item_icons/cloth_107b_shoulders_a.dds",
"Data/UI/item_icons/cloth_107_belt.dds",
"Data/UI/item_icons/cloth_107_boots.dds",
"Data/UI/item_icons/cloth_107_chest.dds",
"Data/UI/item_icons/cloth_107_gloves.dds",
"Data/UI/item_icons/cloth_107_helm.dds",
"Data/UI/item_icons/cloth_107_leggings.dds",
"Data/UI/item_icons/cloth_107_shoulders.dds",
"Data/UI/item_icons/cloth_108_belt.dds",
"Data/UI/item_icons/cloth_108_boots.dds",
"Data/UI/item_icons/cloth_108_chest.dds",
"Data/UI/item_icons/cloth_108_gloves.dds",
"Data/UI/item_icons/cloth_108_helm.dds",
"Data/UI/item_icons/cloth_108_leggings.dds",
"Data/UI/item_icons/cloth_108_shoulders.dds",
"Data/UI/item_icons/cloth_110_belt.dds",
"Data/UI/item_icons/cloth_110_belt_a.dds",
"Data/UI/item_icons/cloth_110_boots.dds",
"Data/UI/item_icons/cloth_110_boots_a.dds",
"Data/UI/item_icons/cloth_110_chest.dds",
"Data/UI/item_icons/cloth_110_chest_a.dds",
"Data/UI/item_icons/cloth_110_gloves.dds",
"Data/UI/item_icons/cloth_110_gloves_a.dds",
"Data/UI/item_icons/cloth_110_helm.dds",
"Data/UI/item_icons/cloth_110_helm_2.dds",
"Data/UI/item_icons/cloth_110_helm_2_a.dds",
"Data/UI/item_icons/cloth_110_helm_a.dds",
"Data/UI/item_icons/cloth_110_leggings.dds",
"Data/UI/item_icons/cloth_110_leggings_a.dds",
"Data/UI/item_icons/cloth_110_leggings_b.dds",
"Data/UI/item_icons/cloth_110_leggings_c.dds",
"Data/UI/item_icons/cloth_110_shoulders.dds",
"Data/UI/item_icons/cloth_110_shoulders_a.dds",
"Data/UI/item_icons/cloth_110_shoulders_b.dds",
"Data/UI/item_icons/cloth_111_belt.dds",
"Data/UI/item_icons/cloth_111_boots.dds",
"Data/UI/item_icons/cloth_111_chest.dds",
"Data/UI/item_icons/cloth_111_gloves.dds",
"Data/UI/item_icons/cloth_111_helm.dds",
"Data/UI/item_icons/cloth_111_leggings.dds",
"Data/UI/item_icons/cloth_111_leggings_a.dds",
"Data/UI/item_icons/cloth_111_shoulders.dds",
"Data/UI/item_icons/cloth_112a_belt.dds",
"Data/UI/item_icons/cloth_112a_belt_a.dds",
"Data/UI/item_icons/cloth_112a_boots.dds",
"Data/UI/item_icons/cloth_112a_boots_a.dds",
"Data/UI/item_icons/cloth_112a_chest.dds",
"Data/UI/item_icons/cloth_112a_chest_a.dds",
"Data/UI/item_icons/cloth_112a_gloves.dds",
"Data/UI/item_icons/cloth_112a_gloves_a.dds",
"Data/UI/item_icons/cloth_112a_helm.dds",
"Data/UI/item_icons/cloth_112a_helm_a.dds",
"Data/UI/item_icons/cloth_112a_leggings.dds",
"Data/UI/item_icons/cloth_112a_leggings_a.dds",
"Data/UI/item_icons/cloth_112a_shoulder.dds",
"Data/UI/item_icons/cloth_112a_shoulder_a.dds",
"Data/UI/item_icons/cloth_112b_belt.dds",
"Data/UI/item_icons/cloth_112b_belt_a.dds",
"Data/UI/item_icons/cloth_112b_boots.dds",
"Data/UI/item_icons/cloth_112b_boots_a.dds",
"Data/UI/item_icons/cloth_112b_chest.dds",
"Data/UI/item_icons/cloth_112b_chest_a.dds",
"Data/UI/item_icons/cloth_112b_gloves.dds",
"Data/UI/item_icons/cloth_112b_gloves_a.dds",
"Data/UI/item_icons/cloth_112b_helm.dds",
"Data/UI/item_icons/cloth_112b_helm_a.dds",
"Data/UI/item_icons/cloth_112b_leggings.dds",
"Data/UI/item_icons/cloth_112b_leggings_a.dds",
"Data/UI/item_icons/cloth_112b_shoulder.dds",
"Data/UI/item_icons/cloth_112b_shoulder_a.dds",
"Data/UI/item_icons/cloth_112c_belt.dds",
"Data/UI/item_icons/cloth_112c_belt_a.dds",
"Data/UI/item_icons/cloth_112c_boots.dds",
"Data/UI/item_icons/cloth_112c_boots_a.dds",
"Data/UI/item_icons/cloth_112c_boots_b.dds",
"Data/UI/item_icons/cloth_112c_chest.dds",
"Data/UI/item_icons/cloth_112c_chest_a.dds",
"Data/UI/item_icons/cloth_112c_gloves.dds",
"Data/UI/item_icons/cloth_112c_gloves_a.dds",
"Data/UI/item_icons/cloth_112c_helm.dds",
"Data/UI/item_icons/cloth_112c_helm_a.dds",
"Data/UI/item_icons/cloth_112c_leggings.dds",
"Data/UI/item_icons/cloth_112c_leggings_a.dds",
"Data/UI/item_icons/cloth_112c_shoulders.dds",
"Data/UI/item_icons/cloth_112c_shoulders_a.dds",
"Data/UI/item_icons/cloth_112d_helm.dds",
"Data/UI/item_icons/cloth_112d_helm_a.dds",
"Data/UI/item_icons/cloth_112d_helm_b.dds",
"Data/UI/item_icons/cloth_112d_shoulders.dds",
"Data/UI/item_icons/cloth_112d_shoulders_a.dds",
"Data/UI/item_icons/cloth_112d_shoulders_b.dds",
"Data/UI/item_icons/cloth_112_belt.dds",
"Data/UI/item_icons/cloth_112_boots.dds",
"Data/UI/item_icons/cloth_112_chest.dds",
"Data/UI/item_icons/cloth_112_gloves.dds",
"Data/UI/item_icons/cloth_112_helm.dds",
"Data/UI/item_icons/cloth_112_leggings.dds",
"Data/UI/item_icons/cloth_112_shoulder.dds",
"Data/UI/item_icons/cloth_112_shoulders_a.dds",
"Data/UI/item_icons/cloth_113_belt.dds",
"Data/UI/item_icons/cloth_113_belt_a.dds",
"Data/UI/item_icons/cloth_113_boots.dds",
"Data/UI/item_icons/cloth_113_boots_a.dds",
"Data/UI/item_icons/cloth_113_chest.dds",
"Data/UI/item_icons/cloth_113_chest_a.dds",
"Data/UI/item_icons/cloth_113_gloves.dds",
"Data/UI/item_icons/cloth_113_gloves_a.dds",
"Data/UI/item_icons/cloth_113_helm.dds",
"Data/UI/item_icons/cloth_113_helm_a.dds",
"Data/UI/item_icons/cloth_113_leggings.dds",
"Data/UI/item_icons/cloth_113_leggings_a.dds",
"Data/UI/item_icons/cloth_113_shoulders.dds",
"Data/UI/item_icons/cloth_113_shoulders_a.dds",
"Data/UI/item_icons/cloth_114_belt.dds",
"Data/UI/item_icons/cloth_114_belt_a.dds",
"Data/UI/item_icons/cloth_114_boots.dds",
"Data/UI/item_icons/cloth_114_boots_a.dds",
"Data/UI/item_icons/cloth_114_chest.dds",
"Data/UI/item_icons/cloth_114_chest_a.dds",
"Data/UI/item_icons/cloth_114_gloves.dds",
"Data/UI/item_icons/cloth_114_gloves_a.dds",
"Data/UI/item_icons/cloth_114_helm.dds",
"Data/UI/item_icons/cloth_114_helm_a.dds",
"Data/UI/item_icons/cloth_114_helm_b.dds",
"Data/UI/item_icons/cloth_114_leggings.dds",
"Data/UI/item_icons/cloth_114_leggings_a.dds",
"Data/UI/item_icons/cloth_114_shoulders.dds",
"Data/UI/item_icons/cloth_114_shoulders_a.dds",
"Data/UI/item_icons/cloth_115A_belt.dds",
"Data/UI/item_icons/cloth_115A_belt_a.dds",
"Data/UI/item_icons/cloth_115A_boots.dds",
"Data/UI/item_icons/cloth_115A_boots_a.dds",
"Data/UI/item_icons/cloth_115A_chest.dds",
"Data/UI/item_icons/cloth_115A_chest_a.dds",
"Data/UI/item_icons/cloth_115A_gloves.dds",
"Data/UI/item_icons/cloth_115A_gloves_a.dds",
"Data/UI/item_icons/cloth_115A_helm.dds",
"Data/UI/item_icons/cloth_115A_helm_a.dds",
"Data/UI/item_icons/cloth_115A_leggings.dds",
"Data/UI/item_icons/cloth_115A_leggings_a.dds",
"Data/UI/item_icons/cloth_115A_shoulders.dds",
"Data/UI/item_icons/cloth_115A_shoulders_a.dds",
"Data/UI/item_icons/cloth_115B_belt.dds",
"Data/UI/item_icons/cloth_115B_belt_a.dds",
"Data/UI/item_icons/cloth_115B_boots.dds",
"Data/UI/item_icons/cloth_115B_boots_a.dds",
"Data/UI/item_icons/cloth_115B_chest.dds",
"Data/UI/item_icons/cloth_115B_chest_a.dds",
"Data/UI/item_icons/cloth_115B_gloves.dds",
"Data/UI/item_icons/cloth_115B_gloves_a.dds",
"Data/UI/item_icons/cloth_115B_helm.dds",
"Data/UI/item_icons/cloth_115B_helm_a.dds",
"Data/UI/item_icons/cloth_115B_leggings.dds",
"Data/UI/item_icons/cloth_115B_leggings_a.dds",
"Data/UI/item_icons/cloth_115B_shoulders.dds",
"Data/UI/item_icons/cloth_115B_shoulders_a.dds",
"Data/UI/item_icons/cloth_115_belt.dds",
"Data/UI/item_icons/cloth_115_belt_a.dds",
"Data/UI/item_icons/cloth_115_boots.dds",
"Data/UI/item_icons/cloth_115_boots_a.dds",
"Data/UI/item_icons/cloth_115_chest.dds",
"Data/UI/item_icons/cloth_115_chest_a.dds",
"Data/UI/item_icons/cloth_115_gloves.dds",
"Data/UI/item_icons/cloth_115_gloves_a.dds",
"Data/UI/item_icons/cloth_115_helm.dds",
"Data/UI/item_icons/cloth_115_helm_a.dds",
"Data/UI/item_icons/cloth_115_leggings.dds",
"Data/UI/item_icons/cloth_115_leggings_a.dds",
"Data/UI/item_icons/cloth_115_shoulders.dds",
"Data/UI/item_icons/cloth_115_shoulders_a.dds",
"Data/UI/item_icons/cloth_116a_helm.dds",
"Data/UI/item_icons/cloth_116a_helm_a.dds",
"Data/UI/item_icons/cloth_116a_shoulders.dds",
"Data/UI/item_icons/cloth_116a_shoulders_a.dds",
"Data/UI/item_icons/cloth_116_belt.dds",
"Data/UI/item_icons/cloth_116_belt_a.dds",
"Data/UI/item_icons/cloth_116_belt_b.dds",
"Data/UI/item_icons/cloth_116_boots.dds",
"Data/UI/item_icons/cloth_116_boots_a.dds",
"Data/UI/item_icons/cloth_116_chest.dds",
"Data/UI/item_icons/cloth_116_chest_a.dds",
"Data/UI/item_icons/cloth_116_gloves.dds",
"Data/UI/item_icons/cloth_116_gloves_a.dds",
"Data/UI/item_icons/cloth_116_helm.dds",
"Data/UI/item_icons/cloth_116_helm_a.dds",
"Data/UI/item_icons/cloth_116_leggings.dds",
"Data/UI/item_icons/cloth_116_leggings_a.dds",
"Data/UI/item_icons/cloth_116_shoulders.dds",
"Data/UI/item_icons/cloth_116_shoulders_a.dds",
"Data/UI/item_icons/cloth_117_belt.dds",
"Data/UI/item_icons/cloth_117_belt_a.dds",
"Data/UI/item_icons/cloth_117_belt_c.dds",
"Data/UI/item_icons/cloth_117_belt_d.dds",
"Data/UI/item_icons/cloth_117_boots.dds",
"Data/UI/item_icons/cloth_117_boots_a.dds",
"Data/UI/item_icons/cloth_117_chest.dds",
"Data/UI/item_icons/cloth_117_chest_a.dds",
"Data/UI/item_icons/cloth_117_gloves.dds",
"Data/UI/item_icons/cloth_117_gloves_a.dds",
"Data/UI/item_icons/cloth_117_helm.dds",
"Data/UI/item_icons/cloth_117_helm_a.dds",
"Data/UI/item_icons/cloth_117_leggings.dds",
"Data/UI/item_icons/cloth_117_leggings_a.dds",
"Data/UI/item_icons/cloth_117_shoulders.dds",
"Data/UI/item_icons/cloth_117_shoulders_a.dds",
"Data/UI/item_icons/cloth_118_belt.dds",
"Data/UI/item_icons/cloth_118_belt_a.dds",
"Data/UI/item_icons/cloth_118_boots.dds",
"Data/UI/item_icons/cloth_118_boots_a.dds",
"Data/UI/item_icons/cloth_118_chest.dds",
"Data/UI/item_icons/cloth_118_chest_a.dds",
"Data/UI/item_icons/cloth_118_gloves.dds",
"Data/UI/item_icons/cloth_118_gloves_a.dds",
"Data/UI/item_icons/cloth_118_helm.dds",
"Data/UI/item_icons/cloth_118_helm_a.dds",
"Data/UI/item_icons/cloth_118_helm_b.dds",
"Data/UI/item_icons/cloth_118_leggings.dds",
"Data/UI/item_icons/cloth_118_leggings_a.dds",
"Data/UI/item_icons/cloth_118_shoulders.dds",
"Data/UI/item_icons/cloth_118_shoulders_a.dds",
"Data/UI/item_icons/cloth_119_belt.dds",
"Data/UI/item_icons/cloth_119_belt_a.dds",
"Data/UI/item_icons/cloth_119_boots.dds",
"Data/UI/item_icons/cloth_119_boots_a.dds",
"Data/UI/item_icons/cloth_119_chest.dds",
"Data/UI/item_icons/cloth_119_chest_a.dds",
"Data/UI/item_icons/cloth_119_gloves.dds",
"Data/UI/item_icons/cloth_119_gloves_a.dds",
"Data/UI/item_icons/cloth_119_helm.dds",
"Data/UI/item_icons/cloth_119_helm_a.dds",
"Data/UI/item_icons/cloth_119_leggings.dds",
"Data/UI/item_icons/cloth_119_leggings_a.dds",
"Data/UI/item_icons/cloth_119_shoulder.dds",
"Data/UI/item_icons/cloth_119_shoulder_a.dds",
"Data/UI/item_icons/cloth_belt_damaged_01.dds",
"Data/UI/item_icons/cloth_belt_damaged_01_a.dds",
"Data/UI/item_icons/cloth_boots_damaged_01.dds",
"Data/UI/item_icons/cloth_boots_damaged_01_a.dds",
"Data/UI/item_icons/cloth_chest_damaged_01.dds",
"Data/UI/item_icons/cloth_chest_damaged_01_a.dds",
"Data/UI/item_icons/cloth_gloves_damaged_01.dds",
"Data/UI/item_icons/cloth_gloves_damaged_01_a.dds",
"Data/UI/item_icons/cloth_helm_damaged_01.dds",
"Data/UI/item_icons/cloth_helm_damaged_01_a.dds",
"Data/UI/item_icons/cloth_leggings_damaged_01.dds",
"Data/UI/item_icons/cloth_leggings_damaged_01_a.dds",
"Data/UI/item_icons/cloth_shoulders_damaged_01.dds",
"Data/UI/item_icons/cloth_shoulders_damaged_01_a.dds",
"Data/UI/item_icons/collection_of_love_letters.dds",
"Data/UI/item_icons/collection_of_love_letters_a.dds",
"Data/UI/item_icons/composite_cloth.dds",
"Data/UI/item_icons/construct_head.dds",
"Data/UI/item_icons/control_collar.dds",
"Data/UI/item_icons/control_collar_a.dds",
"Data/UI/item_icons/cooked_ham.dds",
"Data/UI/item_icons/cooked_ham_slice.dds",
"Data/UI/item_icons/copper_bars.dds",
"Data/UI/item_icons/copper_bars_a.dds",
"Data/UI/item_icons/coral_shrimp.dds",
"Data/UI/item_icons/coral_shrimp_a.dds",
"Data/UI/item_icons/coral_shrimp_b.dds",
"Data/UI/item_icons/cord_composite.dds",
"Data/UI/item_icons/cord_composite_a.dds",
"Data/UI/item_icons/cord_runewoven.dds",
"Data/UI/item_icons/cord_runewoven_a.dds",
"Data/UI/item_icons/cord_sinew.dds",
"Data/UI/item_icons/cord_steeled.dds",
"Data/UI/item_icons/corrupted_sourcestone.dds",
"Data/UI/item_icons/corrupted_sourcestone_a.dds",
"Data/UI/item_icons/corundum.dds",
"Data/UI/item_icons/cosmic_potion.dds",
"Data/UI/item_icons/cosmic_potion_a.dds",
"Data/UI/item_icons/crackedhoneycomb.dds",
"Data/UI/item_icons/cracked_gossamer.dds",
"Data/UI/item_icons/cracked_honeycomb.dds",
"Data/UI/item_icons/cracked_rib.dds",
"Data/UI/item_icons/cracked_rib_a.dds",
"Data/UI/item_icons/cracked_spyglass.dds",
"Data/UI/item_icons/crackling_potion_bottle.dds",
"Data/UI/item_icons/crackling_potion_bottle_a.dds",
"Data/UI/item_icons/crackling_potion_bottle_b.dds",
"Data/UI/item_icons/crate_arcane_hand.dds",
"Data/UI/item_icons/crate_arcane_hand_a.dds",
"Data/UI/item_icons/crate_arcane_hand_b.dds",
"Data/UI/item_icons/crate_dawn_seekers.dds",
"Data/UI/item_icons/crate_defiants.dds",
"Data/UI/item_icons/crate_guardians.dds",
"Data/UI/item_icons/crate_iron_circle.dds",
"Data/UI/item_icons/crate_iron_circle_a.dds",
"Data/UI/item_icons/crate_iron_circle_b.dds",
"Data/UI/item_icons/crate_of_contraband.dds",
"Data/UI/item_icons/crate_scion_cartel.dds",
"Data/UI/item_icons/crocodile_tooth.dds",
"Data/UI/item_icons/crystalline_potion_bottle.dds",
"Data/UI/item_icons/crystalline_potion_bottle_a.dds",
"Data/UI/item_icons/crystallized_nothing.dds",
"Data/UI/item_icons/cup.dds",
"Data/UI/item_icons/cupcake_01.dds",
"Data/UI/item_icons/cupcake_02.dds",
"Data/UI/item_icons/cupcake_02_a.dds",
"Data/UI/item_icons/curvednagaclaw.dds",
"Data/UI/item_icons/curved_naga_claw.dds",
"Data/UI/item_icons/damascus.dds",
"Data/UI/item_icons/damascus2.dds",
"Data/UI/item_icons/damascus2a.dds",
"Data/UI/item_icons/damascus2b.dds",
"Data/UI/item_icons/daniels_torch.dds",
"Data/UI/item_icons/darach_family_locket.dds",
"Data/UI/item_icons/darkwood_chalice1.dds",
"Data/UI/item_icons/darkwood_chalice1_a.dds",
"Data/UI/item_icons/darkwood_chalice2.dds",
"Data/UI/item_icons/darkwood_chalice2_a.dds",
"Data/UI/item_icons/dark_rock1.dds",
"Data/UI/item_icons/dark_rock2.dds",
"Data/UI/item_icons/dark_rock2_a.dds",
"Data/UI/item_icons/death_1h_sword_mundane.dds",
"Data/UI/item_icons/death_1h_sword_mundane_a.dds",
"Data/UI/item_icons/death_2h_hammer_mundane.dds",
"Data/UI/item_icons/death_2h_hammer_mundane_a.dds",
"Data/UI/item_icons/death_2h_hammer_mundane_b.dds",
"Data/UI/item_icons/death_bow_mundane.dds",
"Data/UI/item_icons/death_bow_mundane_a.dds",
"Data/UI/item_icons/death_offhand_totem_epic.dds",
"Data/UI/item_icons/death_offhand_totem_epic_a.dds",
"Data/UI/item_icons/death_polearm_mundane.dds",
"Data/UI/item_icons/death_polearm_mundane_a.dds",
"Data/UI/item_icons/death_quiver_mundane.dds",
"Data/UI/item_icons/death_quiver_mundane_a.dds",
"Data/UI/item_icons/death_shield_mundane.dds",
"Data/UI/item_icons/death_shield_mundane_a.dds",
"Data/UI/item_icons/death_staff_mundane.dds",
"Data/UI/item_icons/death_staff_mundane_a.dds",
"Data/UI/item_icons/decayed_bone.dds",
"Data/UI/item_icons/decayed_carapace.dds",
"Data/UI/item_icons/deck1.dds",
"Data/UI/item_icons/deck1a.dds",
"Data/UI/item_icons/deck2.dds",
"Data/UI/item_icons/deck3.dds",
"Data/UI/item_icons/deck3a.dds",
"Data/UI/item_icons/deck4.dds",
"Data/UI/item_icons/deck5.dds",
"Data/UI/item_icons/deck5a.dds",
"Data/UI/item_icons/deck5aa.dds",
"Data/UI/item_icons/deck6.dds",
"Data/UI/item_icons/deck7.dds",
"Data/UI/item_icons/deck8.dds",
"Data/UI/item_icons/deck9.dds",
"Data/UI/item_icons/defenders_helm.dds",
"Data/UI/item_icons/defenders_helm_a.dds",
"Data/UI/item_icons/defenders_shield.dds",
"Data/UI/item_icons/demonic_potion_a.dds",
"Data/UI/item_icons/demonic_potion_b.dds",
"Data/UI/item_icons/dented_goblet.dds",
"Data/UI/item_icons/detached_fairy_shadow_a.dds",
"Data/UI/item_icons/detached_fairy_shadow_b.dds",
"Data/UI/item_icons/detached_fairy_shadow_c.dds",
"Data/UI/item_icons/detached_fairy_shadow_d.dds",
"Data/UI/item_icons/devils_head.dds",
"Data/UI/item_icons/diaphanous_lace_accesory.dds",
"Data/UI/item_icons/dirty_socks.dds",
"Data/UI/item_icons/dissection_tools.dds",
"Data/UI/item_icons/dissection_tools_a.dds",
"Data/UI/item_icons/disturbing_stone_image.dds",
"Data/UI/item_icons/double-stitched_composite_cloth.dds",
"Data/UI/item_icons/double-stitched_composite_cloth_a.dds",
"Data/UI/item_icons/double-stitched_composite_cloth_b.dds",
"Data/UI/item_icons/draining_device.dds",
"Data/UI/item_icons/dreadbone_crowbar.dds",
"Data/UI/item_icons/dreadbone_crowbar_a.dds",
"Data/UI/item_icons/dried_parrot_head.dds",
"Data/UI/item_icons/dried_parrot_head2.dds",
"Data/UI/item_icons/dried_parrot_head2_a.dds",
"Data/UI/item_icons/drum_01.dds",
"Data/UI/item_icons/drum_02.dds",
"Data/UI/item_icons/drum_03.dds",
"Data/UI/item_icons/drum_04.dds",
"Data/UI/item_icons/drum_05.dds",
"Data/UI/item_icons/drum_05_a.dds",
"Data/UI/item_icons/drum_06.dds",
"Data/UI/item_icons/drum_06_a.dds",
"Data/UI/item_icons/drum_07.dds",
"Data/UI/item_icons/drum_07_a.dds",
"Data/UI/item_icons/drum_08.dds",
"Data/UI/item_icons/drum_09.dds",
"Data/UI/item_icons/dust1.dds",
"Data/UI/item_icons/dust10.dds",
"Data/UI/item_icons/dust10a.dds",
"Data/UI/item_icons/dust11a.dds",
"Data/UI/item_icons/dust11b.dds",
"Data/UI/item_icons/dust11c.dds",
"Data/UI/item_icons/dust12.dds",
"Data/UI/item_icons/dust12a.dds",
"Data/UI/item_icons/dust12b.dds",
"Data/UI/item_icons/dust12_x.dds",
"Data/UI/item_icons/dust13.dds",
"Data/UI/item_icons/dust13a.dds",
"Data/UI/item_icons/dust13b.dds",
"Data/UI/item_icons/dust14.dds",
"Data/UI/item_icons/dust14a.dds",
"Data/UI/item_icons/dust1a.dds",
"Data/UI/item_icons/dust2.dds",
"Data/UI/item_icons/dust3.dds",
"Data/UI/item_icons/dust3a.dds",
"Data/UI/item_icons/dust4.dds",
"Data/UI/item_icons/dust4a.dds",
"Data/UI/item_icons/dust4b.dds",
"Data/UI/item_icons/dust4_brown.dds",
"Data/UI/item_icons/dust4_grey.dds",
"Data/UI/item_icons/dust4_white.dds",
"Data/UI/item_icons/dust5.dds",
"Data/UI/item_icons/dust6.dds",
"Data/UI/item_icons/dust7.dds",
"Data/UI/item_icons/dust7a.dds",
"Data/UI/item_icons/dust8.dds",
"Data/UI/item_icons/dust8a.dds",
"Data/UI/item_icons/dust8b.dds",
"Data/UI/item_icons/dust9.dds",
"Data/UI/item_icons/dust9a.dds",
"Data/UI/item_icons/dust9b.dds",
"Data/UI/item_icons/dustys_stolen_goods.dds",
"Data/UI/item_icons/dye_a.dds",
"Data/UI/item_icons/dye_a1.dds",
"Data/UI/item_icons/dye_a2.dds",
"Data/UI/item_icons/dye_b.dds",
"Data/UI/item_icons/dye_b1.dds",
"Data/UI/item_icons/dye_b2.dds",
"Data/UI/item_icons/dye_blue.dds",
"Data/UI/item_icons/dye_blue_dark.dds",
"Data/UI/item_icons/dye_blue_light.dds",
"Data/UI/item_icons/dye_brown.dds",
"Data/UI/item_icons/dye_brown_dark.dds",
"Data/UI/item_icons/dye_brown_light.dds",
"Data/UI/item_icons/dye_c.dds",
"Data/UI/item_icons/dye_c1.dds",
"Data/UI/item_icons/dye_c2.dds",
"Data/UI/item_icons/dye_c3.dds",
"Data/UI/item_icons/dye_cyan.dds",
"Data/UI/item_icons/dye_cyan_dark.dds",
"Data/UI/item_icons/dye_cyan_light.dds",
"Data/UI/item_icons/dye_d.dds",
"Data/UI/item_icons/dye_d1.dds",
"Data/UI/item_icons/dye_d2.dds",
"Data/UI/item_icons/dye_d3.dds",
"Data/UI/item_icons/dye_e.dds",
"Data/UI/item_icons/dye_e1.dds",
"Data/UI/item_icons/dye_e2.dds",
"Data/UI/item_icons/dye_e3.dds",
"Data/UI/item_icons/dye_f.dds",
"Data/UI/item_icons/dye_f1.dds",
"Data/UI/item_icons/dye_f2.dds",
"Data/UI/item_icons/dye_f3.dds",
"Data/UI/item_icons/dye_fuscha.dds",
"Data/UI/item_icons/dye_fuscha_dark.dds",
"Data/UI/item_icons/dye_fuscha_light.dds",
"Data/UI/item_icons/dye_g.dds",
"Data/UI/item_icons/dye_g1.dds",
"Data/UI/item_icons/dye_g2.dds",
"Data/UI/item_icons/dye_g3.dds",
"Data/UI/item_icons/dye_green.dds",
"Data/UI/item_icons/dye_green_dark.dds",
"Data/UI/item_icons/dye_green_light.dds",
"Data/UI/item_icons/dye_h.dds",
"Data/UI/item_icons/dye_h1.dds",
"Data/UI/item_icons/dye_h2.dds",
"Data/UI/item_icons/dye_h3.dds",
"Data/UI/item_icons/dye_i.dds",
"Data/UI/item_icons/dye_i1.dds",
"Data/UI/item_icons/dye_i2.dds",
"Data/UI/item_icons/dye_i3.dds",
"Data/UI/item_icons/dye_j.dds",
"Data/UI/item_icons/dye_j1.dds",
"Data/UI/item_icons/dye_j2.dds",
"Data/UI/item_icons/dye_j3.dds",
"Data/UI/item_icons/dye_k.dds",
"Data/UI/item_icons/dye_k1.dds",
"Data/UI/item_icons/dye_k2.dds",
"Data/UI/item_icons/dye_k3.dds",
"Data/UI/item_icons/dye_l.dds",
"Data/UI/item_icons/dye_l1.dds",
"Data/UI/item_icons/dye_l2.dds",
"Data/UI/item_icons/dye_l3.dds",
"Data/UI/item_icons/dye_magenta.dds",
"Data/UI/item_icons/dye_magenta_dark.dds",
"Data/UI/item_icons/dye_magenta_light.dds",
"Data/UI/item_icons/dye_orange.dds",
"Data/UI/item_icons/dye_orange_dark.dds",
"Data/UI/item_icons/dye_orange_light.dds",
"Data/UI/item_icons/dye_purple.dds",
"Data/UI/item_icons/dye_purple_dark.dds",
"Data/UI/item_icons/dye_purple_light.dds",
"Data/UI/item_icons/dye_red.dds",
"Data/UI/item_icons/dye_red_dark.dds",
"Data/UI/item_icons/dye_red_light.dds",
"Data/UI/item_icons/dye_seafoam.dds",
"Data/UI/item_icons/dye_seafoam_dark.dds",
"Data/UI/item_icons/dye_seafoam_light.dds",
"Data/UI/item_icons/dye_skyblue.dds",
"Data/UI/item_icons/dye_skyblue_dark.dds",
"Data/UI/item_icons/dye_skyblue_light.dds",
"Data/UI/item_icons/dye_yellow.dds",
"Data/UI/item_icons/dye_yellowgreen.dds",
"Data/UI/item_icons/dye_yellowgreen_dark.dds",
"Data/UI/item_icons/dye_yellowgreen_light.dds",
"Data/UI/item_icons/dye_yellow_dark.dds",
"Data/UI/item_icons/dye_yellow_light.dds",
"Data/UI/item_icons/earth_shoulder.dds",
"Data/UI/item_icons/earth_shoulder_a.dds",
"Data/UI/item_icons/ebony_flesh.dds",
"Data/UI/item_icons/ebony_flesh_a.dds",
"Data/UI/item_icons/edgemore_family_crest.dds",
"Data/UI/item_icons/edgemore_family_crest_a.dds",
"Data/UI/item_icons/egg_gold.dds",
"Data/UI/item_icons/egg_gold_1.dds",
"Data/UI/item_icons/egg_gold_2.dds",
"Data/UI/item_icons/egg_sac_membrane.dds",
"Data/UI/item_icons/einhorn_spindle.dds",
"Data/UI/item_icons/elven_hair_accesory.dds",
"Data/UI/item_icons/elven_remembance.dds",
"Data/UI/item_icons/embroided_silk_cloth.dds",
"Data/UI/item_icons/empty_bucket.dds",
"Data/UI/item_icons/empty_clay_jug.dds",
"Data/UI/item_icons/empty_jug.dds",
"Data/UI/item_icons/empty_keg.dds",
"Data/UI/item_icons/empyreal_spellspun_silk.dds",
"Data/UI/item_icons/empyreal_spellspun_silk_a.dds",
"Data/UI/item_icons/empyreal_spellspun_silk_b.dds",
"Data/UI/item_icons/enchanting_material_1.dds",
"Data/UI/item_icons/enchanting_material_10.dds",
"Data/UI/item_icons/enchanting_material_10a.dds",
"Data/UI/item_icons/enchanting_material_10b.dds",
"Data/UI/item_icons/enchanting_material_11.dds",
"Data/UI/item_icons/enchanting_material_11a.dds",
"Data/UI/item_icons/enchanting_material_11b.dds",
"Data/UI/item_icons/enchanting_material_12.dds",
"Data/UI/item_icons/enchanting_material_13.dds",
"Data/UI/item_icons/enchanting_material_13a.dds",
"Data/UI/item_icons/enchanting_material_14.dds",
"Data/UI/item_icons/enchanting_material_14a.dds",
"Data/UI/item_icons/enchanting_material_14b.dds",
"Data/UI/item_icons/enchanting_material_14c.dds",
"Data/UI/item_icons/enchanting_material_16.dds",
"Data/UI/item_icons/enchanting_material_16a.dds",
"Data/UI/item_icons/enchanting_material_16b.dds",
"Data/UI/item_icons/enchanting_material_16c.dds",
"Data/UI/item_icons/enchanting_material_17.dds",
"Data/UI/item_icons/enchanting_material_17a.dds",
"Data/UI/item_icons/enchanting_material_17b.dds",
"Data/UI/item_icons/enchanting_material_18.dds",
"Data/UI/item_icons/enchanting_material_18a.dds",
"Data/UI/item_icons/enchanting_material_18b.dds",
"Data/UI/item_icons/enchanting_material_19.dds",
"Data/UI/item_icons/enchanting_material_19a.dds",
"Data/UI/item_icons/enchanting_material_1a.dds",
"Data/UI/item_icons/enchanting_material_1b.dds",
"Data/UI/item_icons/enchanting_material_1_x.dds",
"Data/UI/item_icons/enchanting_material_2.dds",
"Data/UI/item_icons/enchanting_material_20.dds",
"Data/UI/item_icons/enchanting_material_20a.dds",
"Data/UI/item_icons/enchanting_material_20b.dds",
"Data/UI/item_icons/enchanting_material_21.dds",
"Data/UI/item_icons/enchanting_material_21b.dds",
"Data/UI/item_icons/enchanting_material_21c.dds",
"Data/UI/item_icons/enchanting_material_21d.dds",
"Data/UI/item_icons/enchanting_material_21e.dds",
"Data/UI/item_icons/enchanting_material_22.dds",
"Data/UI/item_icons/enchanting_material_22a.dds",
"Data/UI/item_icons/enchanting_material_23.dds",
"Data/UI/item_icons/enchanting_material_23a.dds",
"Data/UI/item_icons/enchanting_material_23b.dds",
"Data/UI/item_icons/enchanting_material_23c.dds",
"Data/UI/item_icons/enchanting_material_24.dds",
"Data/UI/item_icons/enchanting_material_24a.dds",
"Data/UI/item_icons/enchanting_material_25.dds",
"Data/UI/item_icons/enchanting_material_25a.dds",
"Data/UI/item_icons/enchanting_material_26.dds",
"Data/UI/item_icons/enchanting_material_26a.dds",
"Data/UI/item_icons/enchanting_material_26c.dds",
"Data/UI/item_icons/enchanting_material_27.dds",
"Data/UI/item_icons/enchanting_material_27a.dds",
"Data/UI/item_icons/enchanting_material_27b.dds",
"Data/UI/item_icons/enchanting_material_28.dds",
"Data/UI/item_icons/enchanting_material_28a.dds",
"Data/UI/item_icons/enchanting_material_28b.dds",
"Data/UI/item_icons/enchanting_material_28c.dds",
"Data/UI/item_icons/enchanting_material_28d.dds",
"Data/UI/item_icons/enchanting_material_2a.dds",
"Data/UI/item_icons/enchanting_material_2b.dds",
"Data/UI/item_icons/enchanting_material_2_x.dds",
"Data/UI/item_icons/enchanting_material_3.dds",
"Data/UI/item_icons/enchanting_material_3a.dds",
"Data/UI/item_icons/enchanting_material_3b.dds",
"Data/UI/item_icons/enchanting_material_4.dds",
"Data/UI/item_icons/enchanting_material_4a.dds",
"Data/UI/item_icons/enchanting_material_5.dds",
"Data/UI/item_icons/enchanting_material_5a.dds",
"Data/UI/item_icons/enchanting_material_5b.dds",
"Data/UI/item_icons/enchanting_material_6a.dds",
"Data/UI/item_icons/enchanting_material_6b.dds",
"Data/UI/item_icons/enchanting_material_6c.dds",
"Data/UI/item_icons/enchanting_material_7.dds",
"Data/UI/item_icons/enchanting_material_7a.dds",
"Data/UI/item_icons/enchanting_material_8.dds",
"Data/UI/item_icons/enchanting_material_8a.dds",
"Data/UI/item_icons/enchanting_material_8b.dds",
"Data/UI/item_icons/enchanting_material_9.dds",
"Data/UI/item_icons/enchanting_material_9a.dds",
"Data/UI/item_icons/enchanting_material_9b.dds",
"Data/UI/item_icons/enchanting_materia_1b_x.dds",
"Data/UI/item_icons/enchanting_materia_6_x.dds",
"Data/UI/item_icons/ench_material_15.dds",
"Data/UI/item_icons/ench_material_15a.dds",
"Data/UI/item_icons/ench_material_15_b.dds",
"Data/UI/item_icons/ench_material_29.dds",
"Data/UI/item_icons/ench_material_29_a.dds",
"Data/UI/item_icons/ench_material_30.dds",
"Data/UI/item_icons/ench_material_30_a.dds",
"Data/UI/item_icons/energized_thorne_fetter.dds",
"Data/UI/item_icons/epic_potion_cylindrical_0.dds",
"Data/UI/item_icons/epic_potion_cylindrical_1.dds",
"Data/UI/item_icons/epic_potion_cylindrical_2.dds",
"Data/UI/item_icons/epic_potion_cylindrical_3.dds",
"Data/UI/item_icons/epic_potion_cylindrical_4.dds",
"Data/UI/item_icons/epic_potion_cylindrical_5.dds",
"Data/UI/item_icons/epic_potion_cylindrical_6.dds",
"Data/UI/item_icons/epic_potion_rectangular_0.dds",
"Data/UI/item_icons/epic_potion_rectangular_1.dds",
"Data/UI/item_icons/epic_potion_rectangular_2.dds",
"Data/UI/item_icons/epic_potion_rectangular_3.dds",
"Data/UI/item_icons/epic_potion_rectangular_4.dds",
"Data/UI/item_icons/epic_potion_rectangular_4a.dds",
"Data/UI/item_icons/epic_potion_rectangular_5.dds",
"Data/UI/item_icons/epic_potion_rectangular_6.dds",
"Data/UI/item_icons/epic_potion_spherical_0.dds",
"Data/UI/item_icons/epic_potion_spherical_1.dds",
"Data/UI/item_icons/epic_potion_spherical_2.dds",
"Data/UI/item_icons/epic_potion_spherical_3.dds",
"Data/UI/item_icons/epic_potion_spherical_4.dds",
"Data/UI/item_icons/epic_potion_spherical_5.dds",
"Data/UI/item_icons/epic_potion_spherical_6.dds",
"Data/UI/item_icons/ever_burning_ember.dds",
"Data/UI/item_icons/ever_burning_ember_a.dds",
"Data/UI/item_icons/excavated_eth_tech.dds",
"Data/UI/item_icons/excavated_eth_tech_a.dds",
"Data/UI/item_icons/exotic_leather_a.dds",
"Data/UI/item_icons/exotic_leather_a2.dds",
"Data/UI/item_icons/exotic_leather_b.dds",
"Data/UI/item_icons/exotic_leather_b2.dds",
"Data/UI/item_icons/exotic_leather_b3.dds",
"Data/UI/item_icons/expert_hide.dds",
"Data/UI/item_icons/expert_hide_a.dds",
"Data/UI/item_icons/expert_hide_b.dds",
"Data/UI/item_icons/expert_hide_b2.dds",
"Data/UI/item_icons/expert_hide_b3.dds",
"Data/UI/item_icons/extra_bowstring.dds",
"Data/UI/item_icons/faded_leaf.dds",
"Data/UI/item_icons/fading_cinder.dds",
"Data/UI/item_icons/faer_embers.dds",
"Data/UI/item_icons/faer_embers_a.dds",
"Data/UI/item_icons/fairie_hairstring.dds",
"Data/UI/item_icons/family_heirloom1.dds",
"Data/UI/item_icons/family_heirloom2.dds",
"Data/UI/item_icons/family_locket.dds",
"Data/UI/item_icons/feather1.dds",
"Data/UI/item_icons/feathered_cravat.dds",
"Data/UI/item_icons/feathered_cravat_a.dds",
"Data/UI/item_icons/feathers2.dds",
"Data/UI/item_icons/feathers2a.dds",
"Data/UI/item_icons/feathers2b.dds",
"Data/UI/item_icons/feathers2c.dds",
"Data/UI/item_icons/femur.dds",
"Data/UI/item_icons/fibbiontic_air_bladder.dds",
"Data/UI/item_icons/fieldstone.dds",
"Data/UI/item_icons/fieldstone_a.dds",
"Data/UI/item_icons/fieldstone_b.dds",
"Data/UI/item_icons/fiery_potion_bottle.dds",
"Data/UI/item_icons/fiery_potion_bottle_a.dds",
"Data/UI/item_icons/fine_silken_bolt.dds",
"Data/UI/item_icons/fine_silken_tassel.dds",
"Data/UI/item_icons/fine_silken_tassel_a.dds",
"Data/UI/item_icons/fine_silken_tassel_b.dds",
"Data/UI/item_icons/fine_stone_engraving.dds",
"Data/UI/item_icons/fine_stone_engraving_a.dds",
"Data/UI/item_icons/finned_isopod.dds",
"Data/UI/item_icons/finned_isopod_a.dds",
"Data/UI/item_icons/fish1.dds",
"Data/UI/item_icons/fish10.dds",
"Data/UI/item_icons/fish11.dds",
"Data/UI/item_icons/fish11a.dds",
"Data/UI/item_icons/fish11b.dds",
"Data/UI/item_icons/fish12.dds",
"Data/UI/item_icons/fish12a.dds",
"Data/UI/item_icons/fish13.dds",
"Data/UI/item_icons/fish13a.dds",
"Data/UI/item_icons/fish14.dds",
"Data/UI/item_icons/fish14a.dds",
"Data/UI/item_icons/fish15.dds",
"Data/UI/item_icons/fish15a.dds",
"Data/UI/item_icons/fish15b.dds",
"Data/UI/item_icons/fish16.dds",
"Data/UI/item_icons/fish17.dds",
"Data/UI/item_icons/fish17a.dds",
"Data/UI/item_icons/fish18.dds",
"Data/UI/item_icons/fish18a.dds",
"Data/UI/item_icons/fish19.dds",
"Data/UI/item_icons/fish1a.dds",
"Data/UI/item_icons/fish2.dds",
"Data/UI/item_icons/fish20.dds",
"Data/UI/item_icons/fish20z.dds",
"Data/UI/item_icons/fish21.dds",
"Data/UI/item_icons/fish22.dds",
"Data/UI/item_icons/fish22a.dds",
"Data/UI/item_icons/fish22z.dds",
"Data/UI/item_icons/fish23.dds",
"Data/UI/item_icons/fish24.dds",
"Data/UI/item_icons/fish24b.dds",
"Data/UI/item_icons/fish25.dds",
"Data/UI/item_icons/fish25a.dds",
"Data/UI/item_icons/fish26a.dds",
"Data/UI/item_icons/fish26b.dds",
"Data/UI/item_icons/fish27.dds",
"Data/UI/item_icons/fish27a.dds",
"Data/UI/item_icons/fish27b.dds",
"Data/UI/item_icons/fish27n.dds",
"Data/UI/item_icons/fish28.dds",
"Data/UI/item_icons/fish29.dds",
"Data/UI/item_icons/fish3.dds",
"Data/UI/item_icons/fish4.dds",
"Data/UI/item_icons/fish4a.dds",
"Data/UI/item_icons/fish4b.dds",
"Data/UI/item_icons/fish5.dds",
"Data/UI/item_icons/fish5a.dds",
"Data/UI/item_icons/fish6.dds",
"Data/UI/item_icons/fish7.dds",
"Data/UI/item_icons/fish7a.dds",
"Data/UI/item_icons/fish8.dds",
"Data/UI/item_icons/fish9.dds",
"Data/UI/item_icons/fish9a.dds",
"Data/UI/item_icons/fishing_pole1.dds",
"Data/UI/item_icons/fishing_pole10.dds",
"Data/UI/item_icons/fishing_pole10a.dds",
"Data/UI/item_icons/fishing_pole2.dds",
"Data/UI/item_icons/fishing_pole3.dds",
"Data/UI/item_icons/fishing_pole4.dds",
"Data/UI/item_icons/fishing_pole5.dds",
"Data/UI/item_icons/fishing_pole6.dds",
"Data/UI/item_icons/fishing_pole7.dds",
"Data/UI/item_icons/fishing_pole8.dds",
"Data/UI/item_icons/fishing_pole8a.dds",
"Data/UI/item_icons/fishing_pole9.dds",
"Data/UI/item_icons/fishing_pole9a.dds",
"Data/UI/item_icons/fissure_soil_sample.dds",
"Data/UI/item_icons/flag_arcane_hand.dds",
"Data/UI/item_icons/flag_arcane_hand_a.dds",
"Data/UI/item_icons/flag_dawn_seeker.dds",
"Data/UI/item_icons/flag_dawn_seeker_a.dds",
"Data/UI/item_icons/flag_iron_circle.dds",
"Data/UI/item_icons/flag_iron_circle_a.dds",
"Data/UI/item_icons/flag_scion_cartel.dds",
"Data/UI/item_icons/flag_scion_cartel_a.dds",
"Data/UI/item_icons/flag_scion_cartel_b.dds",
"Data/UI/item_icons/flag_scion_cartel_c.dds",
"Data/UI/item_icons/flag_smith_haven.dds",
"Data/UI/item_icons/flag_smith_haven_a.dds",
"Data/UI/item_icons/flaming_torch.dds",
"Data/UI/item_icons/flare_defiant.dds",
"Data/UI/item_icons/flare_defiant_a.dds",
"Data/UI/item_icons/flare_defiant_b.dds",
"Data/UI/item_icons/flare_guardian.dds",
"Data/UI/item_icons/flare_guardian_a.dds",
"Data/UI/item_icons/flint_and_tinder.dds",
"Data/UI/item_icons/fluid_injector.dds",
"Data/UI/item_icons/flute_01.dds",
"Data/UI/item_icons/flute_02.dds",
"Data/UI/item_icons/flute_03.dds",
"Data/UI/item_icons/flute_04.dds",
"Data/UI/item_icons/flute_05.dds",
"Data/UI/item_icons/flute_06.dds",
"Data/UI/item_icons/flute_07.dds",
"Data/UI/item_icons/flute_07_a.dds",
"Data/UI/item_icons/flute_08.dds",
"Data/UI/item_icons/flute_09.dds",
"Data/UI/item_icons/flute_10.dds",
"Data/UI/item_icons/flute_10_a.dds",
"Data/UI/item_icons/focus_defiant_01.dds",
"Data/UI/item_icons/focus_defiant_02.dds",
"Data/UI/item_icons/focus_defiant_03.dds",
"Data/UI/item_icons/focus_defiant_03_a.dds",
"Data/UI/item_icons/focus_defiant_04.dds",
"Data/UI/item_icons/focus_defiant_05.dds",
"Data/UI/item_icons/focus_defiant_06.dds",
"Data/UI/item_icons/focus_defiant_06_a.dds",
"Data/UI/item_icons/focus_defiant_07.dds",
"Data/UI/item_icons/focus_defiant_07_a.dds",
"Data/UI/item_icons/focus_defiant_08.dds",
"Data/UI/item_icons/focus_defiant_09.dds",
"Data/UI/item_icons/focus_defiant_09_a.dds",
"Data/UI/item_icons/focus_defiant_09_b.dds",
"Data/UI/item_icons/focus_defiant_10.dds",
"Data/UI/item_icons/focus_defiant_10_a.dds",
"Data/UI/item_icons/focus_defiant_10_b.dds",
"Data/UI/item_icons/focus_defiant_11.dds",
"Data/UI/item_icons/focus_defiant_11_a.dds",
"Data/UI/item_icons/focus_defiant_12_a.dds",
"Data/UI/item_icons/focus_defiant_12_b.dds",
"Data/UI/item_icons/focus_guardian_01.dds",
"Data/UI/item_icons/focus_guardian_01_a.dds",
"Data/UI/item_icons/focus_guardian_02.dds",
"Data/UI/item_icons/focus_guardian_03.dds",
"Data/UI/item_icons/focus_guardian_04.dds",
"Data/UI/item_icons/focus_guardian_04_a.dds",
"Data/UI/item_icons/focus_guardian_05.dds",
"Data/UI/item_icons/focus_guardian_06.dds",
"Data/UI/item_icons/focus_guardian_06_a.dds",
"Data/UI/item_icons/focus_guardian_07.dds",
"Data/UI/item_icons/focus_guardian_07_a.dds",
"Data/UI/item_icons/focus_guardian_07_b.dds",
"Data/UI/item_icons/focus_guardian_08.dds",
"Data/UI/item_icons/focus_guardian_08_a.dds",
"Data/UI/item_icons/focus_guardian_09.dds",
"Data/UI/item_icons/focus_guardian_09_a.dds",
"Data/UI/item_icons/focus_guardian_10.dds",
"Data/UI/item_icons/focus_guardian_10_a.dds",
"Data/UI/item_icons/focus_guardian_10_b.dds",
"Data/UI/item_icons/focus_guardian_11.dds",
"Data/UI/item_icons/focus_guardian_11_a.dds",
"Data/UI/item_icons/focus_guardian_12.dds",
"Data/UI/item_icons/focus_guardian_12_a.dds",
"Data/UI/item_icons/folly_of_the_misled.dds",
"Data/UI/item_icons/food1.dds",
"Data/UI/item_icons/food2.dds",
"Data/UI/item_icons/food3.dds",
"Data/UI/item_icons/food4.dds",
"Data/UI/item_icons/food5.dds",
"Data/UI/item_icons/food6.dds",
"Data/UI/item_icons/food7.dds",
"Data/UI/item_icons/food_10.dds",
"Data/UI/item_icons/food_11.dds",
"Data/UI/item_icons/food_11a.dds",
"Data/UI/item_icons/food_12.dds",
"Data/UI/item_icons/food_13.dds",
"Data/UI/item_icons/food_13a.dds",
"Data/UI/item_icons/food_14.dds",
"Data/UI/item_icons/food_14a.dds",
"Data/UI/item_icons/food_14b.dds",
"Data/UI/item_icons/food_15.dds",
"Data/UI/item_icons/food_16.dds",
"Data/UI/item_icons/food_17.dds",
"Data/UI/item_icons/food_18.dds",
"Data/UI/item_icons/food_19.dds",
"Data/UI/item_icons/food_20.dds",
"Data/UI/item_icons/food_8.dds",
"Data/UI/item_icons/food_9.dds",
"Data/UI/item_icons/food_9a.dds",
"Data/UI/item_icons/forged_deed.dds",
"Data/UI/item_icons/fouled_finger_bone.dds",
"Data/UI/item_icons/fouled_finger_bone_a.dds",
"Data/UI/item_icons/frank_birlen_head.dds",
"Data/UI/item_icons/freemarch_rabbit.dds",
"Data/UI/item_icons/freemarch_rabbit_a.dds",
"Data/UI/item_icons/furrytail1.dds",
"Data/UI/item_icons/furrytail2.dds",
"Data/UI/item_icons/furrytail2a.dds",
"Data/UI/item_icons/fur_black.dds",
"Data/UI/item_icons/fur_brown.dds",
"Data/UI/item_icons/fur_grey.dds",
"Data/UI/item_icons/fur_grey_a.dds",
"Data/UI/item_icons/fur_redbrown.dds",
"Data/UI/item_icons/fur_redbrown_a.dds",
"Data/UI/item_icons/fur_tan.dds",
"Data/UI/item_icons/fur_white.dds",
"Data/UI/item_icons/fur_x_1.dds",
"Data/UI/item_icons/fur_x_1_a.dds",
"Data/UI/item_icons/gem1.dds",
"Data/UI/item_icons/gem1a.dds",
"Data/UI/item_icons/gem1b.dds",
"Data/UI/item_icons/gem2.dds",
"Data/UI/item_icons/gem3.dds",
"Data/UI/item_icons/gem4.dds",
"Data/UI/item_icons/gem4a.dds",
"Data/UI/item_icons/gem5.dds",
"Data/UI/item_icons/gem5a.dds",
"Data/UI/item_icons/gem6.dds",
"Data/UI/item_icons/generic_ability_032.dds",
"Data/UI/item_icons/geode.dds",
"Data/UI/item_icons/ghost_horse.dds",
"Data/UI/item_icons/ghost_horse2.dds",
"Data/UI/item_icons/ghost_horse2_a.dds",
"Data/UI/item_icons/ghost_horse2_b.dds",
"Data/UI/item_icons/giftbox1.dds",
"Data/UI/item_icons/giftbox10.dds",
"Data/UI/item_icons/giftbox1a.dds",
"Data/UI/item_icons/giftbox2.dds",
"Data/UI/item_icons/giftbox2a.dds",
"Data/UI/item_icons/giftbox3.dds",
"Data/UI/item_icons/giftbox3a.dds",
"Data/UI/item_icons/giftbox3b.dds",
"Data/UI/item_icons/giftbox3c.dds",
"Data/UI/item_icons/giftbox4.dds",
"Data/UI/item_icons/giftbox5.dds",
"Data/UI/item_icons/giftbox6.dds",
"Data/UI/item_icons/giftbox6a.dds",
"Data/UI/item_icons/giftbox7.dds",
"Data/UI/item_icons/giftbox7a.dds",
"Data/UI/item_icons/giftbox8.dds",
"Data/UI/item_icons/giftbox9.dds",
"Data/UI/item_icons/glassy_fang.dds",
"Data/UI/item_icons/glassy_fang_a.dds",
"Data/UI/item_icons/glazed_eyestalk.dds",
"Data/UI/item_icons/gloamseed1.dds",
"Data/UI/item_icons/gloamseed1_a.dds",
"Data/UI/item_icons/gloamseed2.dds",
"Data/UI/item_icons/gloamseed3.dds",
"Data/UI/item_icons/glowing_ash.dds",
"Data/UI/item_icons/glowing_ash_2.dds",
"Data/UI/item_icons/goblet_01.dds",
"Data/UI/item_icons/goblet_02.dds",
"Data/UI/item_icons/goblet_03.dds",
"Data/UI/item_icons/goblet_04.dds",
"Data/UI/item_icons/goblet_04_a.dds",
"Data/UI/item_icons/goblin_bait.dds",
"Data/UI/item_icons/goblin_bait_a.dds",
"Data/UI/item_icons/goblin_boss_head.dds",
"Data/UI/item_icons/goblin_key.dds",
"Data/UI/item_icons/goblin_necklace.dds",
"Data/UI/item_icons/goblin_sketches.dds",
"Data/UI/item_icons/golden_goblet.dds",
"Data/UI/item_icons/gold_bars.dds",
"Data/UI/item_icons/gold_bars_a.dds",
"Data/UI/item_icons/goo1.dds",
"Data/UI/item_icons/goo2.dds",
"Data/UI/item_icons/goo3.dds",
"Data/UI/item_icons/goo4.dds",
"Data/UI/item_icons/goo4a.dds",
"Data/UI/item_icons/goo5.dds",
"Data/UI/item_icons/goo7.dds",
"Data/UI/item_icons/goo7a.dds",
"Data/UI/item_icons/goo7b.dds",
"Data/UI/item_icons/granitewood_lumber.dds",
"Data/UI/item_icons/granitewood_timber.dds",
"Data/UI/item_icons/granitewood_timber_a.dds",
"Data/UI/item_icons/graveyard_dust.dds",
"Data/UI/item_icons/greaterpotion_rectangular_0.dds",
"Data/UI/item_icons/greaterpotion_rectangular_1.dds",
"Data/UI/item_icons/greaterpotion_rectangular_2.dds",
"Data/UI/item_icons/greaterpotion_rectangular_3.dds",
"Data/UI/item_icons/greaterpotion_rectangular_4.dds",
"Data/UI/item_icons/greaterpotion_rectangular_5.dds",
"Data/UI/item_icons/greaterpotion_rectangular_6.dds",
"Data/UI/item_icons/greater_air_01.dds",
"Data/UI/item_icons/greater_air_02.dds",
"Data/UI/item_icons/greater_air_02_a.dds",
"Data/UI/item_icons/greater_air_02_b.dds",
"Data/UI/item_icons/greater_air_02_c.dds",
"Data/UI/item_icons/greater_air_03.dds",
"Data/UI/item_icons/greater_air_04.dds",
"Data/UI/item_icons/greater_air_04_a.dds",
"Data/UI/item_icons/greater_air_05.dds",
"Data/UI/item_icons/greater_death_01.dds",
"Data/UI/item_icons/greater_death_02.dds",
"Data/UI/item_icons/greater_death_03.dds",
"Data/UI/item_icons/greater_death_04.dds",
"Data/UI/item_icons/greater_death_05.dds",
"Data/UI/item_icons/greater_death_05_a.dds",
"Data/UI/item_icons/greater_earth_01.dds",
"Data/UI/item_icons/greater_earth_01_a.dds",
"Data/UI/item_icons/greater_earth_02.dds",
"Data/UI/item_icons/greater_earth_02_a.dds",
"Data/UI/item_icons/greater_earth_02_b.dds",
"Data/UI/item_icons/greater_earth_03_a.dds",
"Data/UI/item_icons/greater_earth_03_b.dds",
"Data/UI/item_icons/greater_earth_04.dds",
"Data/UI/item_icons/greater_earth_05.dds",
"Data/UI/item_icons/greater_earth_06.dds",
"Data/UI/item_icons/greater_earth_06_a.dds",
"Data/UI/item_icons/greater_earth_07.dds",
"Data/UI/item_icons/greater_earth_08.dds",
"Data/UI/item_icons/greater_earth_08_a.dds",
"Data/UI/item_icons/greater_earth_09.dds",
"Data/UI/item_icons/greater_earth_09_a.dds",
"Data/UI/item_icons/greater_earth_09_b.dds",
"Data/UI/item_icons/greater_earth_10.dds",
"Data/UI/item_icons/greater_earth_10_a.dds",
"Data/UI/item_icons/greater_fire_01.dds",
"Data/UI/item_icons/greater_fire_01_a.dds",
"Data/UI/item_icons/greater_fire_02.dds",
"Data/UI/item_icons/greater_fire_02_a.dds",
"Data/UI/item_icons/greater_fire_03.dds",
"Data/UI/item_icons/greater_fire_03_a.dds",
"Data/UI/item_icons/greater_fire_04.dds",
"Data/UI/item_icons/greater_fire_04_a.dds",
"Data/UI/item_icons/greater_fire_04_b.dds",
"Data/UI/item_icons/greater_fire_05.dds",
"Data/UI/item_icons/greater_life_01.dds",
"Data/UI/item_icons/greater_life_02.dds",
"Data/UI/item_icons/greater_life_03.dds",
"Data/UI/item_icons/greater_life_04.dds",
"Data/UI/item_icons/greater_life_05.dds",
"Data/UI/item_icons/greater_potion_cylindrical_0.dds",
"Data/UI/item_icons/greater_potion_cylindrical_1.dds",
"Data/UI/item_icons/greater_potion_cylindrical_2.dds",
"Data/UI/item_icons/greater_potion_cylindrical_3.dds",
"Data/UI/item_icons/greater_potion_cylindrical_4.dds",
"Data/UI/item_icons/greater_potion_cylindrical_5.dds",
"Data/UI/item_icons/greater_potion_cylindrical_6.dds",
"Data/UI/item_icons/greater_potion_spherical_0.dds",
"Data/UI/item_icons/greater_potion_spherical_1.dds",
"Data/UI/item_icons/greater_potion_spherical_2.dds",
"Data/UI/item_icons/greater_potion_spherical_3.dds",
"Data/UI/item_icons/greater_potion_spherical_4.dds",
"Data/UI/item_icons/greater_potion_spherical_5.dds",
"Data/UI/item_icons/greater_potion_spherical_6.dds",
"Data/UI/item_icons/greater_potion_spherical_a_0.dds",
"Data/UI/item_icons/greater_potion_spherical_a_1.dds",
"Data/UI/item_icons/greater_potion_spherical_a_2.dds",
"Data/UI/item_icons/greater_potion_spherical_a_3.dds",
"Data/UI/item_icons/greater_potion_spherical_a_4.dds",
"Data/UI/item_icons/greater_potion_spherical_a_5.dds",
"Data/UI/item_icons/greater_potion_spherical_a_6.dds",
"Data/UI/item_icons/greater_water_01.dds",
"Data/UI/item_icons/greater_water_02.dds",
"Data/UI/item_icons/greater_water_03.dds",
"Data/UI/item_icons/greater_water_04.dds",
"Data/UI/item_icons/greater_water_05.dds",
"Data/UI/item_icons/green_flesh.dds",
"Data/UI/item_icons/green_flesh_a.dds",
"Data/UI/item_icons/Gregory_last_request.dds",
"Data/UI/item_icons/grinding_stone1.dds",
"Data/UI/item_icons/grinding_stone2.dds",
"Data/UI/item_icons/grinding_stone3.dds",
"Data/UI/item_icons/grinding_stone4.dds",
"Data/UI/item_icons/grinding_stone4a.dds",
"Data/UI/item_icons/grinding_stone4b.dds",
"Data/UI/item_icons/grinding_stone5.dds",
"Data/UI/item_icons/grinding_stone5a.dds",
"Data/UI/item_icons/grips_01.dds",
"Data/UI/item_icons/grips_01_a.dds",
"Data/UI/item_icons/grips_02.dds",
"Data/UI/item_icons/grips_02_a.dds",
"Data/UI/item_icons/gritty_wax.dds",
"Data/UI/item_icons/growing_energy_01.dds",
"Data/UI/item_icons/growing_energy_02.dds",
"Data/UI/item_icons/growing_energy_03.dds",
"Data/UI/item_icons/growing_energy_03_a.dds",
"Data/UI/item_icons/grubby_ivory_dice.dds",
"Data/UI/item_icons/guardians_badge.dds",
"Data/UI/item_icons/guardian_mount_epic.dds",
"Data/UI/item_icons/guardian_mount_epic_a.dds",
"Data/UI/item_icons/guardian_mount_normal.dds",
"Data/UI/item_icons/guardian_mount_normal_a.dds",
"Data/UI/item_icons/GuildCharter_Defiants.dds",
"Data/UI/item_icons/GuildCharter_Guardians.dds",
"Data/UI/item_icons/gyle_fiddle.dds",
"Data/UI/item_icons/half_eaten_chocolate.dds",
"Data/UI/item_icons/handful_of_hay.dds",
"Data/UI/item_icons/hanna_dern_head.dds",
"Data/UI/item_icons/head_of_slain_young_man.dds",
"Data/UI/item_icons/heartstone.dds",
"Data/UI/item_icons/heart_shaped_potion_bottle.dds",
"Data/UI/item_icons/heart_shaped_potion_bottle_a.dds",
"Data/UI/item_icons/heavytinbar.dds",
"Data/UI/item_icons/heavytinbar_a.dds",
"Data/UI/item_icons/heavy_hide.dds",
"Data/UI/item_icons/heavy_hide_a.dds",
"Data/UI/item_icons/heavy_hide_a_1.dds",
"Data/UI/item_icons/heavy_hide_b.dds",
"Data/UI/item_icons/helenas_flare.dds",
"Data/UI/item_icons/helenas_flare_a.dds",
"Data/UI/item_icons/helenas_flare_b.dds",
"Data/UI/item_icons/helenas_flare_c.dds",
"Data/UI/item_icons/helm_firelord.dds",
"Data/UI/item_icons/helm_firelord_a.dds",
"Data/UI/item_icons/helm_maggot.dds",
"Data/UI/item_icons/helm_maggot_a.dds",
"Data/UI/item_icons/helm_mouse.dds",
"Data/UI/item_icons/helm_nature.dds",
"Data/UI/item_icons/helm_seacritter.dds",
"Data/UI/item_icons/helm_skeletal.dds",
"Data/UI/item_icons/helm_spider.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_0.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_1.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_2.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_3.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_4.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_5.dds",
"Data/UI/item_icons/heroic_potion_cylindrical_6.dds",
"Data/UI/item_icons/heroic_potion_rectangular_0.dds",
"Data/UI/item_icons/heroic_potion_rectangular_1.dds",
"Data/UI/item_icons/heroic_potion_rectangular_2.dds",
"Data/UI/item_icons/heroic_potion_rectangular_3.dds",
"Data/UI/item_icons/heroic_potion_rectangular_4.dds",
"Data/UI/item_icons/heroic_potion_rectangular_5.dds",
"Data/UI/item_icons/heroic_potion_rectangular_6.dds",
"Data/UI/item_icons/heroic_potion_spherical_0.dds",
"Data/UI/item_icons/heroic_potion_spherical_1.dds",
"Data/UI/item_icons/heroic_potion_spherical_2.dds",
"Data/UI/item_icons/heroic_potion_spherical_3.dds",
"Data/UI/item_icons/heroic_potion_spherical_4.dds",
"Data/UI/item_icons/heroic_potion_spherical_5.dds",
"Data/UI/item_icons/heroic_potion_spherical_6.dds",
"Data/UI/item_icons/hertum_attack_plans.dds",
"Data/UI/item_icons/highsilverbar.dds",
"Data/UI/item_icons/highsilverbar_a.dds",
"Data/UI/item_icons/hill_giant_head.dds",
"Data/UI/item_icons/hill_giant_head_a.dds",
"Data/UI/item_icons/hk_chain_belt.dds",
"Data/UI/item_icons/hk_chain_belt_a.dds",
"Data/UI/item_icons/hk_chain_boots.dds",
"Data/UI/item_icons/hk_chain_boots_a.dds",
"Data/UI/item_icons/hk_chain_chest.dds",
"Data/UI/item_icons/hk_chain_chest_a.dds",
"Data/UI/item_icons/hk_chain_gloves.dds",
"Data/UI/item_icons/hk_chain_gloves_a.dds",
"Data/UI/item_icons/hk_chain_helm.dds",
"Data/UI/item_icons/hk_chain_helm_a.dds",
"Data/UI/item_icons/hk_chain_leggings.dds",
"Data/UI/item_icons/hk_chain_leggings_a.dds",
"Data/UI/item_icons/hk_chain_shoulders.dds",
"Data/UI/item_icons/hk_chain_shoulders_a.dds",
"Data/UI/item_icons/hk_cloth_belt.dds",
"Data/UI/item_icons/hk_cloth_belt_a.dds",
"Data/UI/item_icons/hk_cloth_boots.dds",
"Data/UI/item_icons/hk_cloth_boots_a.dds",
"Data/UI/item_icons/hk_cloth_boots_b.dds",
"Data/UI/item_icons/hk_cloth_chest.dds",
"Data/UI/item_icons/hk_cloth_chest_a.dds",
"Data/UI/item_icons/hk_cloth_gloves.dds",
"Data/UI/item_icons/hk_cloth_gloves_a.dds",
"Data/UI/item_icons/hk_cloth_gloves_b.dds",
"Data/UI/item_icons/hk_cloth_helm.dds",
"Data/UI/item_icons/hk_cloth_helm_a.dds",
"Data/UI/item_icons/hk_cloth_helm_b.dds",
"Data/UI/item_icons/hk_cloth_helm_c.dds",
"Data/UI/item_icons/hk_cloth_leggings.dds",
"Data/UI/item_icons/hk_cloth_leggings_a.dds",
"Data/UI/item_icons/hk_cloth_shoulder.dds",
"Data/UI/item_icons/hk_cloth_shoulder_a.dds",
"Data/UI/item_icons/hk_leather_belt.dds",
"Data/UI/item_icons/hk_leather_belt_a.dds",
"Data/UI/item_icons/hk_leather_belt_b.dds",
"Data/UI/item_icons/hk_leather_boots.dds",
"Data/UI/item_icons/hk_leather_boots_a.dds",
"Data/UI/item_icons/hk_leather_boots_b.dds",
"Data/UI/item_icons/hk_leather_boots_c.dds",
"Data/UI/item_icons/hk_leather_chest.dds",
"Data/UI/item_icons/hk_leather_chest_a.dds",
"Data/UI/item_icons/hk_leather_gloves.dds",
"Data/UI/item_icons/hk_leather_gloves_a.dds",
"Data/UI/item_icons/hk_leather_helm.dds",
"Data/UI/item_icons/hk_leather_helm_a.dds",
"Data/UI/item_icons/hk_leather_helm_b.dds",
"Data/UI/item_icons/hk_leather_helm_c.dds",
"Data/UI/item_icons/hk_leather_leggings.dds",
"Data/UI/item_icons/hk_leather_leggings_a.dds",
"Data/UI/item_icons/hk_leather_shoulder.dds",
"Data/UI/item_icons/hk_leather_shoulder_a.dds",
"Data/UI/item_icons/hk_plate_belt.dds",
"Data/UI/item_icons/hk_plate_belt_a.dds",
"Data/UI/item_icons/hk_plate_boots.dds",
"Data/UI/item_icons/hk_plate_boots_a.dds",
"Data/UI/item_icons/hk_plate_boots_b.dds",
"Data/UI/item_icons/hk_plate_chest.dds",
"Data/UI/item_icons/hk_plate_chest_a.dds",
"Data/UI/item_icons/hk_plate_gloves.dds",
"Data/UI/item_icons/hk_plate_gloves_a.dds",
"Data/UI/item_icons/hk_plate_helm.dds",
"Data/UI/item_icons/hk_plate_helm_a.dds",
"Data/UI/item_icons/hk_plate_helm_b.dds",
"Data/UI/item_icons/hk_plate_leggings.dds",
"Data/UI/item_icons/hk_plate_leggings_a.dds",
"Data/UI/item_icons/hk_plate_shoulders.dds",
"Data/UI/item_icons/hk_plate_shoulders_a.dds",
"Data/UI/item_icons/holy_potion.dds",
"Data/UI/item_icons/holy_potion_a.dds",
"Data/UI/item_icons/honey_comb.dds",
"Data/UI/item_icons/honey_comb_a.dds",
"Data/UI/item_icons/horn_01.dds",
"Data/UI/item_icons/horn_02.dds",
"Data/UI/item_icons/horn_03.dds",
"Data/UI/item_icons/horn_04.dds",
"Data/UI/item_icons/horn_05.dds",
"Data/UI/item_icons/horn_06.dds",
"Data/UI/item_icons/horn_06_a.dds",
"Data/UI/item_icons/horn_07.dds",
"Data/UI/item_icons/horn_07_a.dds",
"Data/UI/item_icons/horn_08.dds",
"Data/UI/item_icons/horn_09.dds",
"Data/UI/item_icons/horn_09_a.dds",
"Data/UI/item_icons/horn_10.dds",
"Data/UI/item_icons/horn_10_a.dds",
"Data/UI/item_icons/horsehoof1.dds",
"Data/UI/item_icons/hughs_hammer.dds",
"Data/UI/item_icons/ice_core.dds",
"Data/UI/item_icons/ice_potion.dds",
"Data/UI/item_icons/idol_protectorate_foot.dds",
"Data/UI/item_icons/illustrated_icon_of_mariel.dds",
"Data/UI/item_icons/indecipherable_parchment.dds",
"Data/UI/item_icons/infused_witchweave_cloth_a.dds",
"Data/UI/item_icons/infused_witchweave_cloth_b.dds",
"Data/UI/item_icons/infused_witchweave_cloth_z.dds",
"Data/UI/item_icons/inlaid_sextant.dds",
"Data/UI/item_icons/inlaid_sextant_a.dds",
"Data/UI/item_icons/insoles_01.dds",
"Data/UI/item_icons/insoles_02.dds",
"Data/UI/item_icons/insoles_03.dds",
"Data/UI/item_icons/insoles_03_a.dds",
"Data/UI/item_icons/insoles_04_a.dds",
"Data/UI/item_icons/insoles_04_b.dds",
"Data/UI/item_icons/insoles_04_c.dds",
"Data/UI/item_icons/intact_shambler_eye.dds",
"Data/UI/item_icons/intact_shell_foliage.dds",
"Data/UI/item_icons/ironhide.dds",
"Data/UI/item_icons/iron_bars.dds",
"Data/UI/item_icons/iron_chain.dds",
"Data/UI/item_icons/iron_chain_a.dds",
"Data/UI/item_icons/ivory_comb.dds",
"Data/UI/item_icons/jagged_sourcestone.dds",
"Data/UI/item_icons/jagged_sourcestone_a.dds",
"Data/UI/item_icons/jagged_sourcestone_b.dds",
"Data/UI/item_icons/jagged_sourcestone_c.dds",
"Data/UI/item_icons/janina_bouquet.dds",
"Data/UI/item_icons/janina_bouquet_a.dds",
"Data/UI/item_icons/jawbone.dds",
"Data/UI/item_icons/jeweled_gold_goblet.dds",
"Data/UI/item_icons/jeweled_gold_goblet_a.dds",
"Data/UI/item_icons/jeweled_scale.dds",
"Data/UI/item_icons/journal_of_gerard.dds",
"Data/UI/item_icons/keg_of_ale.dds",
"Data/UI/item_icons/key01.dds",
"Data/UI/item_icons/key02.dds",
"Data/UI/item_icons/key03.dds",
"Data/UI/item_icons/key04.dds",
"Data/UI/item_icons/key04a.dds",
"Data/UI/item_icons/key05.dds",
"Data/UI/item_icons/key06.dds",
"Data/UI/item_icons/key07.dds",
"Data/UI/item_icons/key07a.dds",
"Data/UI/item_icons/key08.dds",
"Data/UI/item_icons/key09.dds",
"Data/UI/item_icons/key10.dds",
"Data/UI/item_icons/key10b.dds",
"Data/UI/item_icons/key10c.dds",
"Data/UI/item_icons/key_x_1.dds",
"Data/UI/item_icons/key_x_10.dds",
"Data/UI/item_icons/key_x_10_a.dds",
"Data/UI/item_icons/key_x_10_b.dds",
"Data/UI/item_icons/key_x_2.dds",
"Data/UI/item_icons/key_x_2_a.dds",
"Data/UI/item_icons/key_x_3.dds",
"Data/UI/item_icons/key_x_4.dds",
"Data/UI/item_icons/key_x_5.dds",
"Data/UI/item_icons/key_x_6.dds",
"Data/UI/item_icons/key_x_6_a.dds",
"Data/UI/item_icons/key_x_7.dds",
"Data/UI/item_icons/key_x_7_a.dds",
"Data/UI/item_icons/key_x_8.dds",
"Data/UI/item_icons/key_x_9.dds",
"Data/UI/item_icons/kobold_hex_doll.dds",
"Data/UI/item_icons/krins_basket.dds",
"Data/UI/item_icons/laquered_sagebrush_lumber.dds",
"Data/UI/item_icons/large_meaty_tongue.dds",
"Data/UI/item_icons/larias_locket.dds",
"Data/UI/item_icons/larias_locket_a.dds",
"Data/UI/item_icons/leather01c.dds",
"Data/UI/item_icons/leather01d.dds",
"Data/UI/item_icons/leather01g.dds",
"Data/UI/item_icons/leather02.dds",
"Data/UI/item_icons/leather02a.dds",
"Data/UI/item_icons/leather02aa.dds",
"Data/UI/item_icons/leather02b.dds",
"Data/UI/item_icons/leather03.dds",
"Data/UI/item_icons/leather03b.dds",
"Data/UI/item_icons/leather03c.dds",
"Data/UI/item_icons/leather03d.dds",
"Data/UI/item_icons/leather04a.dds",
"Data/UI/item_icons/leather05.dds",
"Data/UI/item_icons/leather05z.dds",
"Data/UI/item_icons/leather06.dds",
"Data/UI/item_icons/leather07.dds",
"Data/UI/item_icons/leather07a.dds",
"Data/UI/item_icons/leather07b.dds",
"Data/UI/item_icons/leather08.dds",
"Data/UI/item_icons/leather09a.dds",
"Data/UI/item_icons/leather09b.dds",
"Data/UI/item_icons/leather10.dds",
"Data/UI/item_icons/leather10a.dds",
"Data/UI/item_icons/leather10ab.dds",
"Data/UI/item_icons/leather10b.dds",
"Data/UI/item_icons/leather11a.dds",
"Data/UI/item_icons/leather_000_boots.dds",
"Data/UI/item_icons/leather_000_boots_a.dds",
"Data/UI/item_icons/leather_000_chest.dds",
"Data/UI/item_icons/leather_000_chest_a.dds",
"Data/UI/item_icons/leather_000_leggings.dds",
"Data/UI/item_icons/leather_000_leggings_a.dds",
"Data/UI/item_icons/leather_100_boots.dds",
"Data/UI/item_icons/leather_100_chest.dds",
"Data/UI/item_icons/leather_101_belt.dds",
"Data/UI/item_icons/leather_101_boots.dds",
"Data/UI/item_icons/leather_101_chest.dds",
"Data/UI/item_icons/leather_101_gloves.dds",
"Data/UI/item_icons/leather_101_helm.dds",
"Data/UI/item_icons/leather_101_leggings.dds",
"Data/UI/item_icons/leather_101_shoulders.dds",
"Data/UI/item_icons/leather_102_belt.dds",
"Data/UI/item_icons/leather_102_boots.dds",
"Data/UI/item_icons/leather_102_chest.dds",
"Data/UI/item_icons/leather_102_gloves.dds",
"Data/UI/item_icons/leather_102_helm.dds",
"Data/UI/item_icons/leather_102_leggings.dds",
"Data/UI/item_icons/leather_102_shoulders.dds",
"Data/UI/item_icons/leather_103_belt.dds",
"Data/UI/item_icons/leather_103_boots.dds",
"Data/UI/item_icons/leather_103_chest.dds",
"Data/UI/item_icons/leather_103_gloves.dds",
"Data/UI/item_icons/leather_103_head.dds",
"Data/UI/item_icons/leather_103_leggings.dds",
"Data/UI/item_icons/leather_103_shoulders.dds",
"Data/UI/item_icons/leather_104_belt.dds",
"Data/UI/item_icons/leather_104_boots.dds",
"Data/UI/item_icons/leather_104_boots_a.dds",
"Data/UI/item_icons/leather_104_chest.dds",
"Data/UI/item_icons/leather_104_gloves.dds",
"Data/UI/item_icons/leather_104_helm.dds",
"Data/UI/item_icons/leather_104_leggings.dds",
"Data/UI/item_icons/leather_104_shoulders.dds",
"Data/UI/item_icons/leather_105_belt.dds",
"Data/UI/item_icons/leather_105_boots.dds",
"Data/UI/item_icons/leather_105_chest.dds",
"Data/UI/item_icons/leather_105_gloves.dds",
"Data/UI/item_icons/leather_105_helm.dds",
"Data/UI/item_icons/leather_105_leggings.dds",
"Data/UI/item_icons/leather_105_shoulders.dds",
"Data/UI/item_icons/leather_106a_defiant_helm.dds",
"Data/UI/item_icons/leather_106a_defiant_helm_a.dds",
"Data/UI/item_icons/leather_106a_guardian_helm.dds",
"Data/UI/item_icons/leather_106a_guardian_helm_a.dds",
"Data/UI/item_icons/leather_106a_guardian_shoulders.dds",
"Data/UI/item_icons/leather_106a_guardian_shoulders_a.dds",
"Data/UI/item_icons/leather_106b_defiant_shoulders.dds",
"Data/UI/item_icons/leather_106b_defiant_shoulders_a.dds",
"Data/UI/item_icons/leather_106b_guardian_helm.dds",
"Data/UI/item_icons/leather_106b_guardian_helm_a.dds",
"Data/UI/item_icons/leather_106_belt.dds",
"Data/UI/item_icons/leather_106_belt_a.dds",
"Data/UI/item_icons/leather_106_boots.dds",
"Data/UI/item_icons/leather_106_boots_a.dds",
"Data/UI/item_icons/leather_106_chest.dds",
"Data/UI/item_icons/leather_106_chest_a.dds",
"Data/UI/item_icons/leather_106_gloves.dds",
"Data/UI/item_icons/leather_106_gloves_a.dds",
"Data/UI/item_icons/leather_106_helm.dds",
"Data/UI/item_icons/leather_106_helm_a.dds",
"Data/UI/item_icons/leather_106_leggings.dds",
"Data/UI/item_icons/leather_106_leggings_a.dds",
"Data/UI/item_icons/leather_106_shoulders.dds",
"Data/UI/item_icons/leather_106_shoulders_a.dds",
"Data/UI/item_icons/leather_107a_helm.dds",
"Data/UI/item_icons/leather_107a_helm_a.dds",
"Data/UI/item_icons/leather_107a_shoulder.dds",
"Data/UI/item_icons/leather_107a_shoulder_a.dds",
"Data/UI/item_icons/leather_107_belt.dds",
"Data/UI/item_icons/leather_107_belt_a.dds",
"Data/UI/item_icons/leather_107_boots.dds",
"Data/UI/item_icons/leather_107_boots_a.dds",
"Data/UI/item_icons/leather_107_chest.dds",
"Data/UI/item_icons/leather_107_chest_a.dds",
"Data/UI/item_icons/leather_107_gloves.dds",
"Data/UI/item_icons/leather_107_gloves_a.dds",
"Data/UI/item_icons/leather_107_helm.dds",
"Data/UI/item_icons/leather_107_helm_a.dds",
"Data/UI/item_icons/leather_107_leggings.dds",
"Data/UI/item_icons/leather_107_leggings_a.dds",
"Data/UI/item_icons/leather_107_shoulders.dds",
"Data/UI/item_icons/leather_107_shoulders_a.dds",
"Data/UI/item_icons/leather_108_belt.dds",
"Data/UI/item_icons/leather_108_boots.dds",
"Data/UI/item_icons/leather_108_chest.dds",
"Data/UI/item_icons/leather_108_gloves.dds",
"Data/UI/item_icons/leather_108_helm.dds",
"Data/UI/item_icons/leather_108_leggings.dds",
"Data/UI/item_icons/leather_108_shoulders.dds",
"Data/UI/item_icons/leather_109a_helm.dds",
"Data/UI/item_icons/leather_109a_helm_a.dds",
"Data/UI/item_icons/leather_109a_shoulders.dds",
"Data/UI/item_icons/leather_109a_shoulders_a.dds",
"Data/UI/item_icons/leather_109b_helm.dds",
"Data/UI/item_icons/leather_109b_helm_a.dds",
"Data/UI/item_icons/leather_109b_shoulders.dds",
"Data/UI/item_icons/leather_109b_shoulders_a.dds",
"Data/UI/item_icons/leather_109_belt.dds",
"Data/UI/item_icons/leather_109_belt_a.dds",
"Data/UI/item_icons/leather_109_boots.dds",
"Data/UI/item_icons/leather_109_chest.dds",
"Data/UI/item_icons/leather_109_gloves.dds",
"Data/UI/item_icons/leather_109_helm.dds",
"Data/UI/item_icons/leather_109_leggings.dds",
"Data/UI/item_icons/leather_109_shoulders.dds",
"Data/UI/item_icons/leather_110_belt.dds",
"Data/UI/item_icons/leather_110_belt_a.dds",
"Data/UI/item_icons/leather_110_boots.dds",
"Data/UI/item_icons/leather_110_boots_a.dds",
"Data/UI/item_icons/leather_110_chest.dds",
"Data/UI/item_icons/leather_110_chest_a.dds",
"Data/UI/item_icons/leather_110_gloves.dds",
"Data/UI/item_icons/leather_110_gloves_a.dds",
"Data/UI/item_icons/leather_110_helm.dds",
"Data/UI/item_icons/leather_110_helm_a.dds",
"Data/UI/item_icons/leather_110_leggings.dds",
"Data/UI/item_icons/leather_110_leggings_a.dds",
"Data/UI/item_icons/leather_110_shoulders.dds",
"Data/UI/item_icons/leather_110_shoulders_a.dds",
"Data/UI/item_icons/leather_111_belt.dds",
"Data/UI/item_icons/leather_111_boots.dds",
"Data/UI/item_icons/leather_111_chest.dds",
"Data/UI/item_icons/leather_111_gloves.dds",
"Data/UI/item_icons/leather_111_helm.dds",
"Data/UI/item_icons/leather_111_leggings.dds",
"Data/UI/item_icons/leather_111_shoulders.dds",
"Data/UI/item_icons/leather_112_belt.dds",
"Data/UI/item_icons/leather_112_boots.dds",
"Data/UI/item_icons/leather_112_chest.dds",
"Data/UI/item_icons/leather_112_gloves.dds",
"Data/UI/item_icons/leather_112_helm.dds",
"Data/UI/item_icons/leather_112_leggings.dds",
"Data/UI/item_icons/leather_112_shoulders.dds",
"Data/UI/item_icons/leather_113a_helm.dds",
"Data/UI/item_icons/leather_113a_helm_a.dds",
"Data/UI/item_icons/leather_113a_shoulder.dds",
"Data/UI/item_icons/leather_113a_shoulder_a.dds",
"Data/UI/item_icons/leather_113_belt.dds",
"Data/UI/item_icons/leather_113_belt_a.dds",
"Data/UI/item_icons/leather_113_boots.dds",
"Data/UI/item_icons/leather_113_boots_a.dds",
"Data/UI/item_icons/leather_113_chest.dds",
"Data/UI/item_icons/leather_113_chest_a.dds",
"Data/UI/item_icons/leather_113_gloves.dds",
"Data/UI/item_icons/leather_113_gloves_a.dds",
"Data/UI/item_icons/leather_113_helm.dds",
"Data/UI/item_icons/leather_113_helm_a.dds",
"Data/UI/item_icons/leather_113_leggings.dds",
"Data/UI/item_icons/leather_113_leggings_a.dds",
"Data/UI/item_icons/leather_113_shoulders.dds",
"Data/UI/item_icons/leather_113_shoulders_a.dds",
"Data/UI/item_icons/leather_114a_belt.dds",
"Data/UI/item_icons/leather_114a_belt_a.dds",
"Data/UI/item_icons/leather_114a_boots.dds",
"Data/UI/item_icons/leather_114a_boots_a.dds",
"Data/UI/item_icons/leather_114a_chest.dds",
"Data/UI/item_icons/leather_114a_chest_a.dds",
"Data/UI/item_icons/leather_114a_gloves.dds",
"Data/UI/item_icons/leather_114a_gloves_a.dds",
"Data/UI/item_icons/leather_114a_leggings.dds",
"Data/UI/item_icons/leather_114a_leggings_a.dds",
"Data/UI/item_icons/leather_114a_shoulders.dds",
"Data/UI/item_icons/leather_114a_shoulders_a.dds",
"Data/UI/item_icons/leather_114b_belt.dds",
"Data/UI/item_icons/leather_114b_belt_a.dds",
"Data/UI/item_icons/leather_114c_belt.dds",
"Data/UI/item_icons/leather_114c_belt_a.dds",
"Data/UI/item_icons/leather_114c_boots.dds",
"Data/UI/item_icons/leather_114c_boots_a.dds",
"Data/UI/item_icons/leather_114c_chest.dds",
"Data/UI/item_icons/leather_114c_chest_a.dds",
"Data/UI/item_icons/leather_114c_gloves.dds",
"Data/UI/item_icons/leather_114c_gloves_a.dds",
"Data/UI/item_icons/leather_114c_leggings.dds",
"Data/UI/item_icons/leather_114c_leggings_a.dds",
"Data/UI/item_icons/leather_114c_shoulders.dds",
"Data/UI/item_icons/leather_114c_shoulders_a.dds",
"Data/UI/item_icons/leather_114_belt.dds",
"Data/UI/item_icons/leather_114_belt_a.dds",
"Data/UI/item_icons/leather_114_boots.dds",
"Data/UI/item_icons/leather_114_boots_a.dds",
"Data/UI/item_icons/leather_114_chest.dds",
"Data/UI/item_icons/leather_114_chest_a.dds",
"Data/UI/item_icons/leather_114_gloves_a.dds",
"Data/UI/item_icons/leather_114_helm.dds",
"Data/UI/item_icons/leather_114_helm_a.dds",
"Data/UI/item_icons/leather_114_leggings.dds",
"Data/UI/item_icons/leather_114_leggings_a.dds",
"Data/UI/item_icons/leather_114_shoulders.dds",
"Data/UI/item_icons/leather_114_shoulders_a.dds",
"Data/UI/item_icons/leather_115_belt.dds",
"Data/UI/item_icons/leather_115_belt_a.dds",
"Data/UI/item_icons/leather_115_boots.dds",
"Data/UI/item_icons/leather_115_boots_a.dds",
"Data/UI/item_icons/leather_115_chest.dds",
"Data/UI/item_icons/leather_115_chest_a.dds",
"Data/UI/item_icons/leather_115_gloves.dds",
"Data/UI/item_icons/leather_115_gloves_a.dds",
"Data/UI/item_icons/leather_115_helm.dds",
"Data/UI/item_icons/leather_115_helm_a.dds",
"Data/UI/item_icons/leather_115_leggings.dds",
"Data/UI/item_icons/leather_115_leggings_a.dds",
"Data/UI/item_icons/leather_115_shoulders.dds",
"Data/UI/item_icons/leather_115_shoulders_a.dds",
"Data/UI/item_icons/leather_116_belt.dds",
"Data/UI/item_icons/leather_116_belt_a.dds",
"Data/UI/item_icons/leather_116_boots.dds",
"Data/UI/item_icons/leather_116_boots_a.dds",
"Data/UI/item_icons/leather_116_chest.dds",
"Data/UI/item_icons/leather_116_chest_a.dds",
"Data/UI/item_icons/leather_116_gloves.dds",
"Data/UI/item_icons/leather_116_gloves_a.dds",
"Data/UI/item_icons/leather_116_helm.dds",
"Data/UI/item_icons/leather_116_helm_a.dds",
"Data/UI/item_icons/leather_116_leggings.dds",
"Data/UI/item_icons/leather_116_leggings_a.dds",
"Data/UI/item_icons/leather_116_shoulders.dds",
"Data/UI/item_icons/leather_116_shoulders_a.dds",
"Data/UI/item_icons/leather_117a_helm.dds",
"Data/UI/item_icons/leather_117a_helm_a.dds",
"Data/UI/item_icons/leather_117a_shoulders.dds",
"Data/UI/item_icons/leather_117a_shoulders_a.dds",
"Data/UI/item_icons/leather_117_belt.dds",
"Data/UI/item_icons/leather_117_belt_a.dds",
"Data/UI/item_icons/leather_117_boots.dds",
"Data/UI/item_icons/leather_117_boots_a.dds",
"Data/UI/item_icons/leather_117_chest.dds",
"Data/UI/item_icons/leather_117_chest_a.dds",
"Data/UI/item_icons/leather_117_gloves.dds",
"Data/UI/item_icons/leather_117_gloves_a_48.dds",
"Data/UI/item_icons/leather_117_helm.dds",
"Data/UI/item_icons/leather_117_helm_a.dds",
"Data/UI/item_icons/leather_117_leggings.dds",
"Data/UI/item_icons/leather_117_leggings_a.dds",
"Data/UI/item_icons/leather_117_shoulders.dds",
"Data/UI/item_icons/leather_117_shoulders_a.dds",
"Data/UI/item_icons/leather_119_belt.dds",
"Data/UI/item_icons/leather_119_belt_a.dds",
"Data/UI/item_icons/leather_119_boots.dds",
"Data/UI/item_icons/leather_119_boots_a.dds",
"Data/UI/item_icons/leather_119_chest.dds",
"Data/UI/item_icons/leather_119_chest_a.dds",
"Data/UI/item_icons/leather_119_gloves.dds",
"Data/UI/item_icons/leather_119_gloves_a.dds",
"Data/UI/item_icons/leather_119_helm.dds",
"Data/UI/item_icons/leather_119_helm_a.dds",
"Data/UI/item_icons/leather_119_leggings.dds",
"Data/UI/item_icons/leather_119_leggings_a.dds",
"Data/UI/item_icons/leather_119_shoulder.dds",
"Data/UI/item_icons/leather_119_shoulder_a.dds",
"Data/UI/item_icons/leather_120_belt.dds",
"Data/UI/item_icons/leather_120_belt_a.dds",
"Data/UI/item_icons/leather_120_boots.dds",
"Data/UI/item_icons/leather_120_boots_a.dds",
"Data/UI/item_icons/leather_120_chest.dds",
"Data/UI/item_icons/leather_120_chest_a.dds",
"Data/UI/item_icons/leather_120_gloves.dds",
"Data/UI/item_icons/leather_120_gloves_a.dds",
"Data/UI/item_icons/leather_120_helm.dds",
"Data/UI/item_icons/leather_120_helm_a.dds",
"Data/UI/item_icons/leather_120_leggings.dds",
"Data/UI/item_icons/leather_120_leggings_a.dds",
"Data/UI/item_icons/leather_120_shoulders.dds",
"Data/UI/item_icons/leather_120_shoulders_a.dds",
"Data/UI/item_icons/leather_121_belt.dds",
"Data/UI/item_icons/leather_121_belt_a.dds",
"Data/UI/item_icons/leather_121_boots.dds",
"Data/UI/item_icons/leather_121_boots_a.dds",
"Data/UI/item_icons/leather_121_chest.dds",
"Data/UI/item_icons/leather_121_chest_a.dds",
"Data/UI/item_icons/leather_121_gloves.dds",
"Data/UI/item_icons/leather_121_gloves_a.dds",
"Data/UI/item_icons/leather_121_helm.dds",
"Data/UI/item_icons/leather_121_helm_a.dds",
"Data/UI/item_icons/leather_121_leggings.dds",
"Data/UI/item_icons/leather_121_leggings_a.dds",
"Data/UI/item_icons/leather_121_shoulder.dds",
"Data/UI/item_icons/leather_121_shoulder_a.dds",
"Data/UI/item_icons/leather_belt_damaged_01.dds",
"Data/UI/item_icons/leather_boots_104b.dds",
"Data/UI/item_icons/leather_boots_104b_a.dds",
"Data/UI/item_icons/leather_boots_114b.dds",
"Data/UI/item_icons/leather_boots_114b_a.dds",
"Data/UI/item_icons/leather_boots_damaged_01.dds",
"Data/UI/item_icons/leather_canteen.dds",
"Data/UI/item_icons/leather_chest_104b.dds",
"Data/UI/item_icons/leather_chest_104b_a.dds",
"Data/UI/item_icons/leather_chest_114b.dds",
"Data/UI/item_icons/leather_chest_114b_a.dds",
"Data/UI/item_icons/leather_chest_damaged_01.dds",
"Data/UI/item_icons/leather_gloves_104b.dds",
"Data/UI/item_icons/leather_gloves_104b_a.dds",
"Data/UI/item_icons/leather_gloves_114b.dds",
"Data/UI/item_icons/leather_gloves_114b_a.dds",
"Data/UI/item_icons/leather_gloves_damaged_01.dds",
"Data/UI/item_icons/leather_helm_104a.dds",
"Data/UI/item_icons/leather_helm_104a_a.dds",
"Data/UI/item_icons/leather_helm_104b.dds",
"Data/UI/item_icons/leather_helm_104b_a.dds",
"Data/UI/item_icons/leather_helm_104c.dds",
"Data/UI/item_icons/leather_helm_104c_a.dds",
"Data/UI/item_icons/leather_helm_114a.dds",
"Data/UI/item_icons/leather_helm_114a_a.dds",
"Data/UI/item_icons/leather_helm_114b.dds",
"Data/UI/item_icons/leather_helm_114b_a.dds",
"Data/UI/item_icons/leather_helm_114c.dds",
"Data/UI/item_icons/leather_helm_114c_a.dds",
"Data/UI/item_icons/leather_helm_damaged_01.dds",
"Data/UI/item_icons/leather_leggings_104b.dds",
"Data/UI/item_icons/leather_leggings_104b_a.dds",
"Data/UI/item_icons/leather_leggings_114b.dds",
"Data/UI/item_icons/leather_leggings_114b_a.dds",
"Data/UI/item_icons/leather_leggings_damaged_01.dds",
"Data/UI/item_icons/leather_shoulders_104b.dds",
"Data/UI/item_icons/leather_shoulders_104b_a.dds",
"Data/UI/item_icons/leather_shoulders_114b.dds",
"Data/UI/item_icons/leather_shoulders_114b_a.dds",
"Data/UI/item_icons/leather_shoulders_damaged_01.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_0.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_1.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_2.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_3.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_4.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_5.dds",
"Data/UI/item_icons/legendary_potion_cylindrical_6.dds",
"Data/UI/item_icons/legendary_potion_rectangular_0.dds",
"Data/UI/item_icons/legendary_potion_rectangular_1.dds",
"Data/UI/item_icons/legendary_potion_rectangular_2.dds",
"Data/UI/item_icons/legendary_potion_rectangular_3.dds",
"Data/UI/item_icons/legendary_potion_rectangular_4.dds",
"Data/UI/item_icons/legendary_potion_rectangular_4a.dds",
"Data/UI/item_icons/legendary_potion_rectangular_5.dds",
"Data/UI/item_icons/legendary_potion_rectangular_6.dds",
"Data/UI/item_icons/legendary_potion_spherical_0.dds",
"Data/UI/item_icons/legendary_potion_spherical_1.dds",
"Data/UI/item_icons/legendary_potion_spherical_2.dds",
"Data/UI/item_icons/legendary_potion_spherical_3.dds",
"Data/UI/item_icons/legendary_potion_spherical_3_1.dds",
"Data/UI/item_icons/legendary_potion_spherical_4.dds",
"Data/UI/item_icons/legendary_potion_spherical_5.dds",
"Data/UI/item_icons/legendary_potion_spherical_6.dds",
"Data/UI/item_icons/length_of_silken_rope.dds",
"Data/UI/item_icons/lesser_air_01.dds",
"Data/UI/item_icons/lesser_air_02.dds",
"Data/UI/item_icons/lesser_air_02_a.dds",
"Data/UI/item_icons/lesser_air_03.dds",
"Data/UI/item_icons/lesser_air_04.dds",
"Data/UI/item_icons/lesser_air_05.dds",
"Data/UI/item_icons/lesser_death_01.dds",
"Data/UI/item_icons/lesser_death_02.dds",
"Data/UI/item_icons/lesser_death_03.dds",
"Data/UI/item_icons/lesser_death_04.dds",
"Data/UI/item_icons/lesser_death_05.dds",
"Data/UI/item_icons/lesser_death_05_a.dds",
"Data/UI/item_icons/lesser_earth_01.dds",
"Data/UI/item_icons/lesser_earth_01_a.dds",
"Data/UI/item_icons/lesser_earth_02.dds",
"Data/UI/item_icons/lesser_earth_03.dds",
"Data/UI/item_icons/lesser_earth_03_a.dds",
"Data/UI/item_icons/lesser_earth_04.dds",
"Data/UI/item_icons/lesser_earth_04_a.dds",
"Data/UI/item_icons/lesser_earth_05.dds",
"Data/UI/item_icons/lesser_fire_01.dds",
"Data/UI/item_icons/lesser_fire_02.dds",
"Data/UI/item_icons/lesser_fire_03.dds",
"Data/UI/item_icons/lesser_fire_04.dds",
"Data/UI/item_icons/lesser_fire_05.dds",
"Data/UI/item_icons/lesser_life_01.dds",
"Data/UI/item_icons/lesser_life_02.dds",
"Data/UI/item_icons/lesser_life_03.dds",
"Data/UI/item_icons/lesser_life_04.dds",
"Data/UI/item_icons/lesser_life_05.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_0.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_1.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_2.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_3.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_4.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_5.dds",
"Data/UI/item_icons/lesser_potion_cylindrical_6.dds",
"Data/UI/item_icons/lesser_potion_rectangular_0.dds",
"Data/UI/item_icons/lesser_potion_rectangular_1.dds",
"Data/UI/item_icons/lesser_potion_rectangular_2.dds",
"Data/UI/item_icons/lesser_potion_rectangular_3.dds",
"Data/UI/item_icons/lesser_potion_rectangular_4.dds",
"Data/UI/item_icons/lesser_potion_rectangular_5.dds",
"Data/UI/item_icons/lesser_potion_rectangular_6.dds",
"Data/UI/item_icons/lesser_potion_spherical_0.dds",
"Data/UI/item_icons/lesser_potion_spherical_1.dds",
"Data/UI/item_icons/lesser_potion_spherical_2.dds",
"Data/UI/item_icons/lesser_potion_spherical_3.dds",
"Data/UI/item_icons/lesser_potion_spherical_4.dds",
"Data/UI/item_icons/lesser_potion_spherical_5.dds",
"Data/UI/item_icons/lesser_potion_spherical_6.dds",
"Data/UI/item_icons/lesser_water_01.dds",
"Data/UI/item_icons/lesser_water_01_a.dds",
"Data/UI/item_icons/lesser_water_02.dds",
"Data/UI/item_icons/lesser_water_03.dds",
"Data/UI/item_icons/lesser_water_04.dds",
"Data/UI/item_icons/lesser_water_05.dds",
"Data/UI/item_icons/level_10.dds",
"Data/UI/item_icons/level_20.dds",
"Data/UI/item_icons/level_30.dds",
"Data/UI/item_icons/level_40.dds",
"Data/UI/item_icons/level_50.dds",
"Data/UI/item_icons/life_1h_mace_epic.dds",
"Data/UI/item_icons/life_1h_mace_epic_a.dds",
"Data/UI/item_icons/life_1h_mace_epic_b.dds",
"Data/UI/item_icons/life_2h_hammer_mundane.dds",
"Data/UI/item_icons/life_2h_hammer_mundane_a.dds",
"Data/UI/item_icons/life_2h_hammer_mundane_b.dds",
"Data/UI/item_icons/life_book_epic.dds",
"Data/UI/item_icons/life_book_epic_a.dds",
"Data/UI/item_icons/life_book_epic_b.dds",
"Data/UI/item_icons/life_infused_leech.dds",
"Data/UI/item_icons/life_infused_leech_a.dds",
"Data/UI/item_icons/lightsbane_diamond.dds",
"Data/UI/item_icons/little_black_book.dds",
"Data/UI/item_icons/loaf_of_baguette_bread.dds",
"Data/UI/item_icons/loaf_of_round_bread.dds",
"Data/UI/item_icons/lockflower_sprig.dds",
"Data/UI/item_icons/loot_gold_coins.dds",
"Data/UI/item_icons/loot_platinum_coins.dds",
"Data/UI/item_icons/loot_silver_coins.dds",
"Data/UI/item_icons/lure_air_01.dds",
"Data/UI/item_icons/lure_air_01_a.dds",
"Data/UI/item_icons/lure_air_02.dds",
"Data/UI/item_icons/lure_air_02_a.dds",
"Data/UI/item_icons/lure_air_03.dds",
"Data/UI/item_icons/lure_air_03_a.dds",
"Data/UI/item_icons/lure_air_03_b.dds",
"Data/UI/item_icons/lure_air_04.dds",
"Data/UI/item_icons/lure_air_04_a.dds",
"Data/UI/item_icons/lure_air_04_b.dds",
"Data/UI/item_icons/lure_air_04_c.dds",
"Data/UI/item_icons/lure_air_05.dds",
"Data/UI/item_icons/lure_air_05_a.dds",
"Data/UI/item_icons/lure_death_01.dds",
"Data/UI/item_icons/lure_death_01_a.dds",
"Data/UI/item_icons/lure_death_02.dds",
"Data/UI/item_icons/lure_death_02_a.dds",
"Data/UI/item_icons/lure_death_02_b.dds",
"Data/UI/item_icons/lure_death_03.dds",
"Data/UI/item_icons/lure_death_03_a.dds",
"Data/UI/item_icons/lure_death_04.dds",
"Data/UI/item_icons/lure_death_04_a.dds",
"Data/UI/item_icons/lure_death_04_b.dds",
"Data/UI/item_icons/lure_death_05.dds",
"Data/UI/item_icons/lure_death_05_a.dds",
"Data/UI/item_icons/lure_earth_01.dds",
"Data/UI/item_icons/lure_earth_01_a.dds",
"Data/UI/item_icons/lure_earth_01_b.dds",
"Data/UI/item_icons/lure_earth_02.dds",
"Data/UI/item_icons/lure_earth_02_a.dds",
"Data/UI/item_icons/lure_earth_02_b.dds",
"Data/UI/item_icons/lure_earth_02_c.dds",
"Data/UI/item_icons/lure_earth_03.dds",
"Data/UI/item_icons/lure_earth_03_a.dds",
"Data/UI/item_icons/lure_earth_03_b.dds",
"Data/UI/item_icons/lure_earth_04.dds",
"Data/UI/item_icons/lure_earth_04_a.dds",
"Data/UI/item_icons/lure_earth_05.dds",
"Data/UI/item_icons/lure_earth_05_a.dds",
"Data/UI/item_icons/lure_earth_05_b.dds",
"Data/UI/item_icons/lure_fire_01.dds",
"Data/UI/item_icons/lure_fire_01_a.dds",
"Data/UI/item_icons/lure_fire_02.dds",
"Data/UI/item_icons/lure_fire_02_a.dds",
"Data/UI/item_icons/lure_fire_02_b.dds",
"Data/UI/item_icons/lure_fire_03.dds",
"Data/UI/item_icons/lure_fire_03_a.dds",
"Data/UI/item_icons/lure_fire_03_b.dds",
"Data/UI/item_icons/lure_fire_04.dds",
"Data/UI/item_icons/lure_fire_04_a.dds",
"Data/UI/item_icons/lure_fire_04_b.dds",
"Data/UI/item_icons/lure_fire_05.dds",
"Data/UI/item_icons/lure_fire_05_a.dds",
"Data/UI/item_icons/lure_fire_05_b.dds",
"Data/UI/item_icons/lure_life_01.dds",
"Data/UI/item_icons/lure_life_01_a.dds",
"Data/UI/item_icons/lure_life_02.dds",
"Data/UI/item_icons/lure_life_02_a.dds",
"Data/UI/item_icons/lure_life_03.dds",
"Data/UI/item_icons/lure_life_03_a.dds",
"Data/UI/item_icons/lure_life_04.dds",
"Data/UI/item_icons/lure_life_04_a.dds",
"Data/UI/item_icons/lure_life_04_e.dds",
"Data/UI/item_icons/lure_life_05.dds",
"Data/UI/item_icons/lure_life_05_a.dds",
"Data/UI/item_icons/lure_water_01.dds",
"Data/UI/item_icons/lure_water_01_a.dds",
"Data/UI/item_icons/lure_water_01_b.dds",
"Data/UI/item_icons/lure_water_02.dds",
"Data/UI/item_icons/lure_water_02_a.dds",
"Data/UI/item_icons/lure_water_02_b.dds",
"Data/UI/item_icons/lure_water_03.dds",
"Data/UI/item_icons/lure_water_03_a.dds",
"Data/UI/item_icons/lure_water_04.dds",
"Data/UI/item_icons/lure_water_04_a.dds",
"Data/UI/item_icons/lure_water_04_b.dds",
"Data/UI/item_icons/lure_water_05.dds",
"Data/UI/item_icons/lure_water_05_a.dds",
"Data/UI/item_icons/lute_01.dds",
"Data/UI/item_icons/lute_02.dds",
"Data/UI/item_icons/lute_03.dds",
"Data/UI/item_icons/lute_03_a.dds",
"Data/UI/item_icons/lute_04.dds",
"Data/UI/item_icons/lute_05.dds",
"Data/UI/item_icons/lute_06.dds",
"Data/UI/item_icons/lute_06_a.dds",
"Data/UI/item_icons/lute_07.dds",
"Data/UI/item_icons/lute_07_a.dds",
"Data/UI/item_icons/lute_08.dds",
"Data/UI/item_icons/lute_09.dds",
"Data/UI/item_icons/lute_09_a.dds",
"Data/UI/item_icons/lute_10.dds",
"Data/UI/item_icons/lute_10_a.dds",
"Data/UI/item_icons/mad_diary_of_vizov.dds",
"Data/UI/item_icons/mad_diary_of_vizov_a.dds",
"Data/UI/item_icons/mad_diary_of_vizov_b.dds",
"Data/UI/item_icons/magical_bars.dds",
"Data/UI/item_icons/magical_bars_a.dds",
"Data/UI/item_icons/magical_bars_b.dds",
"Data/UI/item_icons/magical_bars_c.dds",
"Data/UI/item_icons/magical_component1.dds",
"Data/UI/item_icons/magical_component2.dds",
"Data/UI/item_icons/magical_component2a.dds",
"Data/UI/item_icons/magical_component3.dds",
"Data/UI/item_icons/magical_component3a.dds",
"Data/UI/item_icons/magical_component3b.dds",
"Data/UI/item_icons/magical_component4.dds",
"Data/UI/item_icons/magical_component4a.dds",
"Data/UI/item_icons/magical_component4b.dds",
"Data/UI/item_icons/magical_component5.dds",
"Data/UI/item_icons/magical_component5a.dds",
"Data/UI/item_icons/magical_component5b.dds",
"Data/UI/item_icons/magical_component6.dds",
"Data/UI/item_icons/magical_component6a.dds",
"Data/UI/item_icons/magical_component7.dds",
"Data/UI/item_icons/magical_component8.dds",
"Data/UI/item_icons/magical_component9.dds",
"Data/UI/item_icons/magical_sap1.dds",
"Data/UI/item_icons/magical_sap1a.dds",
"Data/UI/item_icons/magical_sap2.dds",
"Data/UI/item_icons/magical_sap2a.dds",
"Data/UI/item_icons/magical_sap3.dds",
"Data/UI/item_icons/magical_sap3a.dds",
"Data/UI/item_icons/magical_sap3_cerulean.dds",
"Data/UI/item_icons/magical_sap3_clear.dds",
"Data/UI/item_icons/magical_sap3_crimson.dds",
"Data/UI/item_icons/magical_sap3_viridian.dds",
"Data/UI/item_icons/magical_sap4.dds",
"Data/UI/item_icons/magical_sap4a.dds",
"Data/UI/item_icons/magical_sap5.dds",
"Data/UI/item_icons/magicitem1.dds",
"Data/UI/item_icons/magicitem2.dds",
"Data/UI/item_icons/magicitem3.dds",
"Data/UI/item_icons/magicitem3a.dds",
"Data/UI/item_icons/magicitem3z.dds",
"Data/UI/item_icons/magicitem4a.dds",
"Data/UI/item_icons/magic_powder.dds",
"Data/UI/item_icons/magic_powder_a.dds",
"Data/UI/item_icons/magic_powder_b.dds",
"Data/UI/item_icons/magic_powder_c.dds",
"Data/UI/item_icons/magic_powder_d.dds",
"Data/UI/item_icons/magic_powder_e.dds",
"Data/UI/item_icons/magic_powder_f.dds",
"Data/UI/item_icons/magic_powder_g.dds",
"Data/UI/item_icons/magic_powder_h.dds",
"Data/UI/item_icons/magic_powder_y.dds",
"Data/UI/item_icons/magic_powder_z.dds",
"Data/UI/item_icons/mag_component10.dds",
"Data/UI/item_icons/mag_component10a.dds",
"Data/UI/item_icons/major_potion_cylindrical_0.dds",
"Data/UI/item_icons/major_potion_cylindrical_1.dds",
"Data/UI/item_icons/major_potion_cylindrical_2.dds",
"Data/UI/item_icons/major_potion_cylindrical_3.dds",
"Data/UI/item_icons/major_potion_cylindrical_3__.dds",
"Data/UI/item_icons/major_potion_cylindrical_4.dds",
"Data/UI/item_icons/major_potion_cylindrical_5.dds",
"Data/UI/item_icons/major_potion_cylindrical_6.dds",
"Data/UI/item_icons/major_potion_rectangular_0.dds",
"Data/UI/item_icons/major_potion_rectangular_1.dds",
"Data/UI/item_icons/major_potion_rectangular_2.dds",
"Data/UI/item_icons/major_potion_rectangular_3.dds",
"Data/UI/item_icons/major_potion_rectangular_4.dds",
"Data/UI/item_icons/major_potion_rectangular_5.dds",
"Data/UI/item_icons/major_potion_rectangular_6.dds",
"Data/UI/item_icons/major_potion_spherical_0.dds",
"Data/UI/item_icons/major_potion_spherical_1.dds",
"Data/UI/item_icons/major_potion_spherical_2.dds",
"Data/UI/item_icons/major_potion_spherical_3.dds",
"Data/UI/item_icons/major_potion_spherical_4.dds",
"Data/UI/item_icons/major_potion_spherical_5.dds",
"Data/UI/item_icons/major_potion_spherical_6.dds",
"Data/UI/item_icons/manastone_01.dds",
"Data/UI/item_icons/manastone_01_a.dds",
"Data/UI/item_icons/manastone_01_b.dds",
"Data/UI/item_icons/manastone_01_c.dds",
"Data/UI/item_icons/manastone_02.dds",
"Data/UI/item_icons/manastone_02_a.dds",
"Data/UI/item_icons/manastone_02_b.dds",
"Data/UI/item_icons/manastone_02_d.dds",
"Data/UI/item_icons/manastone_03.dds",
"Data/UI/item_icons/manastone_03_a.dds",
"Data/UI/item_icons/manastone_03_b.dds",
"Data/UI/item_icons/manastone_03_c.dds",
"Data/UI/item_icons/manastone_03_cc.dds",
"Data/UI/item_icons/manastone_04.dds",
"Data/UI/item_icons/manastone_04_a.dds",
"Data/UI/item_icons/mangled_flute.dds",
"Data/UI/item_icons/map1.dds",
"Data/UI/item_icons/map1a.dds",
"Data/UI/item_icons/map2a.dds",
"Data/UI/item_icons/map3.dds",
"Data/UI/item_icons/map4.dds",
"Data/UI/item_icons/map5.dds",
"Data/UI/item_icons/map5a.dds",
"Data/UI/item_icons/map6.dds",
"Data/UI/item_icons/map7.dds",
"Data/UI/item_icons/map7a.dds",
"Data/UI/item_icons/maren_last_note.dds",
"Data/UI/item_icons/marred_idol.dds",
"Data/UI/item_icons/maurice_head.dds",
"Data/UI/item_icons/maurice_head_a.dds",
"Data/UI/item_icons/mead_barrel_cork.dds",
"Data/UI/item_icons/meat1.dds",
"Data/UI/item_icons/meat10.dds",
"Data/UI/item_icons/meat11.dds",
"Data/UI/item_icons/meat12.dds",
"Data/UI/item_icons/meat13.dds",
"Data/UI/item_icons/meat13a.dds",
"Data/UI/item_icons/meat14.dds",
"Data/UI/item_icons/meat14a.dds",
"Data/UI/item_icons/meat2.dds",
"Data/UI/item_icons/meat3.dds",
"Data/UI/item_icons/meat4.dds",
"Data/UI/item_icons/meat5.dds",
"Data/UI/item_icons/meat5a.dds",
"Data/UI/item_icons/meat6.dds",
"Data/UI/item_icons/meat7.dds",
"Data/UI/item_icons/meat8.dds",
"Data/UI/item_icons/meat9.dds",
"Data/UI/item_icons/meat_15.dds",
"Data/UI/item_icons/meat_16.dds",
"Data/UI/item_icons/meat_16a.dds",
"Data/UI/item_icons/meat_17.dds",
"Data/UI/item_icons/meat_17a.dds",
"Data/UI/item_icons/meat_18.dds",
"Data/UI/item_icons/meat_19.dds",
"Data/UI/item_icons/meat_19a.dds",
"Data/UI/item_icons/meat_20.dds",
"Data/UI/item_icons/meat_21.dds",
"Data/UI/item_icons/meat_22.dds",
"Data/UI/item_icons/meat_22a.dds",
"Data/UI/item_icons/meat_23.dds",
"Data/UI/item_icons/meat_24.dds",
"Data/UI/item_icons/meat_24a.dds",
"Data/UI/item_icons/meat_25.dds",
"Data/UI/item_icons/meat_26.dds",
"Data/UI/item_icons/meat_26a.dds",
"Data/UI/item_icons/meat_27.dds",
"Data/UI/item_icons/meat_27a.dds",
"Data/UI/item_icons/meat_28.dds",
"Data/UI/item_icons/meat_28a.dds",
"Data/UI/item_icons/meat_28b.dds",
"Data/UI/item_icons/meat_pie.dds",
"Data/UI/item_icons/mechanical_bottle.dds",
"Data/UI/item_icons/medical_supplies.dds",
"Data/UI/item_icons/medium_flesh.dds",
"Data/UI/item_icons/medium_flesh_a.dds",
"Data/UI/item_icons/medium_flesh_b.dds",
"Data/UI/item_icons/medium_hide.dds",
"Data/UI/item_icons/medium_hide_a.dds",
"Data/UI/item_icons/medium_leather_a.dds",
"Data/UI/item_icons/medium_leather_a_1.dds",
"Data/UI/item_icons/medium_leather_b.dds",
"Data/UI/item_icons/medium_potion_cylindrical_0.dds",
"Data/UI/item_icons/medium_potion_cylindrical_1.dds",
"Data/UI/item_icons/medium_potion_cylindrical_2.dds",
"Data/UI/item_icons/medium_potion_cylindrical_3.dds",
"Data/UI/item_icons/medium_potion_cylindrical_4.dds",
"Data/UI/item_icons/medium_potion_cylindrical_5.dds",
"Data/UI/item_icons/medium_potion_cylindrical_6.dds",
"Data/UI/item_icons/medium_potion_rectangular_0.dds",
"Data/UI/item_icons/medium_potion_rectangular_1.dds",
"Data/UI/item_icons/medium_potion_rectangular_2.dds",
"Data/UI/item_icons/medium_potion_rectangular_3.dds",
"Data/UI/item_icons/medium_potion_rectangular_4.dds",
"Data/UI/item_icons/medium_potion_rectangular_5.dds",
"Data/UI/item_icons/medium_potion_rectangular_6.dds",
"Data/UI/item_icons/medium_potion_spherical_0.dds",
"Data/UI/item_icons/medium_potion_spherical_1.dds",
"Data/UI/item_icons/medium_potion_spherical_2.dds",
"Data/UI/item_icons/medium_potion_spherical_3.dds",
"Data/UI/item_icons/medium_potion_spherical_4.dds",
"Data/UI/item_icons/medium_potion_spherical_5.dds",
"Data/UI/item_icons/medium_potion_spherical_6.dds",
"Data/UI/item_icons/metal_flask.dds",
"Data/UI/item_icons/metal_fragment.dds",
"Data/UI/item_icons/minor_potion_cylindrical_0.dds",
"Data/UI/item_icons/minor_potion_cylindrical_1.dds",
"Data/UI/item_icons/minor_potion_cylindrical_2.dds",
"Data/UI/item_icons/minor_potion_cylindrical_3.dds",
"Data/UI/item_icons/minor_potion_cylindrical_4.dds",
"Data/UI/item_icons/minor_potion_cylindrical_5.dds",
"Data/UI/item_icons/minor_potion_cylindrical_6.dds",
"Data/UI/item_icons/minor_potion_rectangular_0.dds",
"Data/UI/item_icons/minor_potion_rectangular_1.dds",
"Data/UI/item_icons/minor_potion_rectangular_2.dds",
"Data/UI/item_icons/minor_potion_rectangular_3.dds",
"Data/UI/item_icons/minor_potion_rectangular_4.dds",
"Data/UI/item_icons/minor_potion_rectangular_5.dds",
"Data/UI/item_icons/minor_potion_rectangular_6.dds",
"Data/UI/item_icons/minor_potion_spherical_0.dds",
"Data/UI/item_icons/minor_potion_spherical_1.dds",
"Data/UI/item_icons/minor_potion_spherical_2.dds",
"Data/UI/item_icons/minor_potion_spherical_3.dds",
"Data/UI/item_icons/minor_potion_spherical_4.dds",
"Data/UI/item_icons/minor_potion_spherical_5.dds",
"Data/UI/item_icons/minor_potion_spherical_6.dds",
"Data/UI/item_icons/mithral_hair_wire.dds",
"Data/UI/item_icons/moist_sapwood.dds",
"Data/UI/item_icons/molly_tag.dds",
"Data/UI/item_icons/moonstone.dds",
"Data/UI/item_icons/moonstone_a.dds",
"Data/UI/item_icons/mosquito_plant.dds",
"Data/UI/item_icons/mossy_twig.dds",
"Data/UI/item_icons/mount6.dds",
"Data/UI/item_icons/mountain_troll_head.dds",
"Data/UI/item_icons/mountain_troll_head_a.dds",
"Data/UI/item_icons/mount_1.dds",
"Data/UI/item_icons/mount_1a.dds",
"Data/UI/item_icons/mount_2.dds",
"Data/UI/item_icons/mount_3.dds",
"Data/UI/item_icons/mount_4.dds",
"Data/UI/item_icons/mount_5.dds",
"Data/UI/item_icons/mount_5a.dds",
"Data/UI/item_icons/mount_5b.dds",
"Data/UI/item_icons/mount_arm_gargazelle.dds",
"Data/UI/item_icons/mount_arm_najamok.dds",
"Data/UI/item_icons/mount_arm_yarnisaur.dds",
"Data/UI/item_icons/mount_croc.dds",
"Data/UI/item_icons/mount_croc_a.dds",
"Data/UI/item_icons/mount_croc_b.dds",
"Data/UI/item_icons/mount_croc_bb.dds",
"Data/UI/item_icons/mount_croc_c.dds",
"Data/UI/item_icons/mount_croc_d.dds",
"Data/UI/item_icons/mount_croc_e.dds",
"Data/UI/item_icons/mount_croc_f.dds",
"Data/UI/item_icons/mount_croc_g.dds",
"Data/UI/item_icons/mount_croc_gg.dds",
"Data/UI/item_icons/mount_croc_h.dds",
"Data/UI/item_icons/mount_croc_hh.dds",
"Data/UI/item_icons/mount_defiant_epic.dds",
"Data/UI/item_icons/mount_defiant_epic_a.dds",
"Data/UI/item_icons/mount_defiant_normal.dds",
"Data/UI/item_icons/mount_epic_murdantix.dds",
"Data/UI/item_icons/mount_epic_murdantix_a.dds",
"Data/UI/item_icons/mount_firehorse.dds",
"Data/UI/item_icons/mount_firehorse_a.dds",
"Data/UI/item_icons/mount_firehorse_b.dds",
"Data/UI/item_icons/mount_horse.dds",
"Data/UI/item_icons/mount_murdantix.dds",
"Data/UI/item_icons/mount_murdantix_a.dds",
"Data/UI/item_icons/mount_murdantix_b.dds",
"Data/UI/item_icons/mount_raptor.dds",
"Data/UI/item_icons/mount_raptor_a.dds",
"Data/UI/item_icons/mount_raptor_b.dds",
"Data/UI/item_icons/mount_raptor_c.dds",
"Data/UI/item_icons/mount_spider.dds",
"Data/UI/item_icons/mount_spider_a.dds",
"Data/UI/item_icons/mount_spider_b.dds",
"Data/UI/item_icons/mount_spider_c.dds",
"Data/UI/item_icons/muck_covered_claw.dds",
"Data/UI/item_icons/muck_covered_claw_a.dds",
"Data/UI/item_icons/mudscale_viper_gland.dds",
"Data/UI/item_icons/mug1.dds",
"Data/UI/item_icons/mug2.dds",
"Data/UI/item_icons/mug3.dds",
"Data/UI/item_icons/mug3a.dds",
"Data/UI/item_icons/mug4.dds",
"Data/UI/item_icons/mug5.dds",
"Data/UI/item_icons/mug_06.dds",
"Data/UI/item_icons/mug_07.dds",
"Data/UI/item_icons/mug_07_a.dds",
"Data/UI/item_icons/mug_08.dds",
"Data/UI/item_icons/mug_08_a.dds",
"Data/UI/item_icons/mushr1.dds",
"Data/UI/item_icons/mushr2.dds",
"Data/UI/item_icons/mushr2a.dds",
"Data/UI/item_icons/mushr2b.dds",
"Data/UI/item_icons/mushr3.dds",
"Data/UI/item_icons/mystery_crumbs.dds",
"Data/UI/item_icons/mystical_potion.dds",
"Data/UI/item_icons/mystical_stone1.dds",
"Data/UI/item_icons/mystical_stone1a.dds",
"Data/UI/item_icons/mystical_stone2.dds",
"Data/UI/item_icons/mystical_stone3.dds",
"Data/UI/item_icons/nagahand.dds",
"Data/UI/item_icons/nagahand_a.dds",
"Data/UI/item_icons/naga_hand.dds",
"Data/UI/item_icons/naga_hand_a.dds",
"Data/UI/item_icons/naga_tail_tip.dds",
"Data/UI/item_icons/nathan_grim_manifesto.dds",
"Data/UI/item_icons/nathan_grim_manifesto_a.dds",
"Data/UI/item_icons/nature_potion.dds",
"Data/UI/item_icons/neck1.dds",
"Data/UI/item_icons/neck10.dds",
"Data/UI/item_icons/neck10b.dds",
"Data/UI/item_icons/neck10c.dds",
"Data/UI/item_icons/neck11.dds",
"Data/UI/item_icons/neck12.dds",
"Data/UI/item_icons/neck12a.dds",
"Data/UI/item_icons/neck13.dds",
"Data/UI/item_icons/neck14.dds",
"Data/UI/item_icons/neck15a.dds",
"Data/UI/item_icons/neck16.dds",
"Data/UI/item_icons/neck17.dds",
"Data/UI/item_icons/neck17a.dds",
"Data/UI/item_icons/neck18.dds",
"Data/UI/item_icons/neck18a.dds",
"Data/UI/item_icons/neck18b.dds",
"Data/UI/item_icons/neck19a.dds",
"Data/UI/item_icons/neck19b.dds",
"Data/UI/item_icons/neck2.dds",
"Data/UI/item_icons/neck20.dds",
"Data/UI/item_icons/neck20a.dds",
"Data/UI/item_icons/neck21.dds",
"Data/UI/item_icons/neck22.dds",
"Data/UI/item_icons/neck22a.dds",
"Data/UI/item_icons/neck22b.dds",
"Data/UI/item_icons/neck23.dds",
"Data/UI/item_icons/neck23a.dds",
"Data/UI/item_icons/neck24.dds",
"Data/UI/item_icons/neck25a.dds",
"Data/UI/item_icons/neck25b.dds",
"Data/UI/item_icons/neck25c.dds",
"Data/UI/item_icons/neck26b.dds",
"Data/UI/item_icons/neck26c.dds",
"Data/UI/item_icons/neck27.dds",
"Data/UI/item_icons/neck28.dds",
"Data/UI/item_icons/neck28b.dds",
"Data/UI/item_icons/neck29.dds",
"Data/UI/item_icons/neck3.dds",
"Data/UI/item_icons/neck30.dds",
"Data/UI/item_icons/neck30a.dds",
"Data/UI/item_icons/neck30b.dds",
"Data/UI/item_icons/neck31b.dds",
"Data/UI/item_icons/neck31c.dds",
"Data/UI/item_icons/neck4.dds",
"Data/UI/item_icons/neck5.dds",
"Data/UI/item_icons/neck5a.dds",
"Data/UI/item_icons/neck6.dds",
"Data/UI/item_icons/neck8.dds",
"Data/UI/item_icons/neck9.dds",
"Data/UI/item_icons/necklace_chain_damaged_01.dds",
"Data/UI/item_icons/necklace_chain_damaged_01_a.dds",
"Data/UI/item_icons/necklace_chain_damaged_01_b.dds",
"Data/UI/item_icons/necklace_chain_normal_01.dds",
"Data/UI/item_icons/necklace_chain_normal_01_a.dds",
"Data/UI/item_icons/necklace_chain_normal_01_b.dds",
"Data/UI/item_icons/necklace_chain_normal_01_c.dds",
"Data/UI/item_icons/necklace_chain_normal_01_d.dds",
"Data/UI/item_icons/necklace_chain_normal_01_e.dds",
"Data/UI/item_icons/necklace_chain_ornate_01.dds",
"Data/UI/item_icons/necklace_chain_ornate_01_a.dds",
"Data/UI/item_icons/necklace_chain_ornate_01_b.dds",
"Data/UI/item_icons/necklace_chain_ornate_01_c.dds",
"Data/UI/item_icons/necklace_choker_damaged_01.dds",
"Data/UI/item_icons/necklace_choker_damaged_01_a.dds",
"Data/UI/item_icons/necklace_choker_damaged_01_b.dds",
"Data/UI/item_icons/necklace_choker_normal_01.dds",
"Data/UI/item_icons/necklace_choker_normal_01_a.dds",
"Data/UI/item_icons/necklace_choker_normal_02.dds",
"Data/UI/item_icons/necklace_choker_normal_02_a.dds",
"Data/UI/item_icons/necklace_choker_normal_02_b.dds",
"Data/UI/item_icons/necklace_choker_ornate_01.dds",
"Data/UI/item_icons/necklace_choker_ornate_01_a.dds",
"Data/UI/item_icons/necklace_choker_ornate_01_b.dds",
"Data/UI/item_icons/necklace_pendant_damaged_01.dds",
"Data/UI/item_icons/necklace_pendant_damaged_01_a.dds",
"Data/UI/item_icons/necklace_pendant_damaged_01_b.dds",
"Data/UI/item_icons/necklace_pendant_normal_01.dds",
"Data/UI/item_icons/necklace_pendant_normal_01_a.dds",
"Data/UI/item_icons/necklace_pendant_normal_01_b.dds",
"Data/UI/item_icons/necklace_pendant_normal_01_c.dds",
"Data/UI/item_icons/necklace_pendant_ornate_01.dds",
"Data/UI/item_icons/necklace_pendant_ornate_01_a.dds",
"Data/UI/item_icons/necklace_pendant_ornate_01_b.dds",
"Data/UI/item_icons/necklace_pendant_ornate_01_c.dds",
"Data/UI/item_icons/necromantic_potion.dds",
"Data/UI/item_icons/nery's_merchandise.dds",
"Data/UI/item_icons/nether_dog_collar.dds",
"Data/UI/item_icons/night_tear.dds",
"Data/UI/item_icons/nothched_hide.dds",
"Data/UI/item_icons/oak_matches.dds",
"Data/UI/item_icons/obsidian_stone_of_elemental_earth.dds",
"Data/UI/item_icons/obsidian_stone_of_elemental_earth_a.dds",
"Data/UI/item_icons/offhand_book_009.dds",
"Data/UI/item_icons/offhand_book_009a.dds",
"Data/UI/item_icons/offhand_book_010.dds",
"Data/UI/item_icons/offhand_book_010a.dds",
"Data/UI/item_icons/offhand_book_011.dds",
"Data/UI/item_icons/offhand_book_011a.dds",
"Data/UI/item_icons/offhand_book_012.dds",
"Data/UI/item_icons/offhand_book_012a.dds",
"Data/UI/item_icons/offhand_book_012b.dds",
"Data/UI/item_icons/offhand_book_013.dds",
"Data/UI/item_icons/offhand_book_013a.dds",
"Data/UI/item_icons/offhand_book_014.dds",
"Data/UI/item_icons/offhand_book_014a.dds",
"Data/UI/item_icons/offhand_book_015.dds",
"Data/UI/item_icons/offhand_book_015a.dds",
"Data/UI/item_icons/offhand_book_016.dds",
"Data/UI/item_icons/offhand_book_016a.dds",
"Data/UI/item_icons/offhand_book_017.dds",
"Data/UI/item_icons/offhand_book_017a.dds",
"Data/UI/item_icons/offhand_book_018.dds",
"Data/UI/item_icons/offhand_book_018a.dds",
"Data/UI/item_icons/offhand_book_019.dds",
"Data/UI/item_icons/offhand_book_019a.dds",
"Data/UI/item_icons/offhand_book_020.dds",
"Data/UI/item_icons/offhand_book_020a.dds",
"Data/UI/item_icons/offhand_book_021.dds",
"Data/UI/item_icons/offhand_book_021a.dds",
"Data/UI/item_icons/offhand_book_022.dds",
"Data/UI/item_icons/offhand_book_022a.dds",
"Data/UI/item_icons/offhand_book_023.dds",
"Data/UI/item_icons/offhand_book_023a.dds",
"Data/UI/item_icons/offhand_book_024.dds",
"Data/UI/item_icons/offhand_book_024a.dds",
"Data/UI/item_icons/offhand_book_025.dds",
"Data/UI/item_icons/offhand_book_025a.dds",
"Data/UI/item_icons/offhand_book_026.dds",
"Data/UI/item_icons/offhand_book_026a.dds",
"Data/UI/item_icons/offhand_book_027.dds",
"Data/UI/item_icons/offhand_book_027a.dds",
"Data/UI/item_icons/offhand_crystal_001.dds",
"Data/UI/item_icons/offhand_crystal_001a.dds",
"Data/UI/item_icons/offhand_crystal_002.dds",
"Data/UI/item_icons/offhand_crystal_002a.dds",
"Data/UI/item_icons/offhand_crystal_003.dds",
"Data/UI/item_icons/offhand_crystal_003a.dds",
"Data/UI/item_icons/offhand_crystal_004.dds",
"Data/UI/item_icons/offhand_crystal_004a.dds",
"Data/UI/item_icons/offhand_crystal_005.dds",
"Data/UI/item_icons/offhand_crystal_005a.dds",
"Data/UI/item_icons/offhand_crystal_006.dds",
"Data/UI/item_icons/offhand_crystal_006a.dds",
"Data/UI/item_icons/offhand_crystal_007.dds",
"Data/UI/item_icons/offhand_crystal_007a.dds",
"Data/UI/item_icons/offhand_crystal_008.dds",
"Data/UI/item_icons/offhand_crystal_008a.dds",
"Data/UI/item_icons/offhand_crystal_009.dds",
"Data/UI/item_icons/offhand_crystal_009a.dds",
"Data/UI/item_icons/offhand_crystal_010.dds",
"Data/UI/item_icons/offhand_crystal_010a.dds",
"Data/UI/item_icons/offhand_crystal_011.dds",
"Data/UI/item_icons/offhand_crystal_011a.dds",
"Data/UI/item_icons/offhand_life_01.dds",
"Data/UI/item_icons/offhand_life_01_a.dds",
"Data/UI/item_icons/offhand_skull_001.dds",
"Data/UI/item_icons/offhand_skull_001a.dds",
"Data/UI/item_icons/offhand_skull_002.dds",
"Data/UI/item_icons/offhand_skull_002a.dds",
"Data/UI/item_icons/offhand_skull_003.dds",
"Data/UI/item_icons/offhand_skull_003a.dds",
"Data/UI/item_icons/offhand_skull_004.dds",
"Data/UI/item_icons/offhand_skull_004a.dds",
"Data/UI/item_icons/offhand_skull_005.dds",
"Data/UI/item_icons/offhand_skull_005a.dds",
"Data/UI/item_icons/offhand_skull_006.dds",
"Data/UI/item_icons/offhand_skull_006a.dds",
"Data/UI/item_icons/offhand_skull_007.dds",
"Data/UI/item_icons/offhand_skull_007a.dds",
"Data/UI/item_icons/offhand_skull_008.dds",
"Data/UI/item_icons/offhand_skull_008a.dds",
"Data/UI/item_icons/old_dragonian_artefact.dds",
"Data/UI/item_icons/old_dwarf_artefact.dds",
"Data/UI/item_icons/old_dwarf_artefact_a.dds",
"Data/UI/item_icons/old_elven_artefact.dds",
"Data/UI/item_icons/old_freemarch_artifact.dds",
"Data/UI/item_icons/old_freemarch_artifact_a.dds",
"Data/UI/item_icons/old_human_artefact.dds",
"Data/UI/item_icons/old_orc_artefact.dds",
"Data/UI/item_icons/old_peanut_brittle.dds",
"Data/UI/item_icons/old_tabard_of_march.dds",
"Data/UI/item_icons/old_tabard_of_march_a.dds",
"Data/UI/item_icons/old_thorne_fetter.dds",
"Data/UI/item_icons/on_nedra.dds",
"Data/UI/item_icons/orange.dds",
"Data/UI/item_icons/ore02.dds",
"Data/UI/item_icons/ore02a.dds",
"Data/UI/item_icons/ore02b.dds",
"Data/UI/item_icons/ore03.dds",
"Data/UI/item_icons/ore03a.dds",
"Data/UI/item_icons/ore03b.dds",
"Data/UI/item_icons/ore04.dds",
"Data/UI/item_icons/ore04a.dds",
"Data/UI/item_icons/ore04b.dds",
"Data/UI/item_icons/ore05.dds",
"Data/UI/item_icons/ore06.dds",
"Data/UI/item_icons/ore06a.dds",
"Data/UI/item_icons/ore07.dds",
"Data/UI/item_icons/ore08.dds",
"Data/UI/item_icons/ore09.dds",
"Data/UI/item_icons/ore10.dds",
"Data/UI/item_icons/ore10a.dds",
"Data/UI/item_icons/ore11a.dds",
"Data/UI/item_icons/ore11b.dds",
"Data/UI/item_icons/ore12.dds",
"Data/UI/item_icons/ore13.dds",
"Data/UI/item_icons/ore13a.dds",
"Data/UI/item_icons/ore13b.dds",
"Data/UI/item_icons/ore14b.dds",
"Data/UI/item_icons/ore14c.dds",
"Data/UI/item_icons/ore16.dds",
"Data/UI/item_icons/ore16b.dds",
"Data/UI/item_icons/ore17.dds",
"Data/UI/item_icons/orichaliumbar_a.dds",
"Data/UI/item_icons/orichaliumbar_b.dds",
"Data/UI/item_icons/ornate_hairpin.dds",
"Data/UI/item_icons/padding_01.dds",
"Data/UI/item_icons/padding_01_a.dds",
"Data/UI/item_icons/padding_02.dds",
"Data/UI/item_icons/padding_02_a.dds",
"Data/UI/item_icons/padding_02_b.dds",
"Data/UI/item_icons/padding_03.dds",
"Data/UI/item_icons/padding_03_a.dds",
"Data/UI/item_icons/padding_04.dds",
"Data/UI/item_icons/padding_04_a.dds",
"Data/UI/item_icons/page_42.dds",
"Data/UI/item_icons/page_42_a.dds",
"Data/UI/item_icons/pale_flesh.dds",
"Data/UI/item_icons/pale_flesh_a.dds",
"Data/UI/item_icons/pale_flesh_b.dds",
"Data/UI/item_icons/parasitic_album_vine.dds",
"Data/UI/item_icons/path_of_valiant.dds",
"Data/UI/item_icons/pear.dds",
"Data/UI/item_icons/pear_a.dds",
"Data/UI/item_icons/petrified_bark.dds",
"Data/UI/item_icons/pet_badger.dds",
"Data/UI/item_icons/pet_badger_a.dds",
"Data/UI/item_icons/pet_badger_b.dds",
"Data/UI/item_icons/pet_badger_c.dds",
"Data/UI/item_icons/pet_badger_d_48.dds",
"Data/UI/item_icons/pet_cat.dds",
"Data/UI/item_icons/pet_cat_a.dds",
"Data/UI/item_icons/pet_cat_b.dds",
"Data/UI/item_icons/pet_cat_c.dds",
"Data/UI/item_icons/pet_cat_d.dds",
"Data/UI/item_icons/pet_cobra.dds",
"Data/UI/item_icons/pet_cobra_b.dds",
"Data/UI/item_icons/pet_corgi.dds",
"Data/UI/item_icons/pet_corgi_a.dds",
"Data/UI/item_icons/pet_coyote.dds",
"Data/UI/item_icons/pet_coyote_a.dds",
"Data/UI/item_icons/pet_coyote_b.dds",
"Data/UI/item_icons/pet_coyote_c.dds",
"Data/UI/item_icons/pet_crab.dds",
"Data/UI/item_icons/pet_crab_a.dds",
"Data/UI/item_icons/pet_crab_b.dds",
"Data/UI/item_icons/pet_crab_c.dds",
"Data/UI/item_icons/pet_deer.dds",
"Data/UI/item_icons/pet_deer_a.dds",
"Data/UI/item_icons/pet_deer_b.dds",
"Data/UI/item_icons/pet_deer_c.dds",
"Data/UI/item_icons/pet_dire-wolf.dds",
"Data/UI/item_icons/pet_dire-wolf_a.dds",
"Data/UI/item_icons/pet_fox.dds",
"Data/UI/item_icons/pet_fox_a.dds",
"Data/UI/item_icons/pet_fox_b.dds",
"Data/UI/item_icons/pet_frog.dds",
"Data/UI/item_icons/pet_frog_a.dds",
"Data/UI/item_icons/pet_frog_b.dds",
"Data/UI/item_icons/pet_gold_oreling.dds",
"Data/UI/item_icons/pet_gold_oreling_a.dds",
"Data/UI/item_icons/pet_lizard.dds",
"Data/UI/item_icons/pet_lizard_a.dds",
"Data/UI/item_icons/pet_lizard_b.dds",
"Data/UI/item_icons/pet_otter.dds",
"Data/UI/item_icons/pet_otter_a.dds",
"Data/UI/item_icons/pet_otter_b.dds",
"Data/UI/item_icons/pet_owl.dds",
"Data/UI/item_icons/pet_owl_a.dds",
"Data/UI/item_icons/pet_owl_b.dds",
"Data/UI/item_icons/pet_raptor.dds",
"Data/UI/item_icons/pet_raptor_a.dds",
"Data/UI/item_icons/pet_rat.dds",
"Data/UI/item_icons/pet_rat_a.dds",
"Data/UI/item_icons/pet_rat_b.dds",
"Data/UI/item_icons/pet_razorbeast.dds",
"Data/UI/item_icons/pet_razorbeast_a.dds",
"Data/UI/item_icons/pet_scarab.dds",
"Data/UI/item_icons/pet_scarab_a.dds",
"Data/UI/item_icons/pet_scarab_b.dds",
"Data/UI/item_icons/pet_scorpion.dds",
"Data/UI/item_icons/pet_scorpion_a.dds",
"Data/UI/item_icons/pet_scorpion_b.dds",
"Data/UI/item_icons/pet_scorpion_c.dds",
"Data/UI/item_icons/pet_scorpion_y.dds",
"Data/UI/item_icons/pet_scorpion_z.dds",
"Data/UI/item_icons/pet_shadowstalker.dds",
"Data/UI/item_icons/pet_shambler.dds",
"Data/UI/item_icons/pet_shambler_a.dds",
"Data/UI/item_icons/pet_sheep.dds",
"Data/UI/item_icons/pet_sheep_a.dds",
"Data/UI/item_icons/pet_sheep_b.dds",
"Data/UI/item_icons/pet_sheep_y.dds",
"Data/UI/item_icons/pet_sheep_z.dds",
"Data/UI/item_icons/pet_silver_oreling.dds",
"Data/UI/item_icons/pet_silver_oreling_a.dds",
"Data/UI/item_icons/pet_squirell.dds",
"Data/UI/item_icons/pet_squirell_a.dds",
"Data/UI/item_icons/pet_squirell_b.dds",
"Data/UI/item_icons/pick1.dds",
"Data/UI/item_icons/pick2.dds",
"Data/UI/item_icons/pick3.dds",
"Data/UI/item_icons/pick4.dds",
"Data/UI/item_icons/pick4_a.dds",
"Data/UI/item_icons/pick5.dds",
"Data/UI/item_icons/pick6.dds",
"Data/UI/item_icons/pick7.dds",
"Data/UI/item_icons/pick8.dds",
"Data/UI/item_icons/Pickled_HnG_pie.dds",
"Data/UI/item_icons/pickled_spider_egg.dds",
"Data/UI/item_icons/pirate_costume.dds",
"Data/UI/item_icons/pirate_costume_a.dds",
"Data/UI/item_icons/pirate_hat.dds",
"Data/UI/item_icons/pitch_covered_brush.dds",
"Data/UI/item_icons/pitch_covered_brush_a.dds",
"Data/UI/item_icons/pitch_covered_brush_b.dds",
"Data/UI/item_icons/planarite_gem.dds",
"Data/UI/item_icons/planarite_gem_a.dds",
"Data/UI/item_icons/planarite_gem_b.dds",
"Data/UI/item_icons/planarite_gem_c.dds",
"Data/UI/item_icons/planarite_orb_a.dds",
"Data/UI/item_icons/planarite_orb_b.dds",
"Data/UI/item_icons/planarite_prism.dds",
"Data/UI/item_icons/planarite_prism_a.dds",
"Data/UI/item_icons/planarite_prism_b.dds",
"Data/UI/item_icons/plane_of_life_1h_axe.dds",
"Data/UI/item_icons/plane_of_life_1h_dagger.dds",
"Data/UI/item_icons/plane_of_life_1h_sword.dds",
"Data/UI/item_icons/plane_of_life_2h_polearm.dds",
"Data/UI/item_icons/plant1.dds",
"Data/UI/item_icons/plant10a.dds",
"Data/UI/item_icons/plant11.dds",
"Data/UI/item_icons/plant12.dds",
"Data/UI/item_icons/plant12a.dds",
"Data/UI/item_icons/plant13.dds",
"Data/UI/item_icons/plant13b.dds",
"Data/UI/item_icons/plant13d.dds",
"Data/UI/item_icons/plant14.dds",
"Data/UI/item_icons/plant14a.dds",
"Data/UI/item_icons/plant15.dds",
"Data/UI/item_icons/plant16.dds",
"Data/UI/item_icons/plant16a.dds",
"Data/UI/item_icons/plant17.dds",
"Data/UI/item_icons/plant18.dds",
"Data/UI/item_icons/plant18a.dds",
"Data/UI/item_icons/plant19a.dds",
"Data/UI/item_icons/plant19b.dds",
"Data/UI/item_icons/plant19c.dds",
"Data/UI/item_icons/plant2.dds",
"Data/UI/item_icons/plant20.dds",
"Data/UI/item_icons/plant21.dds",
"Data/UI/item_icons/plant22.dds",
"Data/UI/item_icons/plant23.dds",
"Data/UI/item_icons/plant24.dds",
"Data/UI/item_icons/plant24b.dds",
"Data/UI/item_icons/plant24c.dds",
"Data/UI/item_icons/plant25a.dds",
"Data/UI/item_icons/plant26.dds",
"Data/UI/item_icons/plant26a.dds",
"Data/UI/item_icons/plant26_x.dds",
"Data/UI/item_icons/plant27a.dds",
"Data/UI/item_icons/plant27b.dds",
"Data/UI/item_icons/plant27c.dds",
"Data/UI/item_icons/plant28a.dds",
"Data/UI/item_icons/plant28b.dds",
"Data/UI/item_icons/plant28c.dds",
"Data/UI/item_icons/plant28d.dds",
"Data/UI/item_icons/plant29.dds",
"Data/UI/item_icons/plant2a.dds",
"Data/UI/item_icons/plant2b.dds",
"Data/UI/item_icons/plant3.dds",
"Data/UI/item_icons/plant30.dds",
"Data/UI/item_icons/plant30a.dds",
"Data/UI/item_icons/plant30_x.dds",
"Data/UI/item_icons/plant31.dds",
"Data/UI/item_icons/plant31a.dds",
"Data/UI/item_icons/plant31aa.dds",
"Data/UI/item_icons/plant32.dds",
"Data/UI/item_icons/plant33b.dds",
"Data/UI/item_icons/plant34.dds",
"Data/UI/item_icons/plant35.dds",
"Data/UI/item_icons/plant35b.dds",
"Data/UI/item_icons/plant35c.dds",
"Data/UI/item_icons/plant36.dds",
"Data/UI/item_icons/plant36a.dds",
"Data/UI/item_icons/plant36b.dds",
"Data/UI/item_icons/plant36c.dds",
"Data/UI/item_icons/plant36d.dds",
"Data/UI/item_icons/plant37.dds",
"Data/UI/item_icons/plant37a.dds",
"Data/UI/item_icons/plant37b.dds",
"Data/UI/item_icons/plant38b.dds",
"Data/UI/item_icons/plant38c.dds",
"Data/UI/item_icons/plant39.dds",
"Data/UI/item_icons/plant39a.dds",
"Data/UI/item_icons/plant39b.dds",
"Data/UI/item_icons/plant3a.dds",
"Data/UI/item_icons/plant3b.dds",
"Data/UI/item_icons/plant4.dds",
"Data/UI/item_icons/plant40.dds",
"Data/UI/item_icons/plant40a.dds",
"Data/UI/item_icons/plant40b.dds",
"Data/UI/item_icons/plant40c.dds",
"Data/UI/item_icons/plant41.dds",
"Data/UI/item_icons/plant41a.dds",
"Data/UI/item_icons/plant41b.dds",
"Data/UI/item_icons/plant42.dds",
"Data/UI/item_icons/plant42a.dds",
"Data/UI/item_icons/plant42b.dds",
"Data/UI/item_icons/plant43.dds",
"Data/UI/item_icons/plant43d.dds",
"Data/UI/item_icons/plant47.dds",
"Data/UI/item_icons/plant47a.dds",
"Data/UI/item_icons/plant47w.dds",
"Data/UI/item_icons/plant47y.dds",
"Data/UI/item_icons/plant47z.dds",
"Data/UI/item_icons/plant48.dds",
"Data/UI/item_icons/plant48c.dds",
"Data/UI/item_icons/plant48d.dds",
"Data/UI/item_icons/plant49.dds",
"Data/UI/item_icons/plant49c.dds",
"Data/UI/item_icons/plant5.dds",
"Data/UI/item_icons/plant50.dds",
"Data/UI/item_icons/plant6.dds",
"Data/UI/item_icons/plant7.dds",
"Data/UI/item_icons/plant7a.dds",
"Data/UI/item_icons/plant7b.dds",
"Data/UI/item_icons/plant7c.dds",
"Data/UI/item_icons/plant7c_x.dds",
"Data/UI/item_icons/plant8.dds",
"Data/UI/item_icons/plaque_of_achievement.dds",
"Data/UI/item_icons/plate_000_boots.dds",
"Data/UI/item_icons/plate_000_boots_a.dds",
"Data/UI/item_icons/plate_000_chest.dds",
"Data/UI/item_icons/plate_000_chest_a.dds",
"Data/UI/item_icons/plate_000_leggings.dds",
"Data/UI/item_icons/plate_000_leggings_a.dds",
"Data/UI/item_icons/plate_100_belt.dds",
"Data/UI/item_icons/plate_100_boots.dds",
"Data/UI/item_icons/plate_100_chest.dds",
"Data/UI/item_icons/plate_100_chest_a.dds",
"Data/UI/item_icons/plate_100_leggings.dds",
"Data/UI/item_icons/plate_100_shoulder.dds",
"Data/UI/item_icons/plate_100_shoulder_a.dds",
"Data/UI/item_icons/plate_101_belt.dds",
"Data/UI/item_icons/plate_101_boots.dds",
"Data/UI/item_icons/plate_101_chest.dds",
"Data/UI/item_icons/plate_101_gloves.dds",
"Data/UI/item_icons/plate_101_helm.dds",
"Data/UI/item_icons/plate_101_helm_a.dds",
"Data/UI/item_icons/plate_101_leggings.dds",
"Data/UI/item_icons/plate_101_leggings_a.dds",
"Data/UI/item_icons/plate_101_shoulders.dds",
"Data/UI/item_icons/plate_101_shoulders2.dds",
"Data/UI/item_icons/plate_102_belt.dds",
"Data/UI/item_icons/plate_102_belt_a.dds",
"Data/UI/item_icons/plate_102_boots.dds",
"Data/UI/item_icons/plate_102_boots_a.dds",
"Data/UI/item_icons/plate_102_chest.dds",
"Data/UI/item_icons/plate_102_chest_a.dds",
"Data/UI/item_icons/plate_102_chest_b.dds",
"Data/UI/item_icons/plate_102_gloves.dds",
"Data/UI/item_icons/plate_102_gloves_a.dds",
"Data/UI/item_icons/plate_102_gloves_b.dds",
"Data/UI/item_icons/plate_102_helm.dds",
"Data/UI/item_icons/plate_102_helm_a.dds",
"Data/UI/item_icons/plate_102_leggings.dds",
"Data/UI/item_icons/plate_102_leggings_a.dds",
"Data/UI/item_icons/plate_102_shoulders.dds",
"Data/UI/item_icons/plate_102_shoulders_a.dds",
"Data/UI/item_icons/plate_103a_helm.dds",
"Data/UI/item_icons/plate_103a_helm_a.dds",
"Data/UI/item_icons/plate_103a_shoulder.dds",
"Data/UI/item_icons/plate_103a_shoulder_a.dds",
"Data/UI/item_icons/plate_103_belt.dds",
"Data/UI/item_icons/plate_103_belt_a.dds",
"Data/UI/item_icons/plate_103_boots.dds",
"Data/UI/item_icons/plate_103_boots_a.dds",
"Data/UI/item_icons/plate_103_chest.dds",
"Data/UI/item_icons/plate_103_chest_a.dds",
"Data/UI/item_icons/plate_103_gloves.dds",
"Data/UI/item_icons/plate_103_gloves_a.dds",
"Data/UI/item_icons/plate_103_helm.dds",
"Data/UI/item_icons/plate_103_helm_a.dds",
"Data/UI/item_icons/plate_103_leggings.dds",
"Data/UI/item_icons/plate_103_leggings_a.dds",
"Data/UI/item_icons/plate_103_shoulders.dds",
"Data/UI/item_icons/plate_103_shoulders_a.dds",
"Data/UI/item_icons/plate_104_belt.dds",
"Data/UI/item_icons/plate_104_boots.dds",
"Data/UI/item_icons/plate_104_chest.dds",
"Data/UI/item_icons/plate_104_gloves.dds",
"Data/UI/item_icons/plate_104_helm.dds",
"Data/UI/item_icons/plate_104_leggings.dds",
"Data/UI/item_icons/plate_104_shoulders.dds",
"Data/UI/item_icons/plate_105_belt.dds",
"Data/UI/item_icons/plate_105_boots.dds",
"Data/UI/item_icons/plate_105_chest.dds",
"Data/UI/item_icons/plate_105_gloves.dds",
"Data/UI/item_icons/plate_105_helm.dds",
"Data/UI/item_icons/plate_105_leggings.dds",
"Data/UI/item_icons/plate_105_shoulders.dds",
"Data/UI/item_icons/plate_106c_helm.dds",
"Data/UI/item_icons/plate_106c_helm_a.dds",
"Data/UI/item_icons/plate_106c_shoulder.dds",
"Data/UI/item_icons/plate_106c_shoulder_a.dds",
"Data/UI/item_icons/plate_106_belt.dds",
"Data/UI/item_icons/plate_106_boots.dds",
"Data/UI/item_icons/plate_106_chest.dds",
"Data/UI/item_icons/plate_106_gloves.dds",
"Data/UI/item_icons/plate_106_helm.dds",
"Data/UI/item_icons/plate_106_leggings.dds",
"Data/UI/item_icons/plate_106_shoulders.dds",
"Data/UI/item_icons/plate_107_belt.dds",
"Data/UI/item_icons/plate_107_boots.dds",
"Data/UI/item_icons/plate_107_chest.dds",
"Data/UI/item_icons/plate_107_gloves.dds",
"Data/UI/item_icons/plate_107_helm.dds",
"Data/UI/item_icons/plate_107_leggings.dds",
"Data/UI/item_icons/plate_107_shoulders.dds",
"Data/UI/item_icons/plate_108_belt.dds",
"Data/UI/item_icons/plate_108_boots.dds",
"Data/UI/item_icons/plate_108_chest.dds",
"Data/UI/item_icons/plate_108_gloves.dds",
"Data/UI/item_icons/plate_108_helm.dds",
"Data/UI/item_icons/plate_108_leggings.dds",
"Data/UI/item_icons/plate_108_shoulders.dds",
"Data/UI/item_icons/plate_109_belt.dds",
"Data/UI/item_icons/plate_109_boots.dds",
"Data/UI/item_icons/plate_109_chest.dds",
"Data/UI/item_icons/plate_109_gloves.dds",
"Data/UI/item_icons/plate_109_helm.dds",
"Data/UI/item_icons/plate_109_leggings.dds",
"Data/UI/item_icons/plate_109_shoulders.dds",
"Data/UI/item_icons/plate_110a_helm.dds",
"Data/UI/item_icons/plate_110a_helm_a.dds",
"Data/UI/item_icons/plate_110a_shoulder.dds",
"Data/UI/item_icons/plate_110a_shoulder_a.dds",
"Data/UI/item_icons/plate_110b_belt.dds",
"Data/UI/item_icons/plate_110b_belt_a.dds",
"Data/UI/item_icons/plate_110b_boots.dds",
"Data/UI/item_icons/plate_110b_boots_a.dds",
"Data/UI/item_icons/plate_110b_chest.dds",
"Data/UI/item_icons/plate_110b_chest_a.dds",
"Data/UI/item_icons/plate_110b_gloves.dds",
"Data/UI/item_icons/plate_110b_gloves_a.dds",
"Data/UI/item_icons/plate_110b_helm.dds",
"Data/UI/item_icons/plate_110b_helm_a.dds",
"Data/UI/item_icons/plate_110b_leggings.dds",
"Data/UI/item_icons/plate_110b_leggings_a.dds",
"Data/UI/item_icons/plate_110b_shoulder.dds",
"Data/UI/item_icons/plate_110b_shoulder_a.dds",
"Data/UI/item_icons/plate_110_belt.dds",
"Data/UI/item_icons/plate_110_belt_a.dds",
"Data/UI/item_icons/plate_110_boots.dds",
"Data/UI/item_icons/plate_110_boots_a.dds",
"Data/UI/item_icons/plate_110_chest.dds",
"Data/UI/item_icons/plate_110_chest_a.dds",
"Data/UI/item_icons/plate_110_gloves.dds",
"Data/UI/item_icons/plate_110_gloves_a.dds",
"Data/UI/item_icons/plate_110_helm.dds",
"Data/UI/item_icons/plate_110_helm_a.dds",
"Data/UI/item_icons/plate_110_leggings.dds",
"Data/UI/item_icons/plate_110_leggings_a.dds",
"Data/UI/item_icons/plate_110_shoulders.dds",
"Data/UI/item_icons/plate_110_shoulders_a.dds",
"Data/UI/item_icons/plate_111_belt.dds",
"Data/UI/item_icons/plate_111_belt_a.dds",
"Data/UI/item_icons/plate_111_boots.dds",
"Data/UI/item_icons/plate_111_boots_a.dds",
"Data/UI/item_icons/plate_111_chest.dds",
"Data/UI/item_icons/plate_111_chest_a.dds",
"Data/UI/item_icons/plate_111_gloves.dds",
"Data/UI/item_icons/plate_111_gloves_a.dds",
"Data/UI/item_icons/plate_111_helm.dds",
"Data/UI/item_icons/plate_111_helm_a.dds",
"Data/UI/item_icons/plate_111_leggings.dds",
"Data/UI/item_icons/plate_111_leggings_a.dds",
"Data/UI/item_icons/plate_111_shoulders.dds",
"Data/UI/item_icons/plate_111_shoulders_a.dds",
"Data/UI/item_icons/plate_112_belt.dds",
"Data/UI/item_icons/plate_112_boots.dds",
"Data/UI/item_icons/plate_112_chest.dds",
"Data/UI/item_icons/plate_112_gloves.dds",
"Data/UI/item_icons/plate_112_helm.dds",
"Data/UI/item_icons/plate_112_leggings.dds",
"Data/UI/item_icons/plate_112_shoulders.dds",
"Data/UI/item_icons/plate_113_A_belt.dds",
"Data/UI/item_icons/plate_113_A_belt_a.dds",
"Data/UI/item_icons/plate_113_A_boots.dds",
"Data/UI/item_icons/plate_113_A_boots_a.dds",
"Data/UI/item_icons/plate_113_A_chest.dds",
"Data/UI/item_icons/plate_113_A_chest_a.dds",
"Data/UI/item_icons/plate_113_A_gloves.dds",
"Data/UI/item_icons/plate_113_A_gloves_a.dds",
"Data/UI/item_icons/plate_113_A_helm.dds",
"Data/UI/item_icons/plate_113_A_helm_a.dds",
"Data/UI/item_icons/plate_113_A_leggings.dds",
"Data/UI/item_icons/plate_113_A_leggings_a.dds",
"Data/UI/item_icons/plate_113_A_shoulders.dds",
"Data/UI/item_icons/plate_113_A_shoulders_a.dds",
"Data/UI/item_icons/plate_113_belt.dds",
"Data/UI/item_icons/plate_113_boots.dds",
"Data/UI/item_icons/plate_113_B_belt.dds",
"Data/UI/item_icons/plate_113_B_belt_a.dds",
"Data/UI/item_icons/plate_113_B_boots.dds",
"Data/UI/item_icons/plate_113_B_boots_a.dds",
"Data/UI/item_icons/plate_113_B_chest.dds",
"Data/UI/item_icons/plate_113_B_chest_a.dds",
"Data/UI/item_icons/plate_113_B_gloves.dds",
"Data/UI/item_icons/plate_113_B_gloves_a.dds",
"Data/UI/item_icons/plate_113_B_helm.dds",
"Data/UI/item_icons/plate_113_B_helm_a.dds",
"Data/UI/item_icons/plate_113_B_leggings.dds",
"Data/UI/item_icons/plate_113_B_leggings_a.dds",
"Data/UI/item_icons/plate_113_B_shoulders.dds",
"Data/UI/item_icons/plate_113_B_shoulders_a.dds",
"Data/UI/item_icons/plate_113_chest.dds",
"Data/UI/item_icons/plate_113_C_belt.dds",
"Data/UI/item_icons/plate_113_C_belt_a.dds",
"Data/UI/item_icons/plate_113_C_boots.dds",
"Data/UI/item_icons/plate_113_C_boots_a.dds",
"Data/UI/item_icons/plate_113_C_chest.dds",
"Data/UI/item_icons/plate_113_C_chest_a.dds",
"Data/UI/item_icons/plate_113_C_gloves.dds",
"Data/UI/item_icons/plate_113_C_gloves_a.dds",
"Data/UI/item_icons/plate_113_C_helm.dds",
"Data/UI/item_icons/plate_113_C_helm_a.dds",
"Data/UI/item_icons/plate_113_C_leggings.dds",
"Data/UI/item_icons/plate_113_C_leggings_a.dds",
"Data/UI/item_icons/plate_113_C_shoulders.dds",
"Data/UI/item_icons/plate_113_C_shoulders_a.dds",
"Data/UI/item_icons/plate_113_D_belt.dds",
"Data/UI/item_icons/plate_113_D_belt_a.dds",
"Data/UI/item_icons/plate_113_D_boots.dds",
"Data/UI/item_icons/plate_113_D_boots_a.dds",
"Data/UI/item_icons/plate_113_D_chest.dds",
"Data/UI/item_icons/plate_113_D_chest_a.dds",
"Data/UI/item_icons/plate_113_D_gloves.dds",
"Data/UI/item_icons/plate_113_D_gloves_a.dds",
"Data/UI/item_icons/plate_113_D_helm.dds",
"Data/UI/item_icons/plate_113_D_helm_a.dds",
"Data/UI/item_icons/plate_113_D_leggings.dds",
"Data/UI/item_icons/plate_113_D_leggings_a.dds",
"Data/UI/item_icons/plate_113_D_shoulders.dds",
"Data/UI/item_icons/plate_113_D_shoulders_a.dds",
"Data/UI/item_icons/plate_113_E_belt.dds",
"Data/UI/item_icons/plate_113_E_belt_a.dds",
"Data/UI/item_icons/plate_113_E_boots.dds",
"Data/UI/item_icons/plate_113_E_boots_a.dds",
"Data/UI/item_icons/plate_113_E_chest.dds",
"Data/UI/item_icons/plate_113_E_chest_a.dds",
"Data/UI/item_icons/plate_113_E_gloves.dds",
"Data/UI/item_icons/plate_113_E_gloves_a.dds",
"Data/UI/item_icons/plate_113_E_helm.dds",
"Data/UI/item_icons/plate_113_E_helm_a.dds",
"Data/UI/item_icons/plate_113_E_leggings.dds",
"Data/UI/item_icons/plate_113_E_leggings_a.dds",
"Data/UI/item_icons/plate_113_E_shoulders.dds",
"Data/UI/item_icons/plate_113_E_shoulders_a.dds",
"Data/UI/item_icons/plate_113_gloves.dds",
"Data/UI/item_icons/plate_113_helm.dds",
"Data/UI/item_icons/plate_113_leggings.dds",
"Data/UI/item_icons/plate_113_shoulders.dds",
"Data/UI/item_icons/plate_114a_belt.dds",
"Data/UI/item_icons/plate_114a_belt_a.dds",
"Data/UI/item_icons/plate_114a_boots.dds",
"Data/UI/item_icons/plate_114a_boots_a.dds",
"Data/UI/item_icons/plate_114a_chest.dds",
"Data/UI/item_icons/plate_114a_chest_a.dds",
"Data/UI/item_icons/plate_114a_gloves.dds",
"Data/UI/item_icons/plate_114a_gloves_a.dds",
"Data/UI/item_icons/plate_114a_helm.dds",
"Data/UI/item_icons/plate_114a_helm_a.dds",
"Data/UI/item_icons/plate_114a_leggings.dds",
"Data/UI/item_icons/plate_114a_leggings_a.dds",
"Data/UI/item_icons/plate_114a_shoulders.dds",
"Data/UI/item_icons/plate_114a_shoulders_a.dds",
"Data/UI/item_icons/plate_114_belt.dds",
"Data/UI/item_icons/plate_114_belt_a.dds",
"Data/UI/item_icons/plate_114_boots.dds",
"Data/UI/item_icons/plate_114_boots_a.dds",
"Data/UI/item_icons/plate_114_chest.dds",
"Data/UI/item_icons/plate_114_chest_a.dds",
"Data/UI/item_icons/plate_114_gloves.dds",
"Data/UI/item_icons/plate_114_gloves_a.dds",
"Data/UI/item_icons/plate_114_helm.dds",
"Data/UI/item_icons/plate_114_helm_a.dds",
"Data/UI/item_icons/plate_114_leggings.dds",
"Data/UI/item_icons/plate_114_leggings_a.dds",
"Data/UI/item_icons/plate_114_shoulders.dds",
"Data/UI/item_icons/plate_114_shoulders_a.dds",
"Data/UI/item_icons/plate_115a_belt.dds",
"Data/UI/item_icons/plate_115a_belt_a.dds",
"Data/UI/item_icons/plate_115a_boots.dds",
"Data/UI/item_icons/plate_115a_boots_a.dds",
"Data/UI/item_icons/plate_115a_chest.dds",
"Data/UI/item_icons/plate_115a_chest_a.dds",
"Data/UI/item_icons/plate_115a_gloves.dds",
"Data/UI/item_icons/plate_115a_gloves_a.dds",
"Data/UI/item_icons/plate_115a_helm.dds",
"Data/UI/item_icons/plate_115a_helm_a.dds",
"Data/UI/item_icons/plate_115a_leggings.dds",
"Data/UI/item_icons/plate_115a_leggings_a.dds",
"Data/UI/item_icons/plate_115a_shoulders.dds",
"Data/UI/item_icons/plate_115a_shoulders_a.dds",
"Data/UI/item_icons/plate_115_belt.dds",
"Data/UI/item_icons/plate_115_belt_a.dds",
"Data/UI/item_icons/plate_115_boots.dds",
"Data/UI/item_icons/plate_115_boots_a.dds",
"Data/UI/item_icons/plate_115_chest.dds",
"Data/UI/item_icons/plate_115_chest_a.dds",
"Data/UI/item_icons/plate_115_gloves.dds",
"Data/UI/item_icons/plate_115_gloves_a.dds",
"Data/UI/item_icons/plate_115_helm.dds",
"Data/UI/item_icons/plate_115_helm_a.dds",
"Data/UI/item_icons/plate_115_leggings.dds",
"Data/UI/item_icons/plate_115_leggings_a.dds",
"Data/UI/item_icons/plate_115_shoulders.dds",
"Data/UI/item_icons/plate_115_shoulders_a.dds",
"Data/UI/item_icons/plate_115_shoulders_b.dds",
"Data/UI/item_icons/plate_116_belt.dds",
"Data/UI/item_icons/plate_116_belt_a.dds",
"Data/UI/item_icons/plate_116_boots.dds",
"Data/UI/item_icons/plate_116_boots_a.dds",
"Data/UI/item_icons/plate_116_chest.dds",
"Data/UI/item_icons/plate_116_chest_a.dds",
"Data/UI/item_icons/plate_116_gloves.dds",
"Data/UI/item_icons/plate_116_gloves_a.dds",
"Data/UI/item_icons/plate_116_helm.dds",
"Data/UI/item_icons/plate_116_helm_a.dds",
"Data/UI/item_icons/plate_116_leggings.dds",
"Data/UI/item_icons/plate_116_leggings_a.dds",
"Data/UI/item_icons/plate_116_shoulder.dds",
"Data/UI/item_icons/plate_116_shoulder_a.dds",
"Data/UI/item_icons/plate_117_boots.dds",
"Data/UI/item_icons/plate_117_boots_a.dds",
"Data/UI/item_icons/plate_117_chest.dds",
"Data/UI/item_icons/plate_117_chest_a.dds",
"Data/UI/item_icons/plate_117_gloves.dds",
"Data/UI/item_icons/plate_117_gloves_a.dds",
"Data/UI/item_icons/plate_117_helm.dds",
"Data/UI/item_icons/plate_117_helm_a.dds",
"Data/UI/item_icons/plate_117_leggings.dds",
"Data/UI/item_icons/plate_117_leggings_a.dds",
"Data/UI/item_icons/plate_117_shoulder.dds",
"Data/UI/item_icons/plate_117_shoulder_a.dds",
"Data/UI/item_icons/plate_118_boots.dds",
"Data/UI/item_icons/plate_118_boots_a.dds",
"Data/UI/item_icons/plate_118_chest.dds",
"Data/UI/item_icons/plate_118_chest_a.dds",
"Data/UI/item_icons/plate_118_gloves.dds",
"Data/UI/item_icons/plate_118_gloves_a.dds",
"Data/UI/item_icons/plate_118_helm.dds",
"Data/UI/item_icons/plate_118_helm_a.dds",
"Data/UI/item_icons/plate_118_leggings.dds",
"Data/UI/item_icons/plate_118_leggings_a.dds",
"Data/UI/item_icons/plate_118_shoulder.dds",
"Data/UI/item_icons/plate_118_shoulder_a.dds",
"Data/UI/item_icons/plate_119_belt.dds",
"Data/UI/item_icons/plate_119_belt_a.dds",
"Data/UI/item_icons/plate_119_belt_b.dds",
"Data/UI/item_icons/plate_119_boots.dds",
"Data/UI/item_icons/plate_119_boots_a.dds",
"Data/UI/item_icons/plate_119_chest.dds",
"Data/UI/item_icons/plate_119_chest_a.dds",
"Data/UI/item_icons/plate_119_gloves.dds",
"Data/UI/item_icons/plate_119_gloves_a.dds",
"Data/UI/item_icons/plate_119_helm.dds",
"Data/UI/item_icons/plate_119_helm_a.dds",
"Data/UI/item_icons/plate_119_helm_b.dds",
"Data/UI/item_icons/plate_119_helm_c.dds",
"Data/UI/item_icons/plate_119_leggings.dds",
"Data/UI/item_icons/plate_119_leggings_a.dds",
"Data/UI/item_icons/plate_119_shoulders.dds",
"Data/UI/item_icons/plate_119_shoulders_a.dds",
"Data/UI/item_icons/plate_120_belt.dds",
"Data/UI/item_icons/plate_120_belt_a.dds",
"Data/UI/item_icons/plate_120_boots.dds",
"Data/UI/item_icons/plate_120_boots_a.dds",
"Data/UI/item_icons/plate_120_chest.dds",
"Data/UI/item_icons/plate_120_chest_a.dds",
"Data/UI/item_icons/plate_120_gloves.dds",
"Data/UI/item_icons/plate_120_gloves_a.dds",
"Data/UI/item_icons/plate_120_helm.dds",
"Data/UI/item_icons/plate_120_helm_a.dds",
"Data/UI/item_icons/plate_120_leggings.dds",
"Data/UI/item_icons/plate_120_leggings_a.dds",
"Data/UI/item_icons/plate_120_shoulders.dds",
"Data/UI/item_icons/plate_120_shoulders_a.dds",
"Data/UI/item_icons/plate_120_shoulders_b.dds",
"Data/UI/item_icons/plate_120_shoulders_c.dds",
"Data/UI/item_icons/plate_121_belt.dds",
"Data/UI/item_icons/plate_121_belt_a.dds",
"Data/UI/item_icons/plate_121_boots.dds",
"Data/UI/item_icons/plate_121_boots_a.dds",
"Data/UI/item_icons/plate_121_chest.dds",
"Data/UI/item_icons/plate_121_chest_a.dds",
"Data/UI/item_icons/plate_121_gloves.dds",
"Data/UI/item_icons/plate_121_gloves_a.dds",
"Data/UI/item_icons/plate_121_helm.dds",
"Data/UI/item_icons/plate_121_helm_a.dds",
"Data/UI/item_icons/plate_121_leggings.dds",
"Data/UI/item_icons/plate_121_leggings_a.dds",
"Data/UI/item_icons/plate_121_shoulder.dds",
"Data/UI/item_icons/plate_121_shoulder_a.dds",
"Data/UI/item_icons/plate_122_belt.dds",
"Data/UI/item_icons/plate_122_belt_a.dds",
"Data/UI/item_icons/plate_122_boots.dds",
"Data/UI/item_icons/plate_122_boots_a.dds",
"Data/UI/item_icons/plate_122_chest.dds",
"Data/UI/item_icons/plate_122_chest_a.dds",
"Data/UI/item_icons/plate_122_gloves.dds",
"Data/UI/item_icons/plate_122_gloves_a.dds",
"Data/UI/item_icons/plate_122_helm.dds",
"Data/UI/item_icons/plate_122_helm_a.dds",
"Data/UI/item_icons/plate_122_leggings.dds",
"Data/UI/item_icons/plate_122_leggings_a.dds",
"Data/UI/item_icons/plate_122_shoulders.dds",
"Data/UI/item_icons/plate_122_shoulders_a.dds",
"Data/UI/item_icons/plate_belt_damaged_01.dds",
"Data/UI/item_icons/plate_belt_damaged_01_a.dds",
"Data/UI/item_icons/plate_boots_damaged_01.dds",
"Data/UI/item_icons/plate_boots_damaged_01_a.dds",
"Data/UI/item_icons/plate_chest_damaged_01.dds",
"Data/UI/item_icons/plate_chest_damaged_01_a.dds",
"Data/UI/item_icons/plate_gloves_damaged_01.dds",
"Data/UI/item_icons/plate_gloves_damaged_01_a.dds",
"Data/UI/item_icons/plate_helm_damaged_01.dds",
"Data/UI/item_icons/plate_helm_damaged_01_a.dds",
"Data/UI/item_icons/plate_leggings_damaged_01.dds",
"Data/UI/item_icons/plate_leggings_damaged_01_a.dds",
"Data/UI/item_icons/plate_shoulders_damaged_01.dds",
"Data/UI/item_icons/plate_shoulders_damaged_01_a.dds",
"Data/UI/item_icons/poison_gland.dds",
"Data/UI/item_icons/poison_gland_a.dds",
"Data/UI/item_icons/poison_sack.dds",
"Data/UI/item_icons/poison_sack_a.dds",
"Data/UI/item_icons/polished_stone_artifact.dds",
"Data/UI/item_icons/poorly_rigged_dice.dds",
"Data/UI/item_icons/pot_of_pea_soup.dds",
"Data/UI/item_icons/pot_of_pea_soup_a.dds",
"Data/UI/item_icons/pot_of_pea_soup_b.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_0.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_1.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_2.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_3.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_4.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_5.dds",
"Data/UI/item_icons/powerful_potion_cylindrical_6.dds",
"Data/UI/item_icons/powerful_potion_rectangular_0.dds",
"Data/UI/item_icons/powerful_potion_rectangular_1.dds",
"Data/UI/item_icons/powerful_potion_rectangular_2.dds",
"Data/UI/item_icons/powerful_potion_rectangular_3.dds",
"Data/UI/item_icons/powerful_potion_rectangular_4.dds",
"Data/UI/item_icons/powerful_potion_rectangular_5.dds",
"Data/UI/item_icons/powerful_potion_rectangular_6.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_0.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_1.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_2.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_3.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_4.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_5.dds",
"Data/UI/item_icons/powerful_potion_rectangular_a_6.dds",
"Data/UI/item_icons/powerful_potion_spherical_0.dds",
"Data/UI/item_icons/powerful_potion_spherical_1.dds",
"Data/UI/item_icons/powerful_potion_spherical_2.dds",
"Data/UI/item_icons/powerful_potion_spherical_3.dds",
"Data/UI/item_icons/powerful_potion_spherical_4.dds",
"Data/UI/item_icons/powerful_potion_spherical_5.dds",
"Data/UI/item_icons/powerful_potion_spherical_6.dds",
"Data/UI/item_icons/prisoners_disguise.dds",
"Data/UI/item_icons/prisoners_disguise_a.dds",
"Data/UI/item_icons/pungent_membrane.dds",
"Data/UI/item_icons/pungent_residue.dds",
"Data/UI/item_icons/pungent_shell.dds",
"Data/UI/item_icons/puresource_01_raw.dds",
"Data/UI/item_icons/puresource_01_raw_a.dds",
"Data/UI/item_icons/puresource_core_01.dds",
"Data/UI/item_icons/puresource_core_01_a.dds",
"Data/UI/item_icons/puresource_core_01_b.dds",
"Data/UI/item_icons/puresource_core_02.dds",
"Data/UI/item_icons/puresource_core_02_a.dds",
"Data/UI/item_icons/puresource_core_03.dds",
"Data/UI/item_icons/puresource_core_03_a.dds",
"Data/UI/item_icons/puresource_core_03_z.dds",
"Data/UI/item_icons/puresource_core_04.dds",
"Data/UI/item_icons/puresource_core_04z.dds",
"Data/UI/item_icons/puresource_core_04zz.dds",
"Data/UI/item_icons/puresource_core_04_a.dds",
"Data/UI/item_icons/puresource_core_04_b.dds",
"Data/UI/item_icons/puresource_core_04_c.dds",
"Data/UI/item_icons/puresource_core_04_d.dds",
"Data/UI/item_icons/puresource_core_05_a.dds",
"Data/UI/item_icons/puresource_core_05_b.dds",
"Data/UI/item_icons/puresource_raw_02.dds",
"Data/UI/item_icons/puresource_raw_03.dds",
"Data/UI/item_icons/puresource_raw_03_a.dds",
"Data/UI/item_icons/puresource_raw_04.dds",
"Data/UI/item_icons/puresource_raw_05.dds",
"Data/UI/item_icons/pure_honey.dds",
"Data/UI/item_icons/pure_honey_a.dds",
"Data/UI/item_icons/purple_grape.dds",
"Data/UI/item_icons/quiver_001.dds",
"Data/UI/item_icons/quiver_002.dds",
"Data/UI/item_icons/quiver_003.dds",
"Data/UI/item_icons/quiver_004.dds",
"Data/UI/item_icons/quiver_004a.dds",
"Data/UI/item_icons/quiver_005.dds",
"Data/UI/item_icons/quiver_006.dds",
"Data/UI/item_icons/quiver_007.dds",
"Data/UI/item_icons/quiver_earth_epic.dds",
"Data/UI/item_icons/quiver_earth_epic_a.dds",
"Data/UI/item_icons/quiver_earth_epic_b.dds",
"Data/UI/item_icons/quiver_fire_epic.dds",
"Data/UI/item_icons/quiver_fire_epic_a.dds",
"Data/UI/item_icons/quiver_fire_epic_b.dds",
"Data/UI/item_icons/quiver_life_01.dds",
"Data/UI/item_icons/quiver_life_01_a.dds",
"Data/UI/item_icons/quiver_water_mundane.dds",
"Data/UI/item_icons/quiver_water_mundane_a.dds",
"Data/UI/item_icons/quiver_water_mundane_b.dds",
"Data/UI/item_icons/quiver_water_mundane_c.dds",
"Data/UI/item_icons/rabbit_skeleton.dds",
"Data/UI/item_icons/radiant_diamond.dds",
"Data/UI/item_icons/radiant_diamond_a.dds",
"Data/UI/item_icons/radiant_diamond_b.dds",
"Data/UI/item_icons/radiant_diamond_z.dds",
"Data/UI/item_icons/radiant_diamond_z1.dds",
"Data/UI/item_icons/radiant_emerald.dds",
"Data/UI/item_icons/radiant_emerald_a.dds",
"Data/UI/item_icons/ragged_leather_b.dds",
"Data/UI/item_icons/ranged_bows_038.dds",
"Data/UI/item_icons/ranged_bows_038a.dds",
"Data/UI/item_icons/ranged_bows_039.dds",
"Data/UI/item_icons/ranged_bows_039a.dds",
"Data/UI/item_icons/ranged_bows_040.dds",
"Data/UI/item_icons/ranged_bows_040a.dds",
"Data/UI/item_icons/ranged_bows_041.dds",
"Data/UI/item_icons/ranged_bows_041a.dds",
"Data/UI/item_icons/ranged_bow_010.dds",
"Data/UI/item_icons/ranged_bow_011.dds",
"Data/UI/item_icons/ranged_bow_011_a.dds",
"Data/UI/item_icons/ranged_bow_012.dds",
"Data/UI/item_icons/ranged_bow_012_a.dds",
"Data/UI/item_icons/ranged_bow_013.dds",
"Data/UI/item_icons/ranged_bow_013_a.dds",
"Data/UI/item_icons/ranged_bow_014.dds",
"Data/UI/item_icons/ranged_bow_014_a.dds",
"Data/UI/item_icons/ranged_bow_014_b.dds",
"Data/UI/item_icons/ranged_bow_015.dds",
"Data/UI/item_icons/ranged_bow_015_a.dds",
"Data/UI/item_icons/ranged_bow_015_b.dds",
"Data/UI/item_icons/ranged_bow_016.dds",
"Data/UI/item_icons/ranged_bow_016_a.dds",
"Data/UI/item_icons/ranged_bow_017.dds",
"Data/UI/item_icons/ranged_bow_017_a.dds",
"Data/UI/item_icons/ranged_bow_018.dds",
"Data/UI/item_icons/ranged_bow_019.dds",
"Data/UI/item_icons/ranged_bow_020.dds",
"Data/UI/item_icons/ranged_bow_021.dds",
"Data/UI/item_icons/ranged_bow_022.dds",
"Data/UI/item_icons/ranged_bow_022a.dds",
"Data/UI/item_icons/ranged_bow_023.dds",
"Data/UI/item_icons/ranged_bow_023a.dds",
"Data/UI/item_icons/ranged_bow_024.dds",
"Data/UI/item_icons/ranged_bow_024a.dds",
"Data/UI/item_icons/ranged_bow_025.dds",
"Data/UI/item_icons/ranged_bow_025a.dds",
"Data/UI/item_icons/ranged_bow_025b.dds",
"Data/UI/item_icons/ranged_bow_026.dds",
"Data/UI/item_icons/ranged_bow_027.dds",
"Data/UI/item_icons/ranged_bow_027a.dds",
"Data/UI/item_icons/ranged_bow_028.dds",
"Data/UI/item_icons/ranged_bow_028a.dds",
"Data/UI/item_icons/ranged_bow_028b.dds",
"Data/UI/item_icons/ranged_bow_028c.dds",
"Data/UI/item_icons/ranged_bow_029.dds",
"Data/UI/item_icons/ranged_bow_029a.dds",
"Data/UI/item_icons/ranged_bow_030.dds",
"Data/UI/item_icons/ranged_bow_031.dds",
"Data/UI/item_icons/ranged_bow_031a.dds",
"Data/UI/item_icons/ranged_bow_032.dds",
"Data/UI/item_icons/ranged_bow_032a.dds",
"Data/UI/item_icons/ranged_bow_032b.dds",
"Data/UI/item_icons/ranged_bow_033.dds",
"Data/UI/item_icons/ranged_bow_034.dds",
"Data/UI/item_icons/ranged_bow_034_a.dds",
"Data/UI/item_icons/ranged_bow_035.dds",
"Data/UI/item_icons/ranged_bow_035_a.dds",
"Data/UI/item_icons/ranged_bow_035_b.dds",
"Data/UI/item_icons/ranged_bow_036.dds",
"Data/UI/item_icons/ranged_bow_036_a.dds",
"Data/UI/item_icons/ranged_bow_037.dds",
"Data/UI/item_icons/ranged_bow_037_a.dds",
"Data/UI/item_icons/ranged_bow_042.dds",
"Data/UI/item_icons/ranged_bow_042a.dds",
"Data/UI/item_icons/ranged_bow_043.dds",
"Data/UI/item_icons/ranged_bow_043a.dds",
"Data/UI/item_icons/ranged_bow_044.dds",
"Data/UI/item_icons/ranged_bow_044a.dds",
"Data/UI/item_icons/ranged_bow_045.dds",
"Data/UI/item_icons/ranged_bow_045a.dds",
"Data/UI/item_icons/ranged_bow_046.dds",
"Data/UI/item_icons/ranged_bow_046a.dds",
"Data/UI/item_icons/ranged_bow_047.dds",
"Data/UI/item_icons/ranged_bow_047a.dds",
"Data/UI/item_icons/ranged_bow_048.dds",
"Data/UI/item_icons/ranged_bow_048a.dds",
"Data/UI/item_icons/ranged_bow_048b.dds",
"Data/UI/item_icons/ranged_bow_049.dds",
"Data/UI/item_icons/ranged_bow_049a.dds",
"Data/UI/item_icons/ranged_bow_050.dds",
"Data/UI/item_icons/ranged_bow_050a.dds",
"Data/UI/item_icons/ranged_bow_051.dds",
"Data/UI/item_icons/ranged_bow_051a.dds",
"Data/UI/item_icons/ranged_bow_052.dds",
"Data/UI/item_icons/ranged_bow_052a.dds",
"Data/UI/item_icons/ranged_bow_053.dds",
"Data/UI/item_icons/ranged_bow_054.dds",
"Data/UI/item_icons/ranged_bow_054a.dds",
"Data/UI/item_icons/ranged_bow_055.dds",
"Data/UI/item_icons/ranged_bow_055a.dds",
"Data/UI/item_icons/ranged_bow_056.dds",
"Data/UI/item_icons/ranged_bow_056a.dds",
"Data/UI/item_icons/ranged_bow_057.dds",
"Data/UI/item_icons/ranged_bow_057a.dds",
"Data/UI/item_icons/ranged_bow_058.dds",
"Data/UI/item_icons/ranged_bow_058a.dds",
"Data/UI/item_icons/ranged_bow_059.dds",
"Data/UI/item_icons/ranged_bow_059a.dds",
"Data/UI/item_icons/ranged_bow_060.dds",
"Data/UI/item_icons/ranged_bow_060a.dds",
"Data/UI/item_icons/ranged_bow_061.dds",
"Data/UI/item_icons/ranged_bow_061a.dds",
"Data/UI/item_icons/ranged_bow_062.dds",
"Data/UI/item_icons/ranged_bow_062a.dds",
"Data/UI/item_icons/ranged_bow_063.dds",
"Data/UI/item_icons/ranged_bow_063a.dds",
"Data/UI/item_icons/ranged_bow_064.dds",
"Data/UI/item_icons/ranged_bow_064a.dds",
"Data/UI/item_icons/ranged_bow_065.dds",
"Data/UI/item_icons/ranged_bow_065a.dds",
"Data/UI/item_icons/ranged_bow_066.dds",
"Data/UI/item_icons/ranged_bow_066a.dds",
"Data/UI/item_icons/ranged_bow_067.dds",
"Data/UI/item_icons/ranged_bow_067a.dds",
"Data/UI/item_icons/ranged_bow_068.dds",
"Data/UI/item_icons/ranged_bow_068a.dds",
"Data/UI/item_icons/ranged_bow_068b.dds",
"Data/UI/item_icons/ranged_bow_069.dds",
"Data/UI/item_icons/ranged_bow_069a.dds",
"Data/UI/item_icons/ranged_bow_air_epic.dds",
"Data/UI/item_icons/ranged_bow_air_epic_a.dds",
"Data/UI/item_icons/ranged_bow_earth_mundane.dds",
"Data/UI/item_icons/ranged_bow_earth_mundane_a.dds",
"Data/UI/item_icons/ranged_bow_earth_mundane_b.dds",
"Data/UI/item_icons/ranged_bow_fire_epic.dds",
"Data/UI/item_icons/ranged_bow_fire_epic_a.dds",
"Data/UI/item_icons/ranged_bow_fire_epic_b.dds",
"Data/UI/item_icons/ranged_bow_life_01.dds",
"Data/UI/item_icons/ranged_bow_life_01_a.dds",
"Data/UI/item_icons/ranged_bow_water_mundane.dds",
"Data/UI/item_icons/ranged_bow_water_mundane_a.dds",
"Data/UI/item_icons/ranged_crossbow_001.dds",
"Data/UI/item_icons/ranged_crossbow_001a.dds",
"Data/UI/item_icons/ranged_crossbow_002.dds",
"Data/UI/item_icons/ranged_crossbow_003.dds",
"Data/UI/item_icons/ranged_crossbow_003a.dds",
"Data/UI/item_icons/ranged_crossbow_004.dds",
"Data/UI/item_icons/ranged_crossbow_004a.dds",
"Data/UI/item_icons/ranged_crossbow_005.dds",
"Data/UI/item_icons/ranged_crossbow_006.dds",
"Data/UI/item_icons/ranged_crossbow_006a.dds",
"Data/UI/item_icons/ranged_crossbow_007.dds",
"Data/UI/item_icons/ranged_crossbow_007a.dds",
"Data/UI/item_icons/ranged_crossbow_008.dds",
"Data/UI/item_icons/ranged_crossbow_009.dds",
"Data/UI/item_icons/ranged_crossbow_009a.dds",
"Data/UI/item_icons/ranged_crossbow_010.dds",
"Data/UI/item_icons/ranged_crossbow_011.dds",
"Data/UI/item_icons/ranged_crossbow_011a.dds",
"Data/UI/item_icons/ranged_crossbow_012.dds",
"Data/UI/item_icons/ranged_crossbow_012a.dds",
"Data/UI/item_icons/ranged_crossbow_013.dds",
"Data/UI/item_icons/ranged_crossbow_013a.dds",
"Data/UI/item_icons/ranged_crossbow_014.dds",
"Data/UI/item_icons/ranged_crossbow_014a.dds",
"Data/UI/item_icons/ranged_crossbow_015.dds",
"Data/UI/item_icons/ranged_crossbow_015a.dds",
"Data/UI/item_icons/ranged_crossbow_016.dds",
"Data/UI/item_icons/ranged_crossbow_016a.dds",
"Data/UI/item_icons/ranged_crossbow_017.dds",
"Data/UI/item_icons/ranged_crossbow_017a.dds",
"Data/UI/item_icons/ranged_crossbow_018.dds",
"Data/UI/item_icons/ranged_crossbow_019.dds",
"Data/UI/item_icons/ranged_crossbow_019a.dds",
"Data/UI/item_icons/ranged_crossbow_020.dds",
"Data/UI/item_icons/ranged_crossbow_020a.dds",
"Data/UI/item_icons/ranged_crossbow_021.dds",
"Data/UI/item_icons/ranged_crossbow_021a.dds",
"Data/UI/item_icons/ranged_crossbow_022.dds",
"Data/UI/item_icons/ranged_crossbow_022a.dds",
"Data/UI/item_icons/ranged_crossbow_023.dds",
"Data/UI/item_icons/ranged_crossbow_024.dds",
"Data/UI/item_icons/ranged_crossbow_024a.dds",
"Data/UI/item_icons/ranged_gun_death_epic.dds",
"Data/UI/item_icons/ranged_gun_death_epic_a.dds",
"Data/UI/item_icons/ranged_gun_fire_mundane.dds",
"Data/UI/item_icons/ranged_gun_fire_mundane_a.dds",
"Data/UI/item_icons/ranged_gun_fire_mundane_b.dds",
"Data/UI/item_icons/ranged_gun_water_epic.dds",
"Data/UI/item_icons/ranged_gun_water_epic_a.dds",
"Data/UI/item_icons/ranged_rifle_001.dds",
"Data/UI/item_icons/ranged_rifle_002.dds",
"Data/UI/item_icons/ranged_rifle_003.dds",
"Data/UI/item_icons/ranged_rifle_004.dds",
"Data/UI/item_icons/ranged_rifle_004a.dds",
"Data/UI/item_icons/ranged_rifle_005.dds",
"Data/UI/item_icons/ranged_rifle_006.dds",
"Data/UI/item_icons/ranged_rifle_007.dds",
"Data/UI/item_icons/ranged_rifle_009.dds",
"Data/UI/item_icons/ranged_rifle_010.dds",
"Data/UI/item_icons/ranged_rifle_012.dds",
"Data/UI/item_icons/ranged_rifle_013.dds",
"Data/UI/item_icons/ranged_rifle_013a.dds",
"Data/UI/item_icons/ranged_rifle_013b.dds",
"Data/UI/item_icons/ranged_rifle_014.dds",
"Data/UI/item_icons/ranged_rifle_015.dds",
"Data/UI/item_icons/ranged_rifle_015a.dds",
"Data/UI/item_icons/ranged_rifle_017.dds",
"Data/UI/item_icons/ranged_rifle_018.dds",
"Data/UI/item_icons/ranged_rifle_018a.dds",
"Data/UI/item_icons/ranged_rifle_019.dds",
"Data/UI/item_icons/ranged_rifle_019a.dds",
"Data/UI/item_icons/ranged_rifle_019b.dds",
"Data/UI/item_icons/ranged_rifle_020.dds",
"Data/UI/item_icons/ranged_rifle_021.dds",
"Data/UI/item_icons/ranged_rifle_022.dds",
"Data/UI/item_icons/ranged_rifle_022a.dds",
"Data/UI/item_icons/ranged_rifle_023.dds",
"Data/UI/item_icons/ranged_rifle_024.dds",
"Data/UI/item_icons/ranged_rifle_024a.dds",
"Data/UI/item_icons/ranged_rifle_025.dds",
"Data/UI/item_icons/ranged_rifle_025_a.dds",
"Data/UI/item_icons/ranged_rifle_026.dds",
"Data/UI/item_icons/ranged_rifle_026_a.dds",
"Data/UI/item_icons/ranged_rifle_027.dds",
"Data/UI/item_icons/ranged_rifle_027_a.dds",
"Data/UI/item_icons/ranged_rifle_028.dds",
"Data/UI/item_icons/ranged_rifle_028_a.dds",
"Data/UI/item_icons/ranged_rifle_028_b.dds",
"Data/UI/item_icons/ranged_rifle_029.dds",
"Data/UI/item_icons/ranged_rifle_029a.dds",
"Data/UI/item_icons/ranged_rifle_030.dds",
"Data/UI/item_icons/ranged_rifle_030a.dds",
"Data/UI/item_icons/ranged_rifle_031.dds",
"Data/UI/item_icons/ranged_rifle_031a.dds",
"Data/UI/item_icons/ranged_rifle_032.dds",
"Data/UI/item_icons/ranged_rifle_032a.dds",
"Data/UI/item_icons/ranged_rifle_033.dds",
"Data/UI/item_icons/ranged_rifle_033a.dds",
"Data/UI/item_icons/ranged_rifle_034.dds",
"Data/UI/item_icons/ranged_rifle_034a.dds",
"Data/UI/item_icons/ranged_rifle_035.dds",
"Data/UI/item_icons/ranged_rifle_035a.dds",
"Data/UI/item_icons/ranged_rifle_036.dds",
"Data/UI/item_icons/ranged_rifle_036a.dds",
"Data/UI/item_icons/ranged_rifle_037.dds",
"Data/UI/item_icons/ranged_rifle_037a.dds",
"Data/UI/item_icons/ranged_rifle_038.dds",
"Data/UI/item_icons/ranged_rifle_038a.dds",
"Data/UI/item_icons/ranged_rifle_039.dds",
"Data/UI/item_icons/ranged_rifle_039a.dds",
"Data/UI/item_icons/ranged_rifle_040.dds",
"Data/UI/item_icons/ranged_rifle_040a.dds",
"Data/UI/item_icons/ranged_rifle_041.dds",
"Data/UI/item_icons/ranged_rifle_041a.dds",
"Data/UI/item_icons/ranged_rifle_042.dds",
"Data/UI/item_icons/ranged_rifle_042a.dds",
"Data/UI/item_icons/ranged_rifle_043.dds",
"Data/UI/item_icons/ranged_rifle_043a.dds",
"Data/UI/item_icons/ranged_rifle_044.dds",
"Data/UI/item_icons/ranged_rifle_044a.dds",
"Data/UI/item_icons/ranged_rifle_045.dds",
"Data/UI/item_icons/ranged_rifle_045a.dds",
"Data/UI/item_icons/ranged_rifle_046.dds",
"Data/UI/item_icons/ranged_rifle_046a.dds",
"Data/UI/item_icons/ranged_rifle_047.dds",
"Data/UI/item_icons/ranged_rifle_047a.dds",
"Data/UI/item_icons/ranged_rifle_048.dds",
"Data/UI/item_icons/ranged_rifle_048a.dds",
"Data/UI/item_icons/ranged_rifle_049.dds",
"Data/UI/item_icons/ranged_rifle_049a.dds",
"Data/UI/item_icons/ranged_rifle_050.dds",
"Data/UI/item_icons/ranged_rifle_050a.dds",
"Data/UI/item_icons/ranged_rifle_051.dds",
"Data/UI/item_icons/ranged_rifle_051a.dds",
"Data/UI/item_icons/ranged_rifle_052.dds",
"Data/UI/item_icons/ranged_rifle_052a.dds",
"Data/UI/item_icons/ranged_rifle_053.dds",
"Data/UI/item_icons/ranged_rifle_053a.dds",
"Data/UI/item_icons/ranged_rifle_054.dds",
"Data/UI/item_icons/ranged_rifle_054a.dds",
"Data/UI/item_icons/ranged_rifle_055.dds",
"Data/UI/item_icons/ranged_rifle_055a.dds",
"Data/UI/item_icons/ranged_rifle_056.dds",
"Data/UI/item_icons/ranged_rifle_056a.dds",
"Data/UI/item_icons/ranged_rifle_057.dds",
"Data/UI/item_icons/ranged_rifle_057a.dds",
"Data/UI/item_icons/ranged_rifle_058.dds",
"Data/UI/item_icons/ranged_rifle_058a.dds",
"Data/UI/item_icons/ranged_rifle_11.dds",
"Data/UI/item_icons/ranged_rifle_air_mundane.dds",
"Data/UI/item_icons/ranged_rifle_air_mundane_a.dds",
"Data/UI/item_icons/ranged_rifle_earth_epic.dds",
"Data/UI/item_icons/ranged_rifle_earth_epic_a.dds",
"Data/UI/item_icons/ranged_rifle_life_01.dds",
"Data/UI/item_icons/ranged_rifle_life_01_a.dds",
"Data/UI/item_icons/ranged_wand_001.dds",
"Data/UI/item_icons/ranged_wand_001a.dds",
"Data/UI/item_icons/ranged_wand_002.dds",
"Data/UI/item_icons/ranged_wand_002a.dds",
"Data/UI/item_icons/ranged_wand_003.dds",
"Data/UI/item_icons/ranged_wand_003a.dds",
"Data/UI/item_icons/ranged_wand_004.dds",
"Data/UI/item_icons/ranged_wand_004a.dds",
"Data/UI/item_icons/ranged_wand_005.dds",
"Data/UI/item_icons/ranged_wand_006.dds",
"Data/UI/item_icons/ranged_wand_006a.dds",
"Data/UI/item_icons/ranged_wand_006b.dds",
"Data/UI/item_icons/ranged_wand_007.dds",
"Data/UI/item_icons/ranged_wand_007a.dds",
"Data/UI/item_icons/ranged_wand_007b.dds",
"Data/UI/item_icons/ranged_wand_008.dds",
"Data/UI/item_icons/ranged_wand_008a.dds",
"Data/UI/item_icons/ranged_wand_009.dds",
"Data/UI/item_icons/ranged_wand_009a.dds",
"Data/UI/item_icons/ranged_wand_009b.dds",
"Data/UI/item_icons/ranged_wand_010.dds",
"Data/UI/item_icons/ranged_wand_011.dds",
"Data/UI/item_icons/ranged_wand_011a.dds",
"Data/UI/item_icons/ranged_wand_011b.dds",
"Data/UI/item_icons/ranged_wand_012.dds",
"Data/UI/item_icons/ranged_wand_012a.dds",
"Data/UI/item_icons/ranged_wand_012b.dds",
"Data/UI/item_icons/ranged_wand_013.dds",
"Data/UI/item_icons/ranged_wand_013a.dds",
"Data/UI/item_icons/ranged_wand_014.dds",
"Data/UI/item_icons/ranged_wand_014a.dds",
"Data/UI/item_icons/ranged_wand_014b.dds",
"Data/UI/item_icons/ranged_wand_015.dds",
"Data/UI/item_icons/ranged_wand_015a.dds",
"Data/UI/item_icons/ranged_wand_015b.dds",
"Data/UI/item_icons/ranged_wand_016.dds",
"Data/UI/item_icons/ranged_wand_016a.dds",
"Data/UI/item_icons/ranged_wand_017.dds",
"Data/UI/item_icons/ranged_wand_017a.dds",
"Data/UI/item_icons/ranged_wand_018.dds",
"Data/UI/item_icons/ranged_wand_018a.dds",
"Data/UI/item_icons/ranged_wand_019.dds",
"Data/UI/item_icons/ranged_wand_019a.dds",
"Data/UI/item_icons/ranged_wand_020.dds",
"Data/UI/item_icons/ranged_wand_020a.dds",
"Data/UI/item_icons/ranged_wand_021.dds",
"Data/UI/item_icons/ranged_wand_021a.dds",
"Data/UI/item_icons/ranged_wand_022.dds",
"Data/UI/item_icons/ranged_wand_022a.dds",
"Data/UI/item_icons/ranged_wand_023.dds",
"Data/UI/item_icons/ranged_wand_023a.dds",
"Data/UI/item_icons/ranged_wand_023b.dds",
"Data/UI/item_icons/ranged_wand_024.dds",
"Data/UI/item_icons/ranged_wand_024a.dds",
"Data/UI/item_icons/ranged_wand_025.dds",
"Data/UI/item_icons/ranged_wand_025a.dds",
"Data/UI/item_icons/ranged_wand_026.dds",
"Data/UI/item_icons/ranged_wand_026a.dds",
"Data/UI/item_icons/ranged_wand_027.dds",
"Data/UI/item_icons/ranged_wand_027a.dds",
"Data/UI/item_icons/ranged_wand_028.dds",
"Data/UI/item_icons/ranged_wand_028a.dds",
"Data/UI/item_icons/ranged_wand_029.dds",
"Data/UI/item_icons/ranged_wand_029a.dds",
"Data/UI/item_icons/ranged_wand_030.dds",
"Data/UI/item_icons/ranged_wand_030a.dds",
"Data/UI/item_icons/ranged_wand_030b.dds",
"Data/UI/item_icons/ranged_wand_life_01.dds",
"Data/UI/item_icons/ranged_wand_life_01_a.dds",
"Data/UI/item_icons/ratnaraj_ruby.dds",
"Data/UI/item_icons/ratnaraj_ruby_a.dds",
"Data/UI/item_icons/raw_diamond.dds",
"Data/UI/item_icons/razorbeast_quills.dds",
"Data/UI/item_icons/razor_claw.dds",
"Data/UI/item_icons/refinedwood1.dds",
"Data/UI/item_icons/refinedwood10.dds",
"Data/UI/item_icons/refinedwood2.dds",
"Data/UI/item_icons/refinedwood3.dds",
"Data/UI/item_icons/refinedwood4.dds",
"Data/UI/item_icons/refinedwood4a.dds",
"Data/UI/item_icons/refinedwood4b.dds",
"Data/UI/item_icons/refinedwood5.dds",
"Data/UI/item_icons/refinedwood6.dds",
"Data/UI/item_icons/refinedwood6a.dds",
"Data/UI/item_icons/refinedwood7.dds",
"Data/UI/item_icons/refinedwood7a.dds",
"Data/UI/item_icons/refinedwood8.dds",
"Data/UI/item_icons/refinedwood9.dds",
"Data/UI/item_icons/Refined_mahogany_lumber.dds",
"Data/UI/item_icons/Refined_mahogany_lumber_a.dds",
"Data/UI/item_icons/refined_oak_lumber.dds",
"Data/UI/item_icons/refined_pelt.dds",
"Data/UI/item_icons/refined_pelt_a.dds",
"Data/UI/item_icons/refined_runebirch.dds",
"Data/UI/item_icons/refined_runebirch_a.dds",
"Data/UI/item_icons/refined_runebirch_lumber.dds",
"Data/UI/item_icons/refined_runebirch_lumber_a.dds",
"Data/UI/item_icons/refined_sagebrush.dds",
"Data/UI/item_icons/refined_sagebrush_a.dds",
"Data/UI/item_icons/refined_scales.dds",
"Data/UI/item_icons/refined_scales_a.dds",
"Data/UI/item_icons/reinforced_composite_cloth.dds",
"Data/UI/item_icons/reinforced_composite_cloth_a.dds",
"Data/UI/item_icons/relic_1.dds",
"Data/UI/item_icons/relic_1a.dds",
"Data/UI/item_icons/relic_1b.dds",
"Data/UI/item_icons/relic_1c.dds",
"Data/UI/item_icons/relic_1d.dds",
"Data/UI/item_icons/relic_1e.dds",
"Data/UI/item_icons/relic_1f.dds",
"Data/UI/item_icons/relic_1g.dds",
"Data/UI/item_icons/relic_1h.dds",
"Data/UI/item_icons/relic_1i.dds",
"Data/UI/item_icons/relic_1j.dds",
"Data/UI/item_icons/relic_1k.dds",
"Data/UI/item_icons/relic_1l.dds",
"Data/UI/item_icons/relic_1m.dds",
"Data/UI/item_icons/remade_artifact.dds",
"Data/UI/item_icons/ribcage.dds",
"Data/UI/item_icons/rift_consumable_air_01.dds",
"Data/UI/item_icons/rift_consumable_air_02.dds",
"Data/UI/item_icons/rift_consumable_air_03.dds",
"Data/UI/item_icons/rift_consumable_air_03_a.dds",
"Data/UI/item_icons/rift_consumable_air_04.dds",
"Data/UI/item_icons/rift_consumable_air_05.dds",
"Data/UI/item_icons/rift_consumable_death_01.dds",
"Data/UI/item_icons/rift_consumable_death_02.dds",
"Data/UI/item_icons/rift_consumable_death_03.dds",
"Data/UI/item_icons/rift_consumable_death_03_a.dds",
"Data/UI/item_icons/rift_consumable_death_04.dds",
"Data/UI/item_icons/rift_consumable_death_05.dds",
"Data/UI/item_icons/rift_consumable_death_05_a.dds",
"Data/UI/item_icons/rift_consumable_earth_01.dds",
"Data/UI/item_icons/rift_consumable_earth_02.dds",
"Data/UI/item_icons/rift_consumable_earth_02b.dds",
"Data/UI/item_icons/rift_consumable_earth_02_a.dds",
"Data/UI/item_icons/rift_consumable_earth_03.dds",
"Data/UI/item_icons/rift_consumable_earth_04.dds",
"Data/UI/item_icons/rift_consumable_earth_05.dds",
"Data/UI/item_icons/rift_consumable_fire_01.dds",
"Data/UI/item_icons/rift_consumable_fire_02.dds",
"Data/UI/item_icons/rift_consumable_fire_03.dds",
"Data/UI/item_icons/rift_consumable_fire_03_a.dds",
"Data/UI/item_icons/rift_consumable_fire_04.dds",
"Data/UI/item_icons/rift_consumable_fire_05.dds",
"Data/UI/item_icons/rift_consumable_life_01.dds",
"Data/UI/item_icons/rift_consumable_life_02.dds",
"Data/UI/item_icons/rift_consumable_life_03.dds",
"Data/UI/item_icons/rift_consumable_life_04.dds",
"Data/UI/item_icons/rift_consumable_life_05.dds",
"Data/UI/item_icons/rift_consumable_life_05_a.dds",
"Data/UI/item_icons/rift_consumable_water_01.dds",
"Data/UI/item_icons/rift_consumable_water_02.dds",
"Data/UI/item_icons/rift_consumable_water_03.dds",
"Data/UI/item_icons/rift_consumable_water_03_a.dds",
"Data/UI/item_icons/rift_consumable_water_04.dds",
"Data/UI/item_icons/rift_consumable_water_05.dds",
"Data/UI/item_icons/rims_of_counsel.dds",
"Data/UI/item_icons/rims_of_counsel_a.dds",
"Data/UI/item_icons/ring1.dds",
"Data/UI/item_icons/ring10a.dds",
"Data/UI/item_icons/ring10c.dds",
"Data/UI/item_icons/ring11.dds",
"Data/UI/item_icons/ring12.dds",
"Data/UI/item_icons/ring13.dds",
"Data/UI/item_icons/ring14.dds",
"Data/UI/item_icons/ring14a.dds",
"Data/UI/item_icons/ring15.dds",
"Data/UI/item_icons/ring16.dds",
"Data/UI/item_icons/ring17.dds",
"Data/UI/item_icons/ring18.dds",
"Data/UI/item_icons/ring19.dds",
"Data/UI/item_icons/ring19a.dds",
"Data/UI/item_icons/ring2.dds",
"Data/UI/item_icons/ring20.dds",
"Data/UI/item_icons/ring21.dds",
"Data/UI/item_icons/ring21a.dds",
"Data/UI/item_icons/ring22.dds",
"Data/UI/item_icons/ring23.dds",
"Data/UI/item_icons/ring23a.dds",
"Data/UI/item_icons/ring24.dds",
"Data/UI/item_icons/ring24a.dds",
"Data/UI/item_icons/ring25.dds",
"Data/UI/item_icons/ring25a.dds",
"Data/UI/item_icons/ring26.dds",
"Data/UI/item_icons/ring27.dds",
"Data/UI/item_icons/ring28.dds",
"Data/UI/item_icons/ring29.dds",
"Data/UI/item_icons/ring3.dds",
"Data/UI/item_icons/ring30.dds",
"Data/UI/item_icons/ring30a.dds",
"Data/UI/item_icons/ring31w.dds",
"Data/UI/item_icons/ring32y.dds",
"Data/UI/item_icons/ring32z.dds",
"Data/UI/item_icons/ring4.dds",
"Data/UI/item_icons/ring4a.dds",
"Data/UI/item_icons/ring5a.dds",
"Data/UI/item_icons/ring5b.dds",
"Data/UI/item_icons/ring6.dds",
"Data/UI/item_icons/ring7.dds",
"Data/UI/item_icons/ring7a.dds",
"Data/UI/item_icons/ring7b.dds",
"Data/UI/item_icons/ring8.dds",
"Data/UI/item_icons/ring9a.dds",
"Data/UI/item_icons/ring_band_damaged_01.dds",
"Data/UI/item_icons/ring_band_damaged_01_a.dds",
"Data/UI/item_icons/ring_band_damaged_01_b.dds",
"Data/UI/item_icons/ring_band_normal_01.dds",
"Data/UI/item_icons/ring_band_normal_01_a.dds",
"Data/UI/item_icons/ring_band_normal_01_b.dds",
"Data/UI/item_icons/ring_band_ornate_01.dds",
"Data/UI/item_icons/ring_band_ornate_01_a.dds",
"Data/UI/item_icons/ring_band_ornate_01_b.dds",
"Data/UI/item_icons/ring_gemmed_damaged_01.dds",
"Data/UI/item_icons/ring_gemmed_damaged_01_a.dds",
"Data/UI/item_icons/ring_gemmed_damaged_01_b.dds",
"Data/UI/item_icons/ring_gemmed_normal_01.dds",
"Data/UI/item_icons/ring_gemmed_normal_01_a.dds",
"Data/UI/item_icons/ring_gemmed_normal_01_b.dds",
"Data/UI/item_icons/ring_gemmed_normal_01_c.dds",
"Data/UI/item_icons/ring_gemmed_normal_02.dds",
"Data/UI/item_icons/ring_gemmed_normal_02_a.dds",
"Data/UI/item_icons/ring_gemmed_normal_02_b.dds",
"Data/UI/item_icons/ring_gemmed_ornate.dds",
"Data/UI/item_icons/ring_gemmed_ornate_a.dds",
"Data/UI/item_icons/ring_gemmed_ornate_b.dds",
"Data/UI/item_icons/ring_signet_damaged_01.dds",
"Data/UI/item_icons/ring_signet_damaged_01_a.dds",
"Data/UI/item_icons/ring_signet_damaged_01_b.dds",
"Data/UI/item_icons/ring_signet_normal_01.dds",
"Data/UI/item_icons/ring_signet_normal_01_a.dds",
"Data/UI/item_icons/ring_signet_normal_01_b.dds",
"Data/UI/item_icons/ring_signet_ornate_01.dds",
"Data/UI/item_icons/ring_signet_ornate_01_a.dds",
"Data/UI/item_icons/ring_signet_ornate_01_b.dds",
"Data/UI/item_icons/ring_signet_ornate_01_c.dds",
"Data/UI/item_icons/ripped_eyepatch.dds",
"Data/UI/item_icons/ripped_eyepatch_a.dds",
"Data/UI/item_icons/ripped_eyepatch_b.dds",
"Data/UI/item_icons/ripped_eyepatch_c.dds",
"Data/UI/item_icons/ripped_lootbag.dds",
"Data/UI/item_icons/ritual_dagger.dds",
"Data/UI/item_icons/ritual_dagger_modified.dds",
"Data/UI/item_icons/ritual_dagger_powered.dds",
"Data/UI/item_icons/road_lord_ear.dds",
"Data/UI/item_icons/road_lord_ear_a.dds",
"Data/UI/item_icons/road_warden_body.dds",
"Data/UI/item_icons/roasted_turkey.dds",
"Data/UI/item_icons/roasted_turkey_leg.dds",
"Data/UI/item_icons/rob8a.dds",
"Data/UI/item_icons/roc_egg.dds",
"Data/UI/item_icons/rod1.dds",
"Data/UI/item_icons/rod1a.dds",
"Data/UI/item_icons/rod1b.dds",
"Data/UI/item_icons/rod1c.dds",
"Data/UI/item_icons/rod2.dds",
"Data/UI/item_icons/rod2a.dds",
"Data/UI/item_icons/rod2b.dds",
"Data/UI/item_icons/rod3.dds",
"Data/UI/item_icons/rod3a.dds",
"Data/UI/item_icons/rod3b.dds",
"Data/UI/item_icons/rod4.dds",
"Data/UI/item_icons/rod4a.dds",
"Data/UI/item_icons/rod4b.dds",
"Data/UI/item_icons/rod5.dds",
"Data/UI/item_icons/rod5a.dds",
"Data/UI/item_icons/rod5_1.dds",
"Data/UI/item_icons/rod5_1a.dds",
"Data/UI/item_icons/rod6.dds",
"Data/UI/item_icons/rod6a.dds",
"Data/UI/item_icons/rod7.dds",
"Data/UI/item_icons/rod7a.dds",
"Data/UI/item_icons/rod7b.dds",
"Data/UI/item_icons/rod8.dds",
"Data/UI/item_icons/rod8a.dds",
"Data/UI/item_icons/rod8b.dds",
"Data/UI/item_icons/rod8c.dds",
"Data/UI/item_icons/rognar_tusk.dds",
"Data/UI/item_icons/rogue_potion.dds",
"Data/UI/item_icons/rogue_potion_a.dds",
"Data/UI/item_icons/rotten_heartwood.dds",
"Data/UI/item_icons/rotten_quiver.dds",
"Data/UI/item_icons/rotting_hive_wall.dds",
"Data/UI/item_icons/rough_opal.dds",
"Data/UI/item_icons/rowan_berries.dds",
"Data/UI/item_icons/runebirch_log.dds",
"Data/UI/item_icons/runebirch_log_a.dds",
"Data/UI/item_icons/runebirch_log_b.dds",
"Data/UI/item_icons/runebirch_lumber.dds",
"Data/UI/item_icons/runebirch_lumber_a.dds",
"Data/UI/item_icons/runebirch_lumber_b.dds",
"Data/UI/item_icons/runebirch_lumber_c.dds",
"Data/UI/item_icons/runed_shell.dds",
"Data/UI/item_icons/rune_0_A_blue.dds",
"Data/UI/item_icons/rune_0_A_green.dds",
"Data/UI/item_icons/rune_0_A_pink.dds",
"Data/UI/item_icons/rune_0_A_red.dds",
"Data/UI/item_icons/rune_0_A_violet.dds",
"Data/UI/item_icons/rune_0_A_yellow.dds",
"Data/UI/item_icons/rune_0_B_blue.dds",
"Data/UI/item_icons/rune_0_B_green.dds",
"Data/UI/item_icons/rune_0_B_pink.dds",
"Data/UI/item_icons/rune_0_B_red.dds",
"Data/UI/item_icons/rune_0_B_violet.dds",
"Data/UI/item_icons/rune_0_B_yellow.dds",
"Data/UI/item_icons/rune_0_C_blue.dds",
"Data/UI/item_icons/rune_0_C_green.dds",
"Data/UI/item_icons/rune_0_C_pink.dds",
"Data/UI/item_icons/rune_0_C_red.dds",
"Data/UI/item_icons/rune_0_C_violet.dds",
"Data/UI/item_icons/rune_0_C_yellow.dds",
"Data/UI/item_icons/rune_0_D_blue.dds",
"Data/UI/item_icons/rune_0_D_green.dds",
"Data/UI/item_icons/rune_0_D_pink.dds",
"Data/UI/item_icons/rune_0_D_red.dds",
"Data/UI/item_icons/rune_0_D_violet.dds",
"Data/UI/item_icons/rune_0_D_yellow.dds",
"Data/UI/item_icons/rune_0_E_blue.dds",
"Data/UI/item_icons/rune_0_E_green.dds",
"Data/UI/item_icons/rune_0_E_pink.dds",
"Data/UI/item_icons/rune_0_E_red.dds",
"Data/UI/item_icons/rune_0_E_violet.dds",
"Data/UI/item_icons/rune_0_E_yellow.dds",
"Data/UI/item_icons/rune_0_F_blue.dds",
"Data/UI/item_icons/rune_0_F_green.dds",
"Data/UI/item_icons/rune_0_F_pink.dds",
"Data/UI/item_icons/rune_0_F_red.dds",
"Data/UI/item_icons/rune_0_F_violet.dds",
"Data/UI/item_icons/rune_0_F_yellow.dds",
"Data/UI/item_icons/rune_0_G_blue.dds",
"Data/UI/item_icons/rune_0_G_green.dds",
"Data/UI/item_icons/rune_0_G_pink.dds",
"Data/UI/item_icons/rune_0_G_red.dds",
"Data/UI/item_icons/rune_0_G_violet.dds",
"Data/UI/item_icons/rune_0_G_yellow.dds",
"Data/UI/item_icons/rune_0_H_blue.dds",
"Data/UI/item_icons/rune_0_H_green.dds",
"Data/UI/item_icons/rune_0_H_pink.dds",
"Data/UI/item_icons/rune_0_H_red.dds",
"Data/UI/item_icons/rune_0_H_violet.dds",
"Data/UI/item_icons/rune_0_H_yellow.dds",
"Data/UI/item_icons/rune_0_I_blue.dds",
"Data/UI/item_icons/rune_0_I_green.dds",
"Data/UI/item_icons/rune_0_I_pink.dds",
"Data/UI/item_icons/rune_0_I_red.dds",
"Data/UI/item_icons/rune_0_I_violet.dds",
"Data/UI/item_icons/rune_0_I_yellow.dds",
"Data/UI/item_icons/rune_0_J_blue.dds",
"Data/UI/item_icons/rune_0_J_green.dds",
"Data/UI/item_icons/rune_0_J_pink.dds",
"Data/UI/item_icons/rune_0_J_red.dds",
"Data/UI/item_icons/rune_0_J_violet.dds",
"Data/UI/item_icons/rune_0_J_yellow.dds",
"Data/UI/item_icons/rune_1_A_blue.dds",
"Data/UI/item_icons/rune_1_A_green.dds",
"Data/UI/item_icons/rune_1_A_pink.dds",
"Data/UI/item_icons/rune_1_A_red.dds",
"Data/UI/item_icons/rune_1_A_violet.dds",
"Data/UI/item_icons/rune_1_A_yellow.dds",
"Data/UI/item_icons/rune_1_B_blue.dds",
"Data/UI/item_icons/rune_1_B_green.dds",
"Data/UI/item_icons/rune_1_B_pink.dds",
"Data/UI/item_icons/rune_1_B_red.dds",
"Data/UI/item_icons/rune_1_B_violet.dds",
"Data/UI/item_icons/rune_1_B_yellow.dds",
"Data/UI/item_icons/rune_1_C_blue.dds",
"Data/UI/item_icons/rune_1_C_green.dds",
"Data/UI/item_icons/rune_1_C_pink.dds",
"Data/UI/item_icons/rune_1_C_red.dds",
"Data/UI/item_icons/rune_1_C_violet.dds",
"Data/UI/item_icons/rune_1_C_yellow.dds",
"Data/UI/item_icons/rune_1_D_blue.dds",
"Data/UI/item_icons/rune_1_D_green.dds",
"Data/UI/item_icons/rune_1_D_pink.dds",
"Data/UI/item_icons/rune_1_D_red.dds",
"Data/UI/item_icons/rune_1_D_violet.dds",
"Data/UI/item_icons/rune_1_D_yellow.dds",
"Data/UI/item_icons/rune_1_E_blue.dds",
"Data/UI/item_icons/rune_1_E_green.dds",
"Data/UI/item_icons/rune_1_E_pink.dds",
"Data/UI/item_icons/rune_1_E_red.dds",
"Data/UI/item_icons/rune_1_E_violet.dds",
"Data/UI/item_icons/rune_1_E_yellow.dds",
"Data/UI/item_icons/rune_1_F_blue.dds",
"Data/UI/item_icons/rune_1_F_green.dds",
"Data/UI/item_icons/rune_1_F_pink.dds",
"Data/UI/item_icons/rune_1_F_red.dds",
"Data/UI/item_icons/rune_1_F_violet.dds",
"Data/UI/item_icons/rune_1_F_yellow.dds",
"Data/UI/item_icons/rune_1_G_blue.dds",
"Data/UI/item_icons/rune_1_G_green.dds",
"Data/UI/item_icons/rune_1_G_pink.dds",
"Data/UI/item_icons/rune_1_G_red.dds",
"Data/UI/item_icons/rune_1_G_violet.dds",
"Data/UI/item_icons/rune_1_G_yellow.dds",
"Data/UI/item_icons/rune_1_H_blue.dds",
"Data/UI/item_icons/rune_1_H_green.dds",
"Data/UI/item_icons/rune_1_H_pink.dds",
"Data/UI/item_icons/rune_1_H_red.dds",
"Data/UI/item_icons/rune_1_H_violet.dds",
"Data/UI/item_icons/rune_1_H_yellow.dds",
"Data/UI/item_icons/rune_1_I_blue.dds",
"Data/UI/item_icons/rune_1_I_green.dds",
"Data/UI/item_icons/rune_1_I_pink.dds",
"Data/UI/item_icons/rune_1_I_red.dds",
"Data/UI/item_icons/rune_1_I_violet.dds",
"Data/UI/item_icons/rune_1_I_yellow.dds",
"Data/UI/item_icons/rune_1_J_blue.dds",
"Data/UI/item_icons/rune_1_J_green.dds",
"Data/UI/item_icons/rune_1_J_pink.dds",
"Data/UI/item_icons/rune_1_J_red.dds",
"Data/UI/item_icons/rune_1_J_violet.dds",
"Data/UI/item_icons/rune_1_J_yellow.dds",
"Data/UI/item_icons/rune_3_A_blue.dds",
"Data/UI/item_icons/rune_3_A_green.dds",
"Data/UI/item_icons/rune_3_A_pink.dds",
"Data/UI/item_icons/rune_3_A_red.dds",
"Data/UI/item_icons/rune_3_A_violet.dds",
"Data/UI/item_icons/rune_3_A_yellow.dds",
"Data/UI/item_icons/rune_3_B_blue.dds",
"Data/UI/item_icons/rune_3_B_green.dds",
"Data/UI/item_icons/rune_3_B_pink.dds",
"Data/UI/item_icons/rune_3_B_red.dds",
"Data/UI/item_icons/rune_3_B_violet.dds",
"Data/UI/item_icons/rune_3_B_yellow.dds",
"Data/UI/item_icons/rune_3_C_blue.dds",
"Data/UI/item_icons/rune_3_C_green.dds",
"Data/UI/item_icons/rune_3_C_pink.dds",
"Data/UI/item_icons/rune_3_C_red.dds",
"Data/UI/item_icons/rune_3_C_violet.dds",
"Data/UI/item_icons/rune_3_C_yellow.dds",
"Data/UI/item_icons/rune_3_D_blue.dds",
"Data/UI/item_icons/rune_3_D_green.dds",
"Data/UI/item_icons/rune_3_D_pink.dds",
"Data/UI/item_icons/rune_3_D_red.dds",
"Data/UI/item_icons/rune_3_D_violet.dds",
"Data/UI/item_icons/rune_3_D_yellow.dds",
"Data/UI/item_icons/rune_3_E_blue.dds",
"Data/UI/item_icons/rune_3_E_green.dds",
"Data/UI/item_icons/rune_3_E_pink.dds",
"Data/UI/item_icons/rune_3_E_red.dds",
"Data/UI/item_icons/rune_3_E_violet.dds",
"Data/UI/item_icons/rune_3_E_yellow.dds",
"Data/UI/item_icons/rune_3_F_blue.dds",
"Data/UI/item_icons/rune_3_F_green.dds",
"Data/UI/item_icons/rune_3_F_pink.dds",
"Data/UI/item_icons/rune_3_F_red.dds",
"Data/UI/item_icons/rune_3_F_violet.dds",
"Data/UI/item_icons/rune_3_F_yellow.dds",
"Data/UI/item_icons/rune_3_G_blue.dds",
"Data/UI/item_icons/rune_3_G_green.dds",
"Data/UI/item_icons/rune_3_G_pink.dds",
"Data/UI/item_icons/rune_3_G_red.dds",
"Data/UI/item_icons/rune_3_G_violet.dds",
"Data/UI/item_icons/rune_3_G_yellow.dds",
"Data/UI/item_icons/rune_3_H_blue.dds",
"Data/UI/item_icons/rune_3_H_green.dds",
"Data/UI/item_icons/rune_3_H_pink.dds",
"Data/UI/item_icons/rune_3_H_red.dds",
"Data/UI/item_icons/rune_3_H_violet.dds",
"Data/UI/item_icons/rune_3_H_yellow.dds",
"Data/UI/item_icons/rune_3_I_blue.dds",
"Data/UI/item_icons/rune_3_I_green.dds",
"Data/UI/item_icons/rune_3_I_pink.dds",
"Data/UI/item_icons/rune_3_I_red.dds",
"Data/UI/item_icons/rune_3_I_violet.dds",
"Data/UI/item_icons/rune_3_I_yellow.dds",
"Data/UI/item_icons/rune_3_J_blue.dds",
"Data/UI/item_icons/rune_3_J_green.dds",
"Data/UI/item_icons/rune_3_J_pink.dds",
"Data/UI/item_icons/rune_3_J_red.dds",
"Data/UI/item_icons/rune_3_J_violet.dds",
"Data/UI/item_icons/rune_3_J_yellow.dds",
"Data/UI/item_icons/rune_4_A_blue.dds",
"Data/UI/item_icons/rune_4_A_blue_a.dds",
"Data/UI/item_icons/rune_4_A_green.dds",
"Data/UI/item_icons/rune_4_A_green_a.dds",
"Data/UI/item_icons/rune_4_A_pink.dds",
"Data/UI/item_icons/rune_4_A_pink_a.dds",
"Data/UI/item_icons/rune_4_A_red.dds",
"Data/UI/item_icons/rune_4_A_red_a.dds",
"Data/UI/item_icons/rune_4_A_violet.dds",
"Data/UI/item_icons/rune_4_A_violet_a.dds",
"Data/UI/item_icons/rune_4_A_yellow.dds",
"Data/UI/item_icons/rune_4_A_yellow_a.dds",
"Data/UI/item_icons/rune_4_B_blue.dds",
"Data/UI/item_icons/rune_4_B_blue_a.dds",
"Data/UI/item_icons/rune_4_B_green.dds",
"Data/UI/item_icons/rune_4_B_green_a.dds",
"Data/UI/item_icons/rune_4_B_pink.dds",
"Data/UI/item_icons/rune_4_B_pink_a.dds",
"Data/UI/item_icons/rune_4_B_red.dds",
"Data/UI/item_icons/rune_4_B_red_a.dds",
"Data/UI/item_icons/rune_4_B_violet.dds",
"Data/UI/item_icons/rune_4_B_violet_a.dds",
"Data/UI/item_icons/rune_4_B_yellow.dds",
"Data/UI/item_icons/rune_4_B_yellow_a.dds",
"Data/UI/item_icons/rune_4_C_blue.dds",
"Data/UI/item_icons/rune_4_C_blue_a.dds",
"Data/UI/item_icons/rune_4_C_green.dds",
"Data/UI/item_icons/rune_4_C_green_a.dds",
"Data/UI/item_icons/rune_4_C_pink.dds",
"Data/UI/item_icons/rune_4_C_pink_a.dds",
"Data/UI/item_icons/rune_4_C_red.dds",
"Data/UI/item_icons/rune_4_C_red_a.dds",
"Data/UI/item_icons/rune_4_C_violet.dds",
"Data/UI/item_icons/rune_4_C_violet_a.dds",
"Data/UI/item_icons/rune_4_C_yellow.dds",
"Data/UI/item_icons/rune_4_C_yellow_a.dds",
"Data/UI/item_icons/rune_4_D_blue.dds",
"Data/UI/item_icons/rune_4_D_blue_a.dds",
"Data/UI/item_icons/rune_4_D_green.dds",
"Data/UI/item_icons/rune_4_D_green_a.dds",
"Data/UI/item_icons/rune_4_D_pink.dds",
"Data/UI/item_icons/rune_4_D_pink_a.dds",
"Data/UI/item_icons/rune_4_D_red.dds",
"Data/UI/item_icons/rune_4_D_red_a.dds",
"Data/UI/item_icons/rune_4_D_violet.dds",
"Data/UI/item_icons/rune_4_D_violet_a.dds",
"Data/UI/item_icons/rune_4_D_yellow.dds",
"Data/UI/item_icons/rune_4_D_yellow_a.dds",
"Data/UI/item_icons/rune_4_E_blue.dds",
"Data/UI/item_icons/rune_4_E_blue_a.dds",
"Data/UI/item_icons/rune_4_E_green.dds",
"Data/UI/item_icons/rune_4_E_green_a.dds",
"Data/UI/item_icons/rune_4_E_pink.dds",
"Data/UI/item_icons/rune_4_E_pink_a.dds",
"Data/UI/item_icons/rune_4_E_red.dds",
"Data/UI/item_icons/rune_4_E_red_a.dds",
"Data/UI/item_icons/rune_4_E_violet.dds",
"Data/UI/item_icons/rune_4_E_violet_a.dds",
"Data/UI/item_icons/rune_4_E_yellow.dds",
"Data/UI/item_icons/rune_4_E_yellow_a.dds",
"Data/UI/item_icons/rune_4_F_blue.dds",
"Data/UI/item_icons/rune_4_F_blue_a.dds",
"Data/UI/item_icons/rune_4_F_green.dds",
"Data/UI/item_icons/rune_4_F_green_a.dds",
"Data/UI/item_icons/rune_4_F_pink.dds",
"Data/UI/item_icons/rune_4_F_pink_a.dds",
"Data/UI/item_icons/rune_4_F_red.dds",
"Data/UI/item_icons/rune_4_F_red_a.dds",
"Data/UI/item_icons/rune_4_F_violet.dds",
"Data/UI/item_icons/rune_4_F_violet_a.dds",
"Data/UI/item_icons/rune_4_F_yellow.dds",
"Data/UI/item_icons/rune_4_F_yellow_a.dds",
"Data/UI/item_icons/rune_4_G_blue.dds",
"Data/UI/item_icons/rune_4_G_blue_a.dds",
"Data/UI/item_icons/rune_4_G_green.dds",
"Data/UI/item_icons/rune_4_G_green_a.dds",
"Data/UI/item_icons/rune_4_G_pink.dds",
"Data/UI/item_icons/rune_4_G_pink_a.dds",
"Data/UI/item_icons/rune_4_G_red.dds",
"Data/UI/item_icons/rune_4_G_red_a.dds",
"Data/UI/item_icons/rune_4_G_violet.dds",
"Data/UI/item_icons/rune_4_G_violet_a.dds",
"Data/UI/item_icons/rune_4_G_yellow.dds",
"Data/UI/item_icons/rune_4_G_yellow_a.dds",
"Data/UI/item_icons/rune_4_H_blue.dds",
"Data/UI/item_icons/rune_4_H_blue_a.dds",
"Data/UI/item_icons/rune_4_H_green.dds",
"Data/UI/item_icons/rune_4_H_green_a.dds",
"Data/UI/item_icons/rune_4_H_pink.dds",
"Data/UI/item_icons/rune_4_H_pink_a.dds",
"Data/UI/item_icons/rune_4_H_red.dds",
"Data/UI/item_icons/rune_4_H_red_a.dds",
"Data/UI/item_icons/rune_4_H_violet.dds",
"Data/UI/item_icons/rune_4_H_violet_a.dds",
"Data/UI/item_icons/rune_4_H_yellow.dds",
"Data/UI/item_icons/rune_4_H_yellow_a.dds",
"Data/UI/item_icons/rune_4_I_blue.dds",
"Data/UI/item_icons/rune_4_I_blue_a.dds",
"Data/UI/item_icons/rune_4_I_green.dds",
"Data/UI/item_icons/rune_4_I_green_a.dds",
"Data/UI/item_icons/rune_4_I_pink.dds",
"Data/UI/item_icons/rune_4_I_pink_a.dds",
"Data/UI/item_icons/rune_4_I_red.dds",
"Data/UI/item_icons/rune_4_I_red_a.dds",
"Data/UI/item_icons/rune_4_I_violet.dds",
"Data/UI/item_icons/rune_4_I_violet_a.dds",
"Data/UI/item_icons/rune_4_I_yellow.dds",
"Data/UI/item_icons/rune_4_I_yellow_a.dds",
"Data/UI/item_icons/rune_4_J_blue.dds",
"Data/UI/item_icons/rune_4_J_blue_a.dds",
"Data/UI/item_icons/rune_4_J_green.dds",
"Data/UI/item_icons/rune_4_J_green_a.dds",
"Data/UI/item_icons/rune_4_J_pink.dds",
"Data/UI/item_icons/rune_4_J_pink_a.dds",
"Data/UI/item_icons/rune_4_J_red.dds",
"Data/UI/item_icons/rune_4_J_red_a.dds",
"Data/UI/item_icons/rune_4_J_violet.dds",
"Data/UI/item_icons/rune_4_J_violet_a.dds",
"Data/UI/item_icons/rune_4_J_yellow.dds",
"Data/UI/item_icons/rune_4_J_yellow_a.dds",
"Data/UI/item_icons/rune_5_A_blue.dds",
"Data/UI/item_icons/rune_5_A_blue_a.dds",
"Data/UI/item_icons/rune_5_A_green.dds",
"Data/UI/item_icons/rune_5_A_green_a.dds",
"Data/UI/item_icons/rune_5_A_pink.dds",
"Data/UI/item_icons/rune_5_A_pink_a.dds",
"Data/UI/item_icons/rune_5_A_red.dds",
"Data/UI/item_icons/rune_5_A_red_a.dds",
"Data/UI/item_icons/rune_5_A_violet.dds",
"Data/UI/item_icons/rune_5_A_violet_a.dds",
"Data/UI/item_icons/rune_5_A_yellow.dds",
"Data/UI/item_icons/rune_5_A_yellow_a.dds",
"Data/UI/item_icons/rune_5_B_blue.dds",
"Data/UI/item_icons/rune_5_B_blue_a.dds",
"Data/UI/item_icons/rune_5_B_green.dds",
"Data/UI/item_icons/rune_5_B_green_a.dds",
"Data/UI/item_icons/rune_5_B_pink.dds",
"Data/UI/item_icons/rune_5_B_pink_a.dds",
"Data/UI/item_icons/rune_5_B_red.dds",
"Data/UI/item_icons/rune_5_B_red_a.dds",
"Data/UI/item_icons/rune_5_B_violet.dds",
"Data/UI/item_icons/rune_5_B_violet_a.dds",
"Data/UI/item_icons/rune_5_B_yellow.dds",
"Data/UI/item_icons/rune_5_B_yellow_a.dds",
"Data/UI/item_icons/rune_5_C_blue.dds",
"Data/UI/item_icons/rune_5_C_blue_a.dds",
"Data/UI/item_icons/rune_5_C_green.dds",
"Data/UI/item_icons/rune_5_C_green_a.dds",
"Data/UI/item_icons/rune_5_C_pink.dds",
"Data/UI/item_icons/rune_5_C_pink_a.dds",
"Data/UI/item_icons/rune_5_C_red.dds",
"Data/UI/item_icons/rune_5_C_red_a.dds",
"Data/UI/item_icons/rune_5_C_violet.dds",
"Data/UI/item_icons/rune_5_C_violet_a.dds",
"Data/UI/item_icons/rune_5_C_yellow.dds",
"Data/UI/item_icons/rune_5_C_yellow_a.dds",
"Data/UI/item_icons/rune_5_D_blue.dds",
"Data/UI/item_icons/rune_5_D_blue_a.dds",
"Data/UI/item_icons/rune_5_D_green.dds",
"Data/UI/item_icons/rune_5_D_green_a.dds",
"Data/UI/item_icons/rune_5_D_pink.dds",
"Data/UI/item_icons/rune_5_D_pink_a.dds",
"Data/UI/item_icons/rune_5_D_red.dds",
"Data/UI/item_icons/rune_5_D_red_a.dds",
"Data/UI/item_icons/rune_5_D_violet.dds",
"Data/UI/item_icons/rune_5_D_violet_a.dds",
"Data/UI/item_icons/rune_5_D_yellow.dds",
"Data/UI/item_icons/rune_5_D_yellow_a.dds",
"Data/UI/item_icons/rune_5_E_blue.dds",
"Data/UI/item_icons/rune_5_E_blue_a.dds",
"Data/UI/item_icons/rune_5_E_green.dds",
"Data/UI/item_icons/rune_5_E_green_a.dds",
"Data/UI/item_icons/rune_5_E_pink.dds",
"Data/UI/item_icons/rune_5_E_pink_a.dds",
"Data/UI/item_icons/rune_5_E_red.dds",
"Data/UI/item_icons/rune_5_E_red_a.dds",
"Data/UI/item_icons/rune_5_E_violet.dds",
"Data/UI/item_icons/rune_5_E_violet_a.dds",
"Data/UI/item_icons/rune_5_E_yellow.dds",
"Data/UI/item_icons/rune_5_E_yellow_a.dds",
"Data/UI/item_icons/rune_5_F_blue.dds",
"Data/UI/item_icons/rune_5_F_blue_a.dds",
"Data/UI/item_icons/rune_5_F_green.dds",
"Data/UI/item_icons/rune_5_F_green_a.dds",
"Data/UI/item_icons/rune_5_F_pink.dds",
"Data/UI/item_icons/rune_5_F_pink_a.dds",
"Data/UI/item_icons/rune_5_F_red.dds",
"Data/UI/item_icons/rune_5_F_red_a.dds",
"Data/UI/item_icons/rune_5_F_violet.dds",
"Data/UI/item_icons/rune_5_F_violet_a.dds",
"Data/UI/item_icons/rune_5_F_yellow.dds",
"Data/UI/item_icons/rune_5_F_yellow_a.dds",
"Data/UI/item_icons/rune_5_G_blue.dds",
"Data/UI/item_icons/rune_5_G_blue_a.dds",
"Data/UI/item_icons/rune_5_G_green.dds",
"Data/UI/item_icons/rune_5_G_green_a.dds",
"Data/UI/item_icons/rune_5_G_pink.dds",
"Data/UI/item_icons/rune_5_G_pink_a.dds",
"Data/UI/item_icons/rune_5_G_red.dds",
"Data/UI/item_icons/rune_5_G_red_a.dds",
"Data/UI/item_icons/rune_5_G_violet.dds",
"Data/UI/item_icons/rune_5_G_violet_a.dds",
"Data/UI/item_icons/rune_5_G_yellow.dds",
"Data/UI/item_icons/rune_5_G_yellow_a.dds",
"Data/UI/item_icons/rune_5_H_blue.dds",
"Data/UI/item_icons/rune_5_H_blue_a.dds",
"Data/UI/item_icons/rune_5_H_green.dds",
"Data/UI/item_icons/rune_5_H_green_a.dds",
"Data/UI/item_icons/rune_5_H_pink.dds",
"Data/UI/item_icons/rune_5_H_pink_a.dds",
"Data/UI/item_icons/rune_5_H_red.dds",
"Data/UI/item_icons/rune_5_H_red_a.dds",
"Data/UI/item_icons/rune_5_H_violet.dds",
"Data/UI/item_icons/rune_5_H_violet_a.dds",
"Data/UI/item_icons/rune_5_H_yellow.dds",
"Data/UI/item_icons/rune_5_H_yellow_a.dds",
"Data/UI/item_icons/rune_5_I_blue.dds",
"Data/UI/item_icons/rune_5_I_blue_a.dds",
"Data/UI/item_icons/rune_5_I_green.dds",
"Data/UI/item_icons/rune_5_I_green_a.dds",
"Data/UI/item_icons/rune_5_I_pink.dds",
"Data/UI/item_icons/rune_5_I_pink_a.dds",
"Data/UI/item_icons/rune_5_I_red.dds",
"Data/UI/item_icons/rune_5_I_red_a.dds",
"Data/UI/item_icons/rune_5_I_violet.dds",
"Data/UI/item_icons/rune_5_I_violet_a.dds",
"Data/UI/item_icons/rune_5_I_yellow.dds",
"Data/UI/item_icons/rune_5_I_yellow_a.dds",
"Data/UI/item_icons/rune_5_J_blue.dds",
"Data/UI/item_icons/rune_5_J_blue_a.dds",
"Data/UI/item_icons/rune_5_J_green.dds",
"Data/UI/item_icons/rune_5_J_green_a.dds",
"Data/UI/item_icons/rune_5_J_pink.dds",
"Data/UI/item_icons/rune_5_J_pink_a.dds",
"Data/UI/item_icons/rune_5_J_red.dds",
"Data/UI/item_icons/rune_5_J_red_a.dds",
"Data/UI/item_icons/rune_5_J_violet.dds",
"Data/UI/item_icons/rune_5_J_violet_a.dds",
"Data/UI/item_icons/rune_5_J_yellow.dds",
"Data/UI/item_icons/rune_5_J_yellow_a.dds",
"Data/UI/item_icons/runic_potion.dds",
"Data/UI/item_icons/sack_of_coins.dds",
"Data/UI/item_icons/sagebrush.dds",
"Data/UI/item_icons/sagebrush_a.dds",
"Data/UI/item_icons/sagebrush_lumber.dds",
"Data/UI/item_icons/sagebrush_lumber_a.dds",
"Data/UI/item_icons/sagebrush_timber.dds",
"Data/UI/item_icons/salve_1.dds",
"Data/UI/item_icons/salve_10.dds",
"Data/UI/item_icons/salve_10a.dds",
"Data/UI/item_icons/salve_10b.dds",
"Data/UI/item_icons/salve_11.dds",
"Data/UI/item_icons/salve_11a.dds",
"Data/UI/item_icons/salve_12.dds",
"Data/UI/item_icons/salve_12a.dds",
"Data/UI/item_icons/salve_13.dds",
"Data/UI/item_icons/salve_13a.dds",
"Data/UI/item_icons/salve_1a.dds",
"Data/UI/item_icons/salve_2.dds",
"Data/UI/item_icons/salve_2a.dds",
"Data/UI/item_icons/salve_3.dds",
"Data/UI/item_icons/salve_4.dds",
"Data/UI/item_icons/salve_5a.dds",
"Data/UI/item_icons/salve_5b.dds",
"Data/UI/item_icons/salve_5c.dds",
"Data/UI/item_icons/salve_6.dds",
"Data/UI/item_icons/salve_6a.dds",
"Data/UI/item_icons/salve_6b.dds",
"Data/UI/item_icons/salve_7.dds",
"Data/UI/item_icons/salve_7a.dds",
"Data/UI/item_icons/salve_8.dds",
"Data/UI/item_icons/salve_8a.dds",
"Data/UI/item_icons/salve_9.dds",
"Data/UI/item_icons/sanctified_thorne_fetter.dds",
"Data/UI/item_icons/sandy_bladder.dds",
"Data/UI/item_icons/sandy_bladder_a.dds",
"Data/UI/item_icons/sapphire.dds",
"Data/UI/item_icons/scales.dds",
"Data/UI/item_icons/scanning_device.dds",
"Data/UI/item_icons/scanning_device_a.dds",
"Data/UI/item_icons/scorched_page.dds",
"Data/UI/item_icons/scorched_page_a.dds",
"Data/UI/item_icons/scottys_braclet.dds",
"Data/UI/item_icons/scottys_braclet_a.dds",
"Data/UI/item_icons/scroll1.dds",
"Data/UI/item_icons/scroll10.dds",
"Data/UI/item_icons/scroll10a.dds",
"Data/UI/item_icons/scroll1a.dds",
"Data/UI/item_icons/scroll2.dds",
"Data/UI/item_icons/scroll2a.dds",
"Data/UI/item_icons/scroll3.dds",
"Data/UI/item_icons/scroll4.dds",
"Data/UI/item_icons/scroll4b.dds",
"Data/UI/item_icons/scroll4c.dds",
"Data/UI/item_icons/scroll4d.dds",
"Data/UI/item_icons/scroll4e.dds",
"Data/UI/item_icons/scroll5.dds",
"Data/UI/item_icons/scroll5b.dds",
"Data/UI/item_icons/scroll6.dds",
"Data/UI/item_icons/scroll6a.dds",
"Data/UI/item_icons/scroll7.dds",
"Data/UI/item_icons/scroll8.dds",
"Data/UI/item_icons/scroll8a.dds",
"Data/UI/item_icons/scroll8b.dds",
"Data/UI/item_icons/scroll9.dds",
"Data/UI/item_icons/scroll9a.dds",
"Data/UI/item_icons/seacap_husk.dds",
"Data/UI/item_icons/seacap_innards.dds",
"Data/UI/item_icons/seacap_leg.dds",
"Data/UI/item_icons/seacap_leg_a.dds",
"Data/UI/item_icons/sealed_parchment.dds",
"Data/UI/item_icons/sea_bell.dds",
"Data/UI/item_icons/sea_bell_a.dds",
"Data/UI/item_icons/sea_charm.dds",
"Data/UI/item_icons/seed_of_the_heart_grove.dds",
"Data/UI/item_icons/seed_of_the_heart_grove_a.dds",
"Data/UI/item_icons/serla_irmgard_head.dds",
"Data/UI/item_icons/severed_orc_head.dds",
"Data/UI/item_icons/severed_wing.dds",
"Data/UI/item_icons/sextant.dds",
"Data/UI/item_icons/shademarked_bone_fragment_1.dds",
"Data/UI/item_icons/shademarked_bone_fragment_1_a.dds",
"Data/UI/item_icons/shademarked_bone_fragment_1_b.dds",
"Data/UI/item_icons/shademarked_bone_fragment_2.dds",
"Data/UI/item_icons/shademarked_bone_fragment_2_a.dds",
"Data/UI/item_icons/shademarked_bone_fragment_2_b.dds",
"Data/UI/item_icons/shademarked_bone_fragment_3.dds",
"Data/UI/item_icons/shademarked_bone_fragment_3_a.dds",
"Data/UI/item_icons/shademarked_bone_fragment_3_b.dds",
"Data/UI/item_icons/shade_touche_ether.dds",
"Data/UI/item_icons/shade_touche_ether_a.dds",
"Data/UI/item_icons/shade_touche_ether_b.dds",
"Data/UI/item_icons/shade_touche_ether_c.dds",
"Data/UI/item_icons/shade_touche_ether_d.dds",
"Data/UI/item_icons/shadowy_bottle.dds",
"Data/UI/item_icons/shadowy_bottle_a.dds",
"Data/UI/item_icons/shaggy_mane.dds",
"Data/UI/item_icons/shambler_claw.dds",
"Data/UI/item_icons/shambler_claw2.dds",
"Data/UI/item_icons/shambler_claw_a.dds",
"Data/UI/item_icons/shambler_claw_b.dds",
"Data/UI/item_icons/shambler_snout.dds",
"Data/UI/item_icons/shambler_snout_a.dds",
"Data/UI/item_icons/shambler_tail.dds",
"Data/UI/item_icons/shambler_tail_a.dds",
"Data/UI/item_icons/shattered_scabbard.dds",
"Data/UI/item_icons/shield_air_epic.dds",
"Data/UI/item_icons/shield_air_epic_a.dds",
"Data/UI/item_icons/shield_death_epic.dds",
"Data/UI/item_icons/shield_death_epic_a.dds",
"Data/UI/item_icons/shield_earth_epic.dds",
"Data/UI/item_icons/shield_earth_epic_a.dds",
"Data/UI/item_icons/shield_fire_mundane.dds",
"Data/UI/item_icons/shield_fire_mundane_a.dds",
"Data/UI/item_icons/shield_water_epic.dds",
"Data/UI/item_icons/shield_water_epic_a.dds",
"Data/UI/item_icons/shimmering_opal.dds",
"Data/UI/item_icons/shimmering_opal_a.dds",
"Data/UI/item_icons/shining_scale.dds",
"Data/UI/item_icons/shiny_shell_fragment.dds",
"Data/UI/item_icons/shiv.dds",
"Data/UI/item_icons/shiv_poisoned.dds",
"Data/UI/item_icons/shovel1.dds",
"Data/UI/item_icons/shovel2.dds",
"Data/UI/item_icons/shovel3.dds",
"Data/UI/item_icons/shovel4_a.dds",
"Data/UI/item_icons/shovel4_b.dds",
"Data/UI/item_icons/shovel4_c.dds",
"Data/UI/item_icons/shovel5.dds",
"Data/UI/item_icons/shovel6.dds",
"Data/UI/item_icons/shovel6_a.dds",
"Data/UI/item_icons/shovel7.dds",
"Data/UI/item_icons/shovel8.dds",
"Data/UI/item_icons/shovel8_a.dds",
"Data/UI/item_icons/shrekenfish_eggs.dds",
"Data/UI/item_icons/silken_elven_kerchief.dds",
"Data/UI/item_icons/silk_entwined_cadaver.dds",
"Data/UI/item_icons/silverscale_perch.dds",
"Data/UI/item_icons/silver_bars.dds",
"Data/UI/item_icons/silver_bars_a.dds",
"Data/UI/item_icons/silver_fleece.dds",
"Data/UI/item_icons/silver_leaf.dds",
"Data/UI/item_icons/silver_leaf_a.dds",
"Data/UI/item_icons/silver_sextant.dds",
"Data/UI/item_icons/six-winged-butterfly.dds",
"Data/UI/item_icons/six-winged-butterfly1.dds",
"Data/UI/item_icons/skull.dds",
"Data/UI/item_icons/skystone_agate.dds",
"Data/UI/item_icons/slime_covered_tongue.dds",
"Data/UI/item_icons/small_guarded_crate.dds",
"Data/UI/item_icons/small_mosscovered_stone.dds",
"Data/UI/item_icons/small_student_experiment.dds",
"Data/UI/item_icons/small_student_experiment2.dds",
"Data/UI/item_icons/smelly_hoof.dds",
"Data/UI/item_icons/smooth_sourcestone.dds",
"Data/UI/item_icons/smooth_sourcestone_z.dds",
"Data/UI/item_icons/smooth_sourcestone_z1.dds",
"Data/UI/item_icons/smooth_sourcestone_z2.dds",
"Data/UI/item_icons/smooth_wax.dds",
"Data/UI/item_icons/snipe_foot.dds",
"Data/UI/item_icons/softpaw_carcass.dds",
"Data/UI/item_icons/softpaw_carcass_a.dds",
"Data/UI/item_icons/softwood.dds",
"Data/UI/item_icons/softwood_timber.dds",
"Data/UI/item_icons/soft_paw.dds",
"Data/UI/item_icons/solid_hardwood_plank2.dds",
"Data/UI/item_icons/solid_softwood_plank.dds",
"Data/UI/item_icons/solid_softwood_plank_a.dds",
"Data/UI/item_icons/songs_of_mistran.dds",
"Data/UI/item_icons/songs_of_mistran_a.dds",
"Data/UI/item_icons/soulhide.dds",
"Data/UI/item_icons/soulhide_2.dds",
"Data/UI/item_icons/soulhide_a.dds",
"Data/UI/item_icons/soulhide_a_2.dds",
"Data/UI/item_icons/soulhide_b.dds",
"Data/UI/item_icons/soulstitched_silk_cloth.dds",
"Data/UI/item_icons/soul_archon.dds",
"Data/UI/item_icons/soul_assassin.dds",
"Data/UI/item_icons/soul_bard.dds",
"Data/UI/item_icons/soul_beastmaster.dds",
"Data/UI/item_icons/soul_bladedancer.dds",
"Data/UI/item_icons/soul_cablist.dds",
"Data/UI/item_icons/soul_champion.dds",
"Data/UI/item_icons/soul_chloromancer.dds",
"Data/UI/item_icons/soul_cleric.dds",
"Data/UI/item_icons/soul_dominator.dds",
"Data/UI/item_icons/soul_druid.dds",
"Data/UI/item_icons/soul_elementalist.dds",
"Data/UI/item_icons/soul_fragment1.dds",
"Data/UI/item_icons/soul_fragment1_a.dds",
"Data/UI/item_icons/soul_fragment2.dds",
"Data/UI/item_icons/soul_fragment2_a.dds",
"Data/UI/item_icons/soul_fragment2_b.dds",
"Data/UI/item_icons/soul_fragment3.dds",
"Data/UI/item_icons/soul_fragment3_a.dds",
"Data/UI/item_icons/soul_icon_slot.dds",
"Data/UI/item_icons/soul_inquisitor.dds",
"Data/UI/item_icons/soul_justicar.dds",
"Data/UI/item_icons/soul_mage.dds",
"Data/UI/item_icons/soul_marksman.dds",
"Data/UI/item_icons/soul_necromancer.dds",
"Data/UI/item_icons/soul_nightblade.dds",
"Data/UI/item_icons/soul_paladin.dds",
"Data/UI/item_icons/soul_paragon.dds",
"Data/UI/item_icons/soul_purifier.dds",
"Data/UI/item_icons/soul_pyromancer.dds",
"Data/UI/item_icons/soul_ranger.dds",
"Data/UI/item_icons/soul_reaver.dds",
"Data/UI/item_icons/soul_riftblade.dds",
"Data/UI/item_icons/soul_riftstalker.dds",
"Data/UI/item_icons/soul_rogue.dds",
"Data/UI/item_icons/soul_saboteur.dds",
"Data/UI/item_icons/soul_sentinel.dds",
"Data/UI/item_icons/soul_shaman.dds",
"Data/UI/item_icons/soul_stormcaller.dds",
"Data/UI/item_icons/soul_voidknight.dds",
"Data/UI/item_icons/soul_warden.dds",
"Data/UI/item_icons/soul_warlock.dds",
"Data/UI/item_icons/soul_warlord.dds",
"Data/UI/item_icons/soul_warrior.dds",
"Data/UI/item_icons/sourceshard_01.dds",
"Data/UI/item_icons/sourceshard_02.dds",
"Data/UI/item_icons/sourceshard_02z.dds",
"Data/UI/item_icons/sourceshard_02z_z.dds",
"Data/UI/item_icons/sourceshard_02_a.dds",
"Data/UI/item_icons/sourceshard_02_b.dds",
"Data/UI/item_icons/sourceshard_03.dds",
"Data/UI/item_icons/sourceshard_03a.dds",
"Data/UI/item_icons/sourceshard_03_a.dds",
"Data/UI/item_icons/sourceshard_04.dds",
"Data/UI/item_icons/sourceshard_04_a.dds",
"Data/UI/item_icons/sourceshard_05_a.dds",
"Data/UI/item_icons/sourcestone_01.dds",
"Data/UI/item_icons/sourcestone_02.dds",
"Data/UI/item_icons/sourcestone_02z.dds",
"Data/UI/item_icons/sourcestone_02z_z.dds",
"Data/UI/item_icons/sourcestone_02_a.dds",
"Data/UI/item_icons/sourcestone_02_b.dds",
"Data/UI/item_icons/sourcestone_03.dds",
"Data/UI/item_icons/sourcestone_03_a.dds",
"Data/UI/item_icons/sourcestone_04.dds",
"Data/UI/item_icons/sourcestone_04_a.dds",
"Data/UI/item_icons/sourcestone_05.dds",
"Data/UI/item_icons/sourcestone_agate.dds",
"Data/UI/item_icons/sourcestone_battery.dds",
"Data/UI/item_icons/spare_boot_hooks.dds",
"Data/UI/item_icons/sparrow.dds",
"Data/UI/item_icons/sparrow_b.dds",
"Data/UI/item_icons/sparrow_c.dds",
"Data/UI/item_icons/spellspun_silk.dds",
"Data/UI/item_icons/spellspun_silk_a.dds",
"Data/UI/item_icons/spellstone_03.dds",
"Data/UI/item_icons/spellstone_03_a.dds",
"Data/UI/item_icons/spellstone_04.dds",
"Data/UI/item_icons/spellstone_04_a.dds",
"Data/UI/item_icons/spellstone_04_b.dds",
"Data/UI/item_icons/spiderleg1.dds",
"Data/UI/item_icons/spiderleg1a.dds",
"Data/UI/item_icons/spiderleg2.dds",
"Data/UI/item_icons/spiderleg2a.dds",
"Data/UI/item_icons/spikes_01.dds",
"Data/UI/item_icons/spikes_02.dds",
"Data/UI/item_icons/spikes_03.dds",
"Data/UI/item_icons/spikes_03_a.dds",
"Data/UI/item_icons/spikes_03_b.dds",
"Data/UI/item_icons/spine1.dds",
"Data/UI/item_icons/spine2.dds",
"Data/UI/item_icons/spiteful_roots.dds",
"Data/UI/item_icons/spiteful_roots_a.dds",
"Data/UI/item_icons/split_hardwood_plank.dds",
"Data/UI/item_icons/split_hardwood_plank2.dds",
"Data/UI/item_icons/split_hardwood_plank_a.dds",
"Data/UI/item_icons/split_softwood_plank.dds",
"Data/UI/item_icons/split_softwood_plank_a.dds",
"Data/UI/item_icons/star_sapphire.dds",
"Data/UI/item_icons/stolen_silverwood_honey.dds",
"Data/UI/item_icons/stone_effigy.dds",
"Data/UI/item_icons/stone_potion.dds",
"Data/UI/item_icons/sturdy_hardwood_plank.dds",
"Data/UI/item_icons/sunstone.dds",
"Data/UI/item_icons/superior_potion_cylindrical_0.dds",
"Data/UI/item_icons/superior_potion_cylindrical_1.dds",
"Data/UI/item_icons/superior_potion_cylindrical_2.dds",
"Data/UI/item_icons/superior_potion_cylindrical_3.dds",
"Data/UI/item_icons/superior_potion_cylindrical_4.dds",
"Data/UI/item_icons/superior_potion_cylindrical_5.dds",
"Data/UI/item_icons/superior_potion_cylindrical_6.dds",
"Data/UI/item_icons/superior_potion_rectangular_0.dds",
"Data/UI/item_icons/superior_potion_rectangular_1.dds",
"Data/UI/item_icons/superior_potion_rectangular_2.dds",
"Data/UI/item_icons/superior_potion_rectangular_3.dds",
"Data/UI/item_icons/superior_potion_rectangular_4.dds",
"Data/UI/item_icons/superior_potion_rectangular_5.dds",
"Data/UI/item_icons/superior_potion_rectangular_6.dds",
"Data/UI/item_icons/superior_potion_spherical_0.dds",
"Data/UI/item_icons/superior_potion_spherical_1.dds",
"Data/UI/item_icons/superior_potion_spherical_2.dds",
"Data/UI/item_icons/superior_potion_spherical_3.dds",
"Data/UI/item_icons/superior_potion_spherical_4.dds",
"Data/UI/item_icons/superior_potion_spherical_5.dds",
"Data/UI/item_icons/superior_potion_spherical_6.dds",
"Data/UI/item_icons/supply_crate.dds",
"Data/UI/item_icons/surveying_object.dds",
"Data/UI/item_icons/suzies_doll.dds",
"Data/UI/item_icons/tacky_wax.dds",
"Data/UI/item_icons/tanned_flesh.dds",
"Data/UI/item_icons/tattered_robes.dds",
"Data/UI/item_icons/tavra.dds",
"Data/UI/item_icons/tavra_a.dds",
"Data/UI/item_icons/tavra_gnarl1.dds",
"Data/UI/item_icons/tavra_gnarl1_a.dds",
"Data/UI/item_icons/tavra_gnarl2.dds",
"Data/UI/item_icons/tavra_gnarl2_a.dds",
"Data/UI/item_icons/tavra_gnarl3.dds",
"Data/UI/item_icons/tavra_gnarl3_a.dds",
"Data/UI/item_icons/tavra_gnarl3_b.dds",
"Data/UI/item_icons/tavra_shard1.dds",
"Data/UI/item_icons/tavra_shard2.dds",
"Data/UI/item_icons/tavra_shard3.dds",
"Data/UI/item_icons/tavra_splinter1.dds",
"Data/UI/item_icons/tavra_splinter1_a.dds",
"Data/UI/item_icons/tavra_splinter2.dds",
"Data/UI/item_icons/tavra_splinter2_a.dds",
"Data/UI/item_icons/tavra_splinter3.dds",
"Data/UI/item_icons/tavra_splinter3_a.dds",
"Data/UI/item_icons/tavra_splinter_grafts.dds",
"Data/UI/item_icons/tavra_splinter_grafts_a.dds",
"Data/UI/item_icons/teardrop_talisman.dds",
"Data/UI/item_icons/teardrop_talisman_a.dds",
"Data/UI/item_icons/tektite_shard.dds",
"Data/UI/item_icons/the_coming_darkness.dds",
"Data/UI/item_icons/the_coming_darkness_a.dds",
"Data/UI/item_icons/thick_leather_a.dds",
"Data/UI/item_icons/thick_shell_fragment.dds",
"Data/UI/item_icons/thimble_of_cream.dds",
"Data/UI/item_icons/thin_hide.dds",
"Data/UI/item_icons/thin_leather_a.dds",
"Data/UI/item_icons/thin_leather_a_1.dds",
"Data/UI/item_icons/thin_leather_b.dds",
"Data/UI/item_icons/thread1.dds",
"Data/UI/item_icons/thread1a.dds",
"Data/UI/item_icons/thread2.dds",
"Data/UI/item_icons/thread3.dds",
"Data/UI/item_icons/thread3a.dds",
"Data/UI/item_icons/thread3b.dds",
"Data/UI/item_icons/thread3z.dds",
"Data/UI/item_icons/thread4.dds",
"Data/UI/item_icons/thread4a.dds",
"Data/UI/item_icons/thread5.dds",
"Data/UI/item_icons/thread6a.dds",
"Data/UI/item_icons/thread6b.dds",
"Data/UI/item_icons/thread6z.dds",
"Data/UI/item_icons/thread7.dds",
"Data/UI/item_icons/thread8.dds",
"Data/UI/item_icons/tiny_totem.dds",
"Data/UI/item_icons/titan_shards.dds",
"Data/UI/item_icons/titan_shards_a.dds",
"Data/UI/item_icons/titan_shards_b.dds",
"Data/UI/item_icons/titan_shards_c.dds",
"Data/UI/item_icons/titan_shards_d.dds",
"Data/UI/item_icons/toad_skin.dds",
"Data/UI/item_icons/tole_family_deed.dds",
"Data/UI/item_icons/tongs1.dds",
"Data/UI/item_icons/tongs1_a.dds",
"Data/UI/item_icons/tongs2.dds",
"Data/UI/item_icons/tongs2_a.dds",
"Data/UI/item_icons/tongs3.dds",
"Data/UI/item_icons/tongs4.dds",
"Data/UI/item_icons/tongs4a.dds",
"Data/UI/item_icons/tongs5.dds",
"Data/UI/item_icons/tongs5a.dds",
"Data/UI/item_icons/tooth1.dds",
"Data/UI/item_icons/tooth2.dds",
"Data/UI/item_icons/tooth3.dds",
"Data/UI/item_icons/tooth3a.dds",
"Data/UI/item_icons/tornpaper01.dds",
"Data/UI/item_icons/tornpaper02.dds",
"Data/UI/item_icons/tornpaper03.dds",
"Data/UI/item_icons/tornpaper04.dds",
"Data/UI/item_icons/tornpaper05.dds",
"Data/UI/item_icons/tornpaper06.dds",
"Data/UI/item_icons/tornpaper07.dds",
"Data/UI/item_icons/tornpaper08.dds",
"Data/UI/item_icons/tornpaper09.dds",
"Data/UI/item_icons/tornpaper10.dds",
"Data/UI/item_icons/tornpaper11.dds",
"Data/UI/item_icons/tornpaper12.dds",
"Data/UI/item_icons/tornpaper13y.dds",
"Data/UI/item_icons/tornpaper14y.dds",
"Data/UI/item_icons/tornpaper15y.dds",
"Data/UI/item_icons/tornpaper16y.dds",
"Data/UI/item_icons/tornscroll1.dds",
"Data/UI/item_icons/torn_barghest_ear.dds",
"Data/UI/item_icons/torn_headwrap.dds",
"Data/UI/item_icons/torn_lootbag.dds",
"Data/UI/item_icons/torn_satin.dds",
"Data/UI/item_icons/torn_silken_kerchief.dds",
"Data/UI/item_icons/torn_spinneret.dds",
"Data/UI/item_icons/totem1c.dds",
"Data/UI/item_icons/totem1z.dds",
"Data/UI/item_icons/totem2b.dds",
"Data/UI/item_icons/totem2bb.dds",
"Data/UI/item_icons/totem3.dds",
"Data/UI/item_icons/totem4.dds",
"Data/UI/item_icons/totem4a.dds",
"Data/UI/item_icons/totem5.dds",
"Data/UI/item_icons/totem5a.dds",
"Data/UI/item_icons/totem6.dds",
"Data/UI/item_icons/totem7.dds",
"Data/UI/item_icons/totem8.dds",
"Data/UI/item_icons/totem8a.dds",
"Data/UI/item_icons/totem9.dds",
"Data/UI/item_icons/totem_air_mundane.dds",
"Data/UI/item_icons/totem_air_mundane_a.dds",
"Data/UI/item_icons/totem_earth_epic.dds",
"Data/UI/item_icons/totem_earth_epic_a.dds",
"Data/UI/item_icons/totem_earth_epic_b.dds",
"Data/UI/item_icons/totem_earth_mundane.dds",
"Data/UI/item_icons/totem_earth_mundane_a.dds",
"Data/UI/item_icons/totem_fire_epic_a.dds",
"Data/UI/item_icons/totem_water_epic.dds",
"Data/UI/item_icons/totem_water_epic_a.dds",
"Data/UI/item_icons/tragedy_of_fennain.dds",
"Data/UI/item_icons/transculent_aether.dds",
"Data/UI/item_icons/treasure_chest_11.dds",
"Data/UI/item_icons/treasure_chest_12.dds",
"Data/UI/item_icons/treasure_chest_13.dds",
"Data/UI/item_icons/trinket1.dds",
"Data/UI/item_icons/trinket10.dds",
"Data/UI/item_icons/trinket10a.dds",
"Data/UI/item_icons/trinket11.dds",
"Data/UI/item_icons/trinket11a.dds",
"Data/UI/item_icons/trinket12.dds",
"Data/UI/item_icons/trinket13.dds",
"Data/UI/item_icons/trinket13a.dds",
"Data/UI/item_icons/trinket14.dds",
"Data/UI/item_icons/trinket14a.dds",
"Data/UI/item_icons/trinket15.dds",
"Data/UI/item_icons/trinket15a.dds",
"Data/UI/item_icons/trinket15b.dds",
"Data/UI/item_icons/trinket16.dds",
"Data/UI/item_icons/trinket17.dds",
"Data/UI/item_icons/trinket18.dds",
"Data/UI/item_icons/trinket18a.dds",
"Data/UI/item_icons/trinket19.dds",
"Data/UI/item_icons/trinket2.dds",
"Data/UI/item_icons/trinket20.dds",
"Data/UI/item_icons/trinket20a.dds",
"Data/UI/item_icons/trinket21.dds",
"Data/UI/item_icons/trinket22.dds",
"Data/UI/item_icons/trinket22a.dds",
"Data/UI/item_icons/trinket23.dds",
"Data/UI/item_icons/trinket24.dds",
"Data/UI/item_icons/trinket24a.dds",
"Data/UI/item_icons/trinket25.dds",
"Data/UI/item_icons/trinket25a.dds",
"Data/UI/item_icons/trinket26a.dds",
"Data/UI/item_icons/trinket27.dds",
"Data/UI/item_icons/trinket27a.dds",
"Data/UI/item_icons/trinket28.dds",
"Data/UI/item_icons/trinket28a.dds",
"Data/UI/item_icons/trinket28z.dds",
"Data/UI/item_icons/trinket29.dds",
"Data/UI/item_icons/trinket3.dds",
"Data/UI/item_icons/trinket30.dds",
"Data/UI/item_icons/trinket30a.dds",
"Data/UI/item_icons/trinket31.dds",
"Data/UI/item_icons/trinket4.dds",
"Data/UI/item_icons/trinket4a.dds",
"Data/UI/item_icons/trinket5.dds",
"Data/UI/item_icons/trinket6.dds",
"Data/UI/item_icons/trinket6a.dds",
"Data/UI/item_icons/trinket7.dds",
"Data/UI/item_icons/trinket8.dds",
"Data/UI/item_icons/trinket9.dds",
"Data/UI/item_icons/troglodyte_head.dds",
"Data/UI/item_icons/troll_skin.dds",
"Data/UI/item_icons/troll_skin_a.dds",
"Data/UI/item_icons/troll_skin_b.dds",
"Data/UI/item_icons/troll_skin_c.dds",
"Data/UI/item_icons/trout_jerky.dds",
"Data/UI/item_icons/trout_jerky_a.dds",
"Data/UI/item_icons/two_headed_coin.dds",
"Data/UI/item_icons/two_heads.dds",
"Data/UI/item_icons/two_heads_a.dds",
"Data/UI/item_icons/unbreakable_potion.dds",
"Data/UI/item_icons/unbreakable_potion_a.dds",
"Data/UI/item_icons/unknown_metal_fragment.dds",
"Data/UI/item_icons/unmarked_stone_artifact.dds",
"Data/UI/item_icons/unrefined_runebirch.dds",
"Data/UI/item_icons/unrefined_sagebrush.dds",
"Data/UI/item_icons/used_bottle_cork.dds",
"Data/UI/item_icons/used_bottle_cork_a.dds",
"Data/UI/item_icons/valeglass_agate.dds",
"Data/UI/item_icons/vampiric_potion.dds",
"Data/UI/item_icons/vanity_109_helm.dds",
"Data/UI/item_icons/vanity_109_helm_a.dds",
"Data/UI/item_icons/vanity_111_shoulder.dds",
"Data/UI/item_icons/vanity_111_shoulder_a.dds",
"Data/UI/item_icons/vanity_112_helm.dds",
"Data/UI/item_icons/vanity_112_helm_a.dds",
"Data/UI/item_icons/vanity_112_shoulder.dds",
"Data/UI/item_icons/vanity_112_shoulder_a.dds",
"Data/UI/item_icons/vanity_113_helm.dds",
"Data/UI/item_icons/vanity_113_helm_a.dds",
"Data/UI/item_icons/vanity_113_helm_b.dds",
"Data/UI/item_icons/vanity_114_helm.dds",
"Data/UI/item_icons/vanity_114_helm_a.dds",
"Data/UI/item_icons/vanity_114_helm_b.dds",
"Data/UI/item_icons/vanity_115_helm.dds",
"Data/UI/item_icons/vanity_115_helm_a.dds",
"Data/UI/item_icons/vanity_115_helm_b.dds",
"Data/UI/item_icons/vanity_116_belt.dds",
"Data/UI/item_icons/vanity_116_belt_a.dds",
"Data/UI/item_icons/vanity_116_boots.dds",
"Data/UI/item_icons/vanity_116_boots_a.dds",
"Data/UI/item_icons/vanity_116_chest.dds",
"Data/UI/item_icons/vanity_116_chest_a.dds",
"Data/UI/item_icons/vanity_116_gloves.dds",
"Data/UI/item_icons/vanity_116_gloves_a.dds",
"Data/UI/item_icons/vanity_116_helm.dds",
"Data/UI/item_icons/vanity_116_helm_a.dds",
"Data/UI/item_icons/vanity_116_leggings.dds",
"Data/UI/item_icons/vanity_116_leggings_a.dds",
"Data/UI/item_icons/vanity_116_shoulder.dds",
"Data/UI/item_icons/vanity_116_shoulder_a.dds",
"Data/UI/item_icons/vanity_117_belt.dds",
"Data/UI/item_icons/vanity_117_belt_a.dds",
"Data/UI/item_icons/vanity_117_boots.dds",
"Data/UI/item_icons/vanity_117_boots_a.dds",
"Data/UI/item_icons/vanity_117_chest.dds",
"Data/UI/item_icons/vanity_117_chest_a.dds",
"Data/UI/item_icons/vanity_117_gloves.dds",
"Data/UI/item_icons/vanity_117_gloves_a.dds",
"Data/UI/item_icons/vanity_117_helm.dds",
"Data/UI/item_icons/vanity_117_helm_a.dds",
"Data/UI/item_icons/vanity_117_helm_b.dds",
"Data/UI/item_icons/vanity_117_leggings.dds",
"Data/UI/item_icons/vanity_117_leggings_a.dds",
"Data/UI/item_icons/vanity_117_shoulder.dds",
"Data/UI/item_icons/vanity_117_shoulder_a.dds",
"Data/UI/item_icons/vanity_118_boots.dds",
"Data/UI/item_icons/vanity_118_boots_a.dds",
"Data/UI/item_icons/vanity_118_chest.dds",
"Data/UI/item_icons/vanity_118_chest_a.dds",
"Data/UI/item_icons/vanity_118_gloves.dds",
"Data/UI/item_icons/vanity_118_gloves_a.dds",
"Data/UI/item_icons/vanity_118_helm.dds",
"Data/UI/item_icons/vanity_118_helm_a.dds",
"Data/UI/item_icons/vanity_118_leggings.dds",
"Data/UI/item_icons/vanity_118_leggings_a.dds",
"Data/UI/item_icons/vanity_118_shoulder.dds",
"Data/UI/item_icons/vanity_118_shoulder_a.dds",
"Data/UI/item_icons/vanity_120_helm.dds",
"Data/UI/item_icons/vanity_120_helm_a.dds",
"Data/UI/item_icons/vanity_120_shoulders.dds",
"Data/UI/item_icons/vanity_120_shoulders_a.dds",
"Data/UI/item_icons/vanity_121_gloves.dds",
"Data/UI/item_icons/vanity_121_gloves_a.dds",
"Data/UI/item_icons/vanity_121_helm.dds",
"Data/UI/item_icons/vanity_121_helm_a.dds",
"Data/UI/item_icons/vanity_122_gloves.dds",
"Data/UI/item_icons/vanity_122_gloves_a.dds",
"Data/UI/item_icons/vanity_122_gloves_y.dds",
"Data/UI/item_icons/vanity_122_gloves_z.dds",
"Data/UI/item_icons/vanity_122_helm.dds",
"Data/UI/item_icons/vanity_122_helm_a.dds",
"Data/UI/item_icons/vanity_122_helm_b.dds",
"Data/UI/item_icons/vanity_122_shoulders.dds",
"Data/UI/item_icons/vanity_122_shoulders_a.dds",
"Data/UI/item_icons/vanity_122_shoulders_b.dds",
"Data/UI/item_icons/vanity_123_helm.dds",
"Data/UI/item_icons/vanity_123_helm_a.dds",
"Data/UI/item_icons/vanity_123_shoulders.dds",
"Data/UI/item_icons/vanity_123_shoulders_a.dds",
"Data/UI/item_icons/vanity_124_helm.dds",
"Data/UI/item_icons/vanity_124_helm_a.dds",
"Data/UI/item_icons/vanity_124_shoulders.dds",
"Data/UI/item_icons/vanity_124_shoulders_a.dds",
"Data/UI/item_icons/vanity_125_boots.dds",
"Data/UI/item_icons/vanity_125_boots_a.dds",
"Data/UI/item_icons/vanity_125_chest.dds",
"Data/UI/item_icons/vanity_125_chest_a.dds",
"Data/UI/item_icons/vanity_125_gloves.dds",
"Data/UI/item_icons/vanity_125_gloves_a.dds",
"Data/UI/item_icons/vanity_125_helm.dds",
"Data/UI/item_icons/vanity_125_helm_a.dds",
"Data/UI/item_icons/vanity_125_leggings.dds",
"Data/UI/item_icons/vanity_125_leggings_a.dds",
"Data/UI/item_icons/vanity_125_shoulder.dds",
"Data/UI/item_icons/vanity_125_shoulder_a.dds",
"Data/UI/item_icons/vanity_129_helm.dds",
"Data/UI/item_icons/vanity_129_helm_a.dds",
"Data/UI/item_icons/vanity_129_shoulders.dds",
"Data/UI/item_icons/vanity_129_shoulders_a.dds",
"Data/UI/item_icons/vanity_131_shoulders.dds",
"Data/UI/item_icons/vanity_132_chest.dds",
"Data/UI/item_icons/vanity_132_chest_a.dds",
"Data/UI/item_icons/vanity_133a_boots.dds",
"Data/UI/item_icons/vanity_133a_boots_a.dds",
"Data/UI/item_icons/vanity_133a_helm.dds",
"Data/UI/item_icons/vanity_133a_helm_a.dds",
"Data/UI/item_icons/vanity_133a_shoulder.dds",
"Data/UI/item_icons/vanity_133a_shoulder_a.dds",
"Data/UI/item_icons/vanity_133b_helm.dds",
"Data/UI/item_icons/vanity_133b_helm_a.dds",
"Data/UI/item_icons/vanity_133_boots.dds",
"Data/UI/item_icons/vanity_133_boots_a.dds",
"Data/UI/item_icons/vanity_133_chest.dds",
"Data/UI/item_icons/vanity_133_chest_a.dds",
"Data/UI/item_icons/vanity_133_gloves.dds",
"Data/UI/item_icons/vanity_133_gloves_a.dds",
"Data/UI/item_icons/vanity_133_leggings.dds",
"Data/UI/item_icons/vanity_133_leggings_a.dds",
"Data/UI/item_icons/vanity_133_shoulder.dds",
"Data/UI/item_icons/vanity_133_shoulder_a.dds",
"Data/UI/item_icons/vanity_134_helm.dds",
"Data/UI/item_icons/vanity_134_helm_a.dds",
"Data/UI/item_icons/vanity_134_helm_b.dds",
"Data/UI/item_icons/vanity_134_shoulders.dds",
"Data/UI/item_icons/vanity_134_shoulders_a.dds",
"Data/UI/item_icons/vanity_135_boots.dds",
"Data/UI/item_icons/vanity_135_boots_a.dds",
"Data/UI/item_icons/vanity_135_chest.dds",
"Data/UI/item_icons/vanity_135_chest_a.dds",
"Data/UI/item_icons/vanity_135_gloves_a.dds",
"Data/UI/item_icons/vanity_140_helm.dds",
"Data/UI/item_icons/vanity_140_helm_a.dds",
"Data/UI/item_icons/vanity_141_chest.dds",
"Data/UI/item_icons/vanity_141_chest_a.dds",
"Data/UI/item_icons/vanity_141_helm.dds",
"Data/UI/item_icons/vanity_141_helm_a.dds",
"Data/UI/item_icons/vanity_142_helm.dds",
"Data/UI/item_icons/vanity_142_helm_a.dds",
"Data/UI/item_icons/vanity_143_helm.dds",
"Data/UI/item_icons/vanity_143_helm_a.dds",
"Data/UI/item_icons/vanity_144_helm.dds",
"Data/UI/item_icons/vanity_144_helm_a.dds",
"Data/UI/item_icons/vanity_145.dds",
"Data/UI/item_icons/vanity_145_a.dds",
"Data/UI/item_icons/vanity_146_chest.dds",
"Data/UI/item_icons/vanity_146_chest_a.dds",
"Data/UI/item_icons/vanity_147_chest.dds",
"Data/UI/item_icons/vanity_147_chest_a.dds",
"Data/UI/item_icons/vanity_helm_119.dds",
"Data/UI/item_icons/vanity_helm_119_a.dds",
"Data/UI/item_icons/venom_sack.dds",
"Data/UI/item_icons/venom_sack_a.dds",
"Data/UI/item_icons/venonous_fang.dds",
"Data/UI/item_icons/venonous_fang_a.dds",
"Data/UI/item_icons/veteran_rewards_1.dds",
"Data/UI/item_icons/veteran_rewards_12.dds",
"Data/UI/item_icons/veteran_rewards_18.dds",
"Data/UI/item_icons/veteran_rewards_24.dds",
"Data/UI/item_icons/veteran_rewards_3.dds",
"Data/UI/item_icons/veteran_rewards_6.dds",
"Data/UI/item_icons/vile_potion.dds",
"Data/UI/item_icons/vile_relics.dds",
"Data/UI/item_icons/vile_relics_a.dds",
"Data/UI/item_icons/vile_relics_b.dds",
"Data/UI/item_icons/volatile_potion.dds",
"Data/UI/item_icons/volatile_potion_a.dds",
"Data/UI/item_icons/warming_stone.dds",
"Data/UI/item_icons/wasp_larva.dds",
"Data/UI/item_icons/weaponstone_01.dds",
"Data/UI/item_icons/weaponstone_02.dds",
"Data/UI/item_icons/weaponstone_02_a.dds",
"Data/UI/item_icons/weaponstone_03.dds",
"Data/UI/item_icons/weaponstone_04.dds",
"Data/UI/item_icons/weaponstone_05.dds",
"Data/UI/item_icons/weaponstone_05_a.dds",
"Data/UI/item_icons/weaponstone_05_b.dds",
"Data/UI/item_icons/weaponstone_06.dds",
"Data/UI/item_icons/weaponstone_07.dds",
"Data/UI/item_icons/web1.dds",
"Data/UI/item_icons/web2.dds",
"Data/UI/item_icons/web3.dds",
"Data/UI/item_icons/web3a.dds",
"Data/UI/item_icons/web4.dds",
"Data/UI/item_icons/web5.dds",
"Data/UI/item_icons/web6.dds",
"Data/UI/item_icons/web6aa.dds",
"Data/UI/item_icons/web6b.dds",
"Data/UI/item_icons/web6c.dds",
"Data/UI/item_icons/whetstone1.dds",
"Data/UI/item_icons/whetstone1_a.dds",
"Data/UI/item_icons/whetstone2.dds",
"Data/UI/item_icons/whetstone2_a.dds",
"Data/UI/item_icons/whetstone3.dds",
"Data/UI/item_icons/whetstone3_a.dds",
"Data/UI/item_icons/whetstone4.dds",
"Data/UI/item_icons/whetstone4_a.dds",
"Data/UI/item_icons/whetstone5.dds",
"Data/UI/item_icons/whetstone5_a.dds",
"Data/UI/item_icons/white_freemarch_rabbit.dds",
"Data/UI/item_icons/white_grape.dds",
"Data/UI/item_icons/wild_freemarch_snipe.dds",
"Data/UI/item_icons/wild_freemarch_snipe_a.dds",
"Data/UI/item_icons/wild_potion.dds",
"Data/UI/item_icons/wild_potion_a.dds",
"Data/UI/item_icons/wineskin.dds",
"Data/UI/item_icons/wine_bottle.dds",
"Data/UI/item_icons/wine_bottle2.dds",
"Data/UI/item_icons/wine_bottle2_a.dds",
"Data/UI/item_icons/wine_bottle2_b.dds",
"Data/UI/item_icons/wine_bottle2_c.dds",
"Data/UI/item_icons/wine_bottle2_d.dds",
"Data/UI/item_icons/wine_bottle2_e.dds",
"Data/UI/item_icons/wine_bottle_a.dds",
"Data/UI/item_icons/witchstone_01.dds",
"Data/UI/item_icons/witchstone_01_a.dds",
"Data/UI/item_icons/witchstone_02.dds",
"Data/UI/item_icons/witchstone_02_a.dds",
"Data/UI/item_icons/witchstone_02_b.dds",
"Data/UI/item_icons/witchstone_03.dds",
"Data/UI/item_icons/witchstone_03_a.dds",
"Data/UI/item_icons/witchstone_04.dds",
"Data/UI/item_icons/witchstone_04_a.dds",
"Data/UI/item_icons/witchweave_cloth.dds",
"Data/UI/item_icons/witchweave_cloth_a.dds",
"Data/UI/item_icons/wolf_fur2.dds",
"Data/UI/item_icons/wolf_fur2_a.dds",
"Data/UI/item_icons/wolf_fur_2.dds",
"Data/UI/item_icons/wood1.dds",
"Data/UI/item_icons/wood10.dds",
"Data/UI/item_icons/wood10a.dds",
"Data/UI/item_icons/wood1a.dds",
"Data/UI/item_icons/wood2.dds",
"Data/UI/item_icons/wood3.dds",
"Data/UI/item_icons/wood4.dds",
"Data/UI/item_icons/wood5.dds",
"Data/UI/item_icons/wood5a.dds",
"Data/UI/item_icons/wood6.dds",
"Data/UI/item_icons/wood6a.dds",
"Data/UI/item_icons/wood7.dds",
"Data/UI/item_icons/wood7a.dds",
"Data/UI/item_icons/wood8.dds",
"Data/UI/item_icons/wood9.dds",
"Data/UI/item_icons/wood9a.dds",
"Data/UI/item_icons/wooden_flask.dds",
"Data/UI/item_icons/wooden_periapt.dds",
"Data/UI/item_icons/wood_gnarl.dds",
"Data/UI/item_icons/wood_shard.dds",
"Data/UI/item_icons/wood_splinter.dds",
"Data/UI/item_icons/wool_cloth.dds",
"Data/UI/item_icons/worn_artifact_a.dds",
"Data/UI/item_icons/worn_artifact_b.dds",
"Data/UI/item_icons/worn_dice.dds",
"Data/UI/item_icons/wrath_weed.dds",
"Data/UI/item_icons/wrath_weed_a.dds"
}
